<?php
/**
 * User Ultra Hook.
 *
 * @package User Activity Log PRO
 */

if ( ! function_exists( 'ualp_hook_add_new_custom_profile_field' ) ) {
	/**
	 * Fires when custom profile field created.
	 */
	function ualp_hook_add_new_custom_profile_field() {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'wp_ajax_add_new_custom_profile_field', $log_hook_list ) ) {
			$obj_type    = 'User Ultra Membership';
			$action      = 'Custom Profile field created.';
			$post_id     = '';
			$post_title  = '';
			$hook        = 'wp_ajax_add_new_custom_profile_field';
			$description = "$obj_type : $action";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}
add_action( 'wp_ajax_add_new_custom_profile_field', 'ualp_hook_add_new_custom_profile_field' );

if ( ! function_exists( 'ualp_hook_save_fields_settings' ) ) {
	/**
	 * Fires when profile fields saved.
	 */
	function ualp_hook_save_fields_settings() {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'wp_ajax_save_fields_settings', $log_hook_list ) ) {
			$obj_type    = 'User Ultra Membership';
			$action      = 'Profile field updated successfully.';
			$post_id     = '';
			$post_title  = '';
			$hook        = 'wp_ajax_save_fields_settings';
			$description = "$obj_type : $action";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}
add_action( 'wp_ajax_save_fields_settings', 'ualp_hook_save_fields_settings' );

if ( ! function_exists( 'ualp_hook_delete_profile_field' ) ) {
	/**
	 * Fires when profile field deleted.
	 */
	function ualp_hook_delete_profile_field() {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'wp_ajax_delete_profile_field', $log_hook_list ) ) {
			$obj_type    = 'User Ultra Membership';
			$action      = 'Profile field deleted successfully.';
			$post_id     = '';
			$post_title  = '';
			$hook        = 'wp_ajax_delete_profile_field';
			$description = "$obj_type : $action";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}
add_action( 'wp_ajax_delete_profile_field', 'ualp_hook_delete_profile_field' );

if ( ! function_exists( 'ualp_hook_sort_fileds_list' ) ) {
	/**
	 * Fires when fields sorted.
	 */
	function ualp_hook_sort_fileds_list() {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'wp_ajax_sort_fileds_list', $log_hook_list ) ) {
			$obj_type    = 'User Ultra Membership';
			$action      = 'Fields order updated successfully.';
			$post_id     = '';
			$post_title  = '';
			$hook        = 'wp_ajax_sort_fileds_list';
			$description = "$obj_type : $action";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}
add_action( 'wp_ajax_sort_fileds_list', 'ualp_hook_sort_fileds_list' );

if ( ! function_exists( 'ualp_hook_custom_fields_reset' ) ) {
	/**
	 * Fires when custom fields reset.
	 */
	function ualp_hook_custom_fields_reset() {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'wp_ajax_custom_fields_reset', $log_hook_list ) ) {
			$obj_type    = 'User Ultra Membership';
			$action      = 'Custom Profile fields reseted successfully.';
			$post_id     = '';
			$post_title  = '';
			$hook        = 'wp_ajax_custom_fields_reset';
			$description = "$obj_type : $action";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}
add_action( 'wp_ajax_custom_fields_reset', 'ualp_hook_custom_fields_reset' );

if ( ! function_exists( 'ualp_hook_create_uploader_folder' ) ) {
	/**
	 * Fires when uploader folder created.
	 */
	function ualp_hook_create_uploader_folder() {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'wp_ajax_create_uploader_folder', $log_hook_list ) ) {
			$obj_type    = 'User Ultra Membership';
			$action      = 'Uploader folder created.';
			$post_id     = '';
			$post_title  = '';
			$hook        = 'wp_ajax_create_uploader_folder';
			$description = "$obj_type : $action";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}
add_action( 'wp_ajax_create_uploader_folder', 'ualp_hook_create_uploader_folder' );

if ( ! function_exists( 'ualp_hook_user_ultra_settings' ) ) {
	/**
	 * Fires when settings updated.
	 *
	 * @param string $option Option.
	 * @param string $oldvalue Old Value.
	 * @param string $newvalue New Value.
	 */
	function ualp_hook_user_ultra_settings( $option, $oldvalue, $newvalue ) {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'user_ultra_settings', $log_hook_list ) ) {
			if ( $oldvalue != $newvalue ) {
				$whitelisted_options = ualp_get_user_ultra_options();
				if ( ! in_array( $option, $whitelisted_options ) ) {
					return;
				}
				$obj_type    = 'User Ultra Membership';
				$action      = 'Settings updated';
				$post_id     = '';
				$post_title  = $option;
				$hook        = 'user_ultra_settings';
				$description = "$obj_type : $action";
				$uactid      = ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
				if ( 0 != $uactid ) {
					ualp_add_wordpress_settings_log_detail( $uactid, $oldvalue, $newvalue, $obj_type, $option );
				}
			}
		}
	}
}
add_action( 'updated_option', 'ualp_hook_user_ultra_settings', 50, 3 );


if ( ! function_exists( 'ualp_get_user_ultra_options' ) ) {
	/**
	 * Get User Ultra Options.
	 */
	function ualp_get_user_ultra_options() {
		$whitelisted_options = apply_filters(
			'ualp_get_user_ultra_options',
			array(
				'xoousersultra_my_account_page',
				'xoousersultra_auto_page_creation',
				'xoousersultra_already_rated_ultra',
				'xoousersultra_pro_annuncement_1463',
				'xoousersultra_auto_page_creation',
				'xoousersultra_custom_css',
				'userultra_options',
				'userultra',
				'userultra_default_user_tabs',
				'uultra_cached_results',
				'get_twitter_auth_url',
			)
		);
		return $whitelisted_options;
	}
}

if ( ! function_exists( 'ualp_hook_package_add_new' ) ) {
	/**
	 * Fires when membership Package created.
	 */
	function ualp_hook_package_add_new() {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'usersultra-bulk-action_inbox' ) ) {
			if ( in_array( 'wp_ajax_package_add_new', $log_hook_list ) ) {
				$obj_type    = 'User Ultra Membership';
				$action      = 'Membership Package created successfully.';
				$post_id     = '';
				$post_title  = ( isset( $_POST['p_name'] ) && '' != $_POST['p_name'] ) ? sanitize_text_field( wp_unslash( $_POST['p_name'] ) ) : '';
				$hook        = 'wp_ajax_package_add_new';
				$description = "$obj_type : '$post_title' $action";
				ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
			}
		}
	}
}
add_action( 'wp_ajax_package_add_new', 'ualp_hook_package_add_new' );

if ( ! function_exists( 'ualp_hook_package_delete' ) ) {
	/**
	 * Fires when membership Package deleted.
	 */
	function ualp_hook_package_delete() {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'usersultra-bulk-action_inbox' ) ) {
			if ( in_array( 'wp_ajax_package_delete', $log_hook_list ) ) {
				$obj_type    = 'User Ultra Membership';
				$action      = 'Membership Package deleted successfully.';
				$post_id     = ( isset( $_POST['p_id'] ) && '' != $_POST['p_id'] ) ? sanitize_text_field( wp_unslash( $_POST['p_id'] ) ) : '';
				$package     = XooPaypalPayment::get_package( $post_id );
				$post_title  = $package->package_name;
				$hook        = 'wp_ajax_package_delete';
				$description = "$obj_type : '$post_title' $action";
				ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
			}
		}
	}
}
add_action( 'wp_ajax_package_delete', 'ualp_hook_package_delete' );

if ( ! function_exists( 'ualp_hook_package_edit_confirm' ) ) {
	/**
	 * Fires when membership Package updated.
	 */
	function ualp_hook_package_edit_confirm() {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'usersultra-bulk-action_inbox' ) ) {
			if ( in_array( 'wp_ajax_package_edit_confirm', $log_hook_list ) ) {
				$obj_type    = 'User Ultra Membership';
				$action      = 'Membership Package updated successfully.';
				$post_id     = '';
				$post_title  = ( isset( $_POST['p_name'] ) && '' != $_POST['p_name'] ) ? sanitize_text_field( wp_unslash( $_POST['p_name'] ) ) : '';
				$hook        = 'wp_ajax_package_edit_confirm';
				$description = "$obj_type : '$post_title' $action";
				ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
			}
		}
	}
}
add_action( 'wp_ajax_package_edit_confirm', 'ualp_hook_package_edit_confirm' );

if ( ! function_exists( 'ualp_hook_sync_users' ) ) {
	/**
	 * Fires when sync users.
	 */
	function ualp_hook_sync_users() {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'wp_ajax_sync_users', $log_hook_list ) ) {
			$obj_type    = 'User Ultra Membership';
			$action      = 'Sync users';
			$post_id     = '';
			$post_title  = '';
			$hook        = 'wp_ajax_sync_users';
			$description = "$obj_type : $action";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}
add_action( 'wp_ajax_sync_users', 'ualp_hook_sync_users' );

if ( ! function_exists( 'ualp_hook_user_resend_activation_link' ) ) {
	/**
	 * Fires when resend Activation Link.
	 */
	function ualp_hook_user_resend_activation_link() {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'wp_ajax_user_resend_activation_link', $log_hook_list ) ) {
			$obj_type    = 'User Ultra Membership';
			$action      = 'Resend Activation Link';
			$post_id     = '';
			$post_title  = '';
			$hook        = 'wp_ajax_user_resend_activation_link';
			$description = "$obj_type : $action";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}
add_action( 'wp_ajax_user_resend_activation_link', 'ualp_hook_user_resend_activation_link' );



if ( ! function_exists( 'ualp_hook_delete_uultra_transtient' ) ) {
	/**
	 * Fires when Delete Users Ultra Transients.
	 */
	function ualp_hook_delete_uultra_transtient() {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'wp_ajax_delete_uultra_transtient', $log_hook_list ) ) {
			$obj_type    = 'User Ultra Membership';
			$post_title  = 'Users Ultra Transients';
			$post_id     = '';
			$action      = 'Deleted';
			$hook        = 'wp_ajax_delete_uultra_transtient';
			$description = "$obj_type : $post_title $action";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}
add_action( 'wp_ajax_delete_uultra_transtient', 'ualp_hook_delete_uultra_transtient' );


if ( ! function_exists( 'ualp_hook_delete_uultra_stats' ) ) {
	/**
	 * Fires when Clean the Users Ultra Stats.
	 */
	function ualp_hook_delete_uultra_stats() {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'wp_ajax_delete_uultra_stats', $log_hook_list ) ) {
			$obj_type    = 'User Ultra Membership';
			$action      = 'Clean';
			$post_id     = '';
			$post_title  = 'Users Ultra Stats';
			$hook        = 'wp_ajax_delete_uultra_stats';
			$description = "$obj_type : $action $post_title";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}
add_action( 'wp_ajax_delete_uultra_stats', 'ualp_hook_delete_uultra_stats' );


if ( ! function_exists( 'ualp_hook_delete_uultra_reviews' ) ) {
	/**
	 * Fires when Clean the Users Ultra Ratings.
	 */
	function ualp_hook_delete_uultra_reviews() {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'wp_ajax_delete_uultra_reviews', $log_hook_list ) ) {
			$obj_type    = 'User Ultra Membership';
			$action      = 'Clean';
			$post_id     = '';
			$post_title  = 'Users Ultra Ratings';
			$hook        = 'wp_ajax_delete_uultra_reviews';
			$description = "$obj_type : $action $post_title";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}
add_action( 'wp_ajax_delete_uultra_reviews', 'ualp_hook_delete_uultra_reviews' );

if ( ! function_exists( 'ualp_hook_conf_photo_cate' ) ) {
	/**
	 * Fires when update photo category.
	 */
	function ualp_hook_conf_photo_cate() {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'wp_ajax_edit_photo_cate_conf', $log_hook_list ) ) {
			if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'usersultra-bulk-action_inbox' ) ) {
				if ( isset( $_POST['cate_id'] ) && '' != $_POST['cate_id'] && isset( $_POST['cate_name'] ) && '' != $_POST['cate_name'] ) {
					$cate_id     = isset( $_POST['cate_id'] ) ? sanitize_text_field( wp_unslash( $_POST['cate_id'] ) ) : '';
					$cate_name   = isset( $_POST['cate_name'] ) ? sanitize_text_field( wp_unslash( $_POST['cate_name'] ) ) : '';
					$obj_type    = 'User Ultra Membership';
					$post_title  = $cate_name;
					$post_id     = $cate_id;
					$action      = 'Photo category updated successfully';
					$hook        = 'wp_ajax_edit_photo_cate_conf';
					$description = "$obj_type : '$post_title' $action";
					ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
				}
			}
		}
	}
}
add_action( 'wp_ajax_edit_photo_cate_conf', 'ualp_hook_conf_photo_cate' );

if ( ! function_exists( 'ualp_hook_photo_cate_delete' ) ) {
	/**
	 * Fires when delete photo category.
	 */
	function ualp_hook_photo_cate_delete() {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'usersultra-bulk-action_inbox' ) ) {
			if ( in_array( 'wp_ajax_edit_photo_cate_del', $log_hook_list ) ) {
				global $wpdb;
				$cate_id        = isset( $_POST['cate_id'] ) ? sanitize_text_field( wp_unslash( $_POST['cate_id'] ) ) : '';
				$photo_cat_name = $wpdb->get_var( 'select photo_cat_name from ' . $wpdb->prefix . "usersultra_photo_categories WHERE `photo_cat_id` = '" . (int) $cate_id . "'  " );
				$obj_type       = 'User Ultra Membership';
				$post_title     = $photo_cat_name;
				$post_id        = $cate_id;
				$action         = 'Photo category deleted successfully';
				$hook           = 'wp_ajax_edit_photo_cate_del';
				$description    = "$obj_type : '$post_title' $action";
				ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
			}
		}
	}
}
add_action( 'wp_ajax_edit_photo_cate_del', 'ualp_hook_photo_cate_delete' );

if ( ! function_exists( 'ualp_hook_edit_photo_cate' ) ) {
	/**
	 * Fires when edit photo category.
	 */
	function ualp_hook_edit_photo_cate() {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'usersultra-bulk-action_inbox' ) ) {
			if ( in_array( 'wp_ajax_edit_photo_cate', $log_hook_list ) ) {
				global $wpdb;
				$cate_id        = isset( $_POST['cate_id'] ) ? sanitize_text_field( wp_unslash( $_POST['cate_id'] ) ) : '';
				$photo_cat_name = $wpdb->get_var( 'select photo_cat_name from ' . $wpdb->prefix . "usersultra_photo_categories WHERE `photo_cat_id` = '" . (int) $cate_id . "'  " );
				$obj_type       = 'User Ultra Membership';
				$post_title     = $photo_cat_name;
				$post_id        = $cate_id;
				$action         = 'Photo category edited successfully';
				$hook           = 'wp_ajax_edit_photo_cate';
				$description    = "$obj_type : '$post_title' $action";
				ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
			}
		}
	}
}
add_action( 'wp_ajax_edit_photo_cate', 'ualp_hook_edit_photo_cate' );

if ( ! function_exists( 'ualp_hook_add_photocategory' ) ) {
	/**
	 * Fires when create photo category.
	 */
	function ualp_hook_add_photocategory() {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( isset( $_GET['page'] ) && 'uultra-photocategory' == $_GET['page'] && in_array( 'userultra_admin_menu_hook', $log_hook_list ) ) {
			if ( isset( $_POST['submit'] ) && isset( $_POST['add-category'] ) && 'add-cate' == $_POST['add-category'] ) {
				if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'usersultra-bulk-action_inbox' ) ) {
					if ( isset( $_POST['photo_cat_name'] ) ) {
						$photo_cat_name = isset( $_POST['photo_cat_name'] ) ? sanitize_text_field( wp_unslash( $_POST['photo_cat_name'] ) ) : '';
						$new_array      = array(
							'photo_cat_id'   => null,
							'photo_cat_name' => $photo_cat_name,
						);
						$obj_type       = 'User Ultra Membership';
						$post_title     = $photo_cat_name;
						$post_id        = '';
						$action         = 'Photo category created successfully';
						$hook           = 'userultra_admin_menu_hook';
						$description    = "$obj_type : '$post_title' $action";
						ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
					}
				}
			}
		}
	}
}

add_action( 'admin_init', 'ualp_hook_add_photocategory' );

if ( ! function_exists( 'ualp_hook_sync_keep_existing_field' ) ) {
	/**
	 * Fires when Auto sync with woocommerce.
	 */
	function ualp_hook_sync_keep_existing_field() {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'user_ultra_settings', $log_hook_list ) ) {
			global $pagenow;
			$tab = isset( $_GET['tab'] ) ? sanitize_text_field( wp_unslash( $_GET['tab'] ) ) : '';
			if ( 'admin.php' == $pagenow && isset( $_GET['page'] ) && ( 'import' == $tab ) && in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
				if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'usersultra-bulk-action_inbox' ) ) {
					if ( ( isset( $_REQUEST['sync'] ) && 'woocommerce' == $_REQUEST['sync'] ) && ! isset( $_POST['submit'] ) && ! isset( $_POST['uultra-add'] ) && ! isset( $_POST['reset-options'] ) && ! isset( $_POST['reset-options-fields'] ) ) {
						$obj_type    = 'User Ultra Membership';
						$post_title  = 'Sync and keep existing fields.';
						$post_id     = '';
						$action      = 'Auto sync with woocommerce';
						$hook        = 'user_ultra_settings';
						$description = "$obj_type : $action - $post_title";
						ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
					}
				}
			}
		}
	}
}

add_action( 'init', 'ualp_hook_sync_keep_existing_field' );

if ( ! function_exists( 'ualp_hook_sync_delete_existing_field' ) ) {
	/**
	 * Fires when Auto sync with woocommerce.
	 */
	function ualp_hook_sync_delete_existing_field() {
		global $pagenow;
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'user_ultra_settings', $log_hook_list ) ) {
			$tab = isset( $_GET['tab'] ) ? sanitize_text_field( wp_unslash( $_GET['tab'] ) ) : '';
			if ( 'admin.php' == $pagenow && isset( $_GET['page'] ) && ( 'import' == $tab ) && in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
				if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'usersultra-bulk-action_inbox' ) ) {
					if ( ( isset( $_REQUEST['sync'] ) && 'woocommerce_clean' == $_REQUEST['sync'] ) && ! isset( $_POST['submit'] ) && ! isset( $_POST['uultra-add'] ) && ! isset( $_POST['reset-options'] ) && ! isset( $_POST['reset-options-fields'] ) ) {
						$obj_type    = 'User Ultra Membership';
						$post_title  = 'Sync and delete existing fields.';
						$post_id     = '';
						$action      = 'Auto sync with woocommerce';
						$hook        = 'user_ultra_settings';
						$description = "$obj_type : $action - $post_title";
						ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
					}
				}
			}
		}
	}
}

add_action( 'init', 'ualp_hook_sync_delete_existing_field' );


if ( ! function_exists( 'ualp_hook_import_user' ) ) {
	/**
	 * Fires when Auto sync with woocommerce.
	 */
	function ualp_hook_import_user() {
		global $pagenow;
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'user_ultra_settings', $log_hook_list ) ) {
			$tab = isset( $_GET['tab'] ) ? sanitize_text_field( wp_unslash( $_GET['tab'] ) ) : '';
			if ( 'admin.php' == $pagenow && isset( $_GET['page'] ) && ( 'import' == $tab ) ) {
				if ( isset( $_POST['uultra-form-cvs-form-conf'] ) ) {
					$obj_type   = 'User Ultra Membership';
					$post_title = 'Users Imported successfully';
					$post_id    = '';
					$action     = 'Import';
					$hook       = 'user_ultra_settings';
					if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'usersultra-bulk-action_inbox' ) ) {
						if ( isset( $_POST['uultra-send-welcome-email'] ) && 1 == $_POST['uultra-send-welcome-email'] ) {
							$post_title .= ' and Send Welcome Email';
						}
						if ( isset( $_POST['uultra-activate-account'] ) && '' != $_POST['uultra-activate-account'] ) {
							if ( 'active' == $_POST['uultra-activate-account'] ) {
								$post_title .= ' and Activate account automatically.';
							} elseif ( 'pending' == $_POST['uultra-activate-account'] ) {
								$post_title .= ' and Send Activation Link.';
							}
						}
					}
					$description = "$obj_type : $post_title";
					ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
				}
			}
		}
	}
}

add_action( 'init', 'ualp_hook_import_user' );
