<?php
/**
 * User Activity Hooks.
 *
 * @package User Activity Log PRO
 */

/**
 * Exit if accessed directly.
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

$old_user_meta = '';

if ( ! function_exists( 'ualp_insert_log' ) ) {
	/**
	 * Insert record into wp_user_activity table.
	 *
	 * @param int    $post_id Post ID.
	 * @param string $post_title Post Title.
	 * @param string $obj_type Object Type (Plugin, Post, User etc.).
	 * @param string $current_user_name current user id.
	 * @param string $action current user's activity name.
	 * @param string $hook Hook.
	 * @param string $description Description.
	 */
	function ualp_insert_log( $post_id, $post_title, $obj_type, $current_user_name, $action, $hook, $description ) {
		global $wpdb, $externaldb;
		$post_title    = addslashes( $post_title );
		$modified_date = current_time( 'mysql' );
		$get_ip_type   = get_option( 'ualpIPtypes', 'REMOTE_ADDR' );
		if ( $get_ip_type ) {
			if ( 'REMOTE_ADDR' == $get_ip_type ) {
				$ips = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
			} elseif ( 'HTTP_X_FORWARDED_FOR' == $get_ip_type ) {
				$ips = isset( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) : '';
			} elseif ( 'HTTP_X_REAL_IP' == $get_ip_type ) {
				$ips = isset( $_SERVER['HTTP_X_REAL_IP'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_REAL_IP'] ) ) : '';
			} else {
				$ips = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
			}
		} else {
			$ips = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
		}
		$user_agents = isset( $_SERVER['HTTP_USER_AGENT'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_USER_AGENT'] ) ) : '';
		$ip          = '';
		$user_agent  = '';
		if ( ualp_allow_ip() ) {
			$ip = $ips;
		}
		$username        = get_user_by( 'login', $current_user_name );
		$user_mail       = '';
		$current_user_id = '';
		$user_role       = '';
		if ( isset( $username ) && ! empty( $username ) ) {
			$current_user_name = $username->data->display_name;
			$user_mail         = $username->data->user_email;
			$current_user_id   = $username->data->ID;
			if ( ! empty( $username->roles ) && is_array( $username->roles ) ) {
				if ( count( $username->roles ) == 1 ) {
					$user_role = $username->roles[0];
				} else {
					$user_role = $username->roles;
					foreach ( $user_role as $all_rols ) {
						$user_role .= $all_rols . ' ,';
					}
				}
			}
		} else {
			$current_user_name = 'Guest';
			$user_mail         = '';
			$current_user_id   = '';
			$user_role         = '';
		}
		$i                         = 0;
		$flag                      = 0;
		$currenr_role              = array();
		$exclude_current_user_name = '';
		if ( isset( $username ) && ! empty( $username ) ) {
			$current_user              = get_user_by( 'login', $username->user_login );
			$exclude_current_user_name = $username->user_login;
			if ( ! empty( $current_user->roles ) && is_array( $current_user->roles ) ) {
				foreach ( $current_user->roles as $role ) {
					$currenr_role[] = $role; }
			}
		}
		$all_user_id    = array();
		$excludelogdata = get_option( 'exclude_log_data' );
		if ( is_array( $excludelogdata ) && ! empty( $excludelogdata ) ) {
			$users1 = get_users();
			foreach ( $users1 as $user_id ) {
				$all_user_id[] = $user_id->user_login;
			}
			foreach ( $excludelogdata as $exclude_log_data_single ) {
				$row_count         = count( $exclude_log_data_single );
				$exclude_logs_hook = $exclude_log_data_single['exclude_logs_hook'];
				$exclude_log_users = $exclude_log_data_single['exclude_log_users'];
				$exclude_logs_ips  = $exclude_log_data_single['exclude_logs_ips'];
				if ( empty( $exclude_logs_ips ) ) {
					$exclude_logs_ips = array();}
				if ( '' != $exclude_log_users ) {
					if ( in_array( $exclude_log_users, $all_user_id ) ) {
						$exclude_log_user     = get_user_by( 'login', $exclude_log_users );
						$exclude_user         = $exclude_log_user->user_login;
						$exclude_current_user = $exclude_current_user_name;
					} elseif ( ! in_array( $exclude_log_users, $all_user_id ) ) {
						if ( '0' == $exclude_log_users ) {
							$exclude_user = 'guest';
							if ( isset( $username ) && ! empty( $username ) ) {
								$exclude_current_user = $exclude_current_user_name;
							} else {
								$exclude_current_user = 'guest';
							}
						} else {
							$exclude_current_user = isset( $currenr_role[0] ) ? $currenr_role[0] : '';
							$exclude_user         = $exclude_log_users;
						}
					}
				}
				if ( 'all' != $exclude_log_users && 'all' != $exclude_logs_hook ) {
					if ( 1 == $row_count && 0 == $i ) {
						if ( ! empty( $exclude_logs_ips ) && $hook == $exclude_logs_hook && $exclude_current_user == $exclude_user && in_array( $ip, $exclude_logs_ips ) ) {
							$flag = 1;
						} elseif ( empty( $exclude_logs_ips ) && $hook == $exclude_logs_hook && $exclude_current_user == $exclude_user ) {
							$flag = 1;
						}
					} elseif ( $row_count > 1 && 1 != $flag ) {
						if ( ! empty( $exclude_logs_ips ) && $hook == $exclude_logs_hook && $exclude_current_user == $exclude_user && in_array( $ip, $exclude_logs_ips ) ) {
							$flag = 1;
						} elseif ( empty( $exclude_logs_ips ) && $hook == $exclude_logs_hook && $exclude_current_user == $exclude_user ) {
							$flag = 1;
						}
					}
				} elseif ( 'all' == $exclude_log_users && 'all' != $exclude_logs_hook ) {
					if ( 1 == $row_count && 0 == $i ) {
						if ( ! empty( $exclude_logs_ips ) && $hook == $exclude_logs_hook && in_array( $ip, $exclude_logs_ips ) ) {
							$flag = 1;
						} elseif ( empty( $exclude_logs_ips ) && $hook == $exclude_logs_hook ) {
							$flag = 1;
						}
					} elseif ( $row_count > 1 && 1 != $flag ) {
						if ( ! empty( $exclude_logs_ips ) && $hook == $exclude_logs_hook && in_array( $ip, $exclude_logs_ips ) ) {
							$flag = 1;
						} elseif ( empty( $exclude_logs_ips ) && $hook == $exclude_logs_hook ) {
							$flag = 1;
						}
					}
				} elseif ( 'all' != $exclude_log_users && 'all' == $exclude_logs_hook ) {
					if ( 1 == $row_count && 0 == $i ) {
						if ( ! empty( $exclude_logs_ips ) && $exclude_current_user == $exclude_user && in_array( $ip, $exclude_logs_ips ) ) {
							$flag = 1;
						} elseif ( empty( $exclude_logs_ips ) && $exclude_current_user == $exclude_user ) {
							$flag = 1;
						}
					} elseif ( $row_count > 1 && 1 != $flag ) {
						if ( ! empty( $exclude_logs_ips ) && $exclude_current_user == $exclude_user && in_array( $ip, $exclude_logs_ips ) ) {
							$flag = 1;
						} elseif ( empty( $exclude_logs_ips ) && $exclude_current_user == $exclude_user ) {
							$flag = 1;
						}
					}
				} elseif ( 'all' == $exclude_log_users && 'all' == $exclude_logs_hook ) {
					if ( ! empty( $exclude_logs_ips ) && 1 != $flag && in_array( $ip, $exclude_logs_ips ) ) {
						$flag = 1;
					} elseif ( empty( $exclude_logs_ips ) ) {
						$flag = 1;
					}
				}
				$i++;
			}
		}

		$site_id = get_current_blog_id();
		ualp_create_table();
		ualp_create_detail_table();
		ualp_create_external_table();
		ualp_create_detail_external_table();
		ualp_create_login_external_table();
		ualp_create_email_notification_table();
		ualp_create_sms_notification_table();
		ualp_create_email_notification_external_table();
		ualp_create_schedule_report();
		ualp_create_schedule_external_report();

		/**
		 * Ualpbeforeinsertloghook.
		 */
		do_action( 'ualpbeforeinsertlog' );
		$country   = '';
		$t_session = '';
		if ( ualp_allow_country() ) {
			$country = ( isset( $ip ) && ( '' != $ip ) ) ? ualp_ip_info( $ip, 'country' ) : '';
		}
		if ( ualp_allow_user_agent() ) {
			$user_agent = $user_agents;
			$user_agent = str_replace( ';', ',', $user_agent );
		}
		$description_result = $externaldb->query( 'SHOW COLUMNS FROM ' . UALP_TABLENAME . " LIKE 'description'" );
		if ( 1 == $description_result && 'wp_head' == $hook ) {
			$alter_table_query = $externaldb->query(
				'ALTER TABLE ' . UALP_TABLENAME
				. ' MODIFY description text'
				. ' AFTER hook'
			);
		}
		if ( '0' == $flag ) {
			if ( is_user_logged_in() ) {
				$t_session = wp_get_session_token();
			}
			$insert_query = $externaldb->insert(
				UALP_TABLENAME,
				array(
					'post_id'       => $post_id,
					'favorite'      => '0',
					'post_title'    => $post_title,
					'user_id'       => $current_user_id,
					'user_name'     => $current_user_name,
					'user_role'     => $user_role,
					'user_email'    => $user_mail,
					'ip_address'    => $ip,
					'country'       => $country,
					'user_agent'    => $user_agent,
					'modified_date' => $modified_date,
					'object_type'   => $obj_type,
					'action'        => $action,
					'hook'          => $hook,
					'description'   => $description,
					't_session'     => $t_session,
					'site_id'       => $site_id,
				)
			);
			/* [1] Email Trigger Start */
			if ( '0' != $insert_query ) {
				if ( is_multisite() ) {
					$all_trigger_email_content = $wpdb->get_results( "SELECT * FROM {$wpdb->base_prefix}ualp_email_notification" );
				} else {
					$all_trigger_email_content = $wpdb->get_results( "SELECT * FROM {$wpdb->prefix}ualp_email_notification" );
				}
				if ( ! empty( $all_trigger_email_content ) ) {
					$email_content       = '';
					$custom_trigger_q    = '';
					$email_address       = '';
					$trigger_count       = count( $all_trigger_email_content );
					$j                   = 1;
					$email_template      = 0;
					$send_notification_m = true;

					$compare_trigger_left_value = '';
					foreach ( $all_trigger_email_content as $trigger_email_content ) {
						$global_trigger_sent_mail = 1;
						$custom_trigger_datas     = maybe_unserialize( $trigger_email_content->custom_trigger_data );
						$row_count                = count( $custom_trigger_datas );
						$email_template           = $trigger_email_content->email_template;
						$email_address            = $trigger_email_content->email_address;
						$email_content            = $trigger_email_content->email_content;
						$send_email               = true;
						$first_false              = true;
						$i                        = 1;
						foreach ( $custom_trigger_datas as $custom_trigger_data_single ) {
							$flag                             = 0;
							$custom_trigger_with_opertor_main = 'AND';
							$custom_trigger_operator_with     = $custom_trigger_data_single['custom_trigger_operator_with'];
							$custom_trigger_name              = $custom_trigger_data_single['custom_trigger_name'];
							$custom_trigger_with              = $custom_trigger_data_single['custom_trigger_with'];
							$compare_trigger_right_value      = $custom_trigger_data_single['custom_trigger_value'];
							if ( 1 == $i ) {
								$custom_trigger_with_opertor_main = 'AND';
							} else {
								$custom_trigger_with_opertor_main = $custom_trigger_operator_with;
							}
							switch ( $custom_trigger_name ) {
								case 'user_id':
									$compare_trigger_left_value = $current_user_id;
									break;
								case 'user_name':
									$compare_trigger_left_value = $current_user_name;
									break;
								case 'ip_address':
									$compare_trigger_left_value = $ip;
									break;
								case 'user_role':
									$compare_trigger_left_value = $user_role;
									break;
								case 'hook':
									$compare_trigger_left_value = $hook;
									break;
							}

							if ( 1 == $global_trigger_sent_mail ) {
								if ( 'AND' == $custom_trigger_with_opertor_main ) {
									if ( 'equal' == $custom_trigger_with && '' != $compare_trigger_left_value && ! empty( $compare_trigger_right_value ) ) {
										if ( in_array( $compare_trigger_left_value, $compare_trigger_right_value ) ) {
											$flag = 1;
										}
									} elseif ( 'not equal' == $custom_trigger_with && '' != $compare_trigger_left_value && ! empty( $compare_trigger_right_value ) ) {
										if ( ! in_array( $compare_trigger_left_value, $compare_trigger_right_value ) ) {
											$flag = 1;
										}
									}
								}
								if ( 1 == $row_count && 1 == $i && 0 == $flag ) {
									$global_trigger_sent_mail = 0;
								} elseif ( $row_count > 1 && 1 == $i && '0' == $flag && 'AND' == $custom_trigger_with_opertor_main ) {
									$first_false = false;
								} elseif ( $row_count > 1 && $i > 1 && '0' == $flag && 'AND' == $custom_trigger_with_opertor_main ) {
									$global_trigger_sent_mail = 0;
									$first_false              = false;
								}
							}
							$i++;
							if ( '0' == $flag ) {
								if ( 'AND' == $custom_trigger_with_opertor_main ) {
									$global_trigger_sent_mail = 0;
								}
								break;
							}
						}

						if ( 1 == $global_trigger_sent_mail && false == $first_false ) {
							$global_trigger_sent_mail = 0;
						}
						if ( 1 == $global_trigger_sent_mail ) {
							$mail_msg = get_option( 'ualpEmailMessage' );
							if ( 0 == $email_template ) {
								$email_content = $mail_msg;
							}
							if ( '' != $username ) {
								$current_user_name = $username->user_login;
								$firstname         = ucfirst( $username->user_firstname );
								$lastname          = ucfirst( $username->user_lastname );
								$user_firstnm      = ! empty( $firstname ) ? ucfirst( $firstname ) : '-';
								$user_lastnm       = ! empty( $lastname ) ? $lastname : '-';
								$user_email        = ! empty( $username->user_email ) ? $username->user_email : '-';
								ualp_email_trigger_body( $current_user_name, $user_firstnm, $user_lastnm, $user_email, $description, $email_address, $email_content );
							} else {
								ualp_email_trigger_body( 'Guest', '', '', '', $description, $email_address, $email_content );
							}
							break;
						}
						$j++;
						$global_trigger_sent_mail = 0;
						$flag                     = 0;
					}
				}
			}
			/* [1] Email Trigger End */

			/* [1] SMS Trigger Start */
			if ( '0' != $insert_query ) {
				if ( is_multisite() ) {
					$all_trigger_sms_content = $wpdb->get_results( "SELECT * FROM {$wpdb->base_prefix}ualp_sms_notification" );
				} else {
					$all_trigger_sms_content = $wpdb->get_results( "SELECT * FROM {$wpdb->prefix}ualp_sms_notification" );
				}
				if ( ! empty( $all_trigger_sms_content ) ) {
					$sms_content         = '';
					$custom_trigger_q    = '';
					$sms_address         = '';
					$trigger_count       = count( $all_trigger_sms_content );
					$j                   = 1;
					$sms_template        = 0;
					$send_notification_m = true;

					$compare_trigger_left_value = '';
					foreach ( $all_trigger_sms_content as $trigger_sms_content ) {
						$global_trigger_sent_sms = 1;
						$custom_trigger_datas    = maybe_unserialize( $trigger_sms_content->custom_trigger_data );
						$row_count               = count( $custom_trigger_datas );
						$sms_template            = $trigger_sms_content->sms_template;
						$sms_address             = $trigger_sms_content->sms_address;
						$sms_content             = $trigger_sms_content->sms_content;
						$send_sms                = true;
						$first_false             = true;
						$i                       = 1;
						foreach ( $custom_trigger_datas as $custom_trigger_data_single ) {
							$flag                             = 0;
							$custom_trigger_with_opertor_main = 'AND';
							$custom_trigger_operator_with     = $custom_trigger_data_single['custom_trigger_operator_with'];
							$custom_trigger_name              = $custom_trigger_data_single['custom_trigger_name'];
							$custom_trigger_with              = $custom_trigger_data_single['custom_trigger_with'];
							$compare_trigger_right_value      = $custom_trigger_data_single['custom_trigger_value'];
							if ( 1 == $i ) {
								$custom_trigger_with_opertor_main = 'AND';
							} else {
								$custom_trigger_with_opertor_main = $custom_trigger_operator_with;
							}
							switch ( $custom_trigger_name ) {
								case 'user_id':
									$compare_trigger_left_value = $current_user_id;
									break;
								case 'user_name':
									$compare_trigger_left_value = $current_user_name;
									break;
								case 'ip_address':
									$compare_trigger_left_value = $ip;
									break;
								case 'user_role':
									$compare_trigger_left_value = $user_role;
									break;
								case 'hook':
									$compare_trigger_left_value = $hook;
									break;
							}

							if ( 1 == $global_trigger_sent_sms ) {
								if ( 'AND' == $custom_trigger_with_opertor_main ) {
									if ( 'equal' == $custom_trigger_with && '' != $compare_trigger_left_value && ! empty( $compare_trigger_right_value ) ) {
										if ( in_array( $compare_trigger_left_value, $compare_trigger_right_value ) ) {
											$flag = 1;
										}
									} elseif ( 'not equal' == $custom_trigger_with && '' != $compare_trigger_left_value && ! empty( $compare_trigger_right_value ) ) {
										if ( ! in_array( $compare_trigger_left_value, $compare_trigger_right_value ) ) {
											$flag = 1;
										}
									}
								}
								if ( 1 == $row_count && 1 == $i && 0 == $flag ) {
									$global_trigger_sent_sms = 0;
								} elseif ( $row_count > 1 && 1 == $i && '0' == $flag && 'AND' == $custom_trigger_with_opertor_main ) {
									$first_false = false;
								} elseif ( $row_count > 1 && $i > 1 && '0' == $flag && 'AND' == $custom_trigger_with_opertor_main ) {
									$global_trigger_sent_sms = 0;
									$first_false             = false;
								}
							}
							$i++;
							if ( '0' == $flag ) {
								if ( 'AND' == $custom_trigger_with_opertor_main ) {
									$global_trigger_sent_sms = 0;
								}
								break;
							}
						}

						if ( 1 == $global_trigger_sent_sms && false == $first_false ) {
							$global_trigger_sent_sms = 0;
						}
						if ( 1 == $global_trigger_sent_sms ) {
							$mail_msg = get_option( 'ualpSmsMessage' );
							if ( 0 == $sms_template ) {
								$sms_content = $mail_msg;
							}
							if ( '' != $username ) {
								$current_user_name = $username->user_login;
								$firstname         = ucfirst( $username->user_firstname );
								$lastname          = ucfirst( $username->user_lastname );
								$user_firstnm      = ! empty( $firstname ) ? ucfirst( $firstname ) : '-';
								$user_lastnm       = ! empty( $lastname ) ? $lastname : '-';
								$user_email        = ! empty( $username->user_email ) ? $username->user_email : '-';
								ualp_sms_trigger_body( $current_user_name, $user_firstnm, $user_lastnm, $user_email, $description, $sms_address, $sms_content );
							} else {
								ualp_sms_trigger_body( 'Guest', '', '', '', $description, $sms_address, $sms_content );
							}
							break;
						}
						$j++;
						$global_trigger_sent_sms = 0;
						$flag                    = 0;
					}
				}
			}
			/* [1] Sms Trigger End */

			/**
			 * Ualpafterinsertlog hook.
			 */
			do_action( 'ualpafterinsertlog' );
			$json_credentials = json_decode( get_option( 'ualpAddJsonCredentials' ) );
			$sheets           = get_option( 'ualpSpreadsheetAndWorksheet' );
			if ( isset( $json_credentials->client_email ) && isset( $sheets ) ) {
				ualp_add_data_google_sheets( $post_title, $obj_type, $current_user_name, $action, $hook, $description );
			}
			return $externaldb->insert_id;
		} else {
			return 0;
		}
	}
}

if ( ! function_exists( 'ualp_insert_post_log' ) ) {
	/**
	 * Insert record into wp_user_activity table.
	 *
	 * @param int    $post_id Post ID.
	 * @param string $post_title Post Title.
	 * @param string $obj_type Object Type (Plugin, Post, User etc.).
	 * @param string $current_user_name current user id.
	 * @param string $action current user's activity name.
	 * @param string $hook Hook.
	 * @param string $description Description.
	 */
	function ualp_insert_post_log( $post_id, $post_title, $obj_type, $current_user_name, $action, $hook, $description ) {
		global $wpdb, $externaldb;
		$post_title    = addslashes( $post_title );
		$modified_date = current_time( 'mysql' );
		$get_ip_type   = get_option( 'ualpIPtypes', 'REMOTE_ADDR' );
		if ( $get_ip_type ) {
			if ( 'REMOTE_ADDR' == $get_ip_type ) {
				$ips = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
			} elseif ( 'HTTP_X_FORWARDED_FOR' == $get_ip_type ) {
				$ips = isset( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) : '';
			} elseif ( 'HTTP_X_REAL_IP' == $get_ip_type ) {
				$ips = isset( $_SERVER['HTTP_X_REAL_IP'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_REAL_IP'] ) ) : '';
			} else {
				$ips = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
			}
		} else {
			$ips = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
		}
		$user_agents = isset( $_SERVER['HTTP_USER_AGENT'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_USER_AGENT'] ) ) : '';
		$ip          = '';
		$user_agent  = '';
		if ( ualp_allow_ip() ) {
			$ip = $ips;
		}
		$username        = get_user_by( 'login', $current_user_name );
		$guest_disable   = '';
		$user_mail       = '';
		$current_user_id = '';
		$user_role       = '';
		if ( isset( $username ) && ! empty( $username ) ) {
			$current_user_name = $username->data->display_name;
			$user_mail         = $username->data->user_email;
			$current_user_id   = $username->data->ID;
			if ( ! empty( $username->roles ) && is_array( $username->roles ) ) {
				if ( count( $username->roles ) == 1 ) {
					$user_role = $username->roles[0];
				} else {
					$user_role = $username->roles;
					foreach ( $user_role as $all_rols ) {
						$user_role .= $all_rols . ' ,';
					}
				}
			}
		} else {
			$current_user_name = 'Guest';
			$user_mail         = '';
			$current_user_id   = '';
			$user_role         = '';
		}
		$i                         = 0;
		$flag                      = 0;
		$currenr_role              = array();
		$exclude_current_user_name = '';
		if ( isset( $username ) && ! empty( $username ) ) {
			$current_user              = get_user_by( 'login', $username->user_login );
			$exclude_current_user_name = $username->user_login;
			if ( ! empty( $current_user->roles ) && is_array( $current_user->roles ) ) {
				foreach ( $current_user->roles as $role ) {
					$currenr_role[] = $role; }
			}
		}
		$all_user_id    = array();
		$excludelogdata = get_option( 'exclude_log_data' );
		if ( is_array( $excludelogdata ) && ! empty( $excludelogdata ) ) {
			$users1 = get_users();
			foreach ( $users1 as $user_id ) {
				$all_user_id[] = $user_id->user_login;
			}
			foreach ( $excludelogdata as $exclude_log_data_single ) {
				$row_count         = count( $exclude_log_data_single );
				$exclude_logs_hook = $exclude_log_data_single['exclude_logs_hook'];
				$exclude_log_users = $exclude_log_data_single['exclude_log_users'];
				$exclude_logs_ips  = $exclude_log_data_single['exclude_logs_ips'];
				if ( empty( $exclude_logs_ips ) ) {
					$exclude_logs_ips = array();}
				if ( '' != $exclude_log_users ) {
					if ( in_array( $exclude_log_users, $all_user_id ) ) {
						$exclude_log_user     = get_user_by( 'login', $exclude_log_users );
						$exclude_user         = $exclude_log_user->user_login;
						$exclude_current_user = $exclude_current_user_name;
					} elseif ( ! in_array( $exclude_log_users, $all_user_id ) ) {
						if ( '0' == $exclude_log_users ) {
							$exclude_user = 'guest';
							if ( isset( $username ) && ! empty( $username ) ) {
								$exclude_current_user = $exclude_current_user_name;
							} else {
								$exclude_current_user = 'guest';
							}
						} else {
							$exclude_current_user = $currenr_role[0];
							$exclude_user         = $exclude_log_users;
						}
					}
				}
				if ( 'all' != $exclude_log_users && 'all' != $exclude_logs_hook ) {
					if ( 1 == $row_count && 0 == $i ) {
						if ( ! empty( $exclude_logs_ips ) && $hook == $exclude_logs_hook && $exclude_current_user == $exclude_user && in_array( $ip, $exclude_logs_ips ) ) {
							$flag = 1;
						} elseif ( empty( $exclude_logs_ips ) && $hook == $exclude_logs_hook && $exclude_current_user == $exclude_user ) {
							$flag = 1;
						}
					} elseif ( $row_count > 1 && 1 != $flag ) {
						if ( ! empty( $exclude_logs_ips ) && $hook == $exclude_logs_hook && $exclude_current_user == $exclude_user && in_array( $ip, $exclude_logs_ips ) ) {
							$flag = 1;
						} elseif ( empty( $exclude_logs_ips ) && $hook == $exclude_logs_hook && $exclude_current_user == $exclude_user ) {
							$flag = 1;
						}
					}
				} elseif ( 'all' == $exclude_log_users && 'all' != $exclude_logs_hook ) {
					if ( 1 == $row_count && 0 == $i ) {
						if ( ! empty( $exclude_logs_ips ) && $hook == $exclude_logs_hook && in_array( $ip, $exclude_logs_ips ) ) {
							$flag = 1;
						} elseif ( empty( $exclude_logs_ips ) && $hook == $exclude_logs_hook ) {
							$flag = 1;
						}
					} elseif ( $row_count > 1 && 1 != $flag ) {
						if ( ! empty( $exclude_logs_ips ) && $hook == $exclude_logs_hook && in_array( $ip, $exclude_logs_ips ) ) {
							$flag = 1;
						} elseif ( empty( $exclude_logs_ips ) && $hook == $exclude_logs_hook ) {
							$flag = 1;
						}
					}
				} elseif ( 'all' != $exclude_log_users && 'all' == $exclude_logs_hook ) {
					if ( 1 == $row_count && 0 == $i ) {
						if ( ! empty( $exclude_logs_ips ) && $exclude_current_user == $exclude_user && in_array( $ip, $exclude_logs_ips ) ) {
							$flag = 1;
						} elseif ( empty( $exclude_logs_ips ) && $exclude_current_user == $exclude_user ) {
							$flag = 1;
						}
					} elseif ( $row_count > 1 && 1 != $flag ) {
						if ( ! empty( $exclude_logs_ips ) && $exclude_current_user == $exclude_user && in_array( $ip, $exclude_logs_ips ) ) {
							$flag = 1;
						} elseif ( empty( $exclude_logs_ips ) && $exclude_current_user == $exclude_user ) {
							$flag = 1;
						}
					}
				} elseif ( 'all' == $exclude_log_users && 'all' == $exclude_logs_hook ) {
					if ( ! empty( $exclude_logs_ips ) && 1 != $flag && in_array( $ip, $exclude_logs_ips ) ) {
						$flag = 1;
					} elseif ( empty( $exclude_logs_ips ) ) {
						$flag = 1;
					}
				}
				$i++;
			}
		}
		$site_id = get_current_blog_id();
		ualp_create_table();
		ualp_create_detail_table();
		ualp_create_external_table();
		ualp_create_detail_external_table();
		ualp_create_login_external_table();
		ualp_create_email_notification_table();
		ualp_create_sms_notification_table();
		ualp_create_email_notification_external_table();
		ualp_create_schedule_report();
		ualp_create_schedule_external_report();
		/**
		 * Ualpbeforeinsertlog hook.
		 */
		do_action( 'ualpbeforeinsertlog' );

		$country   = '';
		$t_session = '';
		if ( ualp_allow_country() ) {
			$country = ( isset( $ip ) && ( '' != $ip ) ) ? ualp_ip_info( $ip, 'country' ) : '';
		}
		if ( ualp_allow_user_agent() ) {
			$user_agent = $user_agents;
			$user_agent = str_replace( ';', ',', $user_agent );
		}
		if ( '0' == $flag ) {
			if ( is_user_logged_in() ) {
				$t_session = wp_get_session_token();
			}
			$insert_query = $externaldb->insert(
				UALP_TABLENAME,
				array(
					'post_id'       => $post_id,
					'favorite'      => '0',
					'post_title'    => $post_title,
					'user_id'       => $current_user_id,
					'user_name'     => $current_user_name,
					'user_role'     => $user_role,
					'user_email'    => $user_mail,
					'ip_address'    => $ip,
					'country'       => $country,
					'user_agent'    => $user_agent,
					'modified_date' => $modified_date,
					'object_type'   => $obj_type,
					'action'        => $action,
					'hook'          => 'transition_post_status',
					'description'   => $description,
					't_session'     => $t_session,
					'site_id'       => $site_id,
				)
			);
			if ( '0' != $insert_query ) {
				/* [2] Email Trigger Start */
				if ( is_multisite() ) {
					$all_trigger_email_content = $wpdb->get_results( "SELECT * FROM {$wpdb->base_prefix}ualp_email_notification" );
				} else {
					$all_trigger_email_content = $externaldb->get_results( 'SELECT * FROM ' . UALP_EMAIL_TABLENAME );
				}
				if ( ! empty( $all_trigger_email_content ) ) {
					$email_content       = '';
					$custom_trigger_q    = '';
					$email_address       = '';
					$trigger_count       = count( $all_trigger_email_content );
					$j                   = 1;
					$email_template      = 0;
					$send_notification_m = true;

					$compare_trigger_left_value = '';
					foreach ( $all_trigger_email_content as $trigger_email_content ) {
						$global_trigger_sent_mail = 1;
						$custom_trigger_datas     = maybe_unserialize( $trigger_email_content->custom_trigger_data );
						$row_count                = count( $custom_trigger_datas );
						$email_template           = $trigger_email_content->email_template;
						$email_address            = $trigger_email_content->email_address;
						$email_content            = $trigger_email_content->email_content;
						$send_email               = true;
						$first_false              = true;
						$i                        = 1;
						foreach ( $custom_trigger_datas as $custom_trigger_data_single ) {
							$flag                             = 0;
							$custom_trigger_with_opertor_main = 'AND';
							$custom_trigger_operator_with     = $custom_trigger_data_single['custom_trigger_operator_with'];
							$custom_trigger_name              = $custom_trigger_data_single['custom_trigger_name'];
							$custom_trigger_with              = $custom_trigger_data_single['custom_trigger_with'];
							$compare_trigger_right_value      = $custom_trigger_data_single['custom_trigger_value'];
							if ( 1 == $i ) {
								$custom_trigger_with_opertor_main = 'AND';
							} else {
								$custom_trigger_with_opertor_main = $custom_trigger_operator_with;
							}
							switch ( $custom_trigger_name ) {
								case 'user_id':
									$compare_trigger_left_value = $current_user_id;
									break;
								case 'user_name':
									$compare_trigger_left_value = $current_user_name;
									break;
								case 'ip_address':
									$compare_trigger_left_value = $ip;
									break;
								case 'user_role':
									$compare_trigger_left_value = $user_role;
									break;
								case 'hook':
									$compare_trigger_left_value = $hook;
									break;
							}

							if ( 1 == $global_trigger_sent_mail ) {
								if ( 'AND' == $custom_trigger_with_opertor_main ) {
									if ( 'equal' == $custom_trigger_with && '' != $compare_trigger_left_value && ! empty( $compare_trigger_right_value ) ) {
										if ( in_array( $compare_trigger_left_value, $compare_trigger_right_value ) ) {
											$flag = 1;
										}
									} elseif ( 'not equal' == $custom_trigger_with && '' != $compare_trigger_left_value && ! empty( $compare_trigger_right_value ) ) {
										if ( ! in_array( $compare_trigger_left_value, $compare_trigger_right_value ) ) {
											$flag = 1;
										}
									}
								}
								if ( 1 == $row_count && 1 == $i && 0 == $flag ) {
									$global_trigger_sent_mail = 0;
								} elseif ( $row_count > 1 && 1 == $i && '0' == $flag && 'AND' == $custom_trigger_with_opertor_main ) {
									$first_false = false;
								} elseif ( $row_count > 1 && $i > 1 && '0' == $flag && 'AND' == $custom_trigger_with_opertor_main ) {
									$global_trigger_sent_mail = 0;
									$first_false              = false;
								}
							}
							$i++;
							if ( '0' == $flag ) {
								if ( 'AND' == $custom_trigger_with_opertor_main ) {
									$global_trigger_sent_mail = 0;
								}
								break;
							}
						}

						if ( 1 == $global_trigger_sent_mail && false == $first_false ) {
							$global_trigger_sent_mail = 0;
						}
						if ( 1 == $global_trigger_sent_mail ) {
							$mail_msg = get_option( 'ualpEmailMessage' );
							if ( 0 == $email_template ) {
								$email_content = $mail_msg;
							}
							if ( '' != $username ) {
								$current_user_name = $username->user_login;
								$firstname         = ucfirst( $username->user_firstname );
								$lastname          = ucfirst( $username->user_lastname );
								$user_firstnm      = ! empty( $firstname ) ? ucfirst( $firstname ) : '-';
								$user_lastnm       = ! empty( $lastname ) ? $lastname : '-';
								$user_email        = ! empty( $username->user_email ) ? $username->user_email : '-';
								ualp_email_trigger_body( $current_user_name, $user_firstnm, $user_lastnm, $user_email, $description, $email_address, $email_content );
							} else {
								ualp_email_trigger_body( 'Guest', '', '', '', $description, $email_address, $email_content );
							}
							break;
						}
						$j++;
						$global_trigger_sent_mail = 0;
						$flag                     = 0;
					}
				}
				/* [2] Email Trigger End */

				/* [2] SMS Trigger Start */
				if ( is_multisite() ) {
					$all_trigger_sms_content = $wpdb->get_results( "SELECT * FROM {$wpdb->base_prefix}ualp_sms_notification" );
				} else {
					$all_trigger_sms_content = $wpdb->get_results( "SELECT * FROM {$wpdb->prefix}ualp_sms_notification" );
				}
				if ( ! empty( $all_trigger_sms_content ) ) {
					$sms_content         = '';
					$custom_trigger_q    = '';
					$sms_address         = '';
					$trigger_count       = count( $all_trigger_sms_content );
					$j                   = 1;
					$sms_template        = 0;
					$send_notification_m = true;

					$compare_trigger_left_value = '';
					foreach ( $all_trigger_sms_content as $trigger_sms_content ) {
						$global_trigger_sent_sms = 1;
						$custom_trigger_datas    = maybe_unserialize( $trigger_sms_content->custom_trigger_data );
						$row_count               = count( $custom_trigger_datas );
						$sms_template            = $trigger_sms_content->sms_template;
						$sms_address             = $trigger_sms_content->sms_address;
						$sms_content             = $trigger_sms_content->sms_content;
						$send_sms                = true;
						$first_false             = true;
						$i                       = 1;
						foreach ( $custom_trigger_datas as $custom_trigger_data_single ) {
							$flag                             = 0;
							$custom_trigger_with_opertor_main = 'AND';
							$custom_trigger_operator_with     = $custom_trigger_data_single['custom_trigger_operator_with'];
							$custom_trigger_name              = $custom_trigger_data_single['custom_trigger_name'];
							$custom_trigger_with              = $custom_trigger_data_single['custom_trigger_with'];
							$compare_trigger_right_value      = $custom_trigger_data_single['custom_trigger_value'];
							if ( 1 == $i ) {
								$custom_trigger_with_opertor_main = 'AND';
							} else {
								$custom_trigger_with_opertor_main = $custom_trigger_operator_with;
							}
							switch ( $custom_trigger_name ) {
								case 'user_id':
									$compare_trigger_left_value = $current_user_id;
									break;
								case 'user_name':
									$compare_trigger_left_value = $current_user_name;
									break;
								case 'ip_address':
									$compare_trigger_left_value = $ip;
									break;
								case 'user_role':
									$compare_trigger_left_value = $user_role;
									break;
								case 'hook':
									$compare_trigger_left_value = $hook;
									break;
							}

							if ( 1 == $global_trigger_sent_sms ) {
								if ( 'AND' == $custom_trigger_with_opertor_main ) {
									if ( 'equal' == $custom_trigger_with && '' != $compare_trigger_left_value && ! empty( $compare_trigger_right_value ) ) {
										if ( in_array( $compare_trigger_left_value, $compare_trigger_right_value ) ) {
											$flag = 1;
										}
									} elseif ( 'not equal' == $custom_trigger_with && '' != $compare_trigger_left_value && ! empty( $compare_trigger_right_value ) ) {
										if ( ! in_array( $compare_trigger_left_value, $compare_trigger_right_value ) ) {
											$flag = 1;
										}
									}
								}
								if ( 1 == $row_count && 1 == $i && 0 == $flag ) {
									$global_trigger_sent_sms = 0;
								} elseif ( $row_count > 1 && 1 == $i && '0' == $flag && 'AND' == $custom_trigger_with_opertor_main ) {
									$first_false = false;
								} elseif ( $row_count > 1 && $i > 1 && '0' == $flag && 'AND' == $custom_trigger_with_opertor_main ) {
									$global_trigger_sent_sms = 0;
									$first_false             = false;
								}
							}
							$i++;
							if ( '0' == $flag ) {
								if ( 'AND' == $custom_trigger_with_opertor_main ) {
									$global_trigger_sent_sms = 0;
								}
								break;
							}
						}

						if ( 1 == $global_trigger_sent_sms && false == $first_false ) {
							$global_trigger_sent_sms = 0;
						}
						if ( 1 == $global_trigger_sent_sms ) {
							$mail_msg = get_option( 'ualpSmsMessage' );
							if ( 0 == $sms_template ) {
								$sms_content = $mail_msg;
							}
							if ( '' != $username ) {
								$current_user_name = $username->user_login;
								$firstname         = ucfirst( $username->user_firstname );
								$lastname          = ucfirst( $username->user_lastname );
								$user_firstnm      = ! empty( $firstname ) ? ucfirst( $firstname ) : '-';
								$user_lastnm       = ! empty( $lastname ) ? $lastname : '-';
								$user_email        = ! empty( $username->user_email ) ? $username->user_email : '-';
								ualp_sms_trigger_body( $current_user_name, $user_firstnm, $user_lastnm, $user_email, $description, $sms_address, $sms_content );
							} else {
								ualp_sms_trigger_body( 'Guest', '', '', '', $description, $sms_address, $sms_content );
							}
							break;
						}
						$j++;
						$global_trigger_sent_sms = 0;
						$flag                    = 0;
					}
				}
				/* [2] SMS Trigger End. */

			}

			/**
			 * Ualpafterinsertlog hook.
			 */
			do_action( 'ualpafterinsertlog' );
			$json_credentials = json_decode( get_option( 'ualpAddJsonCredentials' ) );
			$sheets           = get_option( 'ualpSpreadsheetAndWorksheet' );
			if ( isset( $json_credentials->client_email ) && isset( $sheets ) ) {
				ualp_add_data_google_sheets( $post_title, $obj_type, $current_user_name, $action, $hook, $description );
			}
			return $externaldb->insert_id;
		} else {
			return 0;
		}

	}
}

if ( ! function_exists( 'ualp_insert_email_notification' ) ) {
	/**
	 * Insert record into wp_email_notifictaion table.
	 *
	 * @param string $current_user_name current user name.
	 * @param string $description Description.
	 * @param string $trigger_data Trigger Data.
	 * @param string $title Title.
	 * @param string $emailid Email ID.
	 * @param string $email_template Email Template.
	 */
	function ualp_insert_email_notification( $current_user_name, $description, $trigger_data, $title, $emailid, $email_template ) {
		global $wpdb, $externaldb;
		$username        = get_user_by( 'login', $current_user_name );
		$user_mail       = '';
		$current_user_id = '';
		$user_role       = '';
		if ( isset( $username ) && ! empty( $username ) ) {
			$current_user_name = $username->data->display_name;
			$user_mail         = $username->data->user_email;
			$current_user_id   = $username->data->ID;
			if ( ! empty( $username->roles ) && is_array( $username->roles ) ) {
				if ( count( $username->roles ) == 1 ) {
					$user_role = $username->roles[0];
				} else {
					$user_role = $username->roles;
					foreach ( $user_role as $all_rols ) {
						$user_role .= $all_rols . ' ,';
					}
				}
			}
		} else {
			$current_user_name = 'Guest';
			$user_mail         = '';
			$current_user_id   = '';
			$user_role         = '';
		}
		$site_id = get_current_blog_id();
		ualp_create_table();
		ualp_create_detail_table();
		ualp_create_external_table();
		ualp_create_detail_external_table();
		ualp_create_login_external_table();
		ualp_create_email_notification_table();
		ualp_create_email_notification_external_table();
		ualp_create_schedule_report();
		ualp_create_schedule_external_report();
		/**
		 * UalpBeforeInsertEmailNotification hook.
		 */
		do_action( 'ualpbeforeinsertemailnotification' );
		$insert_query = $externaldb->insert(
			UALP_EMAIL_TABLENAME,
			array(
				'email_title'         => $title,
				'email_content'       => $description,
				'user_id'             => $current_user_id,
				'user_name'           => $current_user_name,
				'user_role'           => $user_role,
				'user_email'          => $user_mail,
				'site_id'             => $site_id,
				'email_address'       => $emailid,
				'email_template'      => $email_template,
				'custom_trigger_data' => maybe_serialize( $trigger_data ),
			)
		);
		/**
		 * UalpAfterInsertEmailNotification hook.
		 */
		do_action( 'ualp_insert_email_notification' );
		// wp_redirect( admin_url( 'admin.php?page=ualp_email_setting' ) );.
		// exit();.
	}
}

if ( ! function_exists( 'ualp_update_notification' ) ) {
	/**
	 * Insert record when user logout.
	 *
	 * @param string $enotification Email Notification.
	 * @param string $all_email_notification All Email Notification.
	 * @param string $trigger_data Trigger Data.
	 * @param string $title Title.
	 * @param string $emailid Email ID.
	 * @param string $email_template Email Template.
	 */
	function ualp_update_notification( $enotification, $all_email_notification, $trigger_data, $title, $emailid, $email_template ) {
		global $wpdb, $externaldb;
		do_action( 'ualp_before_update_notification' );
		$table_names = UALP_EMAIL_TABLENAME;
		$externaldb->update(
			UALP_EMAIL_TABLENAME,
			array(
				'email_content'       => $all_email_notification,
				'custom_trigger_data' => maybe_serialize( $trigger_data ),
				'email_title'         => $title,
				'email_address'       => $emailid,
				'email_template'      => $email_template,
			),
			array( 'uacteid' => $enotification )
		);
		do_action( 'ualp_update_email_notification' );
		wp_redirect( admin_url( 'admin.php?page=ualp_email_setting' ) );
		// exit();.
	}
}

if ( ! function_exists( 'ualp_insert_login_log' ) ) {
	/**
	 * Insert record when user login.
	 *
	 * @param string $logged_in_cookie Cookie Logged in.
	 * @param string $expire Expire.
	 * @param string $expiration Expiration.
	 * @param string $user_id User ID.
	 * @param string $logged_in_text Logged in Text.
	 * @param string $token Token.
	 */
	function ualp_insert_login_log( $logged_in_cookie, $expire, $expiration, $user_id, $logged_in_text, $token ) {

		global $wpdb, $externaldb;
		$get_ip_type = get_option( 'ualpIPtypes', 'REMOTE_ADDR' );
		if ( $get_ip_type ) {
			if ( 'REMOTE_ADDR' == $get_ip_type ) {
				$ips = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
			} elseif ( 'HTTP_X_FORWARDED_FOR' == $get_ip_type ) {
				$ips = isset( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) : '';
			} elseif ( 'HTTP_X_REAL_IP' == $get_ip_type ) {
				$ips = isset( $_SERVER['HTTP_X_REAL_IP'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_REAL_IP'] ) ) : '';
			} else {
				$ips = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
			}
		} else {
			$ips = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
		}
		$user_agents    = isset( $_SERVER['HTTP_USER_AGENT'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_USER_AGENT'] ) ) : '';
		$time_login     = current_time( 'mysql' );
		$time_last_seen = current_time( 'mysql' );
		$ip             = '';
		$user_agent     = '';
		if ( ualp_allow_ip() ) {
			$ip = $ips;
		}
		$current_user_id = $user_id;
		$username        = get_user_by( 'id', $current_user_id );
		$user_mail       = '';
		$user_role       = '';
		if ( isset( $username ) && ! empty( $username ) ) {
			$current_user_name = $username->data->display_name;
			$user_mail         = $username->data->user_email;
			if ( ! empty( $username->roles ) && is_array( $username->roles ) ) {
				if ( count( $username->roles ) == 1 ) {
					$user_role = $username->roles[0];
				} else {
					$user_role = $username->roles;
					foreach ( $user_role as $all_rols ) {
						$user_role .= $all_rols . ' ,';
					}
				}
			}
		}
		$site_id = get_current_blog_id();
		/**
		 * Ualpbeforeinsertloghook.
		 */
		do_action( 'ualpbeforeinsertloginlog' );
		$country = '';
		if ( ualp_allow_country() ) {
			$country = ( isset( $ip ) && ( '' != $ip ) ) ? ualp_ip_info( $ip, 'country' ) : '';
		}
		if ( ualp_allow_user_agent() ) {
			$user_agent = $user_agents;
			$user_agent = str_replace( ';', ',', $user_agent );
		}
		$insert_query = $externaldb->insert(
			UALP_LOGIN_TABLENAME,
			array(
				'user_id'        => $current_user_id,
				'user_name'      => $current_user_name,
				'user_role'      => $user_role,
				'user_email'     => $user_mail,
				'time_login'     => $time_login,
				'time_last_seen' => $time_last_seen,
				'session_token'  => $token,
				'ip_address'     => $ip,
				'country'        => $country,
				'user_agent'     => $user_agent,
				'login_status'   => 'login',
				'site_id'        => $site_id,
			)
		);
		do_action( 'ualpafterinsertloginlog' );
	}
}

if ( ! function_exists( 'ualp_insert_logout_log' ) ) {
	/**
	 * Insert record when user logout.
	 */
	function ualp_insert_logout_log() {
		global $wpdb, $externaldb;
		$session_token = wp_get_session_token();
		$time_logout   = current_time( 'mysql' );
		/**
		 * Ualpbeforeinsertloghook.
		 */
		do_action( 'ualpbeforeinsertlogoutlog' );
		$externaldb->update(
			UALP_LOGIN_TABLENAME,
			array(
				'time_logout'    => $time_logout,
				'time_last_seen' => $time_logout,
				'login_status'   => 'logout',
			),
			array( 'session_token' => $session_token )
		);
		do_action( 'ualpafterinsertlogoutlog' );
	}
}

if ( ! function_exists( 'ualp_update_last_seen' ) ) {
	/**
	 * Insert record of user last seen.
	 */
	function ualp_update_last_seen() {
		global $wpdb, $externaldb;
		$session_token = wp_get_session_token();
		if ( '' != $session_token ) {
			$time_last_seen = current_time( 'mysql' );
			$externaldb->update(
				UALP_LOGIN_TABLENAME,
				array( 'time_last_seen' => $time_last_seen ),
				array( 'session_token' => $session_token )
			);
		}
	}
}

if ( ! function_exists( 'ualp_get_activity_log' ) ) {
	/**
	 * Get activity.
	 *
	 * @param string $action current user's activity name.
	 * @param string $obj_type Object Type (Plugin, Post, User etc.).
	 * @param int    $post_id Post ID.
	 * @param string $post_title Post Title.
	 * @param string $hook Hook Name.
	 * @param string $description Description.
	 * @return int $ualp_insert_log Log Id.
	 */
	function ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description ) {
		$current_user_id   = get_current_user_id();
		$current_user      = wp_get_current_user();
		$current_user_name = $current_user->user_login;
		$ualp_insert_log   = ualp_insert_log( $post_id, $post_title, $obj_type, $current_user_name, $action, $hook, $description );
		return $ualp_insert_log;
	}
}

if ( ! function_exists( 'ualp_get_post_activity_log' ) ) {
	/**
	 * Get activity.
	 *
	 * @param string $action current user's activity name.
	 * @param string $obj_type Object Type (Plugin, Post, User etc.).
	 * @param int    $post_id Post ID.
	 * @param string $post_title Post Title.
	 * @param string $hook Hook Name.
	 * @param string $description Description.
	 * @return int $ualp_insert_log Log Id.
	 */
	function ualp_get_post_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description ) {
		$current_user_id   = get_current_user_id();
		$current_user      = wp_get_current_user();
		$current_user_name = $current_user->user_login;
		$ualp_insert_log   = ualp_insert_post_log( $post_id, $post_title, $obj_type, $current_user_name, $action, $hook, $description );
		return $ualp_insert_log;
	}
}

if ( ! function_exists( 'ualp_logout_get_activity_log' ) ) {
	/**
	 * Get activity.
	 *
	 * @param string $action current user's activity name.
	 * @param string $obj_type Object Type (Plugin, Post, User etc.).
	 * @param int    $post_id Post ID.
	 * @param string $post_title Post Title.
	 * @param string $hook Hook Name.
	 * @param string $description Description.
	 * @param string $current_user_name Current User name.
	 * @return int $ualp_insert_log Log Id.
	 */
	function ualp_logout_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description, $current_user_name ) {
		$ualp_insert_log = ualp_insert_log( $post_id, $post_title, $obj_type, $current_user_name, $action, $hook, $description );
		return $ualp_insert_log;
	}
}

if ( ! function_exists( 'ualp_email_trigger_body' ) ) {
	/**
	 * Send email notification content.
	 *
	 * @param string $current_user Current User.
	 * @param string $user_firstnm First name of user.
	 * @param string $user_lastnm Last name of user.
	 * @param string $user_email Email of user.
	 * @param string $activity Activity.
	 * @param string $to_email_address Email Address.
	 * @param string $email_content Email Content.
	 */
	function ualp_email_trigger_body( $current_user, $user_firstnm, $user_lastnm, $user_email, $activity, $to_email_address, $email_content ) {
		if ( ualp_allow_ip() ) {
			$get_ip_type = get_option( 'ualpIPtypes', 'REMOTE_ADDR' );
			if ( $get_ip_type ) {
				if ( 'REMOTE_ADDR' == $get_ip_type ) {
					$ip = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
				} elseif ( 'HTTP_X_FORWARDED_FOR' == $get_ip_type ) {
					$ip = isset( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) : '';
				} elseif ( 'HTTP_X_REAL_IP' == $get_ip_type ) {
					$ip = isset( $_SERVER['HTTP_X_REAL_IP'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_REAL_IP'] ) ) : '';
				} else {
					$ip = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
				}
			} else {
				$ip = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
			}
		} else {
			$ip = '';
		}
		$modified_date = current_time( 'mysql' );
		$modified_date = strtotime( $modified_date );

		$date_format = get_option( 'date_format' );
		$time_format = get_option( 'time_format' );

		$date = gmdate( $date_format, $modified_date );
		$time = gmdate( $time_format, $modified_date );

		$user_reg = $date . ' ' . $time;

		ob_start();
		?>
		<table cellspacing='0' style='margin-top:30px;border: 1px solid #cccccc;border-collapse: collapse;'>
			<tr>
				<th style='padding:5px 10px;border-bottom: 1px solid #cccccc;border-right: 1px solid #cccccc;'><?php echo esc_html__( 'Username', 'user_activity_log_pro' ); ?></th>
				<th style='padding:5px 10px;border-bottom: 1px solid #cccccc;border-right: 1px solid #cccccc;'><?php echo esc_html__( 'Firstname', 'user_activity_log_pro' ); ?></th>
				<th style='padding:5px 10px;border-bottom: 1px solid #cccccc;border-right: 1px solid #cccccc;'><?php echo esc_html__( 'Lastname', 'user_activity_log_pro' ); ?></th>
				<th style='padding:5px 10px;border-bottom: 1px solid #cccccc;border-right: 1px solid #cccccc;'><?php echo esc_html__( 'E-mail', 'user_activity_log_pro' ); ?></th>
				<th style='padding:5px 10px;border-bottom: 1px solid #cccccc;border-right: 1px solid #cccccc;'><?php echo esc_html__( 'Date Time', 'user_activity_log_pro' ); ?></th>
				<?php if ( ualp_allow_ip() ) { ?>
				<th style='padding:5px 10px;border-bottom: 1px solid #cccccc;border-right: 1px solid #cccccc;'><?php echo esc_html__( 'IP Address', 'user_activity_log_pro' ); ?></th>
				<?php } ?>
				<th style='padding:5px 10px;border-bottom: 1px solid #cccccc;'><?php echo esc_html__( 'Activity', 'user_activity_log_pro' ); ?></th>
			</tr>
			<tr>
				<td style='padding:5px 10px;border-right: 1px solid #cccccc;'><?php echo esc_html( $current_user ); ?></td>
				<td style='padding:5px 10px;border-right: 1px solid #cccccc;'><?php echo esc_html( $user_firstnm ); ?></td>
				<td style='padding:5px 10px;border-right: 1px solid #cccccc;'><?php echo esc_html( $user_lastnm ); ?></td>
				<td style='padding:5px 10px;border-right: 1px solid #cccccc;'><?php echo esc_html( $user_email ); ?></td>
				<td style='padding:5px 10px;border-right: 1px solid #cccccc;'><?php echo esc_html( $user_reg ); ?></td>
				<?php if ( ualp_allow_ip() ) { ?>
				<td style='padding:5px 10px;border-right: 1px solid #cccccc;'><?php echo esc_html( $ip ); ?></td>
				<?php } ?>
				<td style='padding:5px 10px;border-right: 1px solid #cccccc;'><?php echo esc_html( $activity ); ?></td>
			</tr>
		</table>
		<br/>
		<br/>
		<?php
		$user_details = ob_get_clean();
		$from_email   = isset( $user_email ) ? $user_email : get_option( 'ualpFromEmail' );
		$mail_msg     = $email_content;
		if ( ! empty( $to_email_address ) ) {
			$to_email = $to_email_address;
		}
		$mail_msg = str_replace( '[user_details]', $user_details, $mail_msg );
		if ( isset( $to_email ) && '' != $to_email && '' != $from_email && '' != $mail_msg ) {

			$headers              = 'From: ' . wp_strip_all_tags( $from_email ) . "\r\n";
			$headers             .= 'Reply-To: ' . wp_strip_all_tags( $from_email ) . "\r\n";
			$headers             .= "MIME-Version: 1.0\r\n";
			$headers             .= "Content-Type: text/html; charset=ISO-8859-1\r\n";
			$to_email_array       = explode( ',', $to_email );
			$to_email_array_count = count( $to_email_array );
			for ( $i = 0; $i < $to_email_array_count;$i++ ) {
				$to_mail = trim( $to_email_array[ $i ], ' ' );
				wp_mail( $to_mail, esc_html__( 'User activity Notification', 'user_activity_log_pro' ), $mail_msg, $headers );
			}
		}
	}
}

if ( ! function_exists( 'ualp_hook_wp_login' ) ) {
	/**
	 * Add activity for the current user when login.
	 *
	 * @param string $current_user_name string user's login name.
	 */
	function ualp_hook_wp_login( $current_user_name ) {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( is_multisite() && empty( $log_hook_list ) ) {
			$log_hook_list = array();
		}
		if ( in_array( 'wp_login', $log_hook_list ) ) {
			$action      = 'logged in';
			$obj_type    = 'User';
			$post_id     = '';
			$username    = get_user_by( 'login', $current_user_name );
			$post_title  = $username->display_name;
			$hook        = 'wp_login';
			$description = "$obj_type $action : $post_title";
			ualp_insert_log( $post_id, $post_title, $obj_type, $current_user_name, $action, $hook, $description );
		}
	}
}
add_action( 'wp_login', 'ualp_hook_wp_login', 9 );
add_action( 'set_logged_in_cookie', 'ualp_insert_login_log', 9, 6 );


if ( ! function_exists( 'ualp_hook_wp_login_failed' ) ) {
	/**
	 * Get activity for the user - Login fail.
	 *
	 * @param string $current_user_name username.
	 */
	function ualp_hook_wp_login_failed( $current_user_name ) {
		$log_hook_list     = get_option( 'enableLogHookList' );
		$logs_failed_login = get_option( 'logs_failed_login' );

		$ualp_user_email_notify          = get_option( 'ualpEmailNotification' );
		$ualp_fail_login_number          = get_option( 'ualpFailLoginNumber' );
		$ualp_fail_login_not_user_number = get_option( 'ualpFailLoginNotUserNumber' );

		$ualp_mobile_notification            = get_option( 'ualpMobileNotification' );
		$ualp_fail_login_not_user_number_att = get_option( 'ualpFailLoginNotUserNumberAtt' );

		$login_failed_non_existing_user = get_option( 'login_failed_non_existing_user' );
		$login_failed_existing_user     = get_option( 'login_failed_existing_user' );
		$action                         = 'login failed';
		$obj_type                       = 'User';
		$post_id                        = '';

		global $wpdb, $externaldb;
		if ( is_multisite() && empty( $log_hook_list ) ) {
			$log_hook_list = array();
		}
		if ( in_array( 'wp_login_failed', $log_hook_list ) && 'yes' == $logs_failed_login ) {
			$username = get_user_by( 'login', $current_user_name );
			if ( isset( $username ) && ! empty( $username ) ) {
				$post_title         = $username->display_name;
				$countwploginfailed = $externaldb->get_var( 'SELECT COUNT(`uactid`) FROM ' . UALP_TABLENAME . ' where hook="wp_login_failed" AND post_title="' . $post_title . '"' );
				$hook               = 'wp_login_failed';
				if ( $countwploginfailed >= $ualp_fail_login_number ) {
					if ( is_array( $ualp_user_email_notify ) && in_array( 'ualpFailLogin', $ualp_user_email_notify ) ) {
						ualp_send_email_notification( $hook, $current_user_name, $post_title . ' Failed login detected' );
					}
				}
				if ( $countwploginfailed >= $ualp_fail_login_not_user_number_att ) {
					if ( is_array( $ualp_mobile_notification ) && in_array( 'ualpFailLogin', $ualp_mobile_notification ) ) {
						ualp_send_sms_notification( $hook, $current_user_name, $post_title . ' Failed login detected' );
					}
				}
				$description      = "$obj_type $action : $post_title";
				$json_credentials = json_decode( get_option( 'ualpAddJsonCredentials' ) );
				$sheets           = get_option( 'ualpSpreadsheetAndWorksheet' );
				if ( empty( $login_failed_existing_user ) ) {
					$login_failed_existing_user = 0;
				}
				if ( $login_failed_existing_user > 0 && $countwploginfailed >= $login_failed_existing_user ) {
					ualp_insert_log( $post_id, $post_title, $obj_type, $current_user_name, $action, $hook, $description );
					if ( isset( $json_credentials->client_email ) && isset( $sheets ) ) {
						ualp_add_data_google_sheets( $post_title, $obj_type, $current_user_name, $action, $hook, $description );
					}
				} else {
					ualp_insert_log( $post_id, $post_title, $obj_type, $current_user_name, $action, $hook, $description );
					if ( isset( $json_credentials->client_email ) && isset( $sheets ) ) {
						ualp_add_data_google_sheets( $post_title, $obj_type, $current_user_name, $action, $hook, $description );
					}
				}
			} else {
				if ( isset( $_REQUEST['log'] ) && '' != $_REQUEST['log'] ) {
					$log                           = sanitize_text_field( wp_unslash( $_REQUEST['log'] ) );
					$username                      = $log;
					$hook                          = 'wp_login_failed';
					$get_all_users                 = get_users();
					$ualp_fail_login_not_user_name = array();
					$countnotuser                  = $externaldb->get_var( 'SELECT COUNT(`uactid`) FROM ' . UALP_TABLENAME . ' where hook="wp_login_failed" AND post_title="' . $log . '"' );
					if ( $countnotuser >= $ualp_fail_login_not_user_number ) {
						ualp_send_email_notification( $hook, '', $log . ' Failed login detected using non existing user' );
					}
					if ( $countnotuser >= $ualp_fail_login_not_user_number_att ) {
						ualp_send_sms_notification( $hook, '', $log . ' Failed login detected using non existing user' );
					}
					$description      = "$obj_type $action : $log";
					$json_credentials = json_decode( get_option( 'ualpAddJsonCredentials' ) );
					$sheets           = get_option( 'ualpSpreadsheetAndWorksheet' );
					if ( empty( $login_failed_non_existing_user ) ) {
						$login_failed_non_existing_user = 0;
					}
					if ( $login_failed_non_existing_user > 0 && $countnotuser >= $login_failed_non_existing_user ) {
						ualp_insert_log( $post_id, $log, $obj_type, $log, $action, $hook, $description );
						if ( isset( $json_credentials->client_email ) && isset( $sheets ) ) {
							ualp_add_data_google_sheets( $log, $obj_type, $current_user_name, $action, $hook, $description );
						}
					} else {
						ualp_insert_log( $post_id, $log, $obj_type, $log, $action, $hook, $description );
						if ( isset( $json_credentials->client_email ) && isset( $sheets ) ) {
							ualp_add_data_google_sheets( $log, $obj_type, $current_user_name, $action, $hook, $description );
						}
					}
				}
			}
		}
	}
}
add_action( 'wp_login_failed', 'ualp_hook_wp_login_failed', 10, 1 );


if ( ! function_exists( 'ualp_hook_wp_logout' ) ) {
	/**
	 * Get activity for the current user when logout.
	 *
	 * @param string $redirect_to Redirect To.
	 * @param string $requested_redirect_to Requested Redirect To.
	 * @param string $user User.
	 */
	function ualp_hook_wp_logout( $redirect_to, $requested_redirect_to, $user ) {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'wp_logout', $log_hook_list ) ) {
			$action   = 'logged out';
			$obj_type = 'User';
			if ( isset( $user ) && ! empty( $user ) ) {
				$post_id           = $user->data->ID;
				$current_user_name = $user->data->user_login;
				$post_title        = $user->data->display_name;
			} else {
				$post_id           = '';
				$current_user_name = 'Guest';
				$post_title        = 'Guest';
			}
			$hook        = 'wp_logout';
			$description = "$obj_type $action : $post_title";
			ualp_logout_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description, $current_user_name );
			$json_credentials = json_decode( get_option( 'ualpAddJsonCredentials' ) );
			$sheets           = get_option( 'ualpSpreadsheetAndWorksheet' );
			if ( isset( $json_credentials->client_email ) && isset( $sheets ) ) {
				ualp_add_data_google_sheets( $post_title, $obj_type, $current_user_name, $action, $hook, $description );
			}
		}
		$requested_redirect_to = wp_login_url( get_permalink() );

		return $requested_redirect_to;

	}
}
add_filter( 'logout_redirect', 'ualp_hook_wp_logout', 10, 3 );
add_action( 'wp_logout', 'ualp_insert_logout_log', 10 );
add_action( 'init', 'ualp_update_last_seen', 10 );

if ( ! function_exists( 'ualp_hook_delete_user' ) ) {
	/**
	 * Get activity for the delete user.
	 *
	 * @param int $user User ID.
	 */
	function ualp_hook_delete_user( $user ) {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'delete_user', $log_hook_list ) ) {
			$action            = 'deleted';
			$obj_type          = 'User';
			$post_id           = $user;
			$user_nm           = get_user_by( 'id', $post_id );
			$current_user_name = $user_nm->user_login;
			$post_title        = $user_nm->display_name;
			$hook              = 'delete_user';
			$description       = "$obj_type $action : $post_title";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}
add_action( 'delete_user', 'ualp_hook_delete_user', 10, 1 );

if ( ! function_exists( 'ualp_hook_user_register' ) ) {
	/**
	 * Get activity for the registered user.
	 *
	 * @param int $user User ID.
	 */
	function ualp_hook_user_register( $user ) {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'user_register', $log_hook_list ) ) {
			$action            = 'registered';
			$obj_type          = 'User';
			$post_id           = $user;
			$user_nm           = get_user_by( 'id', $post_id );
			$current_user_name = $user_nm->user_login;
			$post_title        = $user_nm->display_name;
			$hook              = 'user_register';
			$description       = "$obj_type $action : $post_title";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}

add_action( 'user_register', 'ualp_hook_user_register', 10, 1 );

if ( ! function_exists( 'ualp_pre_user_update' ) ) {
	/**
	 * Get activity for the update user.
	 *
	 * @param object $user User.
	 */
	function ualp_pre_user_update( $user ) {
		$user_id             = $user->data->ID;
		$user_full_key_array = ualp_get_user_primary_data();
		foreach ( $user_full_key_array as $user_key ) {
			$old_user_meta[ $user_key ] = $user->data->$user_key;
		}
		$user_full_metakey_array = ualp_get_user_meta_data();
		foreach ( $user_full_metakey_array as $user_meta_key ) {
			$old_user_meta[ $user_meta_key ] = get_user_meta( $user_id, $user_meta_key, true );
		}
		set_transient( 'ualp_old_user_data_' . $user_id, $old_user_meta, 60 * 30 );
	}
}
add_action( 'show_user_profile', 'ualp_pre_user_update' );
add_action( 'edit_user_profile', 'ualp_pre_user_update' );

if ( ! function_exists( 'ualp_user_update' ) ) {
	/**
	 * Get activity for the update user.
	 *
	 * @param int $user_id User ID.
	 */
	function ualp_user_update( $user_id ) {
		$post_id             = $user_id;
		$old_user_data       = get_transient( 'ualp_old_user_data_' . $user_id );
		$new_user_data       = array();
		$user                = get_userdata( $user_id );
		$user_full_key_array = ualp_get_user_primary_data();
		if ( isset( $_POST['_wpnonce'] ) && ! empty( $_POST['_wpnonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wpnonce'] ) ), 'update-user_' . $user_id ) ) {
			foreach ( $user_full_key_array as $user_key_single => $user_value_single ) {
				if ( ! is_numeric( $user_key_single ) ) {
					if ( isset( $_POST[ $user_key_single ] ) ) {
						$new_user_data[ $user_value_single ] = sanitize_text_field( wp_unslash( $_POST[ $user_key_single ] ) );
					}
				} else {
					if ( isset( $_POST[ $user_value_single ] ) ) {
						$new_user_data[ $user_value_single ] = sanitize_text_field( wp_unslash( $_POST[ $user_value_single ] ) );
					}
				}
			}
		}
		$user_full_metakey_array = ualp_get_user_meta_data();
		foreach ( $user_full_metakey_array as $user_meta_key_single => $user_meta_value_single ) {
			$value = '';
			if ( ! is_numeric( $user_meta_key_single ) ) {
				if ( 'rich_editing' == $user_meta_key_single ) {
					$value = isset( $_POST[ $user_meta_key_single ] ) ? 'false' : 'true';
				} elseif ( 'admin_bar_front' == $user_meta_key_single ) {
					$value = isset( $_POST[ $user_meta_key_single ] ) ? 'true' : 'false';
				} elseif ( isset( $_POST[ $user_meta_key_single ] ) ) {
					$value = sanitize_text_field( wp_unslash( $_POST[ $user_meta_key_single ] ) );
				}
				$new_user_data[ $user_meta_value_single ] = $value;
			} else {
				if ( 'rich_editing' == $user_meta_value_single ) {
					$value = isset( $_POST[ $user_meta_value_single ] ) ? 'false' : 'true';
				} elseif ( 'admin_bar_front' == $user_meta_value_single ) {
					$value = isset( $_POST[ $user_meta_value_single ] ) ? 'true' : 'false';
				} elseif ( isset( $_POST[ $user_meta_value_single ] ) ) {
					$value = sanitize_text_field( wp_unslash( $_POST[ $user_meta_value_single ] ) );
				}
				$new_user_data[ $user_meta_value_single ] = $value;
			}
		}
		$action            = 'profile updated';
		$obj_type          = 'User';
		$current_user_name = $user->user_login;
		$post_title        = $user->display_name;
		$hook              = 'insert_user_meta';
		$description       = "$obj_type $action : $post_title";
		$uactid            = ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		delete_transient( 'ualp_old_user_data_' . $user_id );
		if ( 0 != $uactid ) {
			ualp_add_activity_log_detail_user_update( $uactid, $old_user_data, $new_user_data, $hook );
		}
	}
}
add_action( 'edit_user_profile_update', 'ualp_user_update', 5000 );
add_action( 'personal_options_update', 'ualp_user_update', 5000 );

if ( ! function_exists( 'ualp_hook_password_reset' ) ) {
	/**
	 * Get activity for the password reset.
	 *
	 * @param object $user User.
	 * @param string $new_pass New Password.
	 */
	function ualp_hook_password_reset( $user, $new_pass ) {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'password_reset', $log_hook_list ) ) {
			$action      = 'Password Reset';
			$obj_type    = 'User';
			$post_id     = '';
			$post_title  = $obj_type;
			$hook        = 'password_reset';
			$description = "$obj_type $action : $post_title";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}
add_action( 'password_reset', 'ualp_hook_password_reset', 10, 2 );

if ( ! function_exists( 'ualp_hook_add_attachment' ) ) {
	/**
	 * Get activity for the user - add attach media file.
	 *
	 * @param int $attach Post ID.
	 */
	function ualp_hook_add_attachment( $attach ) {
		$user          = wp_get_current_user();
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'add_attachment', $log_hook_list ) ) {
			$action      = 'added';
			$obj_type    = 'Attachment';
			$post_id     = $attach;
			$post_title  = get_the_title( $post_id );
			$hook        = 'add_attachment';
			$description = "$obj_type $action : $post_title";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}

add_action( 'add_attachment', 'ualp_hook_add_attachment' );

if ( ! function_exists( 'ualp_hook_edit_attachment' ) ) {
	/**
	 * Get activity for the user - edit attach media file.
	 *
	 * @param int $attach Post ID.
	 */
	function ualp_hook_edit_attachment( $attach ) {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'edit_attachment', $log_hook_list ) ) {
			$post_id     = $attach;
			$post_title  = get_the_title( $post_id );
			$action      = 'updated';
			$obj_type    = 'Attachment';
			$hook        = 'edit_attachment';
			$description = "$obj_type $action : $post_title";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}

add_action( 'edit_attachment', 'ualp_hook_edit_attachment' );

if ( ! function_exists( 'ualp_hook_delete_attachment' ) ) {
	/**
	 * Get activity for the user - delete attach media file.
	 *
	 * @param int $attach Post ID.
	 */
	function ualp_hook_delete_attachment( $attach ) {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'delete_attachment', $log_hook_list ) ) {
			$post_id     = $attach;
			$post_title  = get_the_title( $post_id );
			$action      = 'deleted';
			$obj_type    = 'Attachment';
			$hook        = 'delete_attachment';
			$description = "$obj_type $action : $post_title";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}

add_action( 'delete_attachment', 'ualp_hook_delete_attachment' );

if ( ! function_exists( 'ualp_hook_wp_insert_comment' ) ) {
	/**
	 * Get activity for the user - Insert Comment.
	 *
	 * @param int $comment Comment ID.
	 */
	function ualp_hook_wp_insert_comment( $comment ) {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'wp_insert_comment', $log_hook_list ) ) {
			$action                                = 'inserted';
			$obj_type                              = 'Comment';
			$comment_id                            = $comment;
			$com                                   = get_comment( $comment_id );
			$post_id                               = $com->comment_post_ID;
			$comment_detail_ary                    = array();
			$comment_detail_ary['ualp_comment']    = $com->comment_content;
			$comment_detail_ary['ualp_comment_id'] = $comment_id;
			$post_title                            = get_the_title( $post_id );
			$post_type                             = get_post_type( $post_id );
			$post_link                             = get_the_permalink( $post_id );
			$hook                                  = 'wp_insert_comment';
			$description                           = "$obj_type $action in <a target='blank' href='$post_link'>$post_title</a>";
			$uactid                                = ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
			if ( 0 != $uactid ) {
				ualp_add_activity_log_detail( $uactid, $comment_detail_ary );
			}
		}
	}
}

add_action( 'wp_insert_comment', 'ualp_hook_wp_insert_comment' );

if ( ! function_exists( 'ualp_hook_edit_comment' ) ) {
	/**
	 * Get activity for the user - Edit Comment.
	 *
	 * @param int $comment Comment ID.
	 */
	function ualp_hook_edit_comment( $comment ) {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'edit_comment', $log_hook_list ) ) {
			$action                                = 'edited';
			$obj_type                              = 'Comment';
			$comment_id                            = $comment;
			$com                                   = get_comment( $comment_id );
			$post_id                               = $com->comment_post_ID;
			$comment_detail_ary                    = array();
			$comment_detail_ary['ualp_comment']    = $com->comment_content;
			$comment_detail_ary['ualp_comment_id'] = $comment_id;
			$post_title                            = get_the_title( $post_id );
			$post_link                             = get_edit_post_link( $post_id );
			if ( '' == $post_link ) {
				$post_link = get_the_permalink( $post_id );
			}
			$hook        = 'edit_comment';
			$description = "$obj_type $action in <a target='blank' href='$post_link'>$post_title</a>";
			$uactid      = ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
			if ( 0 != $uactid ) {
				ualp_add_activity_log_detail( $uactid, $comment_detail_ary );}
		}
	}
}

add_action( 'edit_comment', 'ualp_hook_edit_comment' );

if ( ! function_exists( 'ualp_hook_delete_comment' ) ) {
	/**
	 * Get activity for the user - Delete Comment.
	 *
	 * @param int $comment Comment ID.
	 */
	function ualp_hook_delete_comment( $comment ) {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'delete_comment', $log_hook_list ) ) {
			$action                                = 'deleted';
			$obj_type                              = 'Comment';
			$com                                   = get_comment( $comment );
			$post_id                               = $com->comment_post_ID;
			$comment_detail_ary                    = array();
			$comment_detail_ary['ualp_comment_id'] = $comment;
			$comment_detail_ary['ualp_comment']    = $com->comment_content;
			$post_title                            = get_the_title( $post_id );
			$post_link                             = get_the_permalink( $post_id );
			$hook                                  = 'delete_comment';
			$description                           = "$obj_type $action from <a target='blank' href='$post_link'>$post_title</a>";
			$uactid                                = ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
			if ( 0 != $uactid ) {
				ualp_add_activity_log_detail( $uactid, $comment_detail_ary );
			}
		}
	}
}

add_action( 'delete_comment', 'ualp_hook_delete_comment', 10, 1 );

if ( ! function_exists( 'ualp_hook_status_comment' ) ) {
	/**
	 * Get activity for the user - Comment status change.
	 *
	 * @param string $new_status New status of comment.
	 * @param string $old_status Old status of comment.
	 * @param int    $comment_id Comment ID.
	 */
	function ualp_hook_status_comment( $new_status, $old_status, $comment_id ) {
		$log_hook_list                      = get_option( 'enableLogHookList' );
		$user                               = wp_get_current_user();
		$obj_type                           = 'Comment';
		$com                                = get_comment( $comment_id );
		$post_id                            = $com->comment_post_ID;
		$comment_detail_ary                 = array();
		$comment_detail_ary['ualp_comment'] = $com->comment_content;
		$post_title                         = get_the_title( $post_id );
		$post_link                          = get_the_permalink( $post_id );
		if ( 'approved' == $new_status ) {
			if ( 'trash' == $old_status || 'spam' == $old_status ) {
				$action      = 'restored';
				$description = "$obj_type $action from $old_status in <a target='blank' href='$post_link'>$post_title</a>";
				$hook        = 'restore_comment';
				if ( in_array( 'restore_comment', $log_hook_list ) ) {
					$uactid = ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
					if ( 0 != $uactid ) {
						ualp_add_activity_log_detail( $uactid, $comment_detail_ary );
					}
				}
			} else {
				$action      = 'approved';
				$description = "$obj_type $action in <a target='blank' href='$post_link'>$post_title</a>";
				$hook        = 'comment_approve';
				if ( in_array( 'comment_approve', $log_hook_list ) ) {
					$uactid = ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
					if ( 0 != $uactid ) {
						ualp_add_activity_log_detail( $uactid, $comment_detail_ary );
					}
				}
			}
		} elseif ( 'unapproved' == $new_status ) {
			if ( 'trash' == $old_status || 'spam' == $old_status ) {
				$action      = 'restored';
				$description = "$obj_type $action from $old_status in <a target='blank' href='$post_link'>$post_title</a>";
				$hook        = 'restore_comment';
				if ( in_array( 'restore_comment', $log_hook_list ) ) {
					$uactid = ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
					if ( 0 != $uactid ) {
						ualp_add_activity_log_detail( $uactid, $comment_detail_ary );
					}
				}
			} else {
				$action      = 'unapproved';
				$description = "$obj_type $action in <a target='blank' href='$post_link'>$post_title</a>";
				$hook        = 'comment_unapprove';
				if ( in_array( 'comment_unapprove', $log_hook_list ) ) {
					$uactid = ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
					if ( 0 != $uactid ) {
						ualp_add_activity_log_detail( $uactid, $comment_detail_ary );
					}
				}
			}
		} elseif ( 'spam' == $new_status ) {
			$action      = 'spam';
			$description = "$obj_type marked as $action in <a target='blank' href='$post_link'>$post_title</a>";
			$hook        = 'spam_comment';
			if ( in_array( 'spam_comment', $log_hook_list ) ) {
				$uactid = ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
				if ( 0 != $uactid ) {
					ualp_add_activity_log_detail( $uactid, $comment_detail_ary );
				}
			}
		} elseif ( 'trash' == $new_status ) {
			$action      = 'trash';
			$description = "$obj_type moved to $action in <a target='blank' href='$post_link'>$post_title</a>";
			$hook        = 'trash_comment';
			if ( in_array( 'trash_comment', $log_hook_list ) ) {
				$uactid = ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
				if ( 0 != $uactid ) {
					ualp_add_activity_log_detail( $uactid, $comment_detail_ary );
				}
			}
		}
	}
}

add_action( 'transition_comment_status', 'ualp_hook_status_comment', 10, 3 ); // new.

if ( ! function_exists( 'ualp_hook_wp_update_menu' ) ) {
	/**
	 * Get activity for the user - Update navigation menu.
	 */
	function ualp_hook_wp_update_menu() {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( ! isset( $_REQUEST['menu'] ) || ! isset( $_REQUEST['action'] ) ) {
			return;
		}
		if ( 'delete' != $_REQUEST['action'] && 'locations' != $_REQUEST['action'] && 'update' != $_REQUEST['action'] ) {
			return;
		}
		$menu_id = sanitize_text_field( wp_unslash( $_REQUEST['menu'] ) );
		if ( ! is_nav_menu( $menu_id ) ) {
			return;
		}

		$user        = wp_get_current_user();
		$menu_object = wp_get_nav_menu_object( $menu_id );
		$obj_type    = 'Menu';
		$post_id     = $menu_id;
		$post_title  = $menu_object->name;
		if ( 'delete' == $_REQUEST['action'] ) {
			if ( in_array( 'delete_nav_menu', $log_hook_list ) ) {
				$action      = 'deleted';
				$hook        = 'delete_nav_menu';
				$description = "$post_title $obj_type $action";
				ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
			}
		} elseif ( 'locations' == $_REQUEST['action'] ) {
			if ( in_array( 'wp_update_nav_menu', $log_hook_list ) ) {
				$action      = 'location updated';
				$hook        = 'wp_update_nav_menu';
				$description = "$post_title $obj_type $action";
				ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
			}
		} else {
			if ( in_array( 'wp_update_nav_menu', $log_hook_list ) ) {
				$action      = 'updated';
				$hook        = 'wp_update_nav_menu';
				$description = "$post_title $obj_type $action";
				ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
			}
		}
	}
}
add_action( 'load-nav-menus.php', 'ualp_hook_wp_update_menu' );

if ( ! function_exists( 'ualp_hook_wp_create_menu' ) ) {
	/**
	 * Get activity for the user - Create navigation menu.
	 *
	 * @param int $menu Post ID.
	 */
	function ualp_hook_wp_create_menu( $menu ) {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'wp_create_nav_menu', $log_hook_list ) ) {
			$action      = 'created';
			$obj_type    = 'Menu';
			$post_id     = $menu;
			$menu_object = wp_get_nav_menu_object( $post_id );
			$post_title  = $menu_object->name;
			$hook        = 'wp_create_nav_menu';
			$description = "$post_title $obj_type $action";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}
add_action( 'wp_create_nav_menu', 'ualp_hook_wp_create_menu' );

if ( ! function_exists( 'ualp_hook_created_term' ) ) {
	/**
	 * Get activity for the user - Create Terms.
	 *
	 * @param string $term term name.
	 * @param string $taxonomy taxonomy name.
	 */
	function ualp_hook_created_term( $term, $taxonomy ) {
		if ( 'nav_menu' === $taxonomy ) {
			return $term;
		}
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( $taxonomy . '_create', $log_hook_list ) ) {
			$taxonomy_details = get_taxonomy( $taxonomy );
			$action           = 'created';
			$obj_type         = 'Term';
			$post_id          = '';
			$post_title       = $taxonomy . ' - ' . $term;
			$hook             = $taxonomy . '_create';

			$description = "$obj_type '$term' $action in taxonomy '$taxonomy_details->label'";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
		return $term;
	}
}
add_filter( 'pre_insert_term', 'ualp_hook_created_term', 10, 2 );

if ( ! function_exists( 'ualp_hook_delete_term' ) ) {
	/**
	 * Get activity for the user - Delete Terms.
	 *
	 * @param int    $term Term ID.
	 * @param string $taxonomy taxonomy name.
	 */
	function ualp_hook_delete_term( $term, $taxonomy ) {
		if ( 'nav_menu' === $taxonomy ) {
			return $term;
		}
		if ( $taxonomy && ! is_wp_error( $taxonomy ) ) {
			$log_hook_list = get_option( 'enableLogHookList' );
			if ( in_array( $taxonomy . '_delete', $log_hook_list ) ) {
				$action           = 'deleted';
				$obj_type         = 'Term';
				$post_id          = $term;
				$post_title       = '';
				$hook             = $taxonomy . '_delete';
				$term_name        = get_term_by( 'id', $term, $taxonomy );
				$term_link        = get_edit_term_link( $term, $taxonomy );
				$taxonomy_details = get_taxonomy( $taxonomy );
				if ( 'product_shipping_class' == $taxonomy ) {
					$description = "$obj_type '$term_name->name' $action from taxonomy '$taxonomy_details->label'";
				} else {
					$description = "$obj_type <a target='blank' href='$term_link'>'$term_name->name'</a> $action from taxonomy '$taxonomy_details->label'";
				}
				ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
			}
		}
	}
}
add_action( 'pre_delete_term', 'ualp_hook_delete_term', 10, 2 );

/**
 * Get activity for the user - Edit Terms.
 *
 * @param int    $parent Parent Term Id.
 * @param int    $term_id Term Id.
 * @param string $taxonomy Taxonomy Name.
 * @param array  $parsed_args Arguments.
 * @param array  $args Arguments.
 * @return int $parent.
 */
function ualp_hook_edited_term( $parent = null, $term_id = null, $taxonomy = null, $parsed_args = null, $args = null ) {

	if ( 'nav_menu' === $args['taxonomy'] ) {
		return;
	}
	$log_hook_list = get_option( 'enableLogHookList' );
	if ( in_array( $args['taxonomy'] . '_update', $log_hook_list ) ) {
		$action   = 'edited';
		$obj_type = 'Term';
		$hook     = $args['taxonomy'] . '_update';
		$old_term = get_term_by( 'id', $term_id, $taxonomy );
		if ( ! $args || empty( $old_term ) ) {
			return $parent;
		}
		$post_id                   = $term_id;
		$ulap_old_term_name        = $old_term->name;
		$ulap_old_term_slug        = $old_term->slug;
		$ulap_old_term_description = $old_term->description;
		$ulap_old_term_parent      = $old_term->parent;
		$ulap_new_term_name        = $args['name'];
		$ulap_new_term_taxonomy    = $args['taxonomy'];
		$ulap_new_term_slug        = $args['slug'];
		$ulap_new_term_description = $args['description'];
		$ulap_new_term_parent      = $args['parent'];

		$post_title = $ulap_new_term_taxonomy . ' - ' . $ulap_new_term_name;
		if ( 'product_shipping_class' == $taxonomy ) {
			$description = "$obj_type '$ulap_new_term_name' $action in taxonomy '$ulap_new_term_taxonomy'";
		} else {
			$description = "$obj_type <a target='blank' href='" . get_edit_term_link( $term_id, $ulap_new_term_taxonomy ) . "'>'$ulap_new_term_name'</a> $action in taxonomy '$ulap_new_term_taxonomy'";
		}
		$uactid          = ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		$user_detail_ary = array();
		if ( $ulap_old_term_name != $ulap_new_term_name ) {
			$user_detail_ary['ulap_old_term_name'] = $ulap_old_term_name;
			$user_detail_ary['ulap_new_term_name'] = $ulap_new_term_name;
		}
		if ( $ulap_old_term_slug != $ulap_new_term_slug ) {
			$user_detail_ary['ulap_old_term_slug'] = $ulap_old_term_slug;
			$user_detail_ary['ulap_new_term_slug'] = $ulap_new_term_slug;
		}
		if ( $ulap_old_term_description != $ulap_new_term_description ) {
			$user_detail_ary['ulap_old_term_description'] = $ulap_old_term_description;
			$user_detail_ary['ulap_new_term_description'] = $ulap_new_term_description;
		}
		if ( $ulap_old_term_parent != $ulap_new_term_parent ) {
			$user_detail_ary['ulap_old_term_parent'] = $ulap_old_term_parent;
			$user_detail_ary['ulap_new_term_parent'] = $ulap_new_term_parent;
		}
		if ( 0 != $uactid ) {
			ualp_add_activity_log_detail( $uactid, $user_detail_ary );
		}
	}
	return $parent;
}
add_filter( 'wp_update_term_parent', 'ualp_hook_edited_term', 10, 5 );


if ( ! function_exists( 'ualp_hook_switch_theme' ) ) {
	/**
	 * Get activity for the user - Switch Theme.
	 *
	 * @param string $themename Theme Name.
	 * @param object $theme Theme Object.
	 */
	function ualp_hook_switch_theme( $themename, $theme ) {
		global $old_theme_data;
		$user          = wp_get_current_user();
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'switch_theme', $log_hook_list ) ) {
			$action      = 'switched';
			$obj_type    = 'Theme';
			$post_id     = '';
			$post_title  = $themename;
			$hook        = 'switch_theme';
			$description = "$obj_type $action : $post_title";
			$new_theme   = array(
				'name'    => $theme->name,
				'version' => $theme->version,
			);
			$uactid      = ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
			if ( 0 != $uactid ) {
				ualp_add_activity_log_detail_switch_theme( $uactid, $old_theme_data, $new_theme );
			}
		}
	}
}
add_action( 'switch_theme', 'ualp_hook_switch_theme', 2, 2 );

if ( ! function_exists( 'ualp_hook_load_theme' ) ) {
	/**
	 * Get activity for the user - Load Theme.
	 */
	function ualp_hook_load_theme() {
		global $old_theme_data;
		if ( ! isset( $_GET['action'] ) || 'activate' != $_GET['action'] ) {
			return;
		}
		$current_theme = wp_get_theme();
		if ( ! is_a( $current_theme, 'WP_Theme' ) ) {
			return;
		}
		$old_theme_data = array(
			'name'    => $current_theme->name,
			'version' => $current_theme->version,
		);
	}
}
add_action( 'load-themes.php', 'ualp_hook_load_theme' );

if ( ! function_exists( 'ualp_hook_customize_save' ) ) {
	/**
	 * Get activity for the user - Customize Theme.
	 *
	 * @param object $instance customize object.
	 */
	function ualp_hook_customize_save( $instance ) {
		$log_hook_list = get_option( 'enableLogHookList' );
		$user          = wp_get_current_user();
		if ( in_array( 'customize_save', $log_hook_list ) ) {
			$action      = 'customize updated';
			$obj_type    = 'Theme';
			$post_id     = '';
			$post_title  = 'Theme Customizer';
			$hook        = 'customize_save';
			$description = "$obj_type $action : $post_title";
			$uactid      = ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
			if ( 0 != $uactid ) {
				ualp_hook_customize_save_detail( $uactid, $instance );
			}
		}
	}
}

add_action( 'customize_save', 'ualp_hook_customize_save', 10, 1 ); // new.

if ( ! function_exists( 'ualp_hook_theme_install_update' ) ) {
	/**
	 * Get activity for Theme update or install.
	 *
	 * @param Theme_Upgrader $upgrader Upgrader.
	 * @param array          $extra Extra.
	 */
	function ualp_hook_theme_install_update( $upgrader, $extra ) {
		$current_user = wp_get_current_user();

		if ( ! isset( $extra['type'] ) || 'theme' !== $extra['type'] ) {
			return;
		}
		global $old_theme_data;
		$obj_type      = 'Theme';
		$post_id       = '';
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( 'install' === $extra['action'] ) {
			$slug = $upgrader->theme_info();
			if ( ! $slug ) {
				return;
			}

			wp_clean_themes_cache();
			$theme       = wp_get_theme( $slug );
			$name        = $theme->name;
			$version     = $theme->version;
			$action      = 'installed';
			$hook        = 'theme_installed';
			$new_theme   = array(
				'name'    => $theme->name,
				'version' => $theme->version,
			);
			$post_title  = $name . ' ' . $version;
			$description = "Theme $action : $post_title";
			if ( in_array( 'theme_installed', $log_hook_list ) ) {
				$uactid = ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
				if ( 0 != $uactid ) {
					ualp_add_activity_log_detail_upgrade_theme( $uactid, $new_theme );
				}
			}
		}
		if ( 'update' === $extra['action'] ) {
			if ( isset( $extra['bulk'] ) && true == $extra['bulk'] ) {
				$slugs = $extra['themes'];
			} else {
				$slugs = array( $upgrader->skin->theme );
			}

			foreach ( $slugs as $slug ) {
				$theme      = wp_get_theme( $slug );
				$stylesheet = $theme['Stylesheet Dir'] . '/style.css';
				$theme_data = get_file_data( $stylesheet, array( 'Version' => 'Version' ) );
				$name       = $theme['Name'];
				$version    = $theme_data['Version'];
				$new_theme  = array(
					'name'    => $name,
					'version' => $version,
				);
				$action     = 'updated';
				$hook       = 'theme_updated';
			}
			$post_title  = $name . ' ' . $version;
			$description = "Theme $action : $post_title";
			if ( in_array( 'theme_updated', $log_hook_list ) ) {
				$uactid = ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
				if ( 0 != $uactid ) {
					ualp_add_activity_log_detail_upgrade_theme( $uactid, $new_theme );
				}
			}
		}
	}
}
add_action( 'upgrader_process_complete', 'ualp_hook_theme_install_update', 10, 2 );

if ( ! function_exists( 'ualp_hook_theme_deleted' ) ) {
	/**
	 * Get activity for the user - Delelte theme.
	 */
	function ualp_hook_theme_deleted() {
		$current_user  = wp_get_current_user();
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'delete_site_transient_update_themes', $log_hook_list ) ) {
			$backtrace_history = debug_backtrace();
			$delete_theme_call = null;
			foreach ( $backtrace_history as $call ) {
				if ( isset( $call['function'] ) && 'delete_theme' === $call['function'] ) {
					$delete_theme_call = $call;
					break;
				}
			}
			if ( empty( $delete_theme_call ) ) {
				return;
			}
			$name        = $delete_theme_call['args'][0];
			$action      = 'deleted';
			$obj_type    = 'Theme';
			$post_title  = $name;
			$post_id     = '';
			$hook        = 'delete_site_transient_update_themes';
			$description = "Theme $action : $post_title";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}
add_action( 'delete_site_transient_update_themes', 'ualp_hook_theme_deleted' );

if ( ! function_exists( 'ualp_hook_theme_modify' ) ) {
	/**
	 * Get activity for the user - update file from editor.
	 *
	 * @param string $location Location.
	 * @param string $status Status.
	 */
	function ualp_hook_theme_modify( $location, $status ) {
		$current_user = wp_get_current_user();
		if ( strpos( $location, 'theme-editor.php?file=' ) !== false ) {
			if ( isset( $_POST ) && ! empty( $_POST ) && isset( $_POST['action'] ) && 'update' === $_POST['action'] ) {
				if ( isset( $_POST['_wpnonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wpnonce'], 'media-form' ) ) ) ) {
					$log_hook_list = get_option( 'enableLogHookList' );
					if ( in_array( 'theme_file_modify', $log_hook_list ) ) {
						$theme       = isset( $_POST['theme'] ) ? sanitize_text_field( wp_unslash( $_POST['theme'] ) ) : '';
						$file        = isset( $_POST['file'] ) ? sanitize_text_field( wp_unslash( $_POST['file'] ) ) : '';
						$action      = 'File Updated';
						$post_title  = $theme . ' - ' . $file;
						$post_id     = '';
						$obj_type    = 'Theme';
						$hook        = 'theme_file_modify';
						$description = "$action : $post_title";
						ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
					}
				}
			}
		}
		// We are need return the instance, for complete the filter.
		return $location;
	}
}

add_filter( 'update_attached_file', 'ualp_hook_theme_modify', 10, 2 ); // new.

if ( ! function_exists( 'ualp_hook_activated_plugin' ) ) {
	/**
	 * Get activity for the user - Activate Plugin.
	 *
	 * @param string $plugin Plugin Name.
	 */
	function ualp_hook_activated_plugin( $plugin ) {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'activated_plugin', $log_hook_list ) ) {
			$action      = 'activated';
			$obj_type    = 'Plugin';
			$post_id     = '';
			$plugin_data = get_plugin_data( WP_PLUGIN_DIR . '/' . $plugin, true, false );
			$post_title  = $plugin_data['Name'];
			$hook        = 'activated_plugin';
			$description = "$obj_type $action : $post_title";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}

add_action( 'activated_plugin', 'ualp_hook_activated_plugin' );

if ( ! function_exists( 'ualp_hook_deactivated_plugin' ) ) {
	/**
	 * Get activity for the user - Deactivate Plugin.
	 *
	 * @param string $plugin Plugin Name.
	 */
	function ualp_hook_deactivated_plugin( $plugin ) {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'deactivated_plugin', $log_hook_list ) ) {
			$action      = 'deactivated';
			$obj_type    = 'Plugin';
			$post_id     = '';
			$plugin_data = get_plugin_data( WP_PLUGIN_DIR . '/' . $plugin, true, false );
			$post_title  = $plugin_data['Name'];
			$hook        = 'deactivated_plugin';
			$description = "$obj_type $action : $post_title";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}

add_action( 'deactivated_plugin', 'ualp_hook_deactivated_plugin' );

if ( ! function_exists( 'ualp_hook_delete_plugin' ) ) {
	/**
	 * Get activity for the user - Delete Plugin.
	 *
	 * @param string $plugin_file file name.
	 */
	function ualp_hook_delete_plugin( $plugin_file ) {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'delete_plugin', $log_hook_list ) ) {
			$action      = 'deleted';
			$obj_type    = 'Plugin';
			$post_id     = '';
			$plugin_data = get_plugin_data( WP_PLUGIN_DIR . '/' . $plugin_file, true, false );
			$post_title  = $plugin_data['Name'];
			$hook        = 'delete_plugin';
			$description = "$obj_type $action : $post_title";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}

add_action( 'delete_plugin', 'ualp_hook_delete_plugin', 10, 1 ); // new.

// add_filter('wp_redirect', 'ualp_hook_plugin_modify', 10); // new.

add_action( 'load-plugin-editor.php', 'ualp_hook_plugin_modify' );
/**
 * Get activity for the user - Modify Plugin.
 */
function ualp_hook_plugin_modify() {
	if ( isset( $_REQUEST['file'] ) && '' != $_REQUEST['file'] && isset( $_REQUEST['action'] ) && 'plugin-editor.php' == $_REQUEST['action'] ) {
		$plugin_explode = explode( '/', sanitize_text_field( wp_unslash( $_REQUEST['file'] ) ) );
		$plugin_data    = get_plugins( '/' . $plugin_explode[0] );
		foreach ( $plugin_data as $plugin ) {
			$plugin_name = $plugin['Name'];
		}
		$action      = 'File Changed';
		$obj_type    = 'Plugin';
		$post_id     = '';
		$post_title  = $plugin_name . ' - ' . sanitize_text_field( wp_unslash( $_REQUEST['file'] ) );
		$hook        = 'plugin_modify';
		$description = "$obj_type $action : $post_title";
		ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
	}
}

if ( ! function_exists( 'ualp_hook_plugin_install_update' ) ) {
	/**
	 * Get activity for the user - Upgrader process complete.
	 *
	 * @param object $upgrader Upgrader.
	 * @param array  $extra Extra.
	 */
	function ualp_hook_plugin_install_update( $upgrader, $extra ) {
		if ( ! isset( $extra['type'] ) || 'plugin' !== $extra['type'] ) {
			return;
		}

		$obj_type      = 'Plugin';
		$post_id       = '';
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( 'install' === $extra['action'] ) {
			$path = $upgrader->plugin_info();
			if ( ! $path ) {
				return;
			}

			$data        = get_plugin_data( $upgrader->skin->result['local_destination'] . '/' . $path, true, false );
			$action      = 'Installed';
			$post_title  = $data['Name'];
			$hook        = 'plugin_installed';
			$description = "$obj_type $action : $post_title";
			if ( in_array( 'plugin_installed', $log_hook_list ) ) {
				ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
				$user = wp_get_current_user();
			}
		}
		if ( 'update' === $extra['action'] ) {
			if ( isset( $extra['bulk'] ) && true == $extra['bulk'] ) {
				$slugs = $extra['plugins'];
			} else {
				if ( ! isset( $upgrader->skin->plugin ) ) {
					return;
				}
				$slugs = array( $upgrader->skin->plugin );
			}
			foreach ( $slugs as $slug ) {
				$data   = get_plugin_data( WP_PLUGIN_DIR . '/' . $slug, true, false );
				$action = 'Updated';
			}
			$post_title  = $data['Name'];
			$hook        = 'plugin_updated';
			$description = "$obj_type $action : $post_title";
			if ( in_array( 'plugin_updated', $log_hook_list ) ) {
				ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
			}
		}
	}
}
add_action( 'upgrader_process_complete', 'ualp_hook_plugin_install_update', 10, 2 ); // new.

if ( ! function_exists( 'ualp_hook_export_wp' ) ) {
	/**
	 * Get activity for the user - Export WordPress data.
	 *
	 * @param array $args Arguments.
	 */
	function ualp_hook_export_wp( $args ) {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'export_wp', $log_hook_list ) ) {
			$action      = 'Downloaded';
			$obj_type    = 'Export';
			$post_id     = '';
			$post_title  = isset( $args['content'] ) ? $args['content'] : 'all';
			$hook        = 'export_wp';
			$description = "$obj_type file $action : $post_title";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}

add_action( 'export_wp', 'ualp_hook_export_wp' );

if ( ! function_exists( 'ualp_hook_import_wp' ) ) {
	/**
	 * Get activity for the user - Import WordPress data.
	 */
	function ualp_hook_import_wp() {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'import_end', $log_hook_list ) ) {
			$action      = 'WordPress';
			$obj_type    = 'Import';
			$post_id     = '';
			$post_title  = $obj_type;
			$hook        = 'import_end';
			$description = "$obj_type $action data";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}

add_action( 'import_end', 'ualp_hook_import_wp' ); // new.

if ( ! function_exists( 'ualp_hook_save_post_status' ) ) {

	/**
	 * Get activity for the user - Save Post.
	 *
	 * @param int    $post_id post id.
	 * @param object $post current post.
	 * @param string $update current post.
	 * @param string $post_before current post.
	 */
	function ualp_hook_save_post_status( $post_id, $post, $update, $post_before ) {
		global $old_post_data, $post, $new_post_data_meta, $new_post_data;
		if ( isset( $_POST['_inline_edit'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_inline_edit'] ) ), 'inlineeditnonce' ) ) {
			$widget_post = json_decode( wp_json_encode( $_POST ) );
		} else {
			if ( isset( $_POST ) && ! empty( $_POST ) ) {
				$widget_post = json_decode( wp_json_encode( $_POST ) );
			} else {
				$widget_post = json_decode( wp_json_encode( $_REQUEST ) );
				$widget_post = isset( $_REQUEST['post'] ) ? get_post( sanitize_text_field( wp_unslash( $_REQUEST['post'] ) ) ) : '';
			}
		}
		@$obj_type            = $widget_post->post_type;
		$log_hook_list        = get_option( 'enableLogHookList' );
		$old_post_data_detail = isset( $old_post_data['post_data'] ) ? $old_post_data['post_data'] : '';
		if ( isset( $old_post_data_detail ) && '' != $old_post_data_detail ) {
			$oldstatus = $old_post_data_detail->post_status;
		}
		@$newstatus = $widget_post->post_status;
		$old_status = isset( $oldstatus ) ? $oldstatus : '';
		$new_status = isset( $newstatus ) ? $newstatus : '';
		$post       = get_post( $post_id );
		$new_status = isset( $post->post_status ) ? $post->post_status : $new_status;
		$obj_type   = isset( $post->post_type ) ? $post->post_type : $new_status;
		if ( in_array( $obj_type . '_create', $log_hook_list ) && in_array( $obj_type . '_update', $log_hook_list ) && in_array( $obj_type . '_delete', $log_hook_list ) ) {
			if ( 'nav_menu_item' == get_post_type( $post ) || 'wpcf7_contact_form' == get_post_type( $post ) || wp_is_post_revision( $post ) || 'customize_changeset' == $obj_type ) {
				return;
			}
			$action = isset( $action ) ? $action : '';
			if ( ( 'forum' == get_post_type( $post ) || 'download' == get_post_type( $post ) ) && ( 'drafted' == $action ) ) {
				return;
			}
			if ( wp_is_post_revision( $post_id ) ) {
				return;
			}

			if ( isset( $widget_post ) && is_object( $widget_post ) ) {
				$post_id = $widget_post->post_ID;
			}
			if ( isset( $widget_post ) && empty( $widget_post ) ) {
				$post_id = $post_before->ID;
			}
			if ( isset( $widget_post->post_title ) && ! empty( $widget_post->post_title ) ) {
				$post_title = $widget_post->post_title;
			} else {
				$post_title = get_the_title( $post_id );
			}
			$action = '';
			$hook   = '';
			if ( 'auto-draft' === $new_status || ( 'new' === $old_status && 'inherit' === $new_status ) ) {
				return;
			} elseif ( 'auto-draft' === $old_status && 'draft' === $new_status ) {
				$action = 'drafted';
				$hook   = 'transition_post_status';
			} elseif ( 'draft' === $old_status && 'publish' === $new_status && '0000-00-00 00:00:00' === $old_post_data['post_data']->post_date_gmt ) {
				$action = 'created';
				$hook   = $obj_type . '_create';
			} elseif ( 'publish' === $old_status && 'trash' != $new_status ) {
				$action = 'updated';
				$hook   = $obj_type . '_update';
			} elseif ( 'publish' === $old_status && 'publish' === $new_status ) {
				$action = 'updated ';
				$hook   = $obj_type . '_update';
			} elseif ( 'publish' === $new_status && 'draft' != $old_status ) {
				$action = 'created ';
				$hook   = $obj_type . '_create';
			} elseif ( 'trash' != $new_status ) {
				$hook   = $obj_type . '_update';
				$action = 'updated ';
			} else {
				return;
			}
			$page_link = get_edit_post_link( $post_id );
			if ( '' == $page_link ) {
				$page_link = get_the_permalink( $post_id );
			}
			$description = "$obj_type $action : <a target='blank' href='" . $page_link . "'>$post_title</a>";
			$uactid      = ualp_get_post_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
			$post_type   = $widget_post->post_type;
			$post_tax    = array();

			if ( '' != $post_type && 'nav_menu_item' != $post_type ) {
				$taxonomy_names = get_object_taxonomies( $post_type );
				if ( is_array( $taxonomy_names ) && ! empty( $taxonomy_names ) ) {
					foreach ( $taxonomy_names as $taxonomy_name ) {
						$post_cats = wp_get_post_terms( $post_id, $taxonomy_name );

						$post_cats_ids = array();
						foreach ( $post_cats as $post_cat ) {
							$post_cats_ids[] = $post_cat->term_id;
						}

						if ( isset( $taxonomy_name ) && ! empty( $taxonomy_name ) ) {
							if ( is_array( $post_cats_ids ) && ! empty( $post_cats_ids ) ) {
								$post_tax[ $taxonomy_name ] = $post_cats_ids;
							}
						}
					}
				}
			}
			$new_post_data = array(
				'post_data' => $post,
				'post_meta' => get_post_custom( $post_id ),
				'post_tax'  => $post_tax,
			);

			if ( 0 != $uactid ) {
				ualp_add_activity_log_detail_post_update( $uactid, $old_post_data, $new_post_data, $post_type, $post_id ); }
		}
	}
}

add_action( 'wp_after_insert_post', 'ualp_hook_save_post_status', 100, 4 );
add_action( 'updated_post_meta', 'ualp_hook_before_edit_post_meta', 100, 4 );
if ( ! function_exists( 'ualp_hook_before_edit_post_meta' ) ) {
	/**
	 * Get activity for the user - Before Save Post.
	 *
	 * @param int    $meta_id Meta ID.
	 * @param int    $object_id Object ID.
	 * @param string $meta_key Meta Key.
	 * @param string $meta_value Meta Value.
	 */
	function ualp_hook_before_edit_post_meta( $meta_id, $object_id, $meta_key, $meta_value ) {
		global $new_post_data_meta;
		$new_post_data_meta = get_post_custom( $object_id );
		$new_post_data      = $new_post_data_meta;
	}
}
if ( ! function_exists( 'ualp_hook_before_edit_post' ) ) {
	/**
	 * Hook for Before Edit Post.
	 *
	 * @param int    $post_ID post id.
	 * @param string $post_data post data.
	 */
	function ualp_hook_before_edit_post( $post_ID, $post_data ) {
		global $old_post_data;
		$prev_post_data = get_post( $post_ID );
		$post_type      = $prev_post_data->post_type;
		$post_tax       = array();
		if ( '' != $post_type && 'nav_menu_item' != $post_type ) {
			$taxonomy_names = get_object_taxonomies( $post_type );
			if ( is_array( $taxonomy_names ) && ! empty( $taxonomy_names ) ) {
				foreach ( $taxonomy_names as $taxonomy_name ) {
					$post_cats     = wp_get_post_terms( $post_ID, $taxonomy_name );
					$post_cats_ids = array();
					foreach ( $post_cats as $post_cat ) {
						$post_cats_ids[] = $post_cat->term_id;
					}

					if ( isset( $taxonomy_name ) ) {
						if ( is_array( $post_cats_ids ) && ! empty( $post_cats_ids ) ) {
							$post_tax[ $taxonomy_name ] = $post_cats_ids;
						}
					}
				}
			}
		}

		$old_post_data = array(
			'post_data' => $prev_post_data,
			'post_meta' => get_post_custom( $post_ID ),
			'post_tax'  => $post_tax,
		);
	}
}
add_action( 'pre_post_update', 'ualp_hook_before_edit_post', 99, 2 );


if ( ! function_exists( 'ualp_hook_untrash_post_status' ) ) {
	/**
	 * Get activity for the user - Restore Post.
	 *
	 * @param int $post_id post id.
	 */
	function ualp_hook_untrash_post_status( $post_id ) {
		$post          = get_post( $post_id );
		$obj_type      = $post->post_type;
		$log_hook_list = get_option( 'enableLogHookList' );

		if ( in_array( $obj_type . '_delete', $log_hook_list ) && 'edd_discount' != $obj_type ) {
			if ( 'nav_menu_item' == $obj_type || 'wpcf7_contact_form' == $obj_type || wp_is_post_revision( $post ) || 'customize_changeset' == $obj_type ) {
				return;
			}
			if ( wp_is_post_revision( $post_id ) ) {
				return;
			}

			$post_id    = $post->ID;
			$post_title = $post->post_title;

			$action = 'restored';
			$hook   = 'transition_post_status';

			$page_link = get_edit_post_link( $post_id );
			if ( '' == $page_link ) {
				$page_link = get_the_permalink( $post_id );
			}
			$description = "$obj_type $action : <a target='blank' href='" . $page_link . "'>$post_title</a>";
			$uactid      = ualp_get_post_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}

add_action( 'untrash_post', 'ualp_hook_untrash_post_status', 100, 1 );


if ( ! function_exists( 'ualp_hook_trash_post_status' ) ) {
	/**
	 * Get activity for the user - Trash Post.
	 *
	 * @param int $post_id post id.
	 */
	function ualp_hook_trash_post_status( $post_id ) {
		$post          = get_post( $post_id );
		$obj_type      = $post->post_type;
		$log_hook_list = get_option( 'enableLogHookList' );

		if ( in_array( $obj_type . '_delete', $log_hook_list ) && 'edd_discount' != $obj_type ) {
			if ( 'nav_menu_item' == $obj_type || 'wpcf7_contact_form' == $obj_type || wp_is_post_revision( $post ) || 'customize_changeset' == $obj_type ) {
				return;
			}
			if ( wp_is_post_revision( $post_id ) ) {
				return;
			}

			$post_id    = $post->ID;
			$post_title = $post->post_title;

			$action = 'trashed';
			$hook   = $obj_type . '_delete';

			$page_link = get_edit_post_link( $post_id );
			if ( '' == $page_link ) {
				$page_link = get_the_permalink( $post_id );
			}
			$description = "$obj_type $action : <a target='blank' href='" . $page_link . "'>$post_title</a>";
			$uactid      = ualp_get_post_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}

add_action( 'wp_trash_post', 'ualp_hook_trash_post_status', 100, 1 );

if ( ! function_exists( 'ualp_hook_delete_post_status' ) ) {
	/**
	 * Get activity for the user - Trash Post.
	 *
	 * @param int $post_id post id.
	 */
	function ualp_hook_delete_post_status( $post_id ) {
		$post          = get_post( $post_id );
		$obj_type      = $post->post_type;
		$log_hook_list = get_option( 'enableLogHookList' );

		if ( in_array( $obj_type . '_delete', $log_hook_list ) && 'edd_discount' != $obj_type ) {
			if ( 'nav_menu_item' == $obj_type || 'wpcf7_contact_form' == $obj_type || wp_is_post_revision( $post ) || 'customize_changeset' == $obj_type ) {
				return;
			}
			if ( wp_is_post_revision( $post_id ) ) {
				return;
			}

			$post_id    = $post->ID;
			$post_title = $post->post_title;

			$action = 'deleted';
			$hook   = $obj_type . '_delete';

			$page_link = get_edit_post_link( $post_id );
			if ( '' == $page_link ) {
				$page_link = get_the_permalink( $post_id );
			}
			$description = "$obj_type $action : <a target='blank' href='" . $page_link . "'>$post_title</a>";
			$uactid      = ualp_get_post_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}

add_action( 'delete_post', 'ualp_hook_delete_post_status', 100, 1 );

if ( ! function_exists( 'ualp_hook_delete_post' ) ) {
	/**
	 * Get activity for the user - Delete Post.
	 *
	 * @param int $pid Post Id.
	 */
	function ualp_hook_delete_post( $pid ) {
		$log_hook_list = get_option( 'enableLogHookList' );
		global $post_type,$wpdb;
		if ( in_array( $post_type . '_delete', $log_hook_list ) ) {
			if ( did_action( 'before_delete_post' ) == 1 ) {
				$action      = 'deleted';
				$obj_type    = $post_type;
				$post_id     = $pid;
				$post_title  = get_the_title( $pid );
				$hook        = $post_type . '_delete';
				$description = "$obj_type $action permanently: $post_title";
				ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
			}
		}
	}
}

add_filter( 'wp_privacy_personal_data_exporters', 'ualp_register_exporter', 10 );

if ( ! function_exists( 'ualp_register_exporter' ) ) {
	/**
	 * Register Exporter Function.
	 *
	 * @param array $exporters_array Exporters Array.
	 */
	function ualp_register_exporter( $exporters_array ) {
		$exporters_array['ualp_exporter'] = array(
			'exporter_friendly_name' => 'UALP exporter',
			'callback'               => 'ualp_exporter_function',
		);
		return $exporters_array;
	}
}

/**
 * Exporter Function.
 *
 * @param string $email_address Email Address.
 * @param string $iteration Iteration.
 */
function ualp_exporter_function( $email_address, $iteration = 1 ) {

	$iteration     = (int) $iteration;
	$export_items  = array();
	$log_hook_list = get_option( 'enableLogHookList' );

	$orders = get_posts(
		array(
			'post_type'      => 'export_personal_data',
			'posts_per_page' => -1,
			'paged'          => $iteration,
			'meta_key'       => 'user_email',
			'meta_value'     => $email_address,
		)
	);
	if ( $orders ) {

		foreach ( (array) $orders as $order ) {

			$data = array(
				array(
					'name'  => 'Full Name',
					'value' => get_post_meta( $order->ID, 'nickname', true ),
				),
				array(
					'name'  => 'Email',
					'value' => $email_address,
				),
			);

			$export_items[] = array(
				'group_id'    => 'orders',
				'group_label' => 'Orders',
				'item_id'     => 'order-' . $order->ID,
				'data'        => $data,
			);
		}
	}

	if ( in_array( 'wp_privacy_personal_data_exporters', $log_hook_list ) ) {
		$action      = 'Downloaded';
		$obj_type    = 'Export Personal Data';
		$post_id     = '';
		$post_title  = '';
		$hook        = 'wp_privacy_personal_data_exporters';
		$description = "$obj_type $action";
		ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
	}

	$done = count( $orders ) < 100;
	return array(
		'data' => $export_items,
		'done' => $done,
	);
}

add_filter( 'wp_privacy_personal_data_erasers', 'ualp_register_my_eraser', 10 );

if ( ! function_exists( 'ualp_register_my_eraser' ) ) {
	/**
	 * Register Eraser Function.
	 *
	 * @param array $erasers Erasers.
	 */
	function ualp_register_my_eraser( $erasers ) {
		$erasers['ualp_eraser'] = array(
			'eraser_friendly_name' => 'UALP eraser',
			'callback'             => 'ualp_eraser_function',
		);
		return $erasers;
	}
}

/**
 * Eraser Function.
 *
 * @param string $email_address Email Address.
 * @param string $iteration Iteration.
 */
function ualp_eraser_function( $email_address, $iteration = 1 ) {

	$iteration     = (int) $iteration;
	$items_removed = false;
	$log_hook_list = get_option( 'enableLogHookList' );

	$orders = get_posts(
		array(
			'post_type'      => 'export_personal_data',
			'posts_per_page' => -1,
			'paged'          => $iteration,
			'meta_key'       => 'user_email',
			'meta_value'     => $email_address,
		)
	);
	if ( $orders ) {

		foreach ( (array) $orders as $order ) {
			delete_post_meta( $order->ID, 'user_email' );
			$items_removed = true;
		}
	}

	if ( in_array( 'wp_privacy_personal_data_erasers', $log_hook_list ) ) {
		$action      = 'successfully';
		$obj_type    = 'Erase Personal Data';
		$post_id     = '';
		$post_title  = '';
		$hook        = 'wp_privacy_personal_data_erasers';
		$description = "$obj_type $action";
		ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
	}

	$done = count( $orders ) < $iteration;
	return array(
		'items_removed'  => $items_removed,
		'items_retained' => false,
		'messages'       => array( '' ),
		'done'           => $done,
	);
}

if ( ! function_exists( 'ualp_hook_widget_update' ) ) {
	/**
	 * Get activity for the user - Widget update.
	 *
	 * @param object $instance Instance.
	 * @param object $new_instance New Instance.
	 * @param object $old_instance Old Instance.
	 * @param object $widget_instance Widget Instance.
	 * @return object $instance
	 */
	function ualp_hook_widget_update( $instance, $new_instance, $old_instance, $widget_instance ) {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'widget_update_callback', $log_hook_list ) ) {
			if ( empty( $old_instance ) ) {
				return $instance;
			}
			$action      = 'updated';
			$obj_type    = 'Widget';
			$post_id     = '';
			$post_title  = $widget_instance->name;
			$hook        = 'widget_update_callback';
			$get_sidebar = '';
			if ( isset( $_POST['sidebar'] ) ) {
				$get_sidebar = sanitize_text_field( wp_unslash( $_POST['sidebar'] ) );
			}
			$sidebar_id   = $get_sidebar;
			$sidebar      = false;
			$sidebars     = isset( $GLOBALS['wp_registered_sidebars'] ) ? $GLOBALS['wp_registered_sidebars'] : false;
			$sidebar_name = '';
			if ( $sidebars ) {
				if ( isset( $sidebars[ $sidebar_id ] ) ) {
					$sidebar = $sidebars[ $sidebar_id ];
				}
			}
			if ( $sidebar ) {
				$sidebar_name = $sidebar['name'];
			}
			$description = "$post_title $obj_type $action in $sidebar_name";
			$uactid      = ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
			if ( 0 != $uactid ) {
				ualp_hook_widget_update_detail( $uactid, $instance, $new_instance, $old_instance, $widget_instance );
			}
		}
		return $instance;
	}
}
add_filter( 'widget_update_callback', 'ualp_hook_widget_update', 8, 4 );

if ( ! function_exists( 'ualp_hook_widget_added_deleted' ) ) {
	/**
	 * Get activity for the user - Widget Add Delete.
	 */
	function ualp_hook_widget_added_deleted() {
		if ( ( isset( $_POST['add_new'] ) && ! empty( $_POST['add_new'] ) && isset( $_POST['sidebar'] ) && isset( $_POST['id_base'] ) ) || isset( $_POST['delete_widget'] ) ) {
			$obj_type          = 'Widget';
			$post_id           = '';
			$hook              = 'sidebar_admin_setup';
			$widget_id_base    = sanitize_text_field( wp_unslash( $_POST['id_base'] ) );
			$widget            = false;
			$widget_detail_ary = array();
			$widget_factory    = isset( $GLOBALS['wp_widget_factory'] ) ? $GLOBALS['wp_widget_factory'] : false;
			if ( $widget_factory ) {
				foreach ( $widget_factory->widgets as $one_widget ) {
					if ( $one_widget->id_base == $widget_id_base ) {
						$widget = $one_widget;
					}
				}
			}
			if ( $widget ) {
				$post_title                                       = $widget->name;
				$widget_detail_ary['ualp_widget_name_translated'] = $widget->name;
				$widget_detail_ary['ualp_widget_id_base']         = $widget_id_base;
			}
			$sidebar_id = sanitize_text_field( wp_unslash( $_POST['sidebar'] ) );
			$sidebar    = false;
			$sidebars   = isset( $GLOBALS['wp_registered_sidebars'] ) ? $GLOBALS['wp_registered_sidebars'] : false;
			if ( $sidebars ) {
				if ( isset( $sidebars[ $sidebar_id ] ) ) {
					$sidebar = $sidebars[ $sidebar_id ];
				}
			}
			if ( $sidebar ) {
				$sidebar_name                                      = $sidebar['name'];
				$widget_detail_ary['ualp_sidebar_name_translated'] = $sidebar['name'];
				$widget_detail_ary['ualp_sidebar_id']              = $sidebar_id;
			}
			if ( isset( $_POST['delete_widget'] ) ) {
				$action      = 'removed';
				$description = "$obj_type $post_title '$action' from '$sidebar_name'";
			} else {
				$action      = 'added';
				$description = "$obj_type $post_title '$action' in '$sidebar_name'";
			}
			$uactid = ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
			ualp_add_activity_log_detail( $uactid, $widget_detail_ary );
			if ( 0 != $uactid ) {
				ualp_add_activity_log_detail( $uactid, $widget_detail_ary );
			}
		}
	}
}
add_action( 'sidebar_admin_setup', 'ualp_hook_widget_added_deleted' );

if ( ! function_exists( 'ualp_hook_core_updated' ) ) {
	/**
	 * Get activity for the user - Core file updated successfully.
	 *
	 * @param float $wp_version WordPress Version.
	 */
	function ualp_hook_core_updated( $wp_version ) {
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( '_core_updated_successfully', $log_hook_list ) ) {
			$action      = 'update';
			$obj_type    = 'Core';
			$post_id     = '';
			$post_title  = $wp_version;
			$hook        = '_core_updated_successfully';
			$description = "WordPress $obj_type $action : $post_title";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}

add_action( '_core_updated_successfully', 'ualp_hook_core_updated' );

if ( ! function_exists( 'ualp_hook_admin_view' ) ) {
	/**
	 * Get activity for Admin View.
	 *
	 * @global string $admin_title.
	 */
	function ualp_hook_admin_view() {
		global $admin_title;
		$current_url   = ualp_get_current_admin_page_url();
		$current_url   = urldecode( $current_url );
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( in_array( 'admin_head', $log_hook_list ) ) {
			$action      = 'Viewed';
			$obj_type    = 'Admin Page';
			$post_id     = '';
			$post_title  = $admin_title;
			$hook        = 'admin_head';
			$description = "$action $obj_type : <a target='blank' href='$current_url'>$post_title</a>";
			ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
		}
	}
}

add_action( 'admin_head', 'ualp_hook_admin_view' ); // new.

if ( ! function_exists( 'ualp_hook_front_view' ) ) {
	/**
	 * Get activity for Front View.
	 */
	function ualp_hook_front_view() {
		$log_hook_list              = get_option( 'enableLogHookList' );
		$ualp_user_email_notify     = get_option( 'ualpEmailNotification' );
		$ualp404_page_access_number = get_option( 'ualp404PageAccessNumber' );
		$ualp_user404_page_access   = get_option( 'ualpUser404PageAccess' );

		$ualp_mobile_notification       = get_option( 'ualpMobileNotification' );
		$ualp404_page_access_number_att = get_option( 'ualp404PageAccessNumberAtt' );

		if ( is_404() ) {
			if ( in_array( '404_page', $log_hook_list ) ) {
				$action      = 'Viewed';
				$obj_type    = '404 Page';
				$current_url = '';
				$post_id     = '';
				$current_url = ualp_get_current_admin_page_url();
				$current_url = urldecode( $current_url );
				$user        = wp_get_current_user();
				$username    = $user->user_login;
				if ( '' == $username ) {
					$display_username = 'Guest';
				} else {
					$display_username = $username;
				}
				$post_title  = '404 page';
				$hook        = '404_page';
				$description = "$action $obj_type $display_username : <a target='blank' href='$current_url'>$current_url</a>";
				$ucid        = ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
				global $wpdb;
				if ( is_multisite() ) {
					$count404page = $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(`uactid`) FROM {$wpdb->base_prefix}ualp_user_activity where hook='404_page' AND object_type=%s", $obj_type ) );
				} else {
					$count404page = $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(`uactid`) FROM {$wpdb->prefix}ualp_user_activity where hook='404_page' AND object_type=%s", $obj_type ) );
				}

				if ( $count404page >= $ualp404_page_access_number && 0 != $ucid ) {
					$user = wp_get_current_user();
					if ( is_array( $ualp_user_email_notify ) && in_array( 'ualpUser404PageAccess', $ualp_user_email_notify ) ) {
						ualp_send_email_notification( $hook, $display_username, 'Has requested a non existing page is ' . $current_url . ' (404 Error Pages)' . $count404page . ' time' );
					}
				}

				if ( $count404page >= $ualp404_page_access_number_att && 0 != $ucid ) {

					$user = wp_get_current_user();
					if ( is_array( $ualp_mobile_notification ) && in_array( 'ualpUser404PageAccess', $ualp_mobile_notification ) ) {
						ualp_send_sms_notification( $hook, $display_username, 'Has requested a non existing page is ' . $current_url . ' (404 Error Pages)' . $count404page . ' time' );
					}
				}
			}
		} else {
			if ( is_multisite() && empty( $log_hook_list ) ) {
				$log_hook_list = array();
			}
			if ( in_array( 'wp_head', $log_hook_list ) ) {
				$action      = 'Viewed';
				$obj_type    = get_post_type( get_the_ID() );
				$current_url = '';
				$post_title  = '';
				$post_id     = '';
				$current_url = ualp_get_current_admin_page_url();
				$current_url = urldecode( $current_url );
				if ( is_single() ) {
					$post_id    = get_the_ID();
					$post_title = get_the_title();
				} else {
					$post_title = $current_url;
				}
				$hook        = 'wp_head';
				$description = "$action $obj_type : <a target='blank' href='$current_url'>$current_url</a>";
				ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );

			}
		}
	}
}

add_action( 'wp_head', 'ualp_hook_front_view' ); // new.


if ( ! function_exists( 'ualp_custom_event_hook' ) ) {
	/**
	 * Function to add custom actions.
	 */
	function ualp_custom_event_hook() {
		global $wp_filter;
		$custom_event_data = get_option( 'custom_event_data' );
		if ( is_array( $custom_event_data ) && ! empty( $custom_event_data ) ) {
			foreach ( $custom_event_data as $custom_event_single ) {
				$number_of_params = apply_filters( 'number_of_params_custom_action', '5', $custom_event_single['custom_event_name'] );
				$priority         = apply_filters( 'priority_custom_action', '10', $custom_event_single['custom_event_name'] );

				add_action( $custom_event_single['custom_event_name'], 'ualp_custom_event_action', $priority, $number_of_params );
			}
		}
	}
}

ualp_custom_event_hook();

/**
 * Callback function to add custom actions.
 *
 * @param string $arg1 Argument1.
 * @param string $arg2 Argument2.
 * @param string $arg3 Argument3.
 * @param string $arg4 Argument4.
 * @param string $arg5 Argument5.
 */
function ualp_custom_event_action( $arg1 = '', $arg2 = '', $arg3 = '', $arg4 = '', $arg5 = '' ) {
	$custom_par_array  = array(
		'arg1' => $arg1,
		'arg2' => $arg2,
		'arg3' => $arg3,
		'arg4' => $arg4,
		'arg5' => $arg5,
	);
	$custom_event_data = get_option( 'custom_event_data' );
	if ( is_array( $custom_event_data ) && ! empty( $custom_event_data ) ) {
		foreach ( $custom_event_data as $custom_event_single ) {
			$action      = $custom_event_single['custom_event_label'];
			$hook        = $custom_event_single['custom_event_name'];
			$obj_type    = $custom_event_single['custom_event_type'];
			$description = $custom_event_single['custom_event_description'];
			$description = apply_filters( 'custom_event_description', $description, $hook, $custom_par_array );
			if ( current_action() == $custom_event_single['custom_event_name'] ) {
				$activity_id = ualp_get_activity_log( $action, $obj_type, '', '', $hook, $description );
				do_action( 'ualpaddcustomeventdetail', $activity_id, $hook, $custom_par_array );
			}
		}
	}
}


/**
 * For ACF Support.
 */
if ( is_plugin_active( 'advanced-custom-fields/acf.php' ) ) {
	if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'export' ) ) {
		if ( isset( $_POST['acf_posts'] ) && is_array( $_POST['acf_posts'] ) && ! empty( $_POST['acf_posts'] ) ) {
			$acf_posts     = array_map( 'sanitize_text_field', wp_unslash( $_POST['acf_posts'] ) );
			$i             = 0;
			$acf_post_data = '';
			foreach ( $acf_posts as $acf_post ) {
				$field_name = get_the_title( $acf_post );
				if ( 0 == $i ) {
					$acf_post_data .= $field_name;
				} else {
					$acf_post_data .= ', ' . $field_name;
				}
			}
			$action_name = 'Downloaded';
			$obj_type    = 'Export';
			$postid      = 0;
			if ( isset( $_POST['export_to_xml'] ) ) {
				$post_title = 'ACF File Exported to xml';
			} elseif ( isset( $_POST['export_to_php'] ) ) {
				$post_title = 'ACF File Exported to php';
			}
			$hook        = 'export_acf';
			$description = "$post_title with fields : $acf_post_data";
			ualp_get_activity_log( $action_name, $obj_type, $postid, $post_title, $hook, $description );
		}
	}
}

if ( ! function_exists( 'ualp_update_options' ) ) {
	/**
	 * Function to add log when options updated.
	 */
	function ualp_update_options() {
		$screen = get_current_screen();
		if ( isset( $_REQUEST['settings-updated'] ) && true == $_REQUEST['settings-updated'] && isset( $_GET['page'] ) && 'bbpress' != $_GET['page'] ) {
			$current_url           = ( isset( $_SERVER['HTTPS'] ) ? 'https' : 'http' ) . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
			$current_url           = str_replace( '&settings-updated=true', '', $current_url );
			$current_url           = str_replace( '?settings-updated=true', '', $current_url );
			$current_url           = substr( $current_url, -20 );
			$transient_name        = 'sp_' . $current_url;
			$settings_page_bbpress = get_transient( $transient_name );
			if ( '' != $settings_page_bbpress ) {
				$action      = 'updated';
				$obj_type    = 'Settings';
				$post_id     = '';
				$post_title  = $screen->base;
				$hook        = 'updated_option';
				$description = "$obj_type $action : $post_title";
				$uactid      = ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
				$oldvalue    = array();
				$_newvalue   = array();
				if ( isset( $settings_page_bbpress ) && is_array( $settings_page_bbpress ) ) {
					foreach ( $settings_page_bbpress as $settings_page_bbpress_key => $settings_page_bbpress_val ) {
						$whitelist_options = ualp_get_options();
						if ( ! in_array( $settings_page_bbpress_key, $whitelist_options ) ) {
							return;
						}
						if ( 'start_of_week' == $settings_page_bbpress_key ) {
							if ( 0 == $settings_page_bbpress_val['old_v'] ) {
								$settings_page_bbpress_val['old_v'] = 'Sunday';
							} elseif ( 1 == $settings_page_bbpress_val['old_v'] ) {
								$settings_page_bbpress_val['old_v'] = 'Monday';
							} elseif ( 2 == $settings_page_bbpress_val['old_v'] ) {
								$settings_page_bbpress_val['old_v'] = 'Tuesday';
							} elseif ( 3 == $settings_page_bbpress_val['old_v'] ) {
								$settings_page_bbpress_val['old_v'] = 'Wednesday';
							} elseif ( 4 == $settings_page_bbpress_val['old_v'] ) {
								$settings_page_bbpress_val['old_v'] = 'Thursday';
							} elseif ( 5 == $settings_page_bbpress_val['old_v'] ) {
								$settings_page_bbpress_val['old_v'] = 'Friday';
							} elseif ( 6 == $settings_page_bbpress_val['old_v'] ) {
								$settings_page_bbpress_val['old_v'] = 'Saturday';
							}
							if ( 0 == $settings_page_bbpress_val['new_v'] ) {
								$settings_page_bbpress_val['new_v'] = 'Sunday';
							} elseif ( 1 == $settings_page_bbpress_val['new_v'] ) {
								$settings_page_bbpress_val['new_v'] = 'Monday';
							} elseif ( 2 == $settings_page_bbpress_val['new_v'] ) {
								$settings_page_bbpress_val['new_v'] = 'Tuesday';
							} elseif ( 3 == $settings_page_bbpress_val['new_v'] ) {
								$settings_page_bbpress_val['new_v'] = 'Wednesday';
							} elseif ( 4 == $settings_page_bbpress_val['new_v'] ) {
								$settings_page_bbpress_val['new_v'] = 'Thursday';
							} elseif ( 5 == $settings_page_bbpress_val['new_v'] ) {
								$settings_page_bbpress_val['new_v'] = 'Friday';
							} elseif ( 6 == $settings_page_bbpress_val['new_v'] ) {
								$settings_page_bbpress_val['new_v'] = 'Saturday';
							}
						}
						$oldvalue[ $settings_page_bbpress_key ]  = $settings_page_bbpress_val['old_v'];
						$_newvalue[ $settings_page_bbpress_key ] = $settings_page_bbpress_val['new_v'];
					}
					ualp_add_wordpress_settings_log_detail( $uactid, $oldvalue, $_newvalue, $obj_type, $option = '' );
				}
			}
			delete_transient( $transient_name );
		}
	}
}
add_action( 'admin_head', 'ualp_update_options', 10 );

add_action( 'updated_option', 'ualp_hook_bbpress_updated_option', 100, 3 );

if ( ! function_exists( 'ualp_hook_bbpress_updated_option' ) ) {
	/**
	 * Fires when settings updated
	 *
	 * @param string $option Option.
	 * @param string $oldvalue Old Value.
	 * @param string $_newvalue New Value.
	 */
	function ualp_hook_bbpress_updated_option( $option, $oldvalue, $_newvalue ) {
		if ( 'rewrite_rules' != $option ) {
			$log_hook_list = get_option( 'enableLogHookList' );
			if ( is_multisite() && empty( $log_hook_list ) ) {
				$log_hook_list = array();
			}
			if ( in_array( 'bbpress_settings', $log_hook_list ) ) {
				$action      = 'updated';
				$obj_type    = 'Settings';
				$post_id     = '';
				$post_title  = $option;
				$hook        = 'bbpress_settings';
				$description = "$obj_type $action : $post_title";
				$uactid      = ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );
				if ( 0 != $uactid ) {
					ualp_add_wordpress_settings_log_detail( $uactid, $oldvalue, $_newvalue, $obj_type, $option );
				}
			}
		}
	}
}

/**
 * Update Hook Option.
 *
 * @param string $option Option.
 * @param string $oldvalue Old Value.
 * @param string $_newvalue New Value.
 */
function ualp_hook_update_option( $option, $oldvalue, $_newvalue ) {
	$current_url       = wp_get_referer();
	$current_url       = str_replace( '&settings-updated=true', '', $current_url );
	$current_url       = str_replace( '?settings-updated=true', '', $current_url );
	$current_url       = substr( $current_url, -20 );
	$transient_name    = 'sp_' . $current_url;
	$whitelist_options = ualp_get_options();
	if ( in_array( $option, $whitelist_options ) ) {
		$settings_page_bbpress = get_transient( $transient_name );
		if ( is_array( $option ) ) {
			$settings_page_bbpress[ $option ] = array(
				'old_v' => $oldvalue,
				'new_v' => $_newvalue,
			);
		}
		set_transient( $transient_name, $settings_page_bbpress );
		$user     = wp_get_current_user();
		$action   = 'updated';
		$obj_type = 'Settings';
		$post_id  = '';
		$hook     = 'update_option';
		ualp_send_email_notification( 'ualpSettingsUpdated', $user->user_login, 'Settings updated' );
		ualp_send_sms_notification( 'ualpSettingsUpdated', $user->user_login, 'Settings updated' );
		ualp_get_activity_log( $action, $obj_type, $post_id, $transient_name, $hook, 'Settings updated' );
	}
}

add_filter( 'dbdelta_queries', 'ualp_event_db_delta_query', 999, 1 );
add_filter( 'query', 'ualp_event_drop_query', 999, 1 );

if ( ! function_exists( 'ualp_event_db_delta_query' ) ) {
	/**
	 * Event DB Query.
	 *
	 * @param string $queries - Query.
	 */
	function ualp_event_db_delta_query( $queries ) {
		global $wpdb;
		$type_queries = array(
			'create' => array(),
			'update' => array(),
			'delete' => array(),
		);

		foreach ( $queries as $qry ) {
			$str = explode( ' ', $qry );
			if ( preg_match( '|CREATE TABLE ([^ ]*)|', $qry ) ) {
				if ( $wpdb->get_var( $wpdb->prepare( 'SHOW TABLES LIKE %s', $str[2] ) ) != $str[2] ) {
					array_push( $type_queries['create'], $str[2] );
				}
			} elseif ( preg_match( '|ALTER TABLE ([^ ]*)|', $qry ) ) {
				array_push( $type_queries['update'], $str[2] );
			} elseif ( preg_match( '|DROP TABLE ([^ ]*)|', $qry ) ) {
				if ( ! empty( $str[4] ) ) {
					array_push( $type_queries['delete'], $str[4] );
				} else {
					array_push( $type_queries['delete'], $str[2] );
				}
			}
		}
		return $queries;
	}
}

if ( ! function_exists( 'ualp_event_drop_query' ) ) {
	/**
	 * Checks for drop query.
	 *
	 * @param WP_Query $query - Query object.
	 */
	function ualp_event_drop_query( $query ) {
		$table_names = array();
		$str         = explode( ' ', $query );

		if ( preg_match( '|DROP TABLE ([^ ]*)|', $query ) ) {
			if ( ! empty( $str[4] ) ) {
				array_push( $table_names, $str[4] );
			} else {
				array_push( $table_names, $str[2] );
			}

			// Filter $_SERVER array for security.
			$server_array = filter_input_array( INPUT_SERVER );

			$actype        = ( isset( $server_array['SCRIPT_NAME'] ) ) ? basename( $server_array['SCRIPT_NAME'], '.php' ) : false;
			$alert_options = ualp_get_action_type( $actype );
		}

		if ( ! empty( $table_names ) ) {
			$alert_options['TableNames'] = implode( ',', $table_names );
			$log_hook_list               = get_option( 'enableLogHookList' );
			if ( is_multisite() && empty( $log_hook_list ) ) {
				$log_hook_list = array();
			}
			if ( in_array( 'dbdelta_queries', $log_hook_list ) ) {
				$action      = 'deleted';
				$obj_type    = 'Database';
				$post_id     = '';
				$post_title  = isset( $alert_options['TableNames'] ) ? "'" . $alert_options['TableNames'] . "'" : '';
				$hook        = 'query';
				$description = $obj_type . ": '$post_title' table " . $action . ' successfully';
				ualp_get_activity_log( $action, $obj_type, $post_id, $post_title, $hook, $description );            }
		}
		return $query;
	}
}

/**
 * Get info by action type.
 *
 * @param string $actype - Plugins, themes or unknown.
 */
function ualp_get_action_type( $actype ) {
	// Filter $_GET array for security.
	$get_array = filter_input_array( INPUT_GET );

	$theme_name  = '';
	$plugin_file = '';
	$is_themes   = 'themes' == $actype;
	$is_plugins  = 'plugins' == $actype;

	// Action Plugin Component.
	$alert_options = array();
	if ( $is_plugins ) {
		if ( isset( $get_array['plugin'] ) ) {
				$plugin_file = $get_array['plugin'];
		} else {
			if ( isset( $get_array['checked'] ) && ! empty( $get_array['checked'] ) ) {
				$plugin_file = $get_array['checked'][0];
			}
		}
		$plugin_name             = basename( $plugin_file, '.php' );
		$plugin_name             = str_replace( array( '_', '-', '  ' ), ' ', $plugin_name );
		$plugin_name             = ucwords( $plugin_name );
		$alert_options['Plugin'] = (object) array(
			'Name' => $plugin_name,
		);
		// Action Theme Component.
	} elseif ( $is_themes ) {
		if ( isset( $get_array['theme'] ) ) {
				$theme_name = $get_array['theme'];
		} else {
			if ( isset( $get_array['checked'] ) && ! empty( $get_array['checked'] ) ) {
				$theme_name = $get_array['checked'][0];
			}
		}
		$theme_name             = str_replace( array( '_', '-', '  ' ), ' ', $theme_name );
		$theme_name             = ucwords( $theme_name );
		$alert_options['Theme'] = (object) array(
			'Name' => $theme_name,
		);
		// Action Unknown Component.
	} else {
		$alert_options['Component'] = 'Unknown';
	}

	return $alert_options;
}
if ( ! function_exists( 'ualp_logged_in_usersupdate' ) ) {
	/**
	 * Update Logged In Users.
	 */
	function ualp_logged_in_usersupdate() {
		global $wpdb;
		if ( is_multisite() ) {
			$logged_in_users = $wpdb->get_results( "Select * from {$wpdb->base_prefix}ualp_user_activity_login order by time_login desc" );
		} else {
			$logged_in_users = $wpdb->get_results( "Select * from {$wpdb->prefix}ualp_user_activity_login order by time_login desc" );
		}
		foreach ( $logged_in_users as $logged_in_user ) {
			$user_id           = $logged_in_user->user_id;
			$user_login_status = $logged_in_user->login_status;
			$session_token     = $logged_in_user->session_token;
			$enc_session_token = ualp_get_current_token( $session_token );
			$login_status      = 'logout';
			$user_metas        = get_user_meta( $user_id, 'session_tokens', true );
			$total_logins      = 0;
			if ( is_array( $user_metas ) ) {
				$total_logins = count( $user_metas );
				if ( array_key_exists( $enc_session_token, $user_metas ) ) {
					$login_status = 'login';
				}
				$time_logout = current_time( 'mysql' );
				if ( 'login' == $user_login_status && 'logout' == $login_status ) {
					if ( is_multisite() ) {
						$userloginstatus = $wpdb->query( $wpdb->prepare( "UPDATE {$wpdb->base_prefix}ualp_user_activity_login SET login_status = %s,time_logout = %s WHERE session_token =%s", $login_status, $time_logout, $session_token ) ) . '<br>';
					} else {
						$userloginstatus = $wpdb->query( $wpdb->prepare( "UPDATE {$wpdb->prefix}ualp_user_activity_login SET login_status = %s,time_logout = %s WHERE session_token =%s", $login_status, $time_logout, $session_token ) ) . '<br>';
					}
				}
			}
		}
	}
}
add_action( 'init', 'ualp_logged_in_usersupdate' );

if ( ! function_exists( 'ualp_fail_login_nofication' ) ) {
	/**
	 * Notification for Failed Login.
	 */
	function ualp_fail_login_nofication() {
		global $wpdb;
		$countwploginfailed = $wpdb->get_var( $wpdb->prepare( 'SELECT COUNT(`uactid`) FROM %s where hook="wp_login_failed"', UALP_TABLENAME ) );
	}
}


if ( ! function_exists( 'ualp_insert_schedule_data' ) ) {
	/**
	 * Insert Schedule Data.
	 *
	 * @param string $schedule_email Schedule Email.
	 * @param string $report_name Report Name.
	 * @param string $report_frequency Reort Frequency.
	 */
	function ualp_insert_schedule_data( $schedule_email, $report_name, $report_frequency ) {
		global $wpdb, $externaldb;
		ualp_create_table();
		ualp_create_detail_table();
		ualp_create_external_table();
		ualp_create_detail_external_table();
		ualp_create_login_external_table();
		ualp_create_email_notification_table();
		ualp_create_sms_notification_table();
		ualp_create_email_notification_external_table();
		ualp_create_schedule_report();
		ualp_create_schedule_external_report();
		/**
		 * UalpBeforeInsertEmailNotification hook.
		 */
		do_action( 'ualpbeforeinsertemailnotification' );

		$insert_query = $externaldb->insert(
			UALP_SCHEDULE_TABLENAME,
			array(
				'schedule_email_id' => $schedule_email,
				'report_frequency'  => $report_frequency,
				'report_title'      => $report_name,
			)
		);

		/**
		 * UalpAfterInsertEmailNotification hook.
		 */
		do_action( 'ualpafterinsertemailnotification' );
		return $externaldb->insert_id;
	}
}
if ( ! function_exists( 'ualp_update_schedule_data' ) ) {
	/**
	 * Update the Schedule Data.
	 *
	 * @param string $schedule_email Schedule Email.
	 * @param string $report_name Report Name.
	 * @param string $report_frequency Reort Frequency.
	 * @param string $ualp_report_id User Report ID.
	 */
	function ualp_update_schedule_data( $schedule_email, $report_name, $report_frequency, $ualp_report_id ) {
		global $wpdb, $externaldb;
		do_action( 'ualp_before_update_notification' );
		$table_names = UALP_SCHEDULE_TABLENAME;
		$externaldb->update(
			UALP_SCHEDULE_TABLENAME,
			array(
				'schedule_email_id' => $schedule_email,
				'report_frequency'  => $report_frequency,
				'report_title'      => $report_name,
			),
			array( 'uactrid' => $ualp_report_id )
		);
		do_action( 'ualp_after_update_notification' );
	}
}

if ( ! function_exists( 'ualp_add_data_google_sheets' ) ) {
	/**
	 * Add Data in Google Sheets
	 *
	 * @param string $post_title Post Title.
	 * @param string $obj_type Object Type (Plugin, Post, User etc.).
	 * @param string $current_user_name current user id.
	 * @param string $action current user's activity name.
	 * @param string $hook Hook.
	 * @param string $description Description.
	 */
	function ualp_add_data_google_sheets( $post_title, $obj_type, $current_user_name, $action, $hook, $description ) {
		$modified_date = current_time( 'mysql' );
		$get_ip_type   = get_option( 'ualpIPtypes', 'REMOTE_ADDR' );
		if ( $get_ip_type ) {
			if ( 'REMOTE_ADDR' == $get_ip_type ) {
				$ips = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
			} elseif ( 'HTTP_X_FORWARDED_FOR' == $get_ip_type ) {
				$ips = isset( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) : '';
			} elseif ( 'HTTP_X_REAL_IP' == $get_ip_type ) {
				$ips = isset( $_SERVER['HTTP_X_REAL_IP'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_REAL_IP'] ) ) : '';
			} else {
				$ips = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
			}
		} else {
			$ips = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
		}
		$user_agents = isset( $_SERVER['HTTP_USER_AGENT'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_USER_AGENT'] ) ) : '';
		$ip          = '';
		$user_agent  = '';
		$country     = '';
		if ( ualp_allow_ip() ) {
			$ip = $ips;
		}
		if ( ualp_allow_country() ) {
			$country = ( isset( $ip ) && ( '' != $ip ) ) ? ualp_ip_info( $ip, 'country' ) : '';
		}
		if ( ualp_allow_user_agent() ) {
			$user_agent = $user_agents;
			$user_agent = str_replace( ';', ',', $user_agent );
		}
		$spreadsheet_id = Ualp_Google_Sheets_API::ualp_spreadsheets();
		if ( isset( $spreadsheet_id ) && is_array( $spreadsheet_id ) ) {
			foreach ( $spreadsheet_id as $id => $name ) {
				$worksheet_id = Ualp_Google_Sheets_API::ualp_worksheets( $id );
				Ualp_Google_Sheets_API::ualp_append_row( $id, $worksheet_id, array( $modified_date, $ip, $country, $user_agent, $post_title, $obj_type, $current_user_name, $action, $hook, $description ) );
			}
		}
	}
}
