<?php
/**
 * Common Functions File.
 *
 * @package User Activity Log PRO
 */

/**
 * Exit if accessed directly
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! function_exists( 'ualp_ip_info' ) ) {
	/**
	 * Get IP information.
	 *
	 * @param string $ip IP Address.
	 * @param string $purpose Purpose.
	 * @param bool   $deep_detect Deep Detect.
	 * @param string $get_ip_type Get IP Type.
	 */
	function ualp_ip_info( $ip = null, $purpose = 'location', $deep_detect = true, $get_ip_type = null ) {
		$output = null;
		if ( null === $get_ip_type ) {
			$get_ip_type = get_option( 'ualpIPtypes', 'REMOTE_ADDR' );
		}
		if ( $get_ip_type ) {
			if ( 'REMOTE_ADDR' == $get_ip_type ) {
				$ipaddress = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
			} elseif ( 'HTTP_X_FORWARDED_FOR' == $get_ip_type ) {
				$ipaddress = isset( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) : '';
			} elseif ( 'HTTP_X_REAL_IP' == $get_ip_type ) {
				$ipaddress = isset( $_SERVER['HTTP_X_REAL_IP'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_REAL_IP'] ) ) : '';
			} else {
				$ipaddress = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
			}
		} else {
			$ipaddress = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
		}
		if ( filter_var( $ipaddress, FILTER_VALIDATE_IP ) ) {
			$ip = $ipaddress;
		} else {
			$ip = '';
		}
		$output     = $ip;
		$purpose    = str_replace( array( 'name', "\n", "\t", ' ', '-', '_' ), '', strtolower( trim( $purpose ) ) );
		$support    = array( 'country', 'countrycode', 'state', 'region', 'city', 'location' );
		$continents = array(
			'AF' => 'Africa',
			'AN' => 'Antarctica',
			'AS' => 'Asia',
			'EU' => 'Europe',
			'OC' => 'Australia (Oceania)',
			'NA' => 'North America',
			'SA' => 'South America',
		);
		if ( filter_var( $ip, FILTER_VALIDATE_IP ) && in_array( $purpose, $support ) ) {
			$ipdat = json_decode( file_get_contents( 'http://www.geoplugin.net/json.gp?ip=' . $ip ) );
			if ( isset( $ipdat->geoplugin_countryCode ) && ! empty( $ipdat->geoplugin_countryCode ) && strlen( trim( $ipdat->geoplugin_countryCode ) ) == 2 ) {
				switch ( $purpose ) {
					case 'location':
						$output = array(
							'city'           => $ipdat->geoplugin_city,
							'state'          => $ipdat->geoplugin_regionName,
							'country'        => $ipdat->geoplugin_countryName,
							'country_code'   => $ipdat->geoplugin_countryCode,
							'continent'      => $continents[ strtoupper( $ipdat->geoplugin_continentCode ) ],
							'continent_code' => $ipdat->geoplugin_continentCode,
						);
						break;
					case 'city':
						$output = $ipdat->geoplugin_city;
						break;
					case 'state':
						$output = $ipdat->geoplugin_regionName;
						break;
					case 'region':
						$output = $ipdat->geoplugin_regionName;
						break;
					case 'country':
						$output = $ipdat->geoplugin_countryName;
						break;
					case 'countrycode':
						$output = $ipdat->geoplugin_countryCode;
						break;
				}
			}
		}
		return $output;
	}
}


if ( ! function_exists( 'ualp_admin_scripts' ) ) {
	/**
	 * Enqueue admin scripts.
	 */
	function ualp_admin_scripts() { ?>
		<script>
			var ajaxurl = "<?php echo esc_url( admin_url( 'admin-ajax.php' ) ); ?>";
		</script>
		<?php
		$screen          = get_current_screen();
		$plugin_data     = get_plugin_data( WP_PLUGIN_DIR . '/user-activity-log-pro/user_activity_log_pro.php', $markup = true, $translate = true );
		$current_version = $plugin_data['Version'];
		$old_version     = get_option( 'ual_version' );
		if ( 'dashboard' == $screen->id || ( isset( $_GET['page'] ) && ( 'ual_pro' == $_GET['page'] || 'ualp_settings' == $_GET['page'] || 'ualp_db_integration' == $_GET['page'] || 'ualp_user_details' == $_GET['page'] || 'ualp_email_setting' == $_GET['page'] || 'ualp_sms_setting' == $_GET['page'] || 'ualp_reports' == $_GET['page'] || 'ualp_add_email_notification' == $_GET['page'] || 'ualp_add_sms_notification' == $_GET['page'] ) ) ) {
			wp_enqueue_script( 'jquery' );
			wp_enqueue_script( 'password-strength-meter', '', array( 'jquery' ), '1.0', false );
			wp_enqueue_script( 'select2-js', UALP_PLUGIN_URL . 'assets/js/select2.js', array( 'jquery' ), '4.0.6-rc.1', false );
			wp_enqueue_script( 'jquery-ui-dialog', '', array( 'jquery' ), '1.0', false );
			wp_enqueue_script( 'jquery-ui-datepicker' );
			wp_enqueue_script( 'ualp-script', UALP_PLUGIN_URL . 'assets/js/ualp_script.js', array( 'select2-js', 'jquery-ui-datepicker', 'jquery-ui-dialog', 'password-strength-meter' ), '2.3.5', false );
			wp_enqueue_script( 'ualp-chart-js', UALP_PLUGIN_URL . 'assets/js/chart.js', array( 'jquery' ), '2.8.0', false );
			wp_localize_script(
				'ualp-script',
				'ualpJSObject',
				array(
					'is_rtl'                           => is_rtl() ? 'rtl' : 'ltr',
					'reset_confirm'                    => esc_html__( 'Are you sure to reset password?', 'user_activity_log_pro' ),
					'note_for_uncheck'                 => esc_html__( 'If you uncheck this option, it will not keep log for', 'user_activity_log_pro' ),
					'select_option_delete_log'         => esc_html__( 'Select any one option to delete log.', 'user_activity_log_pro' ),
					'file_save_success'                => esc_html__( 'File Saved successfully in uploads folder and log deleted successfully.', 'user_activity_log_pro' ),
					'email_send_error'                 => esc_html__( 'Error in sending email.', 'user_activity_log_pro' ),
					'log_delete_successfully'          => esc_html__( 'Log deleted successfully.', 'user_activity_log_pro' ),
					'file_save_and_error_email'        => esc_html__( 'File Saved successfully in uploads folder, But error in sending email.', 'user_activity_log_pro' ),
					'email_send_and_log_delete'        => esc_html__( 'Email sent successfully and log deleted successfully.', 'user_activity_log_pro' ),
					'file_save_and_send_email'         => esc_html__( 'File Saved successfully in uploads folder, Email sent successfully and log deleted successfully', 'user_activity_log_pro' ),
					'enter_valid_pwd'                  => esc_html__( 'Enter valid password.', 'user_activity_log_pro' ),
					'select_option_for_delete_log'     => esc_html__( 'Select any one option to delete log', 'user_activity_log_pro' ),
					'file_save_and_log_delete_success' => esc_html__( 'File Saved successfully in uploads folder and log deleted successfully', 'user_activity_log_pro' ),
					'email_send_and_delete_success'    => esc_html__( 'Email sent successfully and log deleted successfully', 'user_activity_log_pro' ),
					'file_save_email_send_and_success_delete_log' => esc_html__( 'File Saved successfully in uploads folder, Email sent successfully and log deleted successfully', 'user_activity_log_pro' ),
					'please_select_start_date'         => esc_html__( 'Please first select start date.', 'user_activity_log_pro' ),
					'submit'                           => esc_html__( 'Submit', 'user_activity_log_pro' ),
					'cancel'                           => esc_html__( 'Cancel', 'user_activity_log_pro' ),
					'log_delete'                       => esc_html__( 'Select Logs to delete', 'user_activity_log_pro' ),
					'favorite_log'                     => esc_html__( 'Select Logs to favorite it', 'user_activity_log_pro' ),
					'unfavorite_log'                   => esc_html__( 'Select Logs to unfavorite it', 'user_activity_log_pro' ),
					'ajax_nonce'                       => wp_create_nonce( 'ajax-nonce' ),
				)
			);
		}
		wp_enqueue_script( 'thickbox' );
		wp_enqueue_style( 'thickbox' );
	}
}

if ( ! function_exists( 'ualp_admin_styles' ) ) {
	/**
	 * Enqueue admin styles
	 */
	function ualp_admin_styles() {
		$screen = get_current_screen();
		if ( 'dashboard' == $screen->id || ( isset( $_GET['page'] ) && ( 'ual_pro' == $_GET['page'] || 'ualp_settings' == $_GET['page'] || 'ualp_license_key' == $_GET['page'] || 'ualp_db_integration' == $_GET['page'] || 'ualp_user_details' == $_GET['page'] || 'ualp_email_setting' == $_GET['page'] || 'ualp_sms_setting' == $_GET['page'] || 'ualp_reports' == $_GET['page'] || 'ualp_add_email_notification' == $_GET['page'] || 'ualp_add_sms_notification' == $_GET['page'] || 'ualp_hook_details' == $_GET['page'] || 'ualp_debug_log' == $_GET['page'] ) ) ) {
			wp_enqueue_style( 'jquery-ui', UALP_PLUGIN_URL . 'assets/css/jquery-ui.css', array(), '1.12.0' );
			wp_enqueue_style( 'select2-css', UALP_PLUGIN_URL . 'assets/css/select2.min.css', array(), '1.0' );
			wp_enqueue_style( 'ualp-style', UALP_PLUGIN_URL . 'assets/css/ualp_style.css', array(), '2.3.5' );
		}
		if ( is_rtl() ) {
			wp_enqueue_style( 'ualp-style-rtl', UALP_PLUGIN_URL . 'assets/css/ualp_style_rtl.css', array(), '2.3.5' );
		}
	}
}

if ( ! function_exists( 'ualp_session_start' ) ) {
	/**
	 * Start session if not.
	 */
	function ualp_session_start() {
		if ( session_id() == '' ) {
			session_start( array( 'read_and_close' => true ) );
		}
	}
}

if ( ! function_exists( 'ualp_session_expire' ) ) {
	/**
	 * Destroy logged in user session.
	 */
	function ualp_session_expire() {
		if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'ajax-nonce' ) ) {
			$t_session = ( isset( $_POST['t_session'] ) && '' != $_POST['t_session'] ) ? sanitize_text_field( wp_unslash( $_POST['t_session'] ) ) : '';
			$id        = ( isset( $_POST['userID'] ) && '' != $_POST['userID'] ) ? sanitize_text_field( wp_unslash( $_POST['userID'] ) ) : '';
			$except    = ( isset( $_POST['except'] ) && '' != $_POST['except'] ) ? sanitize_text_field( wp_unslash( $_POST['except'] ) ) : '';
			if ( '' == $t_session ) {
				delete_user_meta( $id, 'session_tokens', '' );
				global $wpdb;
				if ( is_multisite() ) {
					$logged_in_users = $wpdb->get_results( "Select * from {$wpdb->base_prefix}ualp_user_activity order by time_login desc" );
				} else {
					$logged_in_users = $wpdb->get_results( "Select * from {$wpdb->prefix}ualp_user_activity order by time_login desc" );
				}
				foreach ( $logged_in_users as $logged_in_user ) {
					$time_logout = current_time( 'mysql' );
					if ( 'login' == $user_login_status && 'logout' == $login_status ) {
						$userloginstatus = $wpdb->query( $wpdb->prepare( 'UPDATE %1s " SET time_logout = %s WHERE user_id = %s', UALP_LOGIN_TABLENAME, $time_logout, $id ) ) . '<br>';
					}
				}
			} else {
				$enc_token      = ualp_get_current_token( $t_session );
				$session_tokens = get_user_meta( $id, 'session_tokens', true );
				if ( $session_tokens ) {
					if ( 'yes' == $except ) {
						foreach ( $session_tokens as $session_token_key => $session_token_val ) {
							if ( $enc_token != $session_token_key ) {
								unset( $session_tokens[ $session_token_key ] );
							}
						}
					} else {
						if ( isset( $session_tokens[ $enc_token ] ) ) {
							unset( $session_tokens[ $enc_token ] );
						}
					}
					update_user_meta( $id, 'session_tokens', $session_tokens );
				}
			}
		}
		exit;
	}
}

if ( ! function_exists( 'latest_news_solwin_feed' ) ) {
	/**
	 * Add Admin Dashboard Widget - News from Solwin Infotech.
	 */
	function latest_news_solwin_feed() {
		// Register the new dashboard widget with the 'wp_dashboard_setup' action.
		add_action( 'wp_dashboard_setup', 'solwin_latest_news_with_product_details' );
		if ( ! function_exists( 'solwin_latest_news_with_product_details' ) ) {
			/**
			 * Latest News with Product Details.
			 */
			function solwin_latest_news_with_product_details() {
				add_screen_option(
					'layout_columns',
					array(
						'max'     => 3,
						'default' => 2,
					)
				);
				$ualp_role_access_list      = get_option( 'ualpRoleAccessList' );
				$ualp_user_access_list      = get_option( 'ualpUserAccessList' );
				$ualp_allow_dashbord_widget = get_option( 'ualpAllowDashbordWidget', '1' );
				if ( 1 == $ualp_allow_dashbord_widget ) {
					$current_user      = wp_get_current_user();
					$current_user_name = $current_user->ID;
					if ( isset( $current_user->roles[0] ) ) {
						$current_user_role = $current_user->roles[0];
					} else {
						if ( is_multisite() ) {
							$id       = get_current_user_id();
							$usermeta = get_user_meta( $id, 'wp_capabilities' );
							if ( is_array( $usermeta[0] ) ) {
								$current_userrolls = array_keys( $usermeta[0] );
							} else {
								$current_userrolls = $usermeta;
							}
							if ( isset( $current_userrolls[0] ) ) {
								$current_user_role = $current_userrolls[0];
							}
						}
					}
					$ualp_role = '';
					$ualp_user = '';
					if ( is_array( $ualp_role_access_list ) && ! empty( $ualp_role_access_list ) ) {
						if ( count( $ualp_role_access_list ) > 1 ) {
							foreach ( $ualp_role_access_list as $ualp_role ) {
								$ualp_role .= $ualp_role;
							}
						}
					}
					if ( is_array( $ualp_user_access_list ) && ! empty( $ualp_user_access_list ) ) {
						if ( count( $ualp_user_access_list ) > 1 ) {
							foreach ( $ualp_user_access_list as $ualp_user ) {
								$ualp_user .= $ualp_user;
							}
						}
					}
					if ( is_array( $ualp_role_access_list ) && is_array( $ualp_user_access_list ) && ! empty( $ualp_role_access_list ) && ! empty( $ualp_user_access_list ) && isset( $current_user_role ) && 'administrator' != $current_user_role ) {
						if ( ( isset( $current_user_role ) && $current_user_role == $ualp_role ) || ( isset( $current_user_name ) && $current_user_name == $ualp_user ) ) {
							add_meta_box( 'wp_ualp_dashboard_widget', esc_html__( 'News From Solwin Infotech', 'user_activity_log_pro' ), 'solwin_dashboard_widget_news', 'dashboard', 'normal', 'high' );
						}
					} elseif ( is_array( $ualp_role_access_list ) && ! empty( $ualp_role_access_list ) && isset( $current_user_role ) && 'administrator' != $current_user_role ) {
						foreach ( $ualp_role_access_list as $ualp_role ) {
							if ( isset( $current_user_role ) && $current_user_role == $ualp_role ) {
								add_meta_box( 'wp_ualp_dashboard_widget', esc_html__( 'News From Solwin Infotech', 'user_activity_log_pro' ), 'solwin_dashboard_widget_news', 'dashboard', 'normal', 'high' );
							}
						}
					} elseif ( is_array( $ualp_user_access_list ) && ! empty( $ualp_user_access_list ) && isset( $current_user_role ) && 'administrator' != $current_user_role ) {

						foreach ( $ualp_user_access_list as $ualp_user ) {
							if ( isset( $current_user_name ) && $current_user_name == $ualp_user ) {
								add_meta_box( 'wp_ualp_dashboard_widget', esc_html__( 'News From Solwin Infotech', 'user_activity_log_pro' ), 'solwin_dashboard_widget_news', 'dashboard', 'normal', 'high' );
							}
						}
					} else {
						if ( isset( $current_user_role ) && 'administrator' == $current_user_role ) {
							add_meta_box( 'wp_ualp_dashboard_widget', esc_html__( 'News From Solwin Infotech', 'user_activity_log_pro' ), 'solwin_dashboard_widget_news', 'dashboard', 'normal', 'high' );
						}
					}
				}
			}
		}
		if ( ! function_exists( 'solwin_dashboard_widget_news' ) ) {
			/**
			 * Dashboard Widget News.
			 */
			function solwin_dashboard_widget_news() {
				echo '<div class="rss-widget">'
				. '<div class="solwin-news"><p><strong>' . esc_html__( 'Solwin Infotech News', 'user_activity_log_pro' ) . '</strong></p>';
				wp_widget_rss_output(
					array(
						'url'          => 'https://www.solwininfotech.com/feed/',
						'title'        => esc_html__( 'News From Solwin Infotech', 'user_activity_log_pro' ),
						'items'        => 5,
						'show_summary' => 0,
						'show_author'  => 0,
						'show_date'    => 1,
					)
				);
				echo '</div>';
				$title     = '';
				$link      = '';
				$thumbnail = '';
				// get Latest product detail from xml file.

				$file = 'https://www.solwininfotech.com/documents/assets/latest_product.xml';
				if ( ! defined( 'LATEST_PRODUCT_FILE' ) ) {
					define( 'LATEST_PRODUCT_FILE', $file );
				}
				echo '<div class="display-product">'
				. '<div class="product-detail"><p><strong>' . esc_html__( 'Latest Product', 'user_activity_log_pro' ) . '</strong></p>';
				$response = wp_remote_get( LATEST_PRODUCT_FILE );
				if ( is_wp_error( $response ) ) {
					$error_message = $response->get_error_message();
					echo '<p>' . esc_html__( 'Something went wrong', 'user_activity_log_pro' ) . ' : ' . esc_html( $error_message ) . '</p>';
				} else {
					$body                = wp_remote_retrieve_body( $response );
					$xml                 = simplexml_load_string( $body );
					$title               = $xml->item->name;
					$thumbnail           = $xml->item->img;
					$link                = $xml->item->link;
					$all_producttext     = $xml->item->viewalltext;
					$all_productlink     = $xml->item->viewalllink;
					$moretext            = $xml->item->moretext;
					$needsupporttext     = $xml->item->needsupporttext;
					$needsupportlink     = $xml->item->needsupportlink;
					$customservicetext   = $xml->item->customservicetext;
					$customservicelink   = $xml->item->customservicelink;
					$joinproductclubtext = $xml->item->joinproductclubtext;
					$joinproductclublink = $xml->item->joinproductclublink;

					echo '<div class="product-name"><a href="' . esc_url( $link ) . '" target="_blank">'
					. '<img alt="' . esc_attr( $title ) . '" src="' . esc_url( $thumbnail ) . '"> </a>'
					. '<a href="' . esc_url( $link ) . '" target="_blank">' . esc_html( $title ) . '</a>'
					. '<p><a href="' . esc_url( $all_productlink ) . '" target="_blank" class="button button-default">' . esc_html( $all_producttext ) . ' &RightArrow;</a></p>'
					. '<hr>'
					. '<p><strong>' . esc_html( $moretext ) . '</strong></p>'
					. '<ul>'
					. '<li><a href="' . esc_url( $needsupportlink ) . '" target="_blank">' . esc_html( $needsupporttext ) . '</a></li>'
					. '<li><a href="' . esc_url( $customservicelink ) . '" target="_blank">' . esc_html( $customservicetext ) . '</a></li>'
					. '<li><a href="' . esc_url( $joinproductclublink ) . '" target="_blank">' . esc_html( $joinproductclubtext ) . '</a></li>'
					. '</ul>'
					. '</div>';
				}
				echo '</div></div><div class="clear"></div>'
				. '</div>';
			}
		}
	}
}


if ( ! function_exists( 'ualp_footer_adv' ) ) {
	/**
	 * Add Footer link.
	 */
	function ualp_footer_adv() {

		if ( ( isset( $_GET['page'] ) && ( 'ual_pro' == $_GET['page'] || 'ualp_settings' == $_GET['page'] ) ) ) {
			add_filter( 'admin_footer_text', 'ualp_footer_admin_adv' ); // change admin footer text.
		}
	}
}


if ( ! function_exists( 'ualp_footer_admin_adv' ) ) {
	/**
	 * Add rating html at footer of admin.
	 *
	 * @return html rating
	 */
	function ualp_footer_admin_adv() {
		ob_start();
		?>
		<p id="footer-left" class="alignleft">
			<?php esc_html_e( 'If you like', 'user_activity_log_pro' ); ?>&nbsp;
			<a href="<?php echo esc_url( 'http://www.solwininfotech.com/product/wordpress-plugins/user-activity-log-pro' ); ?>" target="_blank">
				<strong><?php esc_html_e( 'User Activity Log PRO', 'user_activity_log_pro' ); ?></strong>
			</a>
			<?php esc_html_e( 'please leave us a', 'user_activity_log_pro' ); ?>
			<a class="bdp-rating-link" data-rated="Thanks :)" target="_blank" href="<?php echo esc_url( 'https://codecanyon.net/item/user-activity-log-pro-for-wordpress/reviews/18201203' ); ?>">&#x2605;&#x2605;&#x2605;&#x2605;&#x2605;</a>
			<?php esc_html_e( 'rating. A heartly thank you from Solwin Infotech in advance!', 'user_activity_log_pro' ); ?>
		</p>
		<?php
		return ob_get_clean();
	}
}

if ( ! function_exists( 'ualp_admin_notice_message' ) ) {
	/**
	 * Display success or error message.
	 *
	 * @param string $class Class.
	 * @param string $message Message.
	 */
	function ualp_admin_notice_message( $class, $message ) {
		$args_kses = ualp_args_kses();
		?>
		<div class="<?php echo esc_attr( $class ); ?> is-dismissible ualpAdminNotice notice settings-error below-h1">
			<p><?php echo wp_kses( $message, $args_kses ); ?></p>
		</div>
		<?php
	}
}


if ( ! function_exists( 'ualp_create_table' ) ) {
	/**
	 * Create log table when activate plugin.
	 */
	function ualp_create_table() {
		global $wpdb, $externaldb;
		$plugin_data       = get_plugin_data( WP_PLUGIN_DIR . '/user-activity-log-pro/user_activity_log_pro.php', $markup = true, $translate = true );
		$current_version   = $plugin_data['Version'];
		$table_not_updated = ualp_checkif_table_updated();
		// table is not created. you may create the table here.
		if ( $wpdb->get_var( $wpdb->prepare( 'SHOW TABLES LIKE  %s ', UALP_TABLENAME ) ) != UALP_TABLENAME ) {
			if ( ! $table_not_updated ) {
				$create_table_query = 'CREATE TABLE ' . UALP_TABLENAME . ' ('
					. 'uactid bigint(20) unsigned NOT NULL auto_increment,'
					. 'post_id int(20) unsigned,'
					. "favorite int(1) default '0',"
					. 'post_title varchar(250),'
					. "user_id bigint(20) unsigned default '0',"
					. 'user_name varchar(250),'
					. 'user_role varchar(250),'
					. 'user_email varchar(250),'
					. 'ip_address varchar(50),'
					. 'country varchar(150),'
					. 'user_agent text,'
					. "modified_date datetime default '0000-00-00 00:00:00',"
					. "object_type varchar(50) default 'post',"
					. 'action varchar(50),'
					. 'hook varchar(100),'
					. 'description text,'
					. 't_session varchar(255),'
					. 'site_id INT(15),'
					. 'PRIMARY KEY (uactid))
                        CHARACTER SET utf8 COLLATE utf8_unicode_ci';
				require_once ABSPATH . 'wp-admin/includes/upgrade.php';
				dbDelta( $create_table_query );
			}
		} elseif ( 'country' == $table_not_updated ) {
			$wpdb->query( $wpdb->prepare( 'ALTER TABLE %s ADD `country` TEXT NULL AFTER `ip_address`', UALP_TABLENAME ) );
		} elseif ( 'user_agent' == $table_not_updated ) {
			$fieldname = 'user_agent';
			if ( $wpdb->get_var( $wpdb->prepare( 'SHOW COLUMNS FROM %1s LIKE %s', UALP_TABLENAME, $fieldname ) ) != $fieldname ) {
				$wpdb->query( $wpdb->prepare( 'ALTER TABLE %1s ADD `user_agent` TEXT NULL AFTER `country`', UALP_TABLENAME ) );
			}
		} elseif ( 't_session' == $table_not_updated ) {
			$fielsessiondname = 't_session';
			if ( $wpdb->get_var( $wpdb->prepare( 'SHOW COLUMNS FROM %1s LIKE %s', UALP_TABLENAME, $fielsessiondname ) ) != $fielsessiondname ) {
				$wpdb->query( $wpdb->prepare( 'ALTER TABLE %1s ADD `t_session` varchar(200) NULL AFTER `description`', UALP_TABLENAME ) );
			}
		} elseif ( 'site_id' == $table_not_updated ) {
			$fieldname = 'site_id';
			if ( $wpdb->get_var( $wpdb->prepare( 'SHOW COLUMNS FROM %1s LIKE %s', UALP_TABLENAME, $fieldname ) ) != $fieldname ) {
				$wpdb->query( $wpdb->prepare( 'ALTER TABLE %1s ADD `site_id` INT(15) NULL AFTER `t_session`', UALP_TABLENAME ) );
			}
		}

		$login_table_not_updated = ualp_checkif_login_table_updated();
		if ( ! $login_table_not_updated ) {
			$create_login_table_query = 'CREATE TABLE ' . UALP_LOGIN_TABLENAME . ' ('
					. 'uacltid bigint(20) unsigned NOT NULL auto_increment,'
					. "user_id bigint(20) unsigned default '0',"
					. 'user_name varchar(250),'
					. 'user_role varchar(250),'
					. 'user_email varchar(250),'
					. 'expiration varchar(250),'
					. 'session_token varchar(250),'
					. 'time_login datetime,'
					. 'time_logout datetime,'
					. 'time_last_seen datetime,'
					. 'ip_address varchar(50),'
					. 'country varchar(150),'
					. 'user_agent text,'
					. 'login_status varchar(50),'
					. 'site_id INT(15),'
					. 'PRIMARY KEY (uacltid))
                        CHARACTER SET utf8 COLLATE utf8_unicode_ci';
				require_once ABSPATH . 'wp-admin/includes/upgrade.php';
				dbDelta( $create_login_table_query );
		}
		$fieldname = 'site_id';
		if ( $wpdb->get_var( $wpdb->prepare( 'SHOW COLUMNS FROM %1s LIKE %s', UALP_LOGIN_TABLENAME, $fieldname ) ) != $fieldname ) {
			// $sql = 'ALTER TABLE ' . UALP_LOGIN_TABLENAME . ' ADD `site_id` INT(15) NULL AFTER `login_status`';
			$wpdb->query( $wpdb->prepare( 'ALTER TABLE %1s ADD `site_id` INT(15) NULL AFTER `login_status`', UALP_LOGIN_TABLENAME ) );
		}

		/* Create table for Email */
		if ( $wpdb->get_var( $wpdb->prepare( 'SHOW TABLES LIKE %s', UALP_EMAIL_TABLENAME ) ) != UALP_EMAIL_TABLENAME ) {
			$charset_collate = '';
			if ( ! empty( $wpdb->charset ) ) {
				$charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";
			}
			if ( ! empty( $wpdb->collate ) ) {
				$charset_collate .= " COLLATE $wpdb->collate";
			}
			$create_table_query = 'CREATE TABLE ' . UALP_EMAIL_TABLENAME . ' ('
				. 'uacteid bigint(20) unsigned NOT NULL auto_increment,'
				. 'email_title text,'
				. 'email_content longtext,'
				. "user_id bigint(20) unsigned default '0',"
				. 'user_name varchar(250),'
				. 'user_role varchar(250),'
				. 'user_email varchar(250),'
				. 'site_id INT(15),'
				. 'custom_trigger_data text,'
				. 'email_address varchar(100),'
				. 'email_template INT(15),'
				. 'PRIMARY KEY (uacteid))'
				. $charset_collate;
			require_once ABSPATH . 'wp-admin/includes/upgrade.php';
			dbDelta( $create_table_query );
		}

		/* Create Table for SMS */

		if ( $wpdb->get_var( $wpdb->prepare( 'SHOW TABLES LIKE %s', UALP_SMS_TABLENAME ) ) != UALP_SMS_TABLENAME ) {
			$charset_collate = '';
			if ( ! empty( $wpdb->charset ) ) {
				$charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";
			}
			if ( ! empty( $wpdb->collate ) ) {
				$charset_collate .= " COLLATE $wpdb->collate";
			}
			$create_table_query = 'CREATE TABLE ' . UALP_SMS_TABLENAME . ' ('
			. 'uacteid bigint(20) unsigned NOT NULL auto_increment,'
			. 'sms_title text,'
			. 'sms_content longtext,'
			. "user_id bigint(20) unsigned default '0',"
			. 'user_name varchar(250),'
			. 'user_role varchar(250),'
			. 'user_email varchar(250),'
			. 'site_id INT(15),'
			. 'custom_trigger_data text,'
			. 'sms_address varchar(100),'
			. 'sms_template INT(15),'
			. 'PRIMARY KEY (uacteid))'
			. $charset_collate;
			require_once ABSPATH . 'wp-admin/includes/upgrade.php';
			dbDelta( $create_table_query );
		}
		/* end: create table for sms*/

		update_option( 'ualp_version', $current_version );
	}
}

if ( ! function_exists( 'ualp_create_external_table' ) ) {
	/**
	 * Create log table in external database.
	 */
	function ualp_create_external_table() {
		global $wpdb, $externaldb;
		$tableprefix = '';
		$tableprefix = get_option( 'table_prefix' );
		// table is not created. you may create the table here.
		if ( $externaldb->get_var( "SHOW TABLES LIKE '" . $tableprefix . "ualp_user_activity'" ) != $tableprefix . 'ualp_user_activity' ) {
			$create_table_query = 'CREATE TABLE ' . $tableprefix . 'ualp_user_activity ('
				. 'uactid bigint(20) unsigned NOT NULL auto_increment,'
				. 'post_id int(20) unsigned,'
				. "favorite int(1) default '0',"
				. 'post_title varchar(250),'
				. "user_id bigint(20) unsigned default '0',"
				. 'user_name varchar(250),'
				. 'user_role varchar(250),'
				. 'user_email varchar(250),'
				. 'ip_address varchar(50),'
				. 'country varchar(150),'
				. "modified_date datetime default '0000-00-00 00:00:00',"
				. "object_type varchar(50) default 'post',"
				. 'action varchar(50),'
				. 'hook varchar(100),'
				. 'description text,'
				. 't_session varchar(255),'
				. 'site_id INT(15),'
				. 'PRIMARY KEY (uactid))
                    CHARACTER SET utf8 COLLATE utf8_unicode_ci';
			require_once ABSPATH . 'wp-admin/includes/upgrade.php';
			$externaldb->query( $create_table_query );
		}
		$fielduseragentname = 'user_agent';
		if ( $externaldb->get_var( 'SHOW COLUMNS FROM ' . $tableprefix . "ualp_user_activity LIKE '" . $fielduseragentname . "'" ) != $fielduseragentname ) {
			$ualp_activity_table = $tableprefix . 'ualp_user_activity';
			$sql                 = "ALTER TABLE $ualp_activity_table ADD `user_agent` TEXT NULL AFTER `country`";
			$externaldb->query( $sql );
		}
		$fielsessiondname = 't_session';
		if ( $externaldb->get_var( 'SHOW COLUMNS FROM ' . $tableprefix . "ualp_user_activity LIKE '" . $fielsessiondname . "'" ) != $fielsessiondname ) {
			$ualp_activity_table = $tableprefix . 'ualp_user_activity';
			$sql                 = "ALTER TABLE $ualp_activity_table ADD `t_session` varchar(200) NULL AFTER `description`";
			$externaldb->query( $sql );
		}
		$fieldname = 'site_id';
		if ( $externaldb->get_var( 'SHOW COLUMNS FROM ' . $tableprefix . "ualp_user_activity LIKE '" . $fieldname . "'" ) != $fieldname ) {
			$ualp_activity_table = $tableprefix . 'ualp_user_activity';
			$sql                 = "ALTER TABLE $ualp_activity_table ADD `site_id` INT(15) NULL AFTER `t_session`";
			$externaldb->query( $sql );
		}
	}
}

if ( ! function_exists( 'ualp_checkif_table_updated' ) ) {
	/**
	 * Add country,user_agent,t_session,site_id column in table.
	 */
	function ualp_checkif_table_updated() {
		global $externaldb;
		if ( $externaldb->get_var( "SHOW TABLES LIKE '" . UALP_TABLENAME . "'" ) == UALP_TABLENAME ) {
			if ( $externaldb->get_var( 'SHOW COLUMNS FROM ' . UALP_TABLENAME . " LIKE 'country'" ) != 'country' ) {
				return 'country';
			} elseif ( $externaldb->get_var( 'SHOW COLUMNS FROM ' . UALP_TABLENAME . " LIKE 'user_agent'" ) != 'user_agent' ) {
				return 'user_agent';
			} elseif ( $externaldb->get_var( 'SHOW COLUMNS FROM ' . UALP_TABLENAME . " LIKE 't_session'" ) != 't_session' ) {
				return 't_session';
			} elseif ( $externaldb->get_var( 'SHOW COLUMNS FROM ' . UALP_TABLENAME . " LIKE 'site_id'" ) != 'site_id' ) {
				return 'site_id';
			}
		} else {
			return false;
		}
		return false;
	}
}


if ( ! function_exists( 'ualp_checkif_login_table_updated' ) ) {
	/**
	 * Check if login table exists.
	 */
	function ualp_checkif_login_table_updated() {
		global $externaldb;
		if ( $externaldb->get_var( "SHOW TABLES LIKE '" . UALP_LOGIN_TABLENAME . "'" ) == UALP_LOGIN_TABLENAME ) {
			return true;
		} else {
			return false;
		}
		return false;
	}
}

if ( ! function_exists( 'ualp_database_upgrade' ) ) {
	/**
	 * Run the database updater.
	 */
	function ualp_database_upgrade() {
		global $wpdb, $externaldb;
		$table_not_updated = ualp_checkif_table_updated();
		$plugin_data       = get_plugin_data( WP_PLUGIN_DIR . '/user-activity-log-pro/user_activity_log_pro.php', $markup = true, $translate = true );
		$current_version   = $plugin_data['Version'];
		$old_version       = get_option( 'ual_version' );
		$run_updater       = get_option( 'ualp_run_updater', '0' );
		if ( '' != $old_version ) {
			if ( $old_version != $current_version && '0' == $run_updater ) {
				?>
				<div class="updated">
					<p>
						<strong>
							<?php esc_html_e( 'User Activity Log Data Update', 'user_activity_log_pro' ); ?>
						</strong> &#8211; <?php esc_html_e( 'We need to update your database to the latest version.', 'user_activity_log_pro' ); ?>
					</p>
					<p class="submit">
						<?php
						if ( is_network_admin() && ualp_multi_site_support_return() ) {
							$adminurl = network_admin_url( 'admin.php?page=ualp_settings' );
						} else {
							$adminurl = admin_url( 'admin.php?page=ualp_settings' );
						}
						?>
						<a href="<?php echo esc_url( add_query_arg( 'do_update_ualp_version', 'do', $adminurl ) ); ?>" class="ualp-update-now button-primary">
							<?php esc_html_e( 'Run the updater', 'user_activity_log_pro' ); ?>
						</a>
					</p>
				</div>
				<script type="text/javascript">
					jQuery( '.ualp-update-now' ).click( 'click', function() {
						return window.confirm( '<?php echo esc_js( esc_html__( 'It is strongly recommended that you backup your database before proceeding. Are you sure you wish to run the updater now?', 'user_activity_log_pro' ) ); ?>' );
					});
				</script>
				<?php
			}
		}
		if ( $table_not_updated ) {
			?>
			<div class="updated">
				<p>
					<strong>
						<?php esc_html_e( 'User Activity Log Data Update', 'user_activity_log_pro' ); ?>
					</strong> &#8211; <?php esc_html_e( 'We need to update your database to the latest version.', 'user_activity_log_pro' ); ?>
				</p>
				<p class="submit">
					<?php
					if ( is_network_admin() && ualp_multi_site_support_return() ) {
						$adminurl = network_admin_url( 'admin.php?page=ualp_settings' );
					} else {
						$adminurl = admin_url( 'admin.php?page=ualp_settings' );
					}
					?>
					<a href="<?php echo esc_url( add_query_arg( 'do_update_ualp', 'do', $adminurl ) ); ?>" class="ualp-update-now button-primary">
						<?php esc_html_e( 'Run the updater', 'user_activity_log_pro' ); ?>
					</a>
				</p>
			</div>
			<script type="text/javascript">
				jQuery( '.ualp-update-now' ).click( 'click', function() {
					return window.confirm( '<?php echo esc_js( esc_html__( 'It is strongly recommended that you backup your database before proceeding. Are you sure you wish to run the updater now?', 'user_activity_log_pro' ) ); ?>' );
				});
			</script>
			<?php
		}
	}
}
add_action( 'admin_init', 'do_update_ualp_version_callback' );

/**
 * Update Version Callback Function.
 */
function do_update_ualp_version_callback() {
	global $externaldb;
	$run_updater = get_option( 'ualp_run_updater', '0' );
	if ( isset( $_GET['do_update_ualp_version'] ) && 'do' == $_GET['do_update_ualp_version'] && '0' == $run_updater ) {
		$enable_log_hook_list = get_option( 'enableLogHookList' );
		$all_post_type        = array();
		$post_type_hook       = array();
		$all_taxonomy         = array();
		if ( ! empty( $enable_log_hook_list ) ) {
			$plugin_data     = get_plugin_data( WP_PLUGIN_DIR . '/user-activity-log-pro/user_activity_log_pro.php', $markup = true, $translate = true );
			$current_version = $plugin_data['Version'];
			$post_types      = ualp_posttypes();
			$taxonomies      = ualp_taxonomies();
			foreach ( $post_types as $post_type ) {
				$all_post_type[] = $post_type->name;
			}
			foreach ( $taxonomies as $taxonomy ) {
				if ( 'wpcf7_contact_form' != $taxonomy->name && 'custom_css' != $taxonomy->name && 'customize_changeset' != $taxonomy->name && 'product_visibility' != $taxonomy->name ) {
					$all_taxonomy[] = $taxonomy->name;
				}
			}
			foreach ( $enable_log_hook_list as $key => $val ) {
				if ( in_array( $val, $all_post_type ) ) {
					array_push( $enable_log_hook_list, $val . '_create' );
					array_push( $enable_log_hook_list, $val . '_update' );
					array_push( $enable_log_hook_list, $val . '_delete' );
					unset( $enable_log_hook_list[ $key ] );
				}
				if ( in_array( $val, $all_taxonomy ) ) {
					array_push( $enable_log_hook_list, $val . '_create' );
					array_push( $enable_log_hook_list, $val . '_update' );
					array_push( $enable_log_hook_list, $val . '_delete' );
					unset( $enable_log_hook_list[ $key ] );
				}
				if ( 'woocommerce_variation' == $val ) {
					array_push( $enable_log_hook_list, 'woocommerce_new_product_variation', 'wp_ajax_woocommerce_remove_variations', 'woocommerce_save_product_variation' );
					unset( $enable_log_hook_list[ $key ] );
				}
				if ( 'buddypress' == $val ) {
					array_push( $enable_log_hook_list, 'groups_create_group', 'groups_update_group', 'groups_before_delete_group', 'groups_leave_group', 'groups_join_group', 'groups_promote_member', 'groups_demote_member', 'groups_ban_member', 'groups_unban_member', 'groups_remove_member', 'xprofile_field_after_save', 'xprofile_fields_deleted_field', 'xprofile_group_after_save', 'xprofile_groups_deleted_group' );
					unset( $enable_log_hook_list[ $key ] );
				}
				if ( 'wordfence' == $val ) {
					array_push( $enable_log_hook_list, 'wordfence_updateIssueStatus', 'wordfence_saveWAFConfig', 'wordfence_wafConfigureAutoPrepend', 'wordfence_whitelistBulkEnable', 'wordfence_whitelistBulkDisable', 'wordfence_whitelistBulkDelete', 'wordfence_updateConfig', 'wordfence_loadLiveTraffic', 'wordfence_loadCacheExclusions', 'wordfence_saveCacheConfig', 'wordfence_cache_clear', 'wordfence_reverseLookup', 'wordfence_whois', 'wordfence_clearPageCache', 'wordfence_saveConfig', 'wordfence_blockIP', 'wordfence_addTwoFactor', 'wordfence_exportSettings', 'wordfence_clearAllBlocked', 'wordfence_blockIPUARange', 'wordfence_unblockRange', 'wordfence_saveCountryBlocking', 'wordfence_saveDebuggingConfig', 'wordfence_startPasswdAudit', 'wordfence_saveScanSchedule', 'wordfence_updateWAFRules', 'wordfence_sendTestEmail', 'wordfence_sendDiagnostic', 'wordfence_updateAllIssues', 'wordfence_sendActivityLog', 'wordfence_addCacheExclusion', 'wordfence_daily_autoUpdate', 'wordfence_update_blocked_IPs', 'wordfence_start_scheduled_scan', 'wordfence_doScan', 'wordfence_daily_cron', 'wordfence_hourly_cron', 'wordfence_email_activity_report', 'wordfence_scan', 'wordfence_updateAlertEmail', 'wordfence_restoreFile', 'wordfence_deletePasswdAudit', 'wordfence_weakPasswordsFix', 'wordfence_killPasswdAudit', 'wordfence_importSettings', 'wordfence_bulkOperation', 'wordfence_deleteFile', 'wordfence_deleteDatabaseOption', 'wordfence_removeExclusion', 'wordfence_deleteIssue', 'wordfence_unlockOutIP', 'wordfence_loadBlockRanges', 'wordfence_unblockIP', 'wordfence_permBlockIP', 'wordfence_downloadHtaccess', 'wordfence_checkFalconHtaccess', 'wordfence_removeFromCache', 'wordfence_autoUpdateChoice', 'wordfence_saveCacheOptions', 'wordfence_adminEmailChoice', 'wordfence_getCacheStats', 'wordfence_killScan', 'wordfence_tourClosed', 'wordfence_removeCacheExclusion', 'wordfence_unblockNetwork', 'wordfence_permanentlyBlockAllIPs', 'wordfence_deleteAdminUser', 'wordfence_revokeAdminUser', 'wordfence_hideFileHtaccess' );
					unset( $enable_log_hook_list[ $key ] );
				}
				if ( 'wordpress-seo' == $val ) {
					array_push( $enable_log_hook_list, 'wpseo_handle_import', 'wpseo_import', 'wpseo_save_auth_code', 'seo_check_admin_referer', 'updated_post_meta' );
					unset( $enable_log_hook_list[ $key ] );
				}
				if ( 'blog_designer' == $val ) {
					array_push( $enable_log_hook_list, 'bdp_update_shortcode', 'bdp_reset_shortcode', 'bdp_reset_archive_page', 'bdp_add_shortcode', 'bdp_update_single_file', 'bdp_add_archive_layout', 'bdp_delete_shortcode', 'bdp_delete_single_template', 'bdp_copy_single_template', 'bdp_duplicate_layout_settings', 'bdp_import_blog_layout_settings', 'bdp_import_archive_layout_settings', 'bdp_export_archive_layout_settings', 'bdp_export_blog_layout_settings', 'bdp_update_archive_page', 'bdp_import_single_layout_settings', 'bdp_export_single_layout_settings', 'bdp_delete_archive', 'bdp_delete_single_layout', 'bdp_update_single_page', 'bdp_reset_single_page', 'bdp_add_single_layout', 'bdp_duplicate_single_layout_settings' );
					unset( $enable_log_hook_list[ $key ] );
				}
				if ( 'portfolio_designer' == $val ) {
					array_push( $enable_log_hook_list, 'import_sample_portfolio_data', 'create_portfolio_layout', 'duplicate_portfolio_layout', 'delete_portfolio_layout', 'update_portfolio_layout', 'reset_portfolio_layout', 'import_portfolio_layout', 'export_portfolio_layout', 'create_post_type', 'update_post_type', 'delete_post_type' );
					unset( $enable_log_hook_list[ $key ] );
				}
				if ( 'autoptimizer_settings' == $val ) {
					array_push( $enable_log_hook_list, 'autoptimizer_settings', 'wp_ajax_autoptimize_delete_cache' );
					unset( $enable_log_hook_list[ $key ] );
				}
				if ( 'user_switching' == $val ) {
					array_push( $enable_log_hook_list, 'switch_to_user', 'switch_back_user', 'switch_off_user' );
					unset( $enable_log_hook_list[ $key ] );
				}
				if ( 'all_in_one_reservation' == $val ) {
					array_push( $enable_log_hook_list, 'reset_reservation_settings', 'rest_update_reservation_status', 'save_reservation_settings' );
					unset( $enable_log_hook_list[ $key ] );
				}
				if ( 'edd_settings' == $val ) {
					array_push( $enable_log_hook_list, 'edd_activate_discount', 'edd_deactivate_discount', 'edd_add_discount', 'edd_delete_discount', 'transition_post_status' );
				}
				if ( 'gravityforms' == $val ) {
					array_push( $enable_log_hook_list, 'gf_export_forms', 'notification_duplicated', 'gform_entry_created', 'gf_resend_notifications', 'gform_pre_note_deleted', 'gravityforms_updated_option', 'gform_post_export_entries', 'gform_post_note_added', 'gform_delete_entries', 'gform_post_form_views_deleted', 'gform_post_form_activated', 'gform_post_form_deactivated', 'gform_before_delete_form', 'gform_post_form_trashed', 'gform_post_form_restored', 'gform_post_form_duplicated', 'gform_after_save_form', 'gform_forms_post_import', 'gform_pre_notification_deleted', 'gform_pre_confirmation_deleted', 'wp_ajax_rg_update_notification_active', 'wp_ajax_rg_update_confirmation_active', 'gform_pre_confirmation_save', 'gform_pre_notification_save', 'gform_delete_lead', 'gform_update_status', 'gform_update_is_read', 'gform_update_is_starred', 'gravityforms_updated_option' );
					unset( $enable_log_hook_list[ $key ] );
				}
				if ( 'user_ultra_settings' == $val ) {
					array_push( $enable_log_hook_list, 'wp_ajax_add_new_custom_profile_field', 'wp_ajax_save_fields_settings', 'wp_ajax_delete_profile_field', 'wp_ajax_sort_fileds_list', 'wp_ajax_custom_fields_reset', 'wp_ajax_create_uploader_folder', 'user_ultra_settings', 'wp_ajax_package_add_new', 'wp_ajax_package_delete', 'wp_ajax_package_edit_confirm', 'wp_ajax_sync_users', 'wp_ajax_user_resend_activation_link', 'wp_ajax_delete_uultra_transtient', 'wp_ajax_delete_uultra_stats', 'wp_ajax_delete_uultra_reviews', 'wp_ajax_edit_photo_cate_conf', 'wp_ajax_edit_photo_cate_del', 'wp_ajax_edit_photo_cate', 'userultra_admin_menu_hook', 'user_ultra_settings', 'user_ultra_settings', 'user_ultra_settings' );
					unset( $enable_log_hook_list[ $key ] );
				}
				if ( 'wc_checkout_fields_editor' == $val ) {
					array_push( $enable_log_hook_list, 'save_wcfe_settings', 'wp_ajax_wcfe_get_checkout_required_fields' );
					unset( $enable_log_hook_list[ $key ] );
				}
			}
			$log_hook_list = array_values( $enable_log_hook_list );
			update_option( 'enableLogHookList', $log_hook_list );
			$ualp_user_email_notify[] = get_option( 'ualpEmailNotification' );
			foreach ( $ualp_user_email_notify as $key => $val ) {
				if ( in_array( $val, $all_post_type ) ) {
					array_push( $ualp_user_email_notify, $val . '_create' );
					array_push( $ualp_user_email_notify, $val . '_update' );
					array_push( $ualp_user_email_notify, $val . '_delete' );
					unset( $ualp_user_email_notify[ $key ] );
				}
				if ( in_array( $val, $all_taxonomy ) ) {
					array_push( $ualp_user_email_notify, $val . '_create' );
					array_push( $ualp_user_email_notify, $val . '_update' );
					array_push( $ualp_user_email_notify, $val . '_delete' );
					unset( $ualp_user_email_notify[ $key ] );
				}
			}
			$email_array = array(
				'ualpUserRegisterNotify'             => 'user_register',
				'ualpUserLoginNotify'                => 'wp_login',
				'ualpUserLoginFailNotify'            => 'wp_login_failed',
				'ualpUserLogoutNotify'               => 'wp_logout',
				'ualpUserChangePasswordNotify'       => 'password_reset',
				'ualpUserProfileUpdated'             => 'insert_user_meta',
				'ualpUserDeleted'                    => 'delete_user',
				'ualpThemeInstalled'                 => 'theme_installed',
				'ualpThemeUpdated'                   => 'theme_updated',
				'ualpThemeCustomizeSave'             => 'customize_save',
				'ualpThemeDeleted'                   => 'delete_themes',
				'ualpThemeSwitched'                  => 'switch_theme',
				'ualpThemeFileModify'                => 'theme_file_modify',
				'ualpPluginInstall'                  => 'plugin_installed',
				'ualpPluginUpdated'                  => 'plugin_updated',
				'ualpPluginDeleted'                  => 'delete_plugin',
				'ualpPluginActivated'                => 'activated_plugin',
				'ualpPluginFileChanged'              => 'plugin_modify',
				'ualpWpcf7AfterCreate'               => 'wpcf7_after_create',
				'ualpWpcf7AfterUpdate'               => 'wpcf7_after_update',
				'ualpWpcf7AdminNotices'              => 'wpcf7_admin_notfualpThemeInstalledices',
				'ualpWpcf7RecaptchaSetup'            => 'wpcf7-recaptcha-setup',
				'ualpExportActivityLog'              => 'ualp_export_activity_log',
				'ualpGeneralSettingsUpdated'         => 'ualp_general_settings_updated',
				'ualpHookLogSettingsUpdated'         => 'ualp_hook_log_settings_updated',
				'ualpManageRole'                     => 'ualp_manage_role',
				'ualpSetPassword'                    => 'ualp_set_password',
				'ualpChangePassword'                 => 'ualp_change_password',
				'ualpEmailSettingsUpdated'           => 'ualp_email_settings_updated',
				'ualpDisableEmailNotification'       => 'ualp_disable_email_notification',
				'ualpUntrackUsersListUpdated'        => 'ualp_untrack_users_list_updated',
				'ualpCustomEventLogUpdated'          => 'ualp_custom_event_log_updated',
				'ualpWpInsertComment'                => 'wp_insert_comment',
				'ualpEditComment'                    => 'edit_comment',
				'ualpDeleteComment'                  => 'delete_comment',
				'ualpTrashComment'                   => 'trash_comment',
				'ualpSpamComment'                    => 'comment_unapprove',
				'ualpRestoreComment'                 => 'restore_comment',
				'ualpCommentApprove'                 => 'comment_approve',
				'ualpWpCreateNavMenu'                => 'wp_create_nav_menu',
				'ualpWpUpdateNavMenu'                => 'wp_update_nav_menu',
				'ualpWpDeleteNavMenu'                => 'delete_nav_menu',
				'ualpAdminHead'                      => 'admin_head',
				'ualpWpHead'                         => 'wp_head',
				'ualpWp404Page'                      => '404_page',
				'ualpWidgetUpdateCallback'           => 'widget_update_callback',
				'ualpCoreUpdated'                    => '_core_updated_successfully',
				'ualpImportData'                     => 'import_end',
				'ualpExportData'                     => 'export_wp',
				'dbdelta_queries'                    => 'dbdelta_queries',
				'wp_privacy_personal_data_exporters' => 'wp_privacy_personal_data_exporters',
				'wp_privacy_personal_data_erasers'   => 'wp_privacy_personal_data_erasers',
				'woocommerceUpdatedOption'           => 'woocommerce_updated_option',
				'ualpPluginDeactivated'              => 'deactivated_plugin',
			);
			array_push( $ualp_user_email_notify, '404_page' );
			foreach ( $ualp_user_email_notify as $mail_key => $mail_val ) {
				if ( 'woocommerceVariation' == $mail_val ) {
					array_push( $ualp_user_email_notify, 'woocommerce_new_product_variation', 'wp_ajax_woocommerce_remove_variations', 'woocommerce_save_product_variation' );
					unset( $ualp_user_email_notify[ $mail_key ] );
				}
				if ( 'buddypress' == $mail_val ) {
					array_push( $ualp_user_email_notify, 'groups_create_group', 'groups_update_group', 'groups_before_delete_group', 'groups_leave_group', 'groups_join_group', 'groups_promote_member', 'groups_demote_member', 'groups_ban_member', 'groups_unban_member', 'groups_remove_member', 'xprofile_field_after_save', 'xprofile_fields_deleted_field', 'xprofile_group_after_save', 'xprofile_groups_deleted_group' );
					unset( $ualp_user_email_notify[ $mail_key ] );
				}
				if ( 'wordfence' == $mail_val ) {
					array_push( $ualp_user_email_notify, 'wordfence_updateIssueStatus', 'wordfence_saveWAFConfig', 'wordfence_wafConfigureAutoPrepend', 'wordfence_whitelistBulkEnable', 'wordfence_whitelistBulkDisable', 'wordfence_whitelistBulkDelete', 'wordfence_updateConfig', 'wordfence_loadLiveTraffic', 'wordfence_loadCacheExclusions', 'wordfence_saveCacheConfig', 'wordfence_cache_clear', 'wordfence_reverseLookup', 'wordfence_whois', 'wordfence_clearPageCache', 'wordfence_saveConfig', 'wordfence_blockIP', 'wordfence_addTwoFactor', 'wordfence_exportSettings', 'wordfence_clearAllBlocked', 'wordfence_blockIPUARange', 'wordfence_unblockRange', 'wordfence_saveCountryBlocking', 'wordfence_saveDebuggingConfig', 'wordfence_startPasswdAudit', 'wordfence_saveScanSchedule', 'wordfence_updateWAFRules', 'wordfence_sendTestEmail', 'wordfence_sendDiagnostic', 'wordfence_updateAllIssues', 'wordfence_sendActivityLog', 'wordfence_addCacheExclusion', 'wordfence_daily_autoUpdate', 'wordfence_update_blocked_IPs', 'wordfence_start_scheduled_scan', 'wordfence_doScan', 'wordfence_daily_cron', 'wordfence_hourly_cron', 'wordfence_email_activity_report', 'wordfence_scan', 'wordfence_updateAlertEmail', 'wordfence_restoreFile', 'wordfence_deletePasswdAudit', 'wordfence_weakPasswordsFix', 'wordfence_killPasswdAudit', 'wordfence_importSettings', 'wordfence_bulkOperation', 'wordfence_deleteFile', 'wordfence_deleteDatabaseOption', 'wordfence_removeExclusion', 'wordfence_deleteIssue', 'wordfence_unlockOutIP', 'wordfence_loadBlockRanges', 'wordfence_unblockIP', 'wordfence_permBlockIP', 'wordfence_downloadHtaccess', 'wordfence_checkFalconHtaccess', 'wordfence_removeFromCache', 'wordfence_autoUpdateChoice', 'wordfence_saveCacheOptions', 'wordfence_adminEmailChoice', 'wordfence_getCacheStats', 'wordfence_killScan', 'wordfence_tourClosed', 'wordfence_removeCacheExclusion', 'wordfence_unblockNetwork', 'wordfence_permanentlyBlockAllIPs', 'wordfence_deleteAdminUser', 'wordfence_revokeAdminUser', 'wordfence_hideFileHtaccess' );
					unset( $ualp_user_email_notify[ $mail_key ] );
				}
				if ( 'wordpressSeo' == $mail_val ) {
					array_push( $ualp_user_email_notify, 'wpseo_handle_import', 'wpseo_import', 'wpseo_save_auth_code', 'seo_check_admin_referer', 'updated_post_meta' );
					unset( $ualp_user_email_notify[ $mail_key ] );
				}
				if ( 'blogDesigner' == $mail_val ) {
					array_push( $ualp_user_email_notify, 'bdp_update_shortcode', 'bdp_reset_shortcode', 'bdp_reset_archive_page', 'bdp_add_shortcode', 'bdp_update_single_file', 'bdp_add_archive_layout', 'bdp_delete_shortcode', 'bdp_delete_single_template', 'bdp_copy_single_template', 'bdp_duplicate_layout_settings', 'bdp_import_blog_layout_settings', 'bdp_import_archive_layout_settings', 'bdp_export_archive_layout_settings', 'bdp_export_blog_layout_settings', 'bdp_update_archive_page', 'bdp_import_single_layout_settings', 'bdp_export_single_layout_settings', 'bdp_delete_archive', 'bdp_delete_single_layout', 'bdp_update_single_page', 'bdp_reset_single_page', 'bdp_add_single_layout', 'bdp_duplicate_single_layout_settings' );
					unset( $ualp_user_email_notify[ $mail_key ] );
				}
				if ( 'portfolioDesigner' == $mail_val ) {
					array_push( $ualp_user_email_notify, 'import_sample_portfolio_data', 'create_portfolio_layout', 'duplicate_portfolio_layout', 'delete_portfolio_layout', 'update_portfolio_layout', 'reset_portfolio_layout', 'import_portfolio_layout', 'export_portfolio_layout', 'create_post_type', 'update_post_type', 'delete_post_type' );
					unset( $ualp_user_email_notify[ $mail_key ] );
				}
				if ( 'autoptimizerSettings' == $mail_val ) {
					array_push( $ualp_user_email_notify, 'autoptimizer_settings', 'wp_ajax_autoptimize_delete_cache' );
					unset( $ualp_user_email_notify[ $mail_key ] );
				}
				if ( 'userSwitching' == $mail_val ) {
					array_push( $ualp_user_email_notify, 'switch_to_user', 'switch_back_user', 'switch_off_user' );
					unset( $ualp_user_email_notify[ $mail_key ] );
				}
				if ( 'allInOneReservation' == $mail_val ) {
					array_push( $ualp_user_email_notify, 'reset_reservation_settings', 'rest_update_reservation_status', 'save_reservation_settings' );
					unset( $ualp_user_email_notify[ $mail_key ] );
				}
				if ( 'eddSettings' == $mail_val ) {
					array_push( $ualp_user_email_notify, 'edd_activate_discount', 'edd_deactivate_discount', 'edd_add_discount', 'edd_delete_discount', 'transition_post_status' );
				}
				if ( 'gravityForms' == $mail_val ) {
					array_push( $ualp_user_email_notify, 'gf_export_forms', 'notification_duplicated', 'gform_entry_created', 'gf_resend_notifications', 'gform_pre_note_deleted', 'gravityforms_updated_option', 'gform_post_export_entries', 'gform_post_note_added', 'gform_delete_entries', 'gform_post_form_views_deleted', 'gform_post_form_activated', 'gform_post_form_deactivated', 'gform_before_delete_form', 'gform_post_form_trashed', 'gform_post_form_restored', 'gform_post_form_duplicated', 'gform_after_save_form', 'gform_forms_post_import', 'gform_pre_notification_deleted', 'gform_pre_confirmation_deleted', 'wp_ajax_rg_update_notification_active', 'wp_ajax_rg_update_confirmation_active', 'gform_pre_confirmation_save', 'gform_pre_notification_save', 'gform_delete_lead', 'gform_update_status', 'gform_update_is_read', 'gform_update_is_starred', 'gravityforms_updated_option' );
					unset( $ualp_user_email_notify[ $mail_key ] );
				}
				if ( 'userUltraSettings' == $mail_val ) {
					array_push( $ualp_user_email_notify, 'wp_ajax_add_new_custom_profile_field', 'wp_ajax_save_fields_settings', 'wp_ajax_delete_profile_field', 'wp_ajax_sort_fileds_list', 'wp_ajax_custom_fields_reset', 'wp_ajax_create_uploader_folder', 'user_ultra_settings', 'wp_ajax_package_add_new', 'wp_ajax_package_delete', 'wp_ajax_package_edit_confirm', 'wp_ajax_sync_users', 'wp_ajax_user_resend_activation_link', 'wp_ajax_delete_uultra_transtient', 'wp_ajax_delete_uultra_stats', 'wp_ajax_delete_uultra_reviews', 'wp_ajax_edit_photo_cate_conf', 'wp_ajax_edit_photo_cate_del', 'wp_ajax_edit_photo_cate', 'userultra_admin_menu_hook', 'user_ultra_settings', 'user_ultra_settings', 'user_ultra_settings' );
					unset( $ualp_user_email_notify[ $mail_key ] );
				}
				if ( 'wcCheckoutFieldsEditor' == $mail_val ) {
					array_push( $ualp_user_email_notify, 'save_wcfe_settings', 'wp_ajax_wcfe_get_checkout_required_fields' );
					unset( $ualp_user_email_notify[ $mail_key ] );
				}
				if ( is_array( $email_array ) && ! empty( $email_array ) ) {
					foreach ( $email_array as $key => $val ) {
						if ( $mail_val == $key ) {
							array_push( $ualp_user_email_notify, $val );
							unset( $ualp_user_email_notify[ $mail_key ] );
						}
					}
				}
			}
			$current_user_id        = get_current_user_id();
			$current_user           = get_user_by( 'id', $current_user_id );
			$current_user_name      = $current_user->user_login;
			$all_email_notification = get_option( 'ualpEmailMessage' );
			$trigger_emailid        = get_option( 'ualpToEmail' );
			if ( '' == $trigger_emailid ) {
				$trigger_emailid = $current_user->user_email;
			}
			$trigger_title                  = 'All Email Notification';
			$email_template                 = 0;
			$custom_trigger_name[]          = 'hook';
			$custom_trigger_operator_with[] = 'AND';
			$custom_trigger_with[]          = 'equal';
			$custom_trigger_data            = array();
			$empty_count                    = 0;
			$i                              = 0;
			if ( ( isset( $custom_trigger_operator_with ) && '' != $custom_trigger_operator_with ) || ( isset( $custom_trigger_name ) && '' != $custom_trigger_name ) || ( isset( $custom_trigger_with ) && '' != $custom_trigger_with ) ) {
				$custom_trigger_operator_with = $custom_trigger_operator_with;
				$custom_trigger_name          = $custom_trigger_name;
				$custom_trigger_with          = $custom_trigger_with;
				$custom_trigger_value[]       = $ualp_user_email_notify;
			}
			if ( is_array( $custom_trigger_operator_with ) && ! empty( $custom_trigger_operator_with ) ) {
				if ( count( $custom_trigger_operator_with ) > 0 ) {
					foreach ( $custom_trigger_operator_with as $custom_trigger_operator_with_single ) {
						if ( '' != $custom_trigger_operator_with_single ) {
							$custom_trigger_data[] = array(
								'custom_trigger_operator_with' => $custom_trigger_operator_with_single,
								'custom_trigger_name'  => $custom_trigger_name[ $i ],
								'custom_trigger_with'  => $custom_trigger_with[ $i ],
								'custom_trigger_value' => $custom_trigger_value[ $i ],
							);
							$i++;
						}
					}
				}
				if ( is_array( $custom_trigger_data ) && ! empty( $custom_trigger_data ) ) {
					if ( count( $custom_trigger_data ) >= 1 ) {
						foreach ( $custom_trigger_data as $custom_trigger_single_data ) {
							$triggeropratorwith = $custom_trigger_single_data['custom_trigger_operator_with'];
							$trigger_name       = $custom_trigger_single_data['custom_trigger_name'];
							$trigger_with       = $custom_trigger_single_data['custom_trigger_with'];
							$trigger_value      = $custom_trigger_single_data['custom_trigger_value'];
							if ( '' == $trigger_name || '' == $trigger_with || '' == $triggeropratorwith || '' == $trigger_value ) {
								$empty_count++;
							}
						}
					}
				}
			}
			$guest_disable = get_option( 'ualp_untrack_guest_users' );
			if ( '1' == $guest_disable ) {
				$exclude_log_data    = array();
				$exclude_log_users   = array();
				$k                   = 0;
				$excludeempty_count  = 0;
				$j                   = 0;
				$exclude_log_users[] = '0';
				$exclude_logs_hook[] = 'all';
				$exclude_logs_ips[]  = array();
				if ( ( isset( $exclude_log_users ) && '' != $exclude_log_users ) ||
					( isset( $exclude_logs_hook ) && '' != $exclude_logs_hook ) || ( isset( $exclude_logs_ips ) ) ) {
					$exclude_log_users = $exclude_log_users;
					$exclude_logs_hook = $exclude_logs_hook;
					$exclude_logs_ip   = $exclude_logs_ips;
					if ( is_array( $exclude_log_users ) && ! empty( $exclude_log_users ) ) {
						if ( count( $exclude_log_users ) > 0 ) {
							foreach ( $exclude_log_users as $exclude_log_users_single ) {
								if ( '' != $exclude_log_users_single ) {
									$exclude_log_data[] = array(
										'exclude_log_users' => $exclude_log_users[ $k ],
										'exclude_logs_hook' => $exclude_logs_hook[ $k ],
										'exclude_logs_ips' => $exclude_logs_ip[ $k ],
									);
									$k++;
								}
							}
						}
					}
					if ( is_array( $exclude_log_data ) && ! empty( $exclude_log_data ) ) {
						if ( count( $exclude_log_data ) >= 1 ) {
							foreach ( $exclude_log_data as $exclude_log_single_data ) {
								$exclude_log  = $exclude_log_single_data['exclude_log_users'];
								$exclude_hook = $exclude_log_single_data['exclude_logs_hook'];
								$excludeips   = $exclude_log_single_data['exclude_logs_ips'];
								if ( '' == $exclude_log || '' == $exclude_hook || '' == $excludeips ) {
									$excludeempty_count++;
								}
							}
						}
					}
					update_option( 'exclude_log_data', $exclude_log_data );
				}
			}
			update_option( 'ual_version', $current_version );
			update_option( 'ualp_run_updater', '1' );
			ualp_insert_email_notification( $current_user_name, $all_email_notification, $custom_trigger_data, $trigger_title, $trigger_emailid, $email_template );
		}
	}
}

if ( ! function_exists( 'ualp_create_detail_table' ) ) {
	/**
	 * Create log detail table when activate plugin.
	 */
	function ualp_create_detail_table() {
		global $wpdb;
		// table is not created. you may create the table here.
		if ( $wpdb->get_var( $wpdb->prepare( 'SHOW TABLES LIKE %s', UALP_DETAIL_TABLENAME ) ) != UALP_DETAIL_TABLENAME ) {
			$create_table_query = 'CREATE TABLE ' . UALP_DETAIL_TABLENAME . ' ('
				. 'uactd_id int(20) NOT NULL auto_increment,'
				. 'uactid int(20),'
				. 'uactd_key varchar(200),'
				. 'uactd_value text,'
				. 'PRIMARY KEY (uactd_id)) CHARACTER SET utf8 COLLATE utf8_unicode_ci';
			require_once ABSPATH . 'wp-admin/includes/upgrade.php';
			dbDelta( $create_table_query );
		}
	}
}

if ( ! function_exists( 'ualp_create_email_notification_table' ) ) {
	/**
	 * Create email notification table when activate plugin.
	 */
	function ualp_create_email_notification_table() {
		global $wpdb;
		// table is not created. you may create the table here.
		if ( $wpdb->get_var( $wpdb->prepare( 'SHOW TABLES LIKE %s', UALP_EMAIL_TABLENAME ) ) != UALP_EMAIL_TABLENAME ) {
			$charset_collate = '';
			if ( ! empty( $wpdb->charset ) ) {
				$charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";
			}
			if ( ! empty( $wpdb->collate ) ) {
				$charset_collate .= " COLLATE $wpdb->collate";
			}
			$create_table_query = 'CREATE TABLE ' . UALP_EMAIL_TABLENAME . ' ('
				. 'uacteid bigint(20) unsigned NOT NULL auto_increment,'
				. 'email_content longtext,'
				. 'email_title text,'
				. "user_id bigint(20) unsigned default '0',"
				. 'user_name varchar(250),'
				. 'user_role varchar(250),'
				. 'user_email varchar(250),'
				. 'site_id INT(15),'
				. 'custom_trigger_data text,'
				. 'email_address varchar(100),'
				. 'email_template INT(15),'
				. 'PRIMARY KEY (uacteid))'
				. $charset_collate;
			require_once ABSPATH . 'wp-admin/includes/upgrade.php';
			dbDelta( $create_table_query );
		}
	}
}

if ( ! function_exists( 'ualp_create_sms_notification_table' ) ) {
	/**
	 * Create sms notification table when activate plugin.
	 */
	function ualp_create_sms_notification_table() {
		global $wpdb;
		// table is not created. you may create the table here.
		if ( $wpdb->get_var( $wpdb->prepare( 'SHOW TABLES LIKE %s', UALP_SMS_TABLENAME ) ) != UALP_SMS_TABLENAME ) {
			$charset_collate = '';
			if ( ! empty( $wpdb->charset ) ) {
				$charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";
			}
			if ( ! empty( $wpdb->collate ) ) {
				$charset_collate .= " COLLATE $wpdb->collate";
			}
			$create_table_query = 'CREATE TABLE ' . UALP_SMS_TABLENAME . ' ('
				. 'uacteid bigint(20) unsigned NOT NULL auto_increment,'
				. 'sms_content longtext,'
				. 'sms_title text,'
				. "user_id bigint(20) unsigned default '0',"
				. 'user_name varchar(250),'
				. 'user_role varchar(250),'
				. 'user_email varchar(250),'
				. 'site_id INT(15),'
				. 'custom_trigger_data text,'
				. 'sms_address varchar(100),'
				. 'sms_template INT(15),'
				. 'PRIMARY KEY (uacteid))'
				. $charset_collate;
			require_once ABSPATH . 'wp-admin/includes/upgrade.php';
			dbDelta( $create_table_query );
		}
	}
}

if ( ! function_exists( 'ualp_create_schedule_report' ) ) {
	/**
	 * Create SCHEDULE table when activate plugin.
	 */
	function ualp_create_schedule_report() {
		global $wpdb;
		// table is not created. you may create the table here.
		if ( $wpdb->get_var( $wpdb->prepare( 'SHOW TABLES LIKE %s', UALP_SCHEDULE_TABLENAME ) ) != UALP_SCHEDULE_TABLENAME ) {
			$charset_collate = '';
			if ( ! empty( $wpdb->charset ) ) {
				$charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";
			}
			if ( ! empty( $wpdb->collate ) ) {
				$charset_collate .= " COLLATE $wpdb->collate";
			}
			$create_table_query = 'CREATE TABLE ' . UALP_SCHEDULE_TABLENAME . ' ('
				. 'uactrid bigint(20) unsigned NOT NULL auto_increment,'
				. 'report_title text,'
				. 'schedule_email_id text,'
				. 'report_frequency varchar(250),'
				. 'PRIMARY KEY (uactrid))'
				. $charset_collate;
			require_once ABSPATH . 'wp-admin/includes/upgrade.php';
			dbDelta( $create_table_query );
		}
	}
}

if ( ! function_exists( 'ualp_create_detail_external_table' ) ) {
	/**
	 * Create log detail table in external database.
	 */
	function ualp_create_detail_external_table() {
		global $externaldb;
		// table is not created. you may create the table here.
		$tableprefix = '';
		$tableprefix = get_option( 'table_prefix' );
		if ( $externaldb->get_var( "SHOW TABLES LIKE '" . $tableprefix . "ualp_user_activity_detail'" ) != $tableprefix . 'ualp_user_activity_detail' ) {
			$create_table_query = 'CREATE TABLE ' . $tableprefix . 'ualp_user_activity_detail ('
				. 'uactd_id int(20) NOT NULL auto_increment,'
				. 'uactid int(20),'
				. 'uactd_key varchar(200),'
				. 'uactd_value text,'
				. 'PRIMARY KEY (uactd_id)) CHARACTER SET utf8 COLLATE utf8_unicode_ci';
			require_once ABSPATH . 'wp-admin/includes/upgrade.php';
			$externaldb->query( $create_table_query );
		}
	}
}

if ( ! function_exists( 'ualp_create_email_notification_external_table' ) ) {
	/**
	 * Create External email notification table when activate plugin.
	 */
	function ualp_create_email_notification_external_table() {
		global $externaldb;
		$tableprefix = '';
		$tableprefix = get_option( 'table_prefix' );
		if ( $externaldb->get_var( "SHOW TABLES LIKE '" . $tableprefix . "ualp_email_notification'" ) != $tableprefix . 'ualp_email_notification' ) {
			$create_table_query = 'CREATE TABLE ' . $tableprefix . 'ualp_email_notification ('
				. 'uacteid bigint(20) unsigned NOT NULL auto_increment,'
				. 'email_title text,'
				. 'email_content longtext,'
				. "user_id bigint(20) unsigned default '0',"
				. 'user_name varchar(250),'
				. 'user_role varchar(250),'
				. 'user_email varchar(250),'
				. 'site_id INT(15),'
				. 'custom_trigger_data text,'
				. 'email_address varchar(100),'
				. 'email_template INT(15),'
				. 'PRIMARY KEY (uacteid)) CHARACTER SET utf8 COLLATE utf8_unicode_ci';
				require_once ABSPATH . 'wp-admin/includes/upgrade.php';
				$externaldb->query( $create_table_query );
		}
	}
}

if ( ! function_exists( 'ualp_create_sms_notification_external_table' ) ) {
	/**
	 * Create External sms notification table when activate plugin.
	 */
	function ualp_create_sms_notification_external_table() {
		global $externaldb;
		$tableprefix = '';
		$tableprefix = get_option( 'table_prefix' );
		if ( $externaldb->get_var( "SHOW TABLES LIKE '" . $tableprefix . "ualp_sms_notification'" ) != $tableprefix . 'ualp_sms_notification' ) {
			$create_table_query = 'CREATE TABLE ' . $tableprefix . 'ualp_sms_notification ('
				. 'uacteid bigint(20) unsigned NOT NULL auto_increment,'
				. 'sms_title text,'
				. 'sms_content longtext,'
				. "user_id bigint(20) unsigned default '0',"
				. 'user_name varchar(250),'
				. 'user_role varchar(250),'
				. 'user_email varchar(250),'
				. 'site_id INT(15),'
				. 'custom_trigger_data text,'
				. 'sms_address varchar(100),'
				. 'sms_template INT(15),'
				. 'PRIMARY KEY (uacteid)) CHARACTER SET utf8 COLLATE utf8_unicode_ci';
				require_once ABSPATH . 'wp-admin/includes/upgrade.php';
				$externaldb->query( $create_table_query );
		}
	}
}

if ( ! function_exists( 'ualp_create_schedule_external_report' ) ) {
	/**
	 * Create External SCHEDULE table when activate plugin.
	 */
	function ualp_create_schedule_external_report() {
		global $externaldb;
		$tableprefix = '';
		$tableprefix = get_option( 'table_prefix' );
		if ( $externaldb->get_var( "SHOW TABLES LIKE '" . $tableprefix . "ualp_schedule_report'" ) != $tableprefix . 'ualp_schedule_report' ) {
			$create_table_query = 'CREATE TABLE ' . $tableprefix . 'ualp_schedule_report ('
			. 'uactrid bigint(20) unsigned NOT NULL auto_increment,'
			. 'report_title text,'
			. 'schedule_email_id text,'
			. 'report_frequency varchar(250),'
			. 'PRIMARY KEY (uactrid)) CHARACTER SET utf8 COLLATE utf8_unicode_ci';

				require_once ABSPATH . 'wp-admin/includes/upgrade.php';
				$externaldb->query( $create_table_query );
		}
	}
}

if ( ! function_exists( 'ualp_create_login_external_table' ) ) {
	/**
	 * Create log detail table when activate plugin.
	 */
	function ualp_create_login_external_table() {
		global $externaldb;
		$tableprefix = '';
		$tableprefix = get_option( 'table_prefix' );
		if ( $externaldb->get_var( "SHOW TABLES LIKE '" . $tableprefix . "ualp_user_activity_login'" ) != $tableprefix . 'ualp_user_activity_login' ) {
			$create_login_table_query = 'CREATE TABLE ' . $tableprefix . 'ualp_user_activity_login ('
			. 'uacltid bigint(20) unsigned NOT NULL auto_increment,'
			. "user_id bigint(20) unsigned default '0',"
			. 'user_name varchar(250),'
			. 'user_role varchar(250),'
			. 'user_email varchar(250),'
			. 'expiration varchar(250),'
			. 'session_token varchar(250),'
			. 'time_login datetime,'
			. 'time_logout datetime,'
			. 'time_last_seen datetime,'
			. 'ip_address varchar(50),'
			. 'country varchar(150),'
			. 'user_agent text,'
			. 'login_status varchar(50),'
			. 'site_id INT(15),'
			. 'PRIMARY KEY (uacltid))
                CHARACTER SET utf8 COLLATE utf8_unicode_ci';
			require_once ABSPATH . 'wp-admin/includes/upgrade.php';
			$externaldb->query( $create_login_table_query );
		}
		$fieldname = 'site_id';
		if ( $externaldb->get_var( 'SHOW COLUMNS FROM ' . $tableprefix . "ualp_user_activity_login LIKE '" . $fieldname . "'" ) != $fieldname ) {
			$ualp_activity_table = $tableprefix . 'ualp_user_activity_login';
			$sql                 = "ALTER TABLE $ualp_activity_table ADD `site_id` INT(15) NULL AFTER `login_status`";
			$externaldb->query( $sql );
		}
	}
}

if ( ! function_exists( 'ualp_add_settings_link' ) ) {
	/**
	 * Add Settings Links.
	 *
	 * @param string $links for take a action for redirection setting.
	 * @return action for setting link
	 */
	function ualp_add_settings_link( $links ) {
		$adminurl      = admin_url( 'admin.php?page=ualp_settings' );
		$settings_link = '<a href="' . $adminurl . '">' . esc_html__( 'Settings', 'user_activity_log_pro' ) . '</a>';
		array_push( $links, $settings_link );
		return $links;
	}
}


if ( ! function_exists( 'ualp_filter_user_role' ) ) {
	/**
	 * Filter user Roles.
	 */
	function ualp_filter_user_role() {
		$paged     = 1;
		$admin_url = get_admin_url();
		$display   = '';
		$search    = '';
		if ( ( isset( $_REQUEST['filter_nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_REQUEST['filter_nonce'] ) ), 'ual_filter_nonce' ) ) ) {
			if ( isset( $_POST['user_role'] ) ) {
				$display = sanitize_text_field( wp_unslash( $_POST['user_role'] ) );
			}
			if ( isset( $_POST['btn_filter_user_role'] ) ) {
				$display = sanitize_text_field( wp_unslash( $_POST['user_role'] ) );
				if ( is_network_admin() && ualp_multi_site_support_return() ) {
					wp_redirect( network_admin_url( 'admin.php?page=ualp_email_setting' ) . "&paged=$paged&display=$display&txtsearch=$search" );
				} else {
					wp_redirect( admin_url( 'admin.php?page=ualp_email_setting' ) . "&paged=$paged&display=$display&txtsearch=$search" );
				}
				exit();
			}
			if ( isset( $_POST['btnSearch_user_role'] ) && isset( $_POST['txtSearchinput'] ) ) {
				$search = sanitize_text_field( wp_unslash( $_POST['txtSearchinput'] ) );
				if ( is_network_admin() && ualp_multi_site_support_return() ) {
					wp_redirect( network_admin_url( 'admin.php?page=ualp_email_setting' ) . "&paged=$paged&display=$display&txtsearch=$search" );
				} else {
					wp_redirect( admin_url( 'admin.php?page=ualp_email_setting' ) . "&paged=$paged&display=$display&txtsearch=$search" );
				}
				exit();
			}
		}
	}
}

if ( ! function_exists( 'ualp_compare_log_password' ) ) {
	/**
	 * Compare password to delete logs.
	 */
	function ualp_compare_log_password() {
		$ualp_delete_log_pwd = get_option( 'ualp_delete_log_pwd' );
		if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'ajax-nonce' ) ) {
			if ( isset( $_POST['logPwd'] ) ) {
				$ualp_get_delete_log_pwd = sanitize_text_field( wp_unslash( $_POST['logPwd'] ) );
				$ualp_get_delete_log_pwd = md5( $ualp_get_delete_log_pwd );
				if ( $ualp_delete_log_pwd == $ualp_get_delete_log_pwd ) {
					echo 'valid';
				} else {
					echo 'in_valid';
				}
			}
		}
		exit();
	}
}

if ( ! function_exists( 'ualp_delete_ualp_table' ) ) {
	/**
	 * Reset user activity database.
	 */
	function ualp_delete_ualp_table() {
		global $wpdb, $externaldb;
		if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'ajax-nonce' ) ) {
			if ( isset( $_POST['downloadLogs'] ) && 'true' == $_POST['downloadLogs'] ) {
				ualp_generate_csv_attachment();
				echo 'downloadLogs';
				$externaldb->query( 'TRUNCATE ' . UALP_TABLENAME );
				$externaldb->query( 'TRUNCATE ' . UALP_DETAIL_TABLENAME );
				$externaldb->query( 'TRUNCATE ' . UALP_LOGIN_TABLENAME );
				$externaldb->query( 'TRUNCATE ' . UALP_EMAIL_TABLENAME );
				$externaldb->query( 'TRUNCATE ' . UALP_SCHEDULE_TABLENAME );
				$files = glob( WP_CONTENT_DIR . '/uploads/ualp/*' ); // get all file names.
				foreach ( $files as $file ) { // iterate files.
					if ( is_file( $file ) ) {
						unlink( $file ); // delete file.
					}
				}
			}
			if ( isset( $_POST['sendLog'] ) && 'true' == $_POST['sendLog'] ) {
				$sent = ualp_send_csv_in_email();
				if ( $sent ) {
					echo 'sendLog';
					$externaldb->query( 'TRUNCATE ' . UALP_TABLENAME );
					$externaldb->query( 'TRUNCATE ' . UALP_DETAIL_TABLENAME );
					$externaldb->query( 'TRUNCATE ' . UALP_LOGIN_TABLENAME );
					$externaldb->query( 'TRUNCATE ' . UALP_EMAIL_TABLENAME );
					$externaldb->query( 'TRUNCATE ' . UALP_SCHEDULE_TABLENAME );
					$files = glob( WP_CONTENT_DIR . '/uploads/ualp/*' ); // get all file names.
					foreach ( $files as $file ) { // iterate files.
						if ( is_file( $file ) ) {
							unlink( $file ); // delete file.
						}
					}
				} else {
					echo 'mailnotsent';
				}
			}
		}
		exit();
	}
}

if ( ! function_exists( 'ualp_delete_single_log' ) ) {
	/**
	 * Delete single logs.
	 */
	function ualp_delete_single_log() {
		global $wpdb, $externaldb;
		$delete_id = isset( $_POST['logid'] ) ? sanitize_text_field( wp_unslash( $_POST['logid'] ) ) : '';
		if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'ajax-nonce' ) ) {
			if ( isset( $_POST['downloadLogs'] ) && 'true' == $_POST['downloadLogs'] ) {
				// ualp_generate_csv_attachment();.
				$uactd_id_list = $externaldb->get_results( 'SELECT uactd_id FROM ' . UALP_TABLENAME . ' left join ' . UALP_DETAIL_TABLENAME . ' on ' . UALP_DETAIL_TABLENAME . '.uactid = ' . UALP_TABLENAME . '.uactid WHERE ' . UALP_TABLENAME . '.favorite = 0 AND ' . UALP_DETAIL_TABLENAME . ".uactid = $delete_id", ARRAY_A );
				foreach ( $uactd_id_list as $uactd_id ) {
					$externaldb->query( $externaldb->prepare( 'DELETE FROM ' . UALP_DETAIL_TABLENAME . ' WHERE uactid = %d AND uactd_id = %d', $delete_id, $uactd_id['uactd_id'] ) );
				}
				$externaldb->query( $externaldb->prepare( 'DELETE FROM ' . UALP_TABLENAME . ' WHERE uactid = %d AND favorite = %d', $delete_id, '0' ) );
				echo 'downloadLogs';
			}
			if ( isset( $_POST['sendLog'] ) && 'true' == $_POST['sendLog'] ) {
				$sent = ualp_send_csv_in_email();
				if ( $sent ) {
					$externaldb->query( $externaldb->prepare( 'DELETE FROM ' . UALP_TABLENAME . ' WHERE uactid = %d AND favorite = %d', $delete_id, '0' ) );
					echo 'sendLog';
				} else {
					echo 'mailnotsent';
				}
			}
		}
		exit();
	}
}
add_action( 'admin_init', 'ualp_delete_single_log_get' );
if ( ! function_exists( 'ualp_delete_single_log_get' ) ) {
	/**
	 * Get Delete single logs.
	 */
	function ualp_delete_single_log_get() {
		global $wpdb, $externaldb;

		if ( isset( $_GET['delete-nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_GET['delete-nonce'] ) ), 'delete-action' ) ) {
			if ( isset( $_GET['id'] ) && isset( $_GET['page'] ) && 'ual_pro' == $_GET['page'] ) {
				$delete_id     = sanitize_text_field( wp_unslash( $_GET['id'] ) );
				$uactd_id_list = $externaldb->get_results( 'SELECT uactd_id FROM ' . UALP_TABLENAME . ' left join ' . UALP_DETAIL_TABLENAME . ' on ' . UALP_DETAIL_TABLENAME . '.uactid = ' . UALP_TABLENAME . '.uactid WHERE ' . UALP_TABLENAME . '.favorite = 0 AND ' . UALP_DETAIL_TABLENAME . ".uactid = $delete_id", ARRAY_A );
				foreach ( $uactd_id_list as $uactd_id ) {
					$externaldb->query( $externaldb->prepare( 'DELETE FROM ' . UALP_DETAIL_TABLENAME . ' WHERE uactid = %d AND uactd_id = %d', $delete_id, $uactd_id['uactd_id'] ) );
				}
				$externaldb->query( $externaldb->prepare( 'DELETE FROM ' . UALP_TABLENAME . ' WHERE uactid = %d AND favorite = %d', $delete_id, '0' ) );
			}
		}
	}
}

if ( ! function_exists( 'ualp_delete_bulk_log' ) ) {
	/**
	 * Delete bulk logs.
	 */
	function ualp_delete_bulk_log() {
		global $wpdb, $externaldb;
		$delete_ids = isset( $_POST['logid'] ) ? array_map( 'sanitize_text_field', wp_unslash( $_POST['logid'] ) ) : '';
		if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'ajax-nonce' ) ) {
			if ( isset( $_POST['downloadLogs'] ) && 'true' == $_POST['downloadLogs'] ) {
				// ualp_generate_csv_attachment();.
				foreach ( $delete_ids as $delete_id ) {
					$uactd_id_list = $externaldb->get_results( 'SELECT uactd_id FROM ' . UALP_TABLENAME . ' left join ' . UALP_DETAIL_TABLENAME . ' on ' . UALP_DETAIL_TABLENAME . '.uactid = ' . UALP_TABLENAME . '.uactid WHERE ' . UALP_TABLENAME . '.favorite = 0 AND ' . UALP_DETAIL_TABLENAME . ".uactid = $delete_id", ARRAY_A );
					foreach ( $uactd_id_list as $uactd_id ) {
						$externaldb->query( $externaldb->prepare( 'DELETE FROM ' . UALP_DETAIL_TABLENAME . ' WHERE uactid = %d AND uactd_id = %d', $delete_id, $uactd_id['uactd_id'] ) );
					}
					$externaldb->query( $externaldb->prepare( 'DELETE FROM ' . UALP_TABLENAME . ' WHERE uactid = %d AND favorite = %d', $delete_id, '0' ) );
				}
				echo 'downloadLogs';
			}
			if ( isset( $_POST['sendLog'] ) && 'true' == $_POST['sendLog'] ) {
				$sent = ualp_send_csv_in_email();
				if ( $sent ) {
					foreach ( $delete_ids as $delete_id ) {
						$externaldb->query( $externaldb->prepare( 'DELETE FROM ' . UALP_TABLENAME . ' WHERE uactid = %d AND favorite = %d', $delete_id, '0' ) );
					}
					echo 'sendLog';
				} else {
					echo 'mailnotsent';
				}
			}
		}
		exit();
	}
}

if ( ! function_exists( 'ualp_dialogbox_content' ) ) {
	/**
	 * Display dialog box.
	 */
	function ualp_dialogbox_content() {
		if ( ( isset( $_GET['page'] ) && ( 'ual_pro' == $_GET['page'] || 'ualp_settings' == $_GET['page'] ) ) ) {

			if ( is_network_admin() && ualp_multi_site_support_return() ) {
				$adminurl = network_admin_url( 'admin.php' );
			} else {
				$adminurl = admin_url( 'admin.php' );
			}
			$qry_str = isset( $_SERVER['QUERY_STRING'] ) ? sanitize_text_field( wp_unslash( $_SERVER['QUERY_STRING'] ) ) : '';
			?>
			<div class="ualp_view_details_ajax" title="<?php esc_attr_e( 'Log Details', 'user_activity_log_pro' ); ?>"></div>
			<div id="dialogPassword" title="<?php esc_attr_e( 'Are you sure to delete activity logs?', 'user_activity_log_pro' ); ?>" style="display:none">
				<div id="dialog-form" title="<?php esc_attr_e( 'Delete activity logs', 'user_activity_log_pro' ); ?>">
					<form method="POST" action="<?php echo esc_attr( $adminurl ) . esc_attr( $qry_str ); ?>">
						<fieldset>
							<br/>
							<label for="deleteLogPassword"><?php esc_html_e( 'Enter Password', 'user_activity_log_pro' ); ?></label>
							<input type="password" name="deleteLogPassword" id="deleteLogPassword" value="" class="text ui-widget-content ui-corner-all">
							<p class="pwdErrorMsg red"></p>
						</fieldset>
					</form>
				</div>
			</div>
			<div id="dialogSinglePassword" title="<?php esc_attr_e( 'Are you sure to delete activity log?', 'user_activity_log_pro' ); ?>" style="display:none">
				<form method="POST" action="<?php echo esc_attr( $adminurl ) . esc_attr( $qry_str ); ?>">
					<fieldset>
						<br/>
						<label for="deleteSingleLogPassword"><?php esc_html_e( 'Enter Password', 'user_activity_log_pro' ); ?></label>
						<input type="password" name="deleteLogPassword" id="deleteSingleLogPassword" value="" class="text ui-widget-content ui-corner-all">
						<p class="pwdErrorMsg red"></p>
					</fieldset>
				</form>
			</div>
			<div id="dialogDownload" title="<?php esc_attr_e( 'Download Activity Logs', 'user_activity_log_pro' ); ?>" style="display:none">
				<form method="POST" action="<?php echo esc_attr( $adminurl ) . esc_attr( $qry_str ); ?>">
					<fieldset>
						<label for="downloadLog">
							<input checked="" type="checkbox" name="downloadLogs" value="downloadLog" id="downloadLog">
							<?php esc_html_e( 'Download Log file', 'user_activity_log_pro' ); ?>
						</label>
					</fieldset>
					<fieldset>
						<label for="sendLog">
							<input type="checkbox" name="sendLog" value="sendLog" id="sendLog">
							<?php esc_html_e( 'Send me log file in email', 'user_activity_log_pro' ); ?>
						</label>
					</fieldset>
					<p class="pwdErrorMsg red"></p>
				</form>
			</div>
			<div id="setDeletePassword" title="<?php esc_attr_e( 'Set Password for delete log', 'user_activity_log_pro' ); ?>" style="display:none">
				<p>
					<?php
					esc_html_e( 'To delete the user activity log, you need to set the password. You can set the password using', 'user_activity_log_pro' );
					?>
					&nbsp;
					<?php
					if ( is_network_admin() && ualp_multi_site_support_return() ) {
						$adminurl = network_admin_url( 'admin.php?page=ualp_settings' );
					} else {
						$adminurl = admin_url( 'admin.php?page=ualp_settings' );
					}
					?>
					<a href='<?php echo esc_url( $adminurl ); ?>'>
						<b><?php esc_html_e( 'password settings', 'user_activity_log_pro' ); ?></b>
					</a>
				</p>
			</div>
			<?php
		}
	}
}

if ( ! function_exists( 'ualp_get_term_by_id' ) ) {
	/**
	 * Get term without knowing it's taxonomy.
	 *
	 * @param int|object $term Term.
	 * @param string     $output Output.
	 * @param string     $filter Filter.
	 */
	function ualp_get_term_by_id( $term, $output = ARRAY_A, $filter = 'raw' ) {
		global $wpdb;
		$null = null;
		if ( empty( $term ) ) {
			$error = new WP_Error( 'invalid_term', esc_html__( 'Empty Term', 'user_activity_log_pro' ) );
			return $error;
		}
		$_tax     = $wpdb->get_row( $wpdb->prepare( "SELECT t.* FROM $wpdb->term_taxonomy AS t WHERE t.term_id = %s LIMIT 1", $term ) );
		$taxonomy = $_tax->taxonomy;
		return get_term( $term, $taxonomy, $output, $filter );
	}
}

if ( ! function_exists( 'ualp_deactivate_ual' ) ) {
	/**
	 * Update database and deactivate user activity plugin.
	 */
	function ualp_deactivate_ual() {
		require_once ABSPATH . 'wp-admin/includes/upgrade.php';
		global $wpdb, $externaldb;
		if ( $externaldb->get_var( "SHOW TABLES LIKE '" . UALP_DETAIL_TABLENAME . "'" ) == UALP_DETAIL_TABLENAME ) {
			$hook_result = $externaldb->query( 'SHOW COLUMNS FROM ' . UALP_TABLENAME . " LIKE 'hook'" );
			if ( 0 == $hook_result ) {
				$alter_table_query = $externaldb->query(
					'ALTER TABLE ' . UALP_TABLENAME
					. ' ADD COLUMN hook varchar(100)'
					. ' AFTER action'
				);
			}
			$favorite_result = $externaldb->query( 'SHOW COLUMNS FROM ' . UALP_TABLENAME . " LIKE 'favorite'" );
			if ( 0 == $favorite_result ) {
				$alter_table_query = $externaldb->query(
					'ALTER TABLE ' . UALP_TABLENAME
					. " ADD COLUMN favorite int(1) DEFAULT '0'"
					. ' AFTER post_id'
				);
			}
			$description_result = $externaldb->query( 'SHOW COLUMNS FROM ' . UALP_TABLENAME . " LIKE 'description'" );
			if ( 0 == $description_result ) {
				$alter_table_query = $externaldb->query(
					'ALTER TABLE ' . UALP_TABLENAME
					. ' ADD COLUMN description text'
					. ' AFTER hook'
				);
			}
			$country_result = $externaldb->query( 'SHOW COLUMNS FROM ' . UALP_TABLENAME . " LIKE 'country'" );
			if ( 0 == $country_result ) {
				$alter_table_query = $externaldb->query(
					'ALTER TABLE ' . UALP_TABLENAME
					. ' ADD COLUMN country TEXT'
					. ' AFTER ip_address'
				);
			}
		}
		if ( is_plugin_active( 'user-activity-log/user_activity_log.php' ) ) {
			deactivate_plugins( 'user-activity-log/user_activity_log.php' );
		}
		$plugin_data     = get_plugin_data( WP_PLUGIN_DIR . '/user-activity-log-pro/user_activity_log_pro.php', $markup = true, $translate = true );
		$current_version = $plugin_data['Version'];
		if ( '1.6' == $current_version ) {
			$exclude_log_data    = array();
			$exclude_log_users   = array();
			$k                   = 0;
			$excludeempty_count  = 0;
			$j                   = 0;
			$exclude_log_users[] = '0';
			$exclude_logs_hook[] = 'all';
			$exclude_logs_ips[]  = array();
			if ( ( isset( $exclude_log_users ) && '' != $exclude_log_users ) ||
				( isset( $exclude_logs_hook ) && '' != $exclude_logs_hook ) || ( isset( $exclude_logs_ips ) ) ) {
				$exclude_log_users = $exclude_log_users;
				$exclude_logs_hook = $exclude_logs_hook;
				$exclude_logs_ip   = $exclude_logs_ips;
				if ( is_array( $exclude_log_users ) && ! empty( $exclude_log_users ) ) {
					if ( count( $exclude_log_users ) > 0 ) {
						foreach ( $exclude_log_users as $exclude_log_users_single ) {
							if ( '' != $exclude_log_users_single ) {
								$exclude_log_data[] = array(
									'exclude_log_users' => $exclude_log_users[ $k ],
									'exclude_logs_hook' => $exclude_logs_hook[ $k ],
									'exclude_logs_ips'  => $exclude_logs_ip[ $k ],
								);
								$k++;
							}
						}
					}
				}
				if ( is_array( $exclude_log_data ) && ! empty( $exclude_log_data ) ) {
					if ( count( $exclude_log_data ) >= 1 ) {
						foreach ( $exclude_log_data as $exclude_log_single_data ) {
							$exclude_log  = $exclude_log_single_data['exclude_log_users'];
							$exclude_hook = $exclude_log_single_data['exclude_logs_hook'];
							$excludeips   = $exclude_log_single_data['exclude_logs_ips'];
							if ( '' == $exclude_log || '' == $exclude_hook || '' == $excludeips ) {
								$excludeempty_count++;
							}
						}
					}
				}
				update_option( 'exclude_log_data', $exclude_log_data );
			}
		}
	}
}

if ( ! function_exists( 'ualp_redirect' ) ) {
	/**
	 * Redirect to settings page on plugin activation.
	 *
	 * @param string $plugin Plugin.
	 * @param string $network_activation Network Activation.
	 */
	function ualp_redirect( $plugin, $network_activation ) {
		if ( 'user-activity-log-pro/user_activity_log_pro.php' == $plugin ) {
			if ( is_network_admin() && ualp_multi_site_support_return() ) {
				wp_redirect( network_admin_url( 'admin.php?page=ual_pro' ) );
			} else {
				wp_redirect( admin_url( 'admin.php?page=ualp_settings' ) );
			}
			exit();
		}
	}
}

if ( ! function_exists( 'ualp_reset_database_settings' ) ) {
	/**
	 * Reset database integration settings.
	 */
	function ualp_reset_database_settings() {
		if ( isset( $_GET['do_update_ualp'] ) && 'do' == $_GET['do_update_ualp'] ) {
			global $wpdb, $externaldb;
			$plugin_data     = get_plugin_data( WP_PLUGIN_DIR . '/user-activity-log-pro/user_activity_log_pro.php', $markup = true, $translate = true );
			$current_version = $plugin_data['Version'];
			$externaldb->query( 'ALTER TABLE ' . UALP_TABLENAME . ' ADD `country` TEXT NULL AFTER `ip_address`' );
			$externaldb->query( 'ALTER TABLE ' . UALP_TABLENAME . ' ADD `user_agent` TEXT NULL AFTER `country`' );
			$externaldb->query( 'ALTER TABLE ' . UALP_TABLENAME . ' ADD `t_session` varchar(200) NULL AFTER `description`' );
			update_option( 'ualp_version', $current_version );
			$mail_msg = get_option( 'ualpEmailMessage', true );
			$mail_msg = str_replace( 'Following user is logged in your site', 'Following user has done some activity in your site', $mail_msg );
			update_option( 'ualpEmailMessage', $mail_msg );
			if ( is_network_admin() && ualp_multi_site_support_return() ) {
				wp_redirect( network_admin_url( 'admin.php?page=ual_pro' ) );
			} else {
				wp_redirect( admin_url( 'admin.php?page=ual_pro' ) );
			}
			exit();
		}
		if ( isset( $_POST['btnResetDbData'] ) && isset( $_POST['_wp_db_settings_nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wp_db_settings_nonce'] ) ), '_wp_db_settings_nonce' ) ) {
			update_option( 'table_prefix', '' );
			update_option( 'database_name', '' );
			update_option( 'database_hostname', '' );
			update_option( 'database_username', '' );
			update_option( 'database_password', '' );
			if ( is_network_admin() && ualp_multi_site_support_return() ) {
				wp_redirect( network_admin_url( 'admin.php?page=ualp_db_integration' ) );
			} else {
				wp_redirect( admin_url( 'admin.php?page=ualp_db_integration' ) );
			}

			exit();
		}
	}
}

if ( ! function_exists( 'ulap_get_admin_pwd' ) ) {
	/**
	 * Get current admin password.
	 */
	function ulap_get_admin_pwd() {
		$current_user = wp_get_current_user();
		$admin_pass   = $current_user->user_pass;
		?>
		<script type="text/javascript">
			var adminPwd = '<?php echo esc_html( $admin_pass ); ?>';
			var currentpwd = '<?php echo esc_html( get_option( 'ualp_delete_log_pwd' ) ); ?>';
		</script>
		<?php
	}
}

if ( ! function_exists( 'ualp_send_email_notification' ) ) {
	/**
	 * Send email when selected user login.
	 *
	 * @param string $hook Hook.
	 * @param string $login Current username when login.
	 * @param string $activity Activity.
	 */
	function ualp_send_email_notification( $hook, $login, $activity ) {
		global $wpdb;
		$get_ip_type = get_option( 'ualpIPtypes', 'REMOTE_ADDR' );
		if ( $get_ip_type ) {
			if ( 'REMOTE_ADDR' == $get_ip_type ) {
				$ips = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
			} elseif ( 'HTTP_X_FORWARDED_FOR' == $get_ip_type ) {
				$ips = isset( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) : '';
			} elseif ( 'HTTP_X_REAL_IP' == $get_ip_type ) {
				$ips = isset( $_SERVER['HTTP_X_REAL_IP'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_REAL_IP'] ) ) : '';
			} else {
				$ips = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
			}
		} else {
			$ips = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
		}
		if ( ualp_allow_ip() ) {
			$ip = $ips;
		}
		$currenr_role              = array();
		$exclude_current_user_name = '';
		if ( '' != $login ) {
			$exclude_current_user      = get_user_by( 'login', $login );
			$exclude_current_user_name = $login;
			if ( ! empty( $exclude_current_user->roles ) && is_array( $exclude_current_user->roles ) ) {
				foreach ( $exclude_current_user->roles as $role ) {
						$currenr_role[] = $role;
				}
			}
		}

			$exclude_logdata = get_option( 'exclude_log_data' );
			$i               = 0;
			$flag            = 0;
		if ( is_array( $exclude_logdata ) && ! empty( $exclude_logdata ) ) {
			$users1 = get_users();
			foreach ( $users1 as $user_id ) {
				$all_user_id[] = $user_id->user_login;
			}
			foreach ( $exclude_logdata as $exclude_log_data_single ) {
				$row_count         = count( $exclude_log_data_single );
				$exclude_logs_hook = $exclude_log_data_single['exclude_logs_hook'];
				$exclude_log_users = $exclude_log_data_single['exclude_log_users'];
				$exclude_logs_ips  = $exclude_log_data_single['exclude_logs_ips'];
				if ( empty( $exclude_logs_ips ) ) {
					$exclude_logs_ips = array();}
				if ( '' != $exclude_log_users ) {
					if ( in_array( $exclude_log_users, $all_user_id ) ) {
						$exclude_log_user     = get_user_by( 'login', $exclude_log_users );
						$exclude_user         = $exclude_log_user->user_login;
						$exclude_current_user = $exclude_current_user_name;
					} elseif ( '0' == $exclude_log_users ) {
						$exclude_user = 'guest';
						if ( '' != $login ) {
							$exclude_current_user = $exclude_current_user_name;
						} else {
							$exclude_current_user = 'guest';
						}
					} else {
						$exclude_current_user = isset( $currenr_role[0] ) ? $currenr_role[0] : '';
						$exclude_user         = $exclude_log_users;
					}
				}
				if ( 'all' != $exclude_log_users && 'all' != $exclude_logs_hook ) {
					if ( 1 == $row_count && 0 == $i ) {
						if ( ! empty( $exclude_logs_ips ) && $hook == $exclude_logs_hook && $exclude_current_user == $exclude_user && in_array( $ip, $exclude_logs_ips ) ) {
							$flag = 1;
						} elseif ( empty( $exclude_logs_ips ) && $hook == $exclude_logs_hook && $exclude_current_user == $exclude_user ) {
							$flag = 1;
						}
					} elseif ( $row_count > 1 && 1 != $flag ) {
						if ( ! empty( $exclude_logs_ips ) && $hook == $exclude_logs_hook && $exclude_current_user == $exclude_user && in_array( $ip, $exclude_logs_ips ) ) {
							$flag = 1;
						} elseif ( empty( $exclude_logs_ips ) && $hook == $exclude_logs_hook && $exclude_current_user == $exclude_user ) {
							$flag = 1;
						}
					}
				} elseif ( 'all' == $exclude_log_users && 'all' != $exclude_logs_hook ) {
					if ( 1 == $row_count && 0 == $i ) {
						if ( ! empty( $exclude_logs_ips ) && $hook == $exclude_logs_hook && in_array( $ip, $exclude_logs_ips ) ) {
							$flag = 1;
						} elseif ( empty( $exclude_logs_ips ) && $hook == $exclude_logs_hook ) {
							$flag = 1;
						}
					} elseif ( $row_count > 1 && 1 != $flag ) {
						if ( ! empty( $exclude_logs_ips ) && $hook == $exclude_logs_hook && in_array( $ip, $exclude_logs_ips ) ) {
							$flag = 1;
						} elseif ( empty( $exclude_logs_ips ) && $hook == $exclude_logs_hook ) {
							$flag = 1;
						}
					}
				} elseif ( 'all' != $exclude_log_users && 'all' == $exclude_logs_hook ) {
					if ( 1 == $row_count && 0 == $i ) {
						if ( ! empty( $exclude_logs_ips ) && $exclude_current_user == $exclude_user && in_array( $ip, $exclude_logs_ips ) ) {
							$flag = 1;
						} elseif ( empty( $exclude_logs_ips ) && $exclude_current_user == $exclude_user ) {
							$flag = 1;
						}
					} elseif ( $row_count > 1 && 1 != $flag ) {
						if ( ! empty( $exclude_logs_ips ) && $exclude_current_user == $exclude_user && in_array( $ip, $exclude_logs_ips ) ) {
							$flag = 1;
						} elseif ( empty( $exclude_logs_ips ) && $exclude_current_user == $exclude_user ) {
							$flag = 1;
						}
					}
				} elseif ( 'all' == $exclude_log_users && 'all' == $exclude_logs_hook ) {
					if ( ! empty( $exclude_logs_ips ) && 1 != $flag && in_array( $ip, $exclude_logs_ips ) ) {
						$flag = 1;
					} elseif ( empty( $exclude_logs_ips ) ) {
						$flag = 1;
					}
				}
				$i++;
			}
		}
		if ( '0' == $flag ) {
			$user_role = '';
			if ( '' != $login ) {
				$current_user1 = get_user_by( 'login', $login );
				$current_user  = ! empty( $current_user1->user_login ) ? $current_user1->user_login : 'Guest';
				if ( ! empty( $current_user1->roles ) && is_array( $current_user1->roles ) ) {
					if ( count( $current_user1->roles ) == 1 ) {
						$user_role = $current_user1->roles[0];
					} else {
						$user_role = $current_user1->roles;
						foreach ( $user_role as $all_rols ) {
								$user_role .= $all_rols . ' ,';
						}
					}
				}
					$firstname    = ucfirst( $current_user1->user_firstname );
					$lastname     = ucfirst( $current_user1->user_lastname );
					$user_firstnm = ! empty( $firstname ) ? ucfirst( $firstname ) : '-';
					$user_lastnm  = ! empty( $lastname ) ? $lastname : '-';
					$user_email   = ! empty( $current_user1->user_email ) ? $current_user1->user_email : '-';
					ualp_email_body( $current_user, $user_firstnm, $user_lastnm, $user_email, $activity );
					$ualp_user_email_notify        = get_option( 'ualpEmailNotification' );
					$ualp_fail_login_not_user_name = array();
				if ( is_array( $ualp_user_email_notify ) && in_array( 'ualpFailLoginNotUser', $ualp_user_email_notify ) ) {
					if ( isset( $_REQUEST['log'] ) && '' != $_REQUEST['log'] ) {
						$current_user  = sanitize_text_field( wp_unslash( $_REQUEST['log'] ) );
						$get_all_users = get_users();
						foreach ( $get_all_users as $get_all_user ) {
							$ualp_fail_login_not_user_name[] = $get_all_user->user_login;
						}
						if ( is_array( $ualp_fail_login_not_user_name ) && ! in_array( $current_user, $ualp_fail_login_not_user_name ) ) {
							ualp_email_body( $login, '', '', '', $activity );
						}
					}
				}
			} else {
				ualp_email_body( 'Guest', '', '', '', $activity );
			}
		}
	}
}

if ( ! function_exists( 'ualp_email_body' ) ) {
	/**
	 * Sent email content.
	 *
	 * @param string $current_user Current User.
	 * @param string $user_firstnm First name of user.
	 * @param string $user_lastnm Last name of user.
	 * @param string $user_email User Email.
	 * @param string $activity Activity.
	 */
	function ualp_email_body( $current_user, $user_firstnm, $user_lastnm, $user_email, $activity ) {
		if ( ualp_allow_ip() ) {
			$get_ip_type = get_option( 'ualpIPtypes', 'REMOTE_ADDR' );
			if ( $get_ip_type ) {
				if ( 'REMOTE_ADDR' == $get_ip_type ) {
					$ip = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
				} elseif ( 'HTTP_X_FORWARDED_FOR' == $get_ip_type ) {
					$ip = isset( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) : '';
				} elseif ( 'HTTP_X_REAL_IP' == $get_ip_type ) {
					$ip = isset( $_SERVER['HTTP_X_REAL_IP'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_REAL_IP'] ) ) : '';
				} else {
					$ip = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
				}
			} else {
				$ip = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
			}
		} else {
			$ip = '';
		}
		$modified_date = current_time( 'mysql' );
		$modified_date = strtotime( $modified_date );

		$date_format = get_option( 'date_format' );
		$time_format = get_option( 'time_format' );

		$date = gmdate( $date_format, $modified_date );
		$time = gmdate( $time_format, $modified_date );

		$user_reg = $date . ' ' . $time;
		ob_start();
		?>
		<table cellspacing='0' style='margin-top:30px;border: 1px solid #cccccc;border-collapse: collapse;'>
			<tr>
				<th style='padding:5px 10px;border-bottom: 1px solid #cccccc;border-right: 1px solid #cccccc;'><?php echo esc_html__( 'Username', 'user_activity_log_pro' ); ?></th>
				<th style='padding:5px 10px;border-bottom: 1px solid #cccccc;border-right: 1px solid #cccccc;'><?php echo esc_html__( 'Firstname', 'user_activity_log_pro' ); ?></th>
				<th style='padding:5px 10px;border-bottom: 1px solid #cccccc;border-right: 1px solid #cccccc;'><?php echo esc_html__( 'Lastname', 'user_activity_log_pro' ); ?></th>
				<th style='padding:5px 10px;border-bottom: 1px solid #cccccc;border-right: 1px solid #cccccc;'><?php echo esc_html__( 'E-mail', 'user_activity_log_pro' ); ?></th>
				<th style='padding:5px 10px;border-bottom: 1px solid #cccccc;border-right: 1px solid #cccccc;'><?php echo esc_html__( 'Date Time', 'user_activity_log_pro' ); ?></th>
				<?php if ( ualp_allow_ip() ) { ?>
				<th style='padding:5px 10px;border-bottom: 1px solid #cccccc;border-right: 1px solid #cccccc;'><?php echo esc_html__( 'IP Address', 'user_activity_log_pro' ); ?></th>
				<?php } ?>
				<th style='padding:5px 10px;border-bottom: 1px solid #cccccc;'><?php echo esc_html__( 'Activity', 'user_activity_log_pro' ); ?></th>
			</tr>
			<tr>
				<td style='padding:5px 10px;border-right: 1px solid #cccccc;'><?php echo esc_html( $current_user ); ?></td>
				<td style='padding:5px 10px;border-right: 1px solid #cccccc;'><?php echo esc_html( $user_firstnm ); ?></td>
				<td style='padding:5px 10px;border-right: 1px solid #cccccc;'><?php echo esc_html( $user_lastnm ); ?></td>
				<td style='padding:5px 10px;border-right: 1px solid #cccccc;'><?php echo esc_html( $user_email ); ?></td>
				<td style='padding:5px 10px;border-right: 1px solid #cccccc;'><?php echo esc_html( $user_reg ); ?></td>
				<?php if ( ualp_allow_ip() ) { ?>
				<td style='padding:5px 10px;border-right: 1px solid #cccccc;'><?php echo esc_html( $ip ); ?></td>
				<?php } ?>
				<td style='padding:5px 10px;border-right: 1px solid #cccccc;'><?php echo esc_html( $activity ); ?></td>
			</tr>
		</table>
		<br/>
		<br/>
		<?php
		$user_details                      = ob_get_clean();
		$from_email                        = get_option( 'ualpFromEmail' );
		$to_email                          = $user_email;
		$mail_msg                          = get_option( 'ualpEmailMessage' );
		$ualp_to_faild_login_email         = get_option( 'ualpToFaildLoginEmail' );
		$ualp_to_fail_login_not_user_email = get_option( 'ualpFailLoginNotUserEmail' );
		$ualp_fail_login_not_user_number   = get_option( 'ualpFailLoginNotUserNumber' );
		$ualp404_login_email               = get_option( 'ualp404LoginEmail' );
		$ualp404_page_access_number        = get_option( 'ualp404PageAccessNumber' );
		$ualp_fail_login_number            = get_option( 'ualpFailLoginNumber' );

		if ( empty( $ualp404_login_email ) ) {
			$ualp404_login_email = $to_email;
		}
		if ( empty( $ualp_to_faild_login_email ) ) {
			$ualp_to_faild_login_email = $to_email;
		}
		if ( empty( $ualp_to_fail_login_not_user_email ) ) {
			$ualp_to_fail_login_not_user_email = $to_email;
		}
		global  $wpdb;
		$get_all_users                 = get_users();
		$ualp_fail_login_not_user_name = array();
		$count404page                  = $wpdb->get_var( $wpdb->prepare( 'SELECT COUNT(`uactid`) FROM %s where hook="wp_head" AND object_type="404 Page"', UALP_TABLENAME ) );
		if ( $count404page >= $ualp404_page_access_number ) {
			$to_email = $ualp404_login_email;
		}

		if ( isset( $_REQUEST['log'] ) && '' != $_REQUEST['log'] ) {
			foreach ( $get_all_users as $get_all_user ) {
				$ualp_fail_login_not_user_name[] = $get_all_user->user_login;
			}
			if ( is_array( $ualp_fail_login_not_user_name ) && ! in_array( $_REQUEST['log'], $ualp_fail_login_not_user_name ) ) {
				$countwploginfailed = $wpdb->get_var( $wpdb->prepare( 'SELECT COUNT(`uactid`) FROM %s where hook="wp_login_failed" AND post_title="$s"', UALP_TABLENAME, sanitize_text_field( wp_unslash( $_REQUEST['log'] ) ) ) );
				if ( $countwploginfailed >= $ualp_fail_login_not_user_number ) {
					$to_email = $ualp_to_fail_login_not_user_email;
				}
			} if ( is_array( $ualp_fail_login_not_user_name ) && in_array( $_REQUEST['log'], $ualp_fail_login_not_user_name ) ) {
				$post_title         = $current_user;
				$countwploginfailed = $wpdb->get_var( $wpdb->prepare( 'SELECT COUNT(`uactid`) FROM %s where hook="wp_login_failed" AND user_name=%s', UALP_TABLENAME, $post_title ) );
				if ( $countwploginfailed >= $ualp_fail_login_number ) {
					$to_email = $ualp_to_faild_login_email;
				}
			}
		}

		$mail_msg = str_replace( '[user_details]', $user_details, $mail_msg );
		if ( isset( $to_email ) && '' != $to_email && '' != $from_email && '' != $mail_msg ) {
			$headers              = 'From: ' . wp_strip_all_tags( $from_email ) . "\r\n";
			$headers             .= 'Reply-To: ' . wp_strip_all_tags( $from_email ) . "\r\n";
			$headers             .= "MIME-Version: 1.0\r\n";
			$headers             .= "Content-Type: text/html; charset=ISO-8859-1\r\n";
			$to_email_array       = explode( ',', $to_email );
			$to_email_array_count = count( $to_email_array );
			for ( $i = 0; $i < $to_email_array_count; $i++ ) {
				$to_mail = trim( $to_email_array[ $i ], ' ' );
				wp_mail( $to_mail, esc_html__( 'User activity Notification', 'user_activity_log_pro' ), $mail_msg, $headers );
			}
		}
	}
}

if ( ! function_exists( 'ualp_set_role_manager' ) ) {
	/**
	 * Set default role manager.
	 */
	function ualp_set_role_manager() {
		$ualp_role_access_list  = get_option( 'ualpRoleAccessList' );
		$ualp_role_access_list1 = array( 0 => 'administrator' );
		if ( empty( $ualp_role_access_list ) ) {
			$ualp_role_access_list = array(
				0 => 'administrator',
				1 => 'editor',
				2 => 'author',
				3 => 'contributor',
				4 => 'subscriber',
			);
		}
		if ( ! in_array( 'administrator', $ualp_role_access_list ) ) {
			$ualp_role_access_list = array_merge( $ualp_role_access_list, $ualp_role_access_list1 );
		}
		update_option( 'ualpRoleAccessList', $ualp_role_access_list );
	}
}

if ( ! function_exists( 'ualp_set_default_log_hook' ) ) {
	/**
	 * Set default log hook.
	 */
	function ualp_set_default_log_hook() {
		global $wpdb;
		$post_types           = ualp_posttypes();
		$taxonomies           = ualp_taxonomies();
		$enable_log_hook_list = get_option( 'enableLogHookList' );
		if ( ! $enable_log_hook_list ) {
			$log_hook       = array();
			$post_type_hook = array();
			$taxonomy_hook  = array();
			foreach ( $post_types as $post_type ) {
				if ( is_multisite() ) {
					$post_type_hook[] = $post_type->name;
				}
				if ( 'attachment' != $post_type->name ) {
					array_push( $post_type_hook, $post_type->name . '_create' );
					array_push( $post_type_hook, $post_type->name . '_update' );
					array_push( $post_type_hook, $post_type->name . '_delete' );
				}
				if ( 'attachment' != $post_type->name ) {
					array_push( $post_type_hook, 'add_' . $post_type->name );
					array_push( $post_type_hook, 'edit_' . $post_type->name );
					array_push( $post_type_hook, 'delete_' . $post_type->name );
				}
			}
			$log_hook = array_merge( $log_hook, $post_type_hook );
			foreach ( $taxonomies as $taxonomy ) {
				if ( is_multisite() ) {
					$taxonomy_hook[] = $taxonomy->name;
				}
				if ( 'wpcf7_contact_form' != $taxonomy->name && 'custom_css' != $taxonomy->name && 'customize_changeset' != $taxonomy->name && 'product_visibility' != $taxonomy->name ) {
					array_push( $taxonomy_hook, $taxonomy->name . '_create' );
					array_push( $taxonomy_hook, $taxonomy->name . '_update' );
					array_push( $taxonomy_hook, $taxonomy->name . '_delete' );
				}
			}
			$log_hook              = array_merge( $log_hook, $taxonomy_hook );
			$plugin_hook           = array( 'activated_plugin', 'deactivated_plugin', 'delete_plugin', 'plugin_modify', 'plugin_updated', 'plugin_installed' );
			$log_hook              = array_merge( $plugin_hook, $log_hook );
			$contact_hook          = array( 'wpcf7_after_create', 'wpcf7_after_update', 'wpcf7_admin_notices', 'wpcf7-recaptcha-setup' );
			$log_hook              = array_merge( $contact_hook, $log_hook );
			$ualp_hook             = array( 'ualp_export_activity_log', 'ualp_general_settings_updated', 'ualp_hook_log_settings_updated', 'ualp_manage_role', 'ualp_set_password', 'ualp_change_password', 'ualp_email_settings_updated' );
			$log_hook              = array_merge( $ualp_hook, $log_hook );
			$user_hook             = array( 'wp_login', 'wp_login_failed', 'wp_logout', 'delete_user', 'user_register', 'insert_user_meta' );
			$log_hook              = array_merge( $user_hook, $log_hook );
			$theme_hook            = array( 'theme_file_modify', 'switch_theme', 'customize_save', 'theme_updated', 'theme_installed', 'delete_site_transient_update_themes' );
			$log_hook              = array_merge( $theme_hook, $log_hook );
			$comment_hook          = array( 'wp_insert_comment', 'edit_comment', 'trash_comment', 'spam_comment', 'restore_comment', 'delete_comment', 'comment_unapprove', 'comment_approve' );
			$log_hook              = array_merge( $comment_hook, $log_hook );
			$menuview_hook         = array( 'wp_create_nav_menu', 'wp_update_nav_menu', 'delete_nav_menu' );
			$log_hook              = array_merge( $menuview_hook, $log_hook );
			$other_hook            = array( '_core_updated_successfully', 'import_end', 'export_wp', 'widget_update_callback', 'updated_option', 'woocommerce_updated_option', 'dbdelta_queries', 'wp_privacy_personal_data_exporters', 'wp_privacy_personal_data_erasers' );
			$log_hook              = array_merge( $other_hook, $log_hook );
			$supported_plugin_hook = array( 'woocommerce_new_product_variation', 'wp_ajax_woocommerce_remove_variations', 'woocommerce_save_product_variation', 'groups_create_group', 'groups_update_group', 'groups_before_delete_group', 'groups_leave_group', 'groups_join_group', 'groups_promote_member', 'groups_demote_member', 'groups_ban_member', 'groups_unban_member', 'groups_remove_member', 'xprofile_field_after_save', 'xprofile_fields_deleted_field', 'xprofile_group_after_save', 'xprofile_groups_deleted_group', 'wordfence_updateIssueStatus', 'wordfence_saveWAFConfig', 'wordfence_wafConfigureAutoPrepend', 'wordfence_whitelistBulkEnable', 'wordfence_whitelistBulkDisable', 'wordfence_whitelistBulkDelete', 'wordfence_updateConfig', 'wordfence_loadLiveTraffic', 'wordfence_loadCacheExclusions', 'wordfence_saveCacheConfig', 'wordfence_cache_clear', 'wordfence_reverseLookup', 'wordfence_whois', 'wordfence_clearPageCache', 'wordfence_saveConfig', 'wordfence_blockIP', 'wordfence_addTwoFactor', 'wordfence_exportSettings', 'wordfence_clearAllBlocked', 'wordfence_blockIPUARange', 'wordfence_unblockRange', 'wordfence_saveCountryBlocking', 'wordfence_saveDebuggingConfig', 'wordfence_startPasswdAudit', 'wordfence_saveScanSchedule', 'wordfence_updateWAFRules', 'wordfence_sendTestEmail', 'wordfence_sendDiagnostic', 'wordfence_updateAllIssues', 'wordfence_sendActivityLog', 'wordfence_addCacheExclusion', 'wordfence_daily_autoUpdate', 'wordfence_update_blocked_IPs', 'wordfence_start_scheduled_scan', 'wordfence_doScan', 'wordfence_daily_cron', 'wordfence_hourly_cron', 'wordfence_email_activity_report', 'wordfence_scan', 'wordfence_updateAlertEmail', 'wordfence_restoreFile', 'wordfence_deletePasswdAudit', 'wordfence_weakPasswordsFix', 'wordfence_killPasswdAudit', 'wordfence_importSettings', 'wordfence_bulkOperation', 'wordfence_deleteFile', 'wordfence_deleteDatabaseOption', 'wordfence_removeExclusion', 'wordfence_deleteIssue', 'wordfence_unlockOutIP', 'wordfence_loadBlockRanges', 'wordfence_unblockIP', 'wordfence_permBlockIP', 'wordfence_downloadHtaccess', 'wordfence_checkFalconHtaccess', 'wordfence_removeFromCache', 'wordfence_autoUpdateChoice', 'wordfence_saveCacheOptions', 'wordfence_adminEmailChoice', 'wordfence_getCacheStats', 'wordfence_killScan', 'wordfence_tourClosed', 'wordfence_removeCacheExclusion', 'wordfence_unblockNetwork', 'wordfence_permanentlyBlockAllIPs', 'wordfence_deleteAdminUser', 'wordfence_revokeAdminUser', 'wordfence_hideFileHtaccess', 'wpseo_handle_import', 'wpseo_import', 'wpseo_save_auth_code', 'seo_check_admin_referer', 'updated_post_meta', 'bdp_update_shortcode', 'bdp_reset_shortcode', 'bdp_reset_archive_page', 'bdp_add_shortcode', 'bdp_update_single_file', 'bdp_add_archive_layout', 'bdp_delete_shortcode', 'bdp_delete_single_template', 'bdp_copy_single_template', 'bdp_duplicate_layout_settings', 'bdp_import_blog_layout_settings', 'bdp_import_archive_layout_settings', 'bdp_export_archive_layout_settings', 'bdp_export_blog_layout_settings', 'bdp_update_archive_page', 'bdp_import_single_layout_settings', 'bdp_export_single_layout_settings', 'bdp_delete_archive', 'bdp_delete_single_layout', 'bdp_update_single_page', 'bdp_reset_single_page', 'bdp_add_single_layout', 'bdp_duplicate_single_layout_settings', 'import_sample_portfolio_data', 'create_portfolio_layout', 'duplicate_portfolio_layout', 'delete_portfolio_layout', 'update_portfolio_layout', 'reset_portfolio_layout', 'import_portfolio_layout', 'export_portfolio_layout', 'create_post_type', 'update_post_type', 'delete_post_type', 'autoptimizer_settings', 'wp_ajax_autoptimize_delete_cache', 'switch_to_user', 'switch_back_user', 'switch_off_user', 'reset_reservation_settings', 'rest_update_reservation_status', 'save_reservation_settings', 'edd_activate_discount', 'edd_deactivate_discount', 'edd_add_discount', 'edd_delete_discount', 'transition_post_status', 'gf_export_forms', 'notification_duplicated', 'gform_entry_created', 'gf_resend_notifications', 'gform_pre_note_deleted', 'gravityforms_updated_option', 'gform_post_export_entries', 'gform_post_note_added', 'gform_delete_entries', 'gform_post_form_views_deleted', 'gform_post_form_activated', 'gform_post_form_deactivated', 'gform_before_delete_form', 'gform_post_form_trashed', 'gform_post_form_restored', 'gform_post_form_duplicated', 'gform_after_save_form', 'gform_forms_post_import', 'gform_pre_notification_deleted', 'gform_pre_confirmation_deleted', 'wp_ajax_rg_update_notification_active', 'wp_ajax_rg_update_confirmation_active', 'gform_pre_confirmation_save', 'gform_pre_notification_save', 'gform_delete_lead', 'gform_update_status', 'gform_update_is_read', 'gform_update_is_starred', 'wp_ajax_add_new_custom_profile_field', 'wp_ajax_save_fields_settings', 'wp_ajax_delete_profile_field', 'wp_ajax_sort_fileds_list', 'wp_ajax_custom_fields_reset', 'wp_ajax_create_uploader_folder', 'user_ultra_settings', 'wp_ajax_package_add_new', 'wp_ajax_package_delete', 'wp_ajax_package_edit_confirm', 'wp_ajax_sync_users', 'wp_ajax_user_resend_activation_link', 'wp_ajax_delete_uultra_transtient', 'wp_ajax_delete_uultra_stats', 'wp_ajax_delete_uultra_reviews', 'wp_ajax_edit_photo_cate_conf', 'wp_ajax_edit_photo_cate_del', 'wp_ajax_edit_photo_cate', 'userultra_admin_menu_hook', 'user_ultra_settings', 'user_ultra_settings', 'user_ultra_settings', 'save_wcfe_settings', 'wp_ajax_wcfe_get_checkout_required_fields' );
			$log_hook              = array_merge( $supported_plugin_hook, $log_hook );
			update_option( 'enableLogHookList', $log_hook );
		}

	}
}

if ( ! function_exists( 'ualp_ajax_favorite_log' ) ) {
	/**
	 * Function to handle AJAX request.
	 *
	 * @since    1.0.0
	 */
	function ualp_ajax_favorite_log() {
		global $wpdb, $externaldb;
		if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'ajax-nonce' ) ) {
			$featured = isset( $_POST['featured'] ) ? sanitize_text_field( wp_unslash( $_POST['featured'] ) ) : '';
			$id       = isset( $_POST['post'] ) ? (int) $_POST['post'] : '';
			if ( isset( $id ) && ! empty( $id ) && null !== $featured ) {
				if ( 'no' == $featured ) {
					$result = $externaldb->update(
						UALP_TABLENAME,
						array( 'favorite' => '0' ),
						array( 'uactid' => $id ),
						array( '%s' ),
						array( '%d' )
					);
				} else {
					$result = $externaldb->update(
						UALP_TABLENAME,
						array( 'favorite' => '1' ),
						array( 'uactid' => $id ),
						array( '%s' ),
						array( '%d' )
					);
				}
			}
			exit;
		}
	}
}


if ( ! function_exists( 'ualp_view_update_detail' ) ) {
	/**
	 * View detailed log.
	 */
	function ualp_view_update_detail() {
		global $wpdb,$externaldb;
		if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'ajax-nonce' ) ) {
			$uactid            = isset( $_POST['logID'] ) ? sanitize_text_field( wp_unslash( $_POST['logID'] ) ) : '';
			$uact_log_activity = $externaldb->get_results( 'SELECT * FROM ' . UALP_TABLENAME . ' WHERE uactid = ' . $uactid );
			$uact_details      = $externaldb->get_results( 'SELECT uactd_key,uactd_value FROM ' . UALP_DETAIL_TABLENAME . ' WHERE uactid = ' . $uactid );
			foreach ( $uact_log_activity as $k => $v ) {
				$object_type = $v->object_type;
				$post_id     = $v->post_id;
				$action      = $v->action;
				$user_id     = $v->user_id;
				$date        = $v->modified_date;
				$username    = $v->user_name;
				$user_role   = $v->user_role;
				$useremail   = $v->user_email;
				$ipaddress   = $v->ip_address;
				$country     = ( isset( $v->country ) && '' != $v->country ) ? $v->country : '';
				$user_agent  = ( isset( $v->user_agent ) && '' != $v->user_agent ) ? $v->user_agent : '';
				$detail      = $v->description;
			}
			$post_type              = get_post_type( $post_id );
			$user_primary_key_array = ualp_get_user_primary_data( $user_id );
			$user_meta_key_array    = ualp_get_user_meta_data( $user_id );
			$user_full_key_array    = array_merge( $user_primary_key_array, $user_meta_key_array );
			$post_primary_key_array = ualp_get_primary_post_data( $object_type );
			$post_meta_key_array    = ualp_get_post_meta_data( $object_type );
			$post_full_key_array    = array_merge( $post_primary_key_array, $post_meta_key_array );
			$get_term_key_array     = array( 'term_name', 'term_slug', 'term_description', 'term_parent' );

			$customizer_activities = array();
			$customizer_activities = $externaldb->get_results( 'SELECT uactd_key,uactd_value FROM ' . UALP_DETAIL_TABLENAME . ' WHERE uactid = ' . $uactid, ARRAY_A );

			$post_types = ualp_posttypes();
			?>
		<div class="ualp_view_details_wrap" title="<?php esc_attr_e( 'Log Details', 'user_activity_log_pro' ); ?>">
			<div class="ualp_view_user_details">
				<table class="striped widefat">
					<tr>
						<td colspan="2" class="ulap_view_title">
							<?php esc_html_e( 'Activity Detail', 'user_activity_log_pro' ); ?>
						</td>
					</tr>
					<tr>
						<td><?php esc_html_e( 'Action ID', 'user_activity_log_pro' ); ?></td>
						<td>
							<?php echo esc_html( $uactid ); ?>
						</td>
					</tr>
					<tr>
						<td><?php esc_html_e( 'Message', 'user_activity_log_pro' ); ?></td>
						<td>
							<?php
							if ( 'shop_order' != $post_type ) {
								echo wp_kses( $detail, ualp_args_kses() ); } elseif ( is_plugin_active( 'woocommerce/woocommerce.php' ) && 'shop_order' == $post_type ) {
								echo esc_html_e( 'Order', 'user_activity_log_pro' ) . ' #' . esc_html( $post_id ) . ' ' . esc_html_e( ' Details', 'user_activity_log_pro' ) . ' <a href="' . esc_url( admin_url( 'post.php?post=' . absint( $post_id ) . '&action=edit' ) ) . '" >View order</a>';
								$order = new WC_Order( $post_id );
								$items = $order->get_items();
								if ( ! empty( $items ) ) {
									?>
									<table class="striped widefat ualp_shop_order">
										<tr>
											<th><?php esc_html_e( 'ID', 'user_activity_log_pro' ); ?></th>
											<th><?php esc_html_e( 'Name', 'user_activity_log_pro' ); ?></th>
											<th><?php esc_html_e( 'SKU', 'user_activity_log_pro' ); ?></th>
											<th><?php esc_html_e( 'Price', 'user_activity_log_pro' ); ?></th>
										</tr>
									<?php
									foreach ( $items as $item ) {
										$product_name         = $item['name'];
										$product_id           = $item['product_id'];
										$subtotal             = $item['subtotal'];
										$product_variation_id = $item['variation_id'];
										// Check if product has variation.
										if ( $product_variation_id ) {
											$product = new WC_Product( $item['variation_id'] );
										} else {
											$product = new WC_Product( $item['product_id'] );
										}

										// Get SKU.
										$sku = $product->get_sku();
										echo '<tr>';
										echo '<td>' . esc_html( $product_id ) . '</td>';
										echo '<td>' . esc_html( $product_name ) . '</td>';
										echo '<td>' . esc_html( $sku ) . '</td>';
										echo '<td>' . esc_html( $subtotal ) . '</td>';
										echo '</tr>';
									}
									?>
									</table>
									<?php
								}
								}
								?>
						</td>
					</tr>
					<?php
					if ( 'Woocommerce' == $object_type && 'product variation deleted' == $action ) {
						?>
						<tr>
							<td><?php esc_html_e( 'Product name', 'user_activity_log_pro' ); ?></td>
							<td>
								<span>
									<?php
										print_r( ualp_get_activity_log_detail( 'ualp_product_name', $uactid ) );
									?>
								</span>
							</td>
						</tr>
						<tr>
							<td><?php esc_html_e( 'Deleted variation data', 'user_activity_log_pro' ); ?></td>
							<td>
								<span>
									<?php
										$variation_data = ualp_get_activity_log_detail( 'ualp_variation_data', $uactid );
										$variation_data = maybe_unserialize( $variation_data );
										print_r( $variation_data );
									?>
								</span>
							</td>
						</tr>
						<?php
					}

					if ( 'Woocommerce' == $object_type && 'product variation saved' == $action ) {
						?>
						<tr>
							<td><?php esc_html_e( 'Product name', 'user_activity_log_pro' ); ?></td>
							<td>
								<span>
									<?php
										print_r( ualp_get_activity_log_detail( 'ualp_product_name', $uactid ) );
									?>
								</span>
							</td>
						</tr>
						<tr>
							<td><?php esc_html_e( 'Variation SKU', 'user_activity_log_pro' ); ?></td>
							<td>
								<?php
								if ( ! empty( ualp_get_activity_log_detail( 'ualp_variation_new_sku', $uactid ) ) && ualp_get_activity_log_detail( 'ualp_variation_sku', $uactid ) != ualp_get_activity_log_detail( 'ualp_variation_new_sku', $uactid ) ) {
									?>
									<del>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_variation_sku', $uactid ) );
										?>
									</del>
									<ins>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_variation_new_sku', $uactid ) );
										?>
									</ins>
								<?php } else { ?>
									<span>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_variation_sku', $uactid ) );
										?>
									</span>
								<?php } ?>
							</td>
						</tr>
						<tr>
							<td><?php esc_html_e( 'Variation regular price', 'user_activity_log_pro' ); ?></td>
							<td>
								<?php
								if ( ! empty( ualp_get_activity_log_detail( 'ualp_variation_new_regular_price', $uactid ) ) && ualp_get_activity_log_detail( 'ualp_variation_regular_price', $uactid ) != ualp_get_activity_log_detail( 'ualp_variation_new_regular_price', $uactid ) ) {
									?>
									<del>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_variation_regular_price', $uactid ) );
										?>
									</del>
									<ins>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_variation_new_regular_price', $uactid ) );
										?>
									</ins>
								<?php } else { ?>
									<span>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_variation_regular_price', $uactid ) );
										?>
									</span>
								<?php } ?>
							</td>
						</tr>
						<tr>
							<td><?php esc_html_e( 'Variation sale price', 'user_activity_log_pro' ); ?></td>
							<td>
								<?php
								if ( ! empty( ualp_get_activity_log_detail( 'ualp_variation_new_sale_price', $uactid ) ) && ualp_get_activity_log_detail( 'ualp_variation_sale_price', $uactid ) != ualp_get_activity_log_detail( 'ualp_variation_new_sale_price', $uactid ) ) {
									?>
									<del>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_variation_sale_price', $uactid ) );
										?>
									</del>
									<ins>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_variation_new_sale_price', $uactid ) );
										?>
									</ins>
								<?php } else { ?>
									<span>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_variation_sale_price', $uactid ) );
										?>
									</span>
								<?php } ?>
							</td>
						</tr>
						<tr>
							<td><?php esc_html_e( 'Virtual', 'user_activity_log_pro' ); ?></td>
							<td>
								<span>
									<?php
										print_r( ualp_get_activity_log_detail( 'ualp_virtual', $uactid ) );
									?>
								</span>
							</td>
						</tr>
						<tr>
							<td><?php esc_html_e( 'Downloadable', 'user_activity_log_pro' ); ?></td>
							<td>
								<span>
									<?php
										print_r( ualp_get_activity_log_detail( 'ualp_downloadable', $uactid ) );
									?>
								</span>
							</td>
						</tr>
						<tr>
							<td><?php esc_html_e( 'Weight', 'user_activity_log_pro' ); ?></td>
							<td>
								<?php
								if ( ! empty( ualp_get_activity_log_detail( 'ualp_new_weight', $uactid ) ) && ualp_get_activity_log_detail( 'ualp_weight', $uactid ) != ualp_get_activity_log_detail( 'ualp_new_weight', $uactid ) ) {
									?>
									<del>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_weight', $uactid ) );
										?>
									</del>
									<ins>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_new_weight', $uactid ) );
										?>
									</ins>
								<?php } else { ?>
									<span>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_weight', $uactid ) );
										?>
									</span>
								<?php } ?>
							</td>
						</tr>
						<tr>
							<td><?php esc_html_e( 'Length', 'user_activity_log_pro' ); ?></td>
							<td>
								<?php
								if ( ! empty( ualp_get_activity_log_detail( 'ualp_new_length', $uactid ) ) && ualp_get_activity_log_detail( 'ualp_length', $uactid ) != ualp_get_activity_log_detail( 'ualp_new_length', $uactid ) ) {
									?>
									<del>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_length', $uactid ) );
										?>
									</del>
									<ins>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_new_length', $uactid ) );
										?>
									</ins>
								<?php } else { ?>
									<span>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_length', $uactid ) );
										?>
									</span>
								<?php } ?>
							</td>
						</tr>
						<tr>
							<td><?php esc_html_e( 'Manage stock', 'user_activity_log_pro' ); ?></td>
							<td>
								<span>
									<?php
										print_r( ualp_get_activity_log_detail( 'ualp_manage_stock', $uactid ) );
									?>
								</span>
							</td>
						</tr>
						<tr>
							<td><?php esc_html_e( 'Backorders', 'user_activity_log_pro' ); ?></td>
							<td>
								<?php
								if ( ! empty( ualp_get_activity_log_detail( 'ualp_new_backorders', $uactid ) ) && ualp_get_activity_log_detail( 'ualp_backorders', $uactid ) != ualp_get_activity_log_detail( 'ualp_new_backorders', $uactid ) ) {
									?>
									<del>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_backorders', $uactid ) );
										?>
									</del>
									<ins>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_new_backorders', $uactid ) );
										?>
									</ins>
								<?php } else { ?>
									<span>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_backorders', $uactid ) );
										?>
									</span>
								<?php } ?>
							</td>
						</tr>
						<tr>
							<td><?php esc_html_e( 'Tax class', 'user_activity_log_pro' ); ?></td>
							<td>
								<span>
									<?php
										print_r( ualp_get_activity_log_detail( 'ualp_tax_class', $uactid ) );
									?>
								</span>
							</td>
						</tr>
						<tr>
							<td><?php esc_html_e( 'Download limit', 'user_activity_log_pro' ); ?></td>
							<td>
								<?php
								if ( ! empty( ualp_get_activity_log_detail( 'ualp_new_download_limit', $uactid ) ) && ualp_get_activity_log_detail( 'ualp_download_limit', $uactid ) != ualp_get_activity_log_detail( 'ualp_new_download_limit', $uactid ) ) {
									?>
									<del>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_download_limit', $uactid ) );
										?>
									</del>
									<ins>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_new_download_limit', $uactid ) );
										?>
									</ins>
								<?php } else { ?>
									<span>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_download_limit', $uactid ) );
										?>
									</span>
								<?php } ?>
							</td>
						</tr>
						<tr>
							<td><?php esc_html_e( 'Download expiry', 'user_activity_log_pro' ); ?></td>
							<td>
								<?php
								if ( ! empty( ualp_get_activity_log_detail( 'ualp_new_download_expiry', $uactid ) ) && ualp_get_activity_log_detail( 'ualp_download_expiry', $uactid ) != ualp_get_activity_log_detail( 'ualp_new_download_expiry', $uactid ) ) {
									?>
									<del>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_download_expiry', $uactid ) );
										?>
									</del>
									<ins>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_new_download_expiry', $uactid ) );
										?>
									</ins>
								<?php } else { ?>
									<span>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_download_expiry', $uactid ) );
										?>
									</span>
								<?php } ?>
							</td>
						</tr>
						<tr>
							<td><?php esc_html_e( 'Variation description', 'user_activity_log_pro' ); ?></td>
							<td>
								<?php
								if ( ! empty( ualp_get_activity_log_detail( 'ualp_new_variation_description', $uactid ) ) && ualp_get_activity_log_detail( 'ualp_variation_description', $uactid ) != ualp_get_activity_log_detail( 'ualp_new_variation_description', $uactid ) ) {
									?>
									<del>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_variation_description', $uactid ) );
										?>
									</del>
									<ins>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_new_variation_description', $uactid ) );
										?>
									</ins>
								<?php } else { ?>
									<span>
										<?php
											print_r( ualp_get_activity_log_detail( 'ualp_variation_description', $uactid ) );
										?>
									</span>
								<?php } ?>
							</td>
						</tr>
						<tr>
							<td><?php esc_html_e( 'Updated variation data', 'user_activity_log_pro' ); ?></td>
							<td>
								<span>
									<?php
										$variation_data = ualp_get_activity_log_detail( 'ualp_variation_data', $uactid );
										$variation_data = maybe_unserialize( $variation_data );
										print_r( $variation_data );
									?>
								</span>
							</td>
						</tr>
						<?php
					}

					if ( 'Theme' == $object_type && 'switched' == $action ) {
						?>
						<tr>
							<td><?php esc_html_e( 'Theme switched', 'user-activity-log-pro' ); ?></td>
							<td>
								<span>
									<del>
										<?php
										print_r( ualp_get_activity_log_detail( 'ualp_old_theme_name', $uactid ) );
										echo '(';
										print_r( ualp_get_activity_log_detail( 'ualp_old_theme_version', $uactid ) );
										echo ')';
										?>
									</del>
									<ins>
										<?php
										print_r( ualp_get_activity_log_detail( 'ualp_new_theme_name', $uactid ) );
										echo '(';
										print_r( ualp_get_activity_log_detail( 'ualp_new_theme_version', $uactid ) );
										echo ')';
										?>
									</ins>
								</span>
							</td>
						</tr>
						<?php
					}

					if ( 'Theme' == $object_type && 'updated' == $action ) {
						?>
						<tr>
							<td><?php esc_html_e( 'Theme Name', 'user_activity_log_pro' ); ?></td>
							<td>
								<span>
									<?php print_r( ualp_get_activity_log_detail( 'ualp_theme_name', $uactid ) ); ?>
								</span>
							</td>
						</tr>
						<tr>
							<td><?php esc_html_e( 'Theme Version', 'user_activity_log_pro' ); ?></td>
							<td>
								<span>
									<?php print_r( ualp_get_activity_log_detail( 'ualp_theme_version', $uactid ) ); ?>
								</span>
							</td>
						</tr>
						<?php
					}

					if ( 'Theme' == $object_type && 'customize updated' == $action ) {
						foreach ( $customizer_activities as $customizer_activity ) {
							$customizer_old_value = ualp_get_activity_log_detail( $customizer_activity['uactd_value'] . '_setting_old_value', $uactid );
							$customizer_new_value = ualp_get_activity_log_detail( $customizer_activity['uactd_value'] . '_setting_new_value', $uactid );
							if ( 'main_text_colorcontrol_id' == $customizer_activity['uactd_key'] ||
								'secondary_text_colorcontrol_id' == $customizer_activity['uactd_key'] ||
								'background_repeatcontrol_id' == $customizer_activity['uactd_key'] ||
								'background_position_xcontrol_id' == $customizer_activity['uactd_key'] ||
								'background_attachmentcontrol_id' == $customizer_activity['uactd_key'] ||
								'page_background_colorcontrol_id' == $customizer_activity['uactd_key'] ||
								'link_colorcontrol_id' == $customizer_activity['uactd_key'] ||
								'header_textcolorcontrol_id' == $customizer_activity['uactd_key'] ||
								'blognamecontrol_id' == $customizer_activity['uactd_key'] ||
								'blogdescriptioncontrol_id' == $customizer_activity['uactd_key'] ||
								'header_textcolor_setting_id' == $customizer_activity['uactd_key'] ||
								'color_scheme_setting_id' == $customizer_activity['uactd_key'] ||
								'sidebar_textcolor_setting_id' == $customizer_activity['uactd_key'] ||
								'header_background_color_setting_id' == $customizer_activity['uactd_key'] ||
								'nav_menu_locations[primary]control_id' == $customizer_activity['uactd_key'] ||
								'nav_menu_locations[social]control_id' == $customizer_activity['uactd_key'] ||
								'background_colorcontrol_id' == $customizer_activity['uactd_key']
							) {
								?>
								<tr>
									<td><?php echo esc_html( ualp_replace_key( $customizer_activity['uactd_value'] ) ); ?></td>
									<td>
										<del><?php print_r( $customizer_old_value ); ?></del>
										<span>
											<ins>
												<?php print_r( $customizer_new_value ); ?>
											</ins>
										</span>
									</td>
								</tr>
								<?php
							}
							$theme      = get_option( 'stylesheet' );
							$custom_css = 'custom_css[' . $theme . ']_setting_id';

							if ( $customizer_activity['uactd_key'] == $custom_css ) {
								?>
								<tr>
									<td><?php echo esc_html( ualp_replace_key( $customizer_activity['uactd_value'] ) ); ?></td>
									<td>
										<table class="diff">
											<colgroup><col class="content"></colgroup>
											<tbody>
												<tr>
													<td class="diff-deletedline">
														<?php
														if ( '' != $customizer_old_value ) {
															echo esc_html( $customizer_old_value );
														}
														?>
													</td>
													<td>&nbsp;</td>
													<td class="diff-addedline">
														<?php
														if ( '' != $customizer_new_value ) {
															echo esc_html( $customizer_new_value );
														}
														?>
													</td>
												</tr>
											</tbody>
										</table>
									</td>
								</tr>
								<?php
							}

							if ( 'page_for_posts_setting_id' == $customizer_activity['uactd_key'] || 'page_on_front_setting_id' == $customizer_activity['uactd_key'] ) {
								?>
								<tr>
									<td><?php echo esc_html( ualp_replace_key( $customizer_activity['uactd_value'] ) ); ?></td>
									<td>
										<del>
										<?php
										if ( '' != $customizer_old_value ) {
											echo esc_html( get_the_title( $customizer_old_value ) );
										}
										?>
										</del>
										<span>
											<ins>
												<?php
												if ( '' != $customizer_new_value ) {
													echo esc_html( get_the_title( $customizer_new_value ) );
												}
												?>
											</ins>
										</span>
									</td>
								</tr>
								<?php
							}

							if ( 'background_image_setting_id' == $customizer_activity['uactd_key'] || 'header_image_setting_id' == $customizer_activity['uactd_key'] ) {
								?>
								<tr>
									<td><?php echo esc_html( ualp_replace_key( $customizer_activity['uactd_value'] ) ); ?></td>
									<td>
									<?php
									if ( '' != $customizer_old_value ) {
										?>
											<del><img class="ualp-tmb-img" src="<?php echo esc_url( $customizer_old_value ); ?>" alt="<?php esc_attr_e( 'Old Background image', 'user_activity_log_pro' ); ?>"></del>
											<?php
									}
									?>
										<span>
										<?php
										if ( '' != $customizer_new_value ) {
											?>
												<ins><img class="ualp-tmb-img" src="<?php echo esc_url( $customizer_new_value ); ?>" alt="<?php esc_attr_e( 'New Background image', 'user_activity_log_pro' ); ?>"></ins>
												<?php
										}
										?>
										</span>
									</td>
								</tr>
								<?php
							}
							if ( 'custom_logocontrol_id' == $customizer_activity['uactd_key'] || 'site_icon_setting_id' == $customizer_activity['uactd_key'] ) {
								?>
								<tr>
									<td><?php echo esc_html( ualp_replace_key( $customizer_activity['uactd_value'] ) ); ?></td>
									<td>
									<?php
									if ( '' != $customizer_old_value ) {
										?>
											<del><img class="ualp-tmb-img" src="<?php echo esc_url( wp_get_attachment_thumb_url( $customizer_old_value ) ); ?>" alt="<?php esc_attr_e( 'Old custom logo', 'user_activity_log_pro' ); ?>"></del>
											<?php
									}
									?>
										<span>
										<?php
										if ( '' != $customizer_new_value ) {
											?>
												<ins><img class="ualp-tmb-img" src="<?php echo esc_url( wp_get_attachment_thumb_url( $customizer_new_value ) ); ?>" alt="<?php esc_attr_e( 'New custom logo', 'user_activity_log_pro' ); ?>"></ins>
												<?php
										}
										?>
										</span>
									</td>
								</tr>
								<?php
							}
						}
					}

					if ( 'Comment' == $object_type && ( 'edited' == $action || 'inserted' == $action ) ) {
						$comment_id = ualp_get_activity_log_detail( 'ualp_comment_id', $uactid );
						$comment    = get_comment( $comment_id );
						?>
						<tr>
							<td><?php esc_html_e( 'Comment ID', 'user_activity_log_pro' ); ?></td>
							<td>
								<span>
									<?php echo esc_html( $comment_id ); ?>
								</span>
							</td>
						</tr>
						<?php
						if ( isset( $comment->comment_author ) && '' != $comment->comment_author ) {
							?>
							<tr>
								<td><?php esc_html_e( 'Comment Author', 'user_activity_log_pro' ); ?></td>
								<td>
									<span>
										<?php print_r( $comment->comment_author ); ?>
									</span>
								</td>
							</tr>
							<?php
						}
						if ( isset( $comment->comment_author_email ) && '' != $comment->comment_author_email ) {
							?>
							<tr>
								<td><?php esc_html_e( 'Comment Author E-mail', 'user_activity_log_pro' ); ?></td>
								<td>
									<span>
										<?php print_r( $comment->comment_author_email ); ?>
									</span>
								</td>
							</tr>
							<?php
						}
						if ( isset( $comment->comment_author_url ) && '' != $comment->comment_author_url ) {
							?>
							<tr>
								<td><?php esc_html_e( 'Comment Author URL', 'user_activity_log_pro' ); ?></td>
								<td>
									<span>
										<?php print_r( $comment->comment_author_url ); ?>
									</span>
								</td>
							</tr>
							<?php
						}
						?>
						<tr>
							<td><?php esc_html_e( 'Comment Content', 'user_activity_log_pro' ); ?></td>
							<td>
								<span>
									<?php print_r( $comment->comment_content ); ?>
								</span>
							</td>
						</tr>
						<tr>
							<td><?php esc_html_e( 'Comment Date', 'user_activity_log_pro' ); ?></td>
							<td>
								<span>
									<?php print_r( $comment->comment_date ); ?>
								</span>
							</td>
						</tr>
						<tr>
							<td><?php esc_html_e( 'Comment Approved', 'user_activity_log_pro' ); ?></td>
							<td>
								<span>
									<?php print_r( $comment->comment_approved ); ?>
								</span>
							</td>
						</tr>
						<tr>
							<td><?php esc_html_e( 'Comment Post Title', 'user_activity_log_pro' ); ?></td>
							<td>
								<span>
									<?php print_r( get_the_title( $comment->comment_post_ID ) ); ?>
								</span>
							</td>
						</tr>
						<?php
					}

					if ( 'Widget' == $object_type && 'updated' == $action ) {
						$args_kses = ualp_args_kses();
						?>
						<tr>
							<td><?php esc_html_e( 'Sidebar Name', 'user_activity_log_pro' ); ?></td>
							<td>
								<span>
									<?php echo esc_html( ualp_get_activity_log_detail( 'ualp_sidebar_name_translated', $uactid ) ); ?>
								</span>
							</td>
						</tr>
						<tr>
							<td><?php esc_html_e( 'Widget Name', 'user_activity_log_pro' ); ?></td>
							<td>
								<span>
									<?php echo esc_html( ualp_get_activity_log_detail( 'ualp_widget_name_translated', $uactid ) ); ?>
								</span>
							</td>
						</tr>
						<tr>
							<td><?php esc_html_e( 'Old instance', 'user_activity_log_pro' ); ?></td>
							<td>
								<span>
									<?php echo wp_kses( ualp_get_activity_log_detail( 'ualp_old_instance', $uactid ), $args_kses ); ?>
								</span>
							</td>
						</tr>
						<tr>
							<td><?php esc_html_e( 'New instance', 'user_activity_log_pro' ); ?></td>
							<td>
								<span>
									<?php echo wp_kses( ualp_get_activity_log_detail( 'ualp_new_instance', $uactid ), $args_kses ); ?>
								</span>
							</td>
						</tr>
						<?php
					}

					if ( 'User' == $object_type && 'profile updated' == $action ) {
						foreach ( $user_full_key_array as $user_full_key_single ) {
							$old_user_meta_key = "ualp_user_old_$user_full_key_single";
							$new_user_meta_key = "ualp_user_new_$user_full_key_single";
							foreach ( $uact_details as $k => $v ) {
								if ( $old_user_meta_key == $v->uactd_key ) {
									$old_value = ualp_get_activity_log_detail( $old_user_meta_key, $uactid );
									$new_value = ualp_get_activity_log_detail( $new_user_meta_key, $uactid );
									?>
									<tr>
										<td>
											<label>
											<?php
												ualp_replace_key( $user_full_key_single );
											?>
												:
											</label>
										</td>
										<td class="user_updated">
											<?php
											if ( '' != $old_value ) {
												?>
												<del><?php echo esc_html( ualp_replace_special_array( $user_full_key_single, $old_value ) ); ?></del>
												<?php
											}
											?>
												<span>
												<ins><?php echo esc_html( ualp_replace_special_array( $user_full_key_single, $new_value ) ); ?></ins>
											</span>
										</td>
									</tr>
									<?php
								}
							}
						}
					}

					if ( 'Term' == $object_type && 'edited' == $action ) {
						foreach ( $get_term_key_array as $term_key_single ) {
							$old_term_meta_key = "ulap_old_$term_key_single";
							$new_term_meta_key = "ulap_new_$term_key_single";
							foreach ( $uact_details as $k => $v ) {
								if ( $old_term_meta_key == $v->uactd_key ) {
									$old_value = ualp_get_activity_log_detail( $old_term_meta_key, $uactid );
									$new_value = ualp_get_activity_log_detail( $new_term_meta_key, $uactid );
									if ( '' != $old_value && '' != $new_value ) {
										?>
										<tr>
											<td><?php ualp_replace_key( $term_key_single ); ?></td>
											<td>
												<?php
												if ( 'term_parent' == $term_key_single ) {

													if ( 0 == $old_value ) {
														$old_value = esc_html__( 'None', 'user_activity_log_pro' );
													} else {
														$old_value = ualp_get_term_by_id( $old_value );
														$old_value = $old_value['name'];
													}
													if ( 0 == $new_value ) {
														$new_value = esc_html__( 'None', 'user_activity_log_pro' );
													} else {
														$new_value = ualp_get_term_by_id( $new_value );
														$new_value = $new_value['name'];
													}
												}
												if ( '1' == $old_value ) {
													$old_value = 'True';
												}
												if ( '0' == $old_value ) {
													$old_value = 'False';
												}
												if ( '1' == $new_value ) {
													$new_value = 'True';
												}
												if ( '0' == $new_value ) {
													$new_value = 'False';
												}
												?>
												<del><?php print_r( $old_value ); ?></del>
												<span>
													<ins><?php print_r( $new_value ); ?></ins>
												</span>
											</td>
										</tr>
										<?php
									}
								}
							}
						}
					}
					foreach ( $post_types as $post_type ) {
						if ( $object_type == $post_type->name ) {
							foreach ( $post_full_key_array as $post_full_key_single ) {
									$old_user_meta_key = "ualp_post_old_$post_full_key_single";
									$new_user_meta_key = "ualp_post_new_$post_full_key_single";
								foreach ( $uact_details as $k => $v ) {
									if ( 'field_' == $post_full_key_single ) {
										if ( strpos( $v->uactd_key, 'ualp_post_old_field_' ) !== false ) {
											$old_user_meta_key = $v->uactd_key;
											$field_name        = ualp_str_after( $v->uactd_key, 'ualp_post_old_field_' );
											$new_user_meta_key = "ualp_post_new_field_$field_name";
										}
									}
									if ( $old_user_meta_key == $v->uactd_key ) {
										$old_value = ualp_get_activity_log_detail( $old_user_meta_key, $uactid );
										$new_value = ualp_get_activity_log_detail( $new_user_meta_key, $uactid );
										?>
											<tr>
												<td>
													<label>
													<?php
													if ( 'field_' == $post_full_key_single ) {
														ualp_replace_key( 'field_' . $field_name );
													} else {
														ualp_replace_key( $post_full_key_single, $object_type );
													}
													?>
													</label>
												</td>
												<td>
													<?php
													if ( 'post_content' == $post_full_key_single ) {
														$diff_table = wp_text_diff( $old_value, $new_value );
														echo esc_html( $diff_table );
													} else {
														if ( 'post_parent' == $post_full_key_single ) {
															$old_value = get_the_title( $old_value );
															$new_value = get_the_title( $new_value );
														}
														if ( 'post_author' == $post_full_key_single ) {
															$old_user = get_user_by( 'id', $old_value );
															$new_user = get_user_by( 'id', $new_value );
															if ( ( isset( $new_user ) && ! empty( $new_user ) ) || ( isset( $old_user ) && ! empty( $old_user ) ) ) {
																$old_value = isset( $old_user->display_name ) ? $old_user->display_name : '';
																$new_value = isset( $new_user->display_name ) ? $new_user->display_name : '';
															} else {
																$old_value = '';
																$new_value = '';
															}
														}
														$old_val = ualp_replace_special_array( $post_full_key_single, $old_value );
														$new_val = ualp_replace_special_array( $post_full_key_single, $new_value );
														if ( '_sale_price_dates_to' == $post_full_key_single || '_sale_price_dates_from' == $post_full_key_single ) {
															$date_format = get_option( 'date_format' );
															$old_val     = gmdate( $date_format, $old_val );
															$new_val     = gmdate( $date_format, $new_val );
														}
														if ( '_tax_class' == $post_full_key_single ) {
															if ( '' == $old_val ) {
																$old_val = 'Standard';
															}
															if ( '' == $new_val ) {
																$new_val = 'Standard';
															}
														}
														if ( '_tax_status' == $post_full_key_single ) {
															if ( '' == $old_val ) {
																$old_val = 'None';
															}
															if ( '' == $new_val ) {
																$new_val = 'None';
															}
														}
														?>
														<span>
															<?php
															if ( '' != $old_val ) {
																?>
																<del><?php echo esc_html( $old_val ); ?></del>
																<?php
															}
															?>
															<ins><?php echo esc_html( $new_val ); ?></ins>
														</span>
													<?php } ?>
												</td>
											</tr>
										<?php
									}
								}
							}
						}
					}

					$all_taxs = ualp_get_all_activity_log_detail( $uactid );

					foreach ( $all_taxs as $all_tax ) {
						$old_value        = '';
						$tax_key          = $all_tax['uactd_key'];
						$ualp_term_in     = ualp_term_in( $tax_key );
						$ualp_term_in_new = ualp_term_in_new( $tax_key );
						if ( $ualp_term_in_new ) {
							$taxname   = ualp_str_after( $tax_key, 'ualp_post_tax_new_' );
							$oldkey    = 'ualp_post_tax_old_' . $taxname;
							$old_value = ualp_get_activity_log_detail( $oldkey, $uactid );
							if ( $ualp_term_in && $ualp_term_in_new ) {
								$new_value = $all_tax['uactd_value'];
								if ( 'ualp_post_tax_new_post_format' == $tax_key || 'ualp_post_tax_old_post_format' == $tax_key ) {
									if ( '' != $old_value ) {
										$old_value = ualp_replace_special_array( $tax_key, $old_value, 'old' );
									}
									$new_value = ualp_replace_special_array( $tax_key, $new_value, 'new' );
									if ( '' == $old_value ) {
										$old_value = 'Standard';
									}
									if ( '' == $new_value ) {
										$new_value = 'Standard';
									}
								} else {
									if ( '' != $old_value ) {
										$old_value = ualp_replace_special_array( $tax_key, $old_value, 'old' );
									}
									$new_value = ualp_replace_special_array( $tax_key, $new_value, 'new' );
								}
								if ( $old_value != $new_value ) {
									if ( '1' == $old_value ) {
										$old_value = 'True';
									}
									if ( '0' == $old_value ) {
										$old_value = 'False';
									}
									if ( '1' == $new_value ) {
										$new_value = 'True';
									}
									if ( '0' == $new_value ) {
										$new_value = 'False';
									}
									?>
									<tr>
										<td><?php ualp_replace_key( $tax_key ); ?></td>
										<td>
											<span>
												<del><?php print_r( $old_value ); ?></del>
												<ins><?php print_r( $new_value ); ?></ins>
											</span>
										</td>
									</tr>
									<?php
								}
							}
						}
					}

					foreach ( $all_taxs as $all_tax ) {
						$old_value          = '';
						$option_key         = $all_tax['uactd_key'];
						$ualp_option_in     = ualp_option_in( $option_key );
						$ualp_option_in_new = ualp_option_in_new( $option_key );
						if ( $ualp_option_in_new ) {
							$option_name = ualp_str_after( $option_key, 'ualp_option_new_' );
							$oldkey      = 'ualp_option_old_' . $option_name;
							$old_value   = ualp_get_activity_log_detail( $oldkey, $uactid );
							if ( $ualp_option_in && $ualp_option_in_new ) {
								$new_value = $all_tax['uactd_value'];
								if ( '' != $old_value ) {
									$old_value = ualp_replace_special_array( $option_key, $old_value, 'old' );
								}
								$new_value = ualp_replace_special_array( $option_key, $new_value, 'new' );
								if ( $old_value != $new_value ) {
									if ( 'ualp_option_new_autoptimize_extra_checkbox_field_1' == $option_key ) {
										$option_label = 'Remove emojis';
									} elseif ( 'ualp_option_new_autoptimize_extra_checkbox_field_0' == $option_key ) {
										$option_label = 'Remove query strings from static resources';
									} elseif ( 'ualp_option_new_autoptimize_extra_radio_field_4' == $option_key ) {
										$option_label = 'Google Fonts';
									} elseif ( 'ualp_option_new_autoptimize_extra_text_field_2' == $option_key ) {
										$option_label = 'Preconnect to 3rd party domains';
									} elseif ( 'ualp_option_new_autoptimize_extra_text_field_3' == $option_key ) {
										$option_label = 'Async Javascript-files';
									} else {
										$option_label = $option_key;
									}
									if ( 'ualp_option_new_default_link_category' != $option_key ) {
										?>
									<tr>
										<td><?php ualp_replace_key( $option_label ); ?></td>
										<td>
											<span>
												<?php
												if ( 'ualp_option_new_rest_front_page' == $option_key ||
														'ualp_option_old_rest_front_page' == $option_key ||
														'ualp_option_new_directory_page_id' == $option_key ||
														'ualp_option_old_directory_page_id' == $option_key ||
														'ualp_option_new_profile_page_id' == $option_key ||
														'ualp_option_old_profile_page_id' == $option_key ||
														'ualp_option_new_login_page_id' == $option_key ||
														'ualp_option_old_login_page_id' == $option_key ||
														'ualp_option_new_registration_page_id' == $option_key ||
														'ualp_option_old_registration_page_id' == $option_key ||
														'ualp_option_new_xoousersultra_my_account_page' == $option_key ||
														'ualp_option_old_xoousersultra_my_account_page' == $option_key ||
														'ualp_option_new_rest_cancel_page' == $option_key ||
														'ualp_option_old_rest_front_page' == $option_key ) {
													$old_value = get_the_title( $old_value );
													$new_value = get_the_title( $new_value );
												}

												if ( 'ualp_option_new_holiday_dates' == $option_key || 'ualp_option_old_holiday_dates' == $option_key ) {
													$new_value = str_replace( 'replace_this :', ' ', $new_value );
													$new_value = str_replace( 'rest_holidays_date_from : , rest_holidays_date_to :', ' ', $new_value );
													$new_value = trim( $new_value, ', ,' );
													$new_value = trim( $new_value, ',' );
													$new_value = explode( ', ,', $new_value );

													$old_value = str_replace( 'replace_this :', ' ', $old_value );
													$old_value = str_replace( 'rest_holidays_date_from : , rest_holidays_date_to :', ' ', $old_value );
													$old_value = trim( $old_value, ', ,' );
													$old_value = trim( $old_value, ',' );
													$old_value = explode( ', ,', $old_value );

													$temp_new_value = array_intersect( $new_value, $old_value );
													$new_value1     = array_diff( $new_value, $temp_new_value );

													$temp_old_value = array_intersect( $old_value, $new_value );
													$old_value1     = array_diff( $old_value, $temp_old_value );

													$new_value = $new_value1;
													$old_value = $old_value1;

													$new_value = implode( ', ', $new_value );
													$old_value = implode( ', ', $old_value );
												}
												if ( 'ualp_option_new_dayno' == $option_key || 'ualp_option_old_dayno' == $option_key ) {
													$new_value = trim( $new_value, ', ' );
													$new_value = ltrim( $new_value, ', ' );
													$new_value = explode( ', ,', $new_value );
													foreach ( $new_value as $key => $value ) {
														if ( isset( $value ) ) {
															$new_value[ $key ] = gmdate( 'l', strtotime( "Monday + $key Days" ) ) . ' : ' . $value;
														}
													}
													$new_value = $new_value;

													$old_value = trim( $old_value, ', ' );
													$old_value = ltrim( $old_value, ', ' );
													$old_value = explode( ', ,', $old_value );
													foreach ( $old_value as $key => $value ) {
														if ( isset( $value ) ) {
															$old_value[ $key ] = gmdate( 'l', strtotime( "Monday + $key Days" ) ) . ' : ' . $value;
														}
													}
													$old_value = $old_value;

													$temp_new_value = array_intersect( $new_value, $old_value );
													$new_value1     = array_diff( $new_value, $temp_new_value );

													$temp_old_value = array_intersect( $old_value, $new_value );
													$old_value1     = array_diff( $old_value, $temp_old_value );

													$new_value = $new_value1;
													$old_value = $old_value1;

													$new_value = implode( ', ', $new_value );
													$old_value = implode( ', ', $old_value );
												}
												if ( 'ualp_option_new_r_closed_days' == $option_key || 'ualp_option_old_r_closed_days' == $option_key ) {
													$new_value = explode( ',', $new_value );
													foreach ( $new_value as $value ) {
														$new_value[] = gmdate( 'l', strtotime( "Sunday + $value Days" ) );
													}
													$new_value = implode( ', ', $new_value );
													$old_value = explode( ',', $old_value );
													foreach ( $old_value as $value ) {
														$old_value[] = gmdate( 'l', strtotime( "Sunday + $value Days" ) );
													}
													$old_value = implode( ', ', $old_value );
												}
												if ( 'ualp_option_new_shipping' == $option_key || 'ualp_option_old_shipping' == $option_key || 'ualp_option_new_billing' == $option_key || 'ualp_option_old_billing' == $option_key ) {
													$old_value = trim( $old_value, ', ' );
													$old_value = ltrim( $old_value, ', ' );
													$old_value = explode( ', , ', $old_value );

													$new_value = trim( $new_value, ', ' );
													$new_value = ltrim( $new_value, ', ' );
													$new_value = explode( ', , ', $new_value );

													$temp_new_value = array_intersect( $new_value, $old_value );
													$new_value1     = array_diff( $new_value, $temp_new_value );

													$temp_old_value = array_intersect( $old_value, $new_value );
													$old_value1     = array_diff( $old_value, $temp_old_value );

													$new_value = $new_value1;
													$old_value = $old_value1;

													$old_value = wp_json_encode( $old_value );
													$old_value = str_replace( ', ', ', <br/>', $old_value );
													$new_value = str_replace( ',"position', ',<br/>"position', $old_value );

													$new_value = wp_json_encode( $new_value );
													$new_value = str_replace( ', ', ', <br/>', $new_value );
													$old_value = str_replace( ',"position', ',<br/>"position', $new_value );

													$new_value = $new_value;
													$old_value = $old_value;
													?>
														<del style="display: inline-block;width: 48%;float: left;margin: 0;padding: 10px;box-sizing: border-box;"><?php print_r( $old_value ); ?></del>
														<ins style="display: inline-block;width: 48%;float: right;margin: 0;padding: 10px;box-sizing: border-box;"><?php print_r( $new_value ); ?></ins>
													<?php
												} else {
													if ( 'ualp_option_new_uultra_front_publisher_default_category' == $option_key || 'ualp_option_old_uultra_front_publisher_default_category' == $option_key ) {
														if ( '0' == $old_value ) {
															$old_value = '-';
														} else {
															$term      = get_term_by( 'id', $old_value, 'category' );
															$old_value = $term->name;
														}
														if ( '0' == $new_value ) {
															$new_value = '-';
														} else {
															$term      = get_term_by( 'id', $new_value, 'category' );
															$new_value = $term->name;
														}
													}
													if ( 'ualp_option_new_redirect_after_registration_login' == $option_key || 'ualp_option_old_redirect_after_registration_login' == $option_key ) {
														if ( '0' == $old_value ) {
															$old_value = '-';
														} else {
															$old_value = get_the_title( $old_value );
														}
														if ( '0' == $new_value ) {
															$new_value = '-';
														} else {
															$new_value = get_the_title( $new_value );
														}
													}
													if ( '1' == $old_value ) {
														$old_value = 'True';
													}
													if ( '0' == $old_value ) {
														$old_value = 'False';
													}
													if ( '1' == $new_value ) {
														$new_value = 'True';
													}
													if ( '0' == $new_value ) {
														$new_value = 'False';
													}
													?>
														<del><?php print_r( $old_value ); ?></del>
														<ins><?php print_r( $new_value ); ?></ins>
														<?php
												}
												?>
											</span>
										</td>
									</tr>
										<?php
									}
								}
							}
						}
					}
					?>
					<tr>
						<td class="ulap_view_title" colspan="2">
							<?php esc_html_e( 'User Detail', 'user_activity_log_pro' ); ?>
						</td>
					</tr>
					<tr>
						<td><?php esc_html_e( 'Date', 'user_activity_log_pro' ); ?></td>
						<td>
							<?php
							$date_format = get_option( 'date_format' );
							$time_format = get_option( 'time_format' );

							$modified_date = gmdate( $date_format, strtotime( $date ) );
							$modified_time = gmdate( $time_format, strtotime( $date ) );

							echo esc_html( $modified_date ) . ' ' . esc_html( $modified_time );
							?>
						</td>
					</tr>
					<tr>
						<td><?php esc_html_e( 'Username', 'user_activity_log_pro' ); ?></td>
						<td>
							<?php echo esc_html( $username ); ?>
						</td>
					</tr>
					<?php
					if ( isset( $user_role ) && '' != $user_role ) {
						?>
						<tr>
							<td><?php esc_html_e( 'Role', 'user_activity_log_pro' ); ?></td>
							<td>
								<?php echo esc_html( $user_role ); ?>
							</td>
						</tr>
						<?php
					}
					if ( isset( $useremail ) && '' != $useremail ) {
						?>
						<tr>
							<td><?php esc_html_e( 'Email Address', 'user_activity_log_pro' ); ?></td>
							<td>
								<?php echo esc_html( $useremail ); ?>
							</td>
						</tr>
						<?php
					}
					if ( ualp_allow_ip() ) {
						?>
					<tr>
						<td><?php esc_html_e( 'IP Address', 'user_activity_log_pro' ); ?></td>
						<td>
							<?php echo esc_html( $ipaddress ); ?>
						</td>
					</tr>
						<?php
					}
					if ( ualp_allow_country() ) {
						?>
					<tr>
						<td><?php esc_html_e( 'Country', 'user_activity_log_pro' ); ?></td>
						<td>
							<?php echo esc_html( $country ); ?>
						</td>
					</tr>
						<?php
					}
					if ( ualp_allow_user_agent() ) {
						?>
					<tr>
						<td><?php esc_html_e( 'User Agent', 'user_activity_log_pro' ); ?></td>
						<td>
							<?php echo esc_html( $user_agent ); ?>
						</td>
					</tr>
					<?php } ?>
				</table>
			</div>
		</div>
			<?php
			exit;
		}
	}
}

if ( ! function_exists( 'ualp_get_current_admin_page_url' ) ) {
	/**
	 * Get current admin page url.
	 *
	 * @param string $server Server.
	 */
	function ualp_get_current_admin_page_url( $server = null ) {
		if ( null === $server ) {
			$server = $_SERVER;
		}
		if ( ! isset( $server['SERVER_PORT'] ) ) {
			return '';
		}
		$ssl = false;
		if ( isset( $server['HTTPS'] ) ) {
			$value = strtolower( $server['HTTPS'] );
			$ssl   = ( ( '' != $value ) && ( 'off' != $value ) );
		}

		$port = $server['SERVER_PORT'];
		if ( $ssl && 443 == $port ) {
			$port = '';
		}
		if ( ! $ssl && 80 == $port ) {
			$port = '';
		}
		if ( '' != $port ) {
			$port = ':' . $port;
		}

		$url = $server['REQUEST_URI'];
		return sprintf( '%s://%s%s%s', $ssl ? 'https' : 'http', $server['HTTP_HOST'], $port, $url );
	}
}

if ( ! function_exists( 'ualp_get_activity_log_detail' ) ) {
	/**
	 * Get detail activity log.
	 *
	 * @param string $option_name Option Name.
	 * @param string $activity_id Activity ID.
	 */
	function ualp_get_activity_log_detail( $option_name, $activity_id ) {
		global $wpdb, $externaldb;
		$row = $externaldb->get_row( 'SELECT uactd_value FROM ' . UALP_DETAIL_TABLENAME . " WHERE uactd_key = '$option_name' AND uactid = '$activity_id'", ARRAY_A );
		return $row['uactd_value'];
	}
}

if ( ! function_exists( 'ualp_get_all_activity_log_detail' ) ) {
	/**
	 * Get all activity log.
	 *
	 * @param string $activity_id Activity ID.
	 */
	function ualp_get_all_activity_log_detail( $activity_id ) {
		global $wpdb, $externaldb;
		$uactd_data = $externaldb->get_results( 'SELECT * FROM ' . UALP_DETAIL_TABLENAME . " WHERE uactid = '$activity_id'", ARRAY_A );
		return $uactd_data;
	}
}

if ( ! function_exists( 'ualp_delete_log' ) ) {
	/**
	 * Delete activity log as per selected days.
	 */
	function ualp_delete_log() {
		global $wpdb, $externaldb;
		$get_log_span = '';
		$get_log_span = get_option( 'ualpKeepLogsDay' );
		$time         = strtotime( '-' . $get_log_span . ' days' );
		$date         = gmdate( 'Y-m-d', $time );
		if ( isset( $get_log_span ) && ! empty( $get_log_span ) ) {
			$externaldb->query( 'DELETE FROM ' . UALP_TABLENAME . " WHERE modified_date < '$date'" );
		}
	}
}

if ( ! function_exists( 'ualp_replace_key' ) ) {
	/**
	 * Replace key width label.
	 *
	 * @param string $val Value.
	 * @param string $post_type Post Type.
	 */
	function ualp_replace_key( $val = '', $post_type = '' ) {
		$dflt_array = array();

		if ( '_upsell_ids' == $val ) {
			$val = '_upsell_products';
		}
		if ( 'ualp_option_new_uultra_password_lenght' == $val ) {
			$val = 'User Ultra Password Length';
		}
		if ( '_crosssell_ids' == $val ) {
			$val = '_crosssell_products';
		}

		$val = str_replace( 'ualp_post_tax_old_', '', $val );
		$val = str_replace( 'ualp_post_tax_new_', '', $val );
		$val = str_replace( 'ualp_option_old_', '', $val );
		$val = str_replace( 'ualp_option_new_', '', $val );

		if ( 0 === strpos( $val, 'rest_' ) ) {
			$val = substr( $val, strlen( 'rest_' ) );
		}

		$val = str_replace( '_', ' ', $val );
		$val = trim( $val );
		$val = ucfirst( $val );
		$val = apply_filters( 'ualp_replace_key', $val, $post_type );
		print_r( $val );
	}
}

if ( ! function_exists( 'ualp_replace_special_array' ) ) {
	/**
	 * Replace value for special key.
	 *
	 * @param string $key Key.
	 * @param string $value Value.
	 * @param string $status Status.
	 */
	function ualp_replace_special_array( $key, $value, $status = '' ) {
		// post id serialized array.
		$post_serialized_array = ualp_get_serialized_post_array();
		// cat id serialized array.
		$cat_serialized_array = ualp_get_serialized_cat_array();
		// data serialized array.
		// $data_serialized_array = ualp_get_serialized_data_array();.
		$n_value = '';
		if ( '_thumbnail_id' == $key ) {
			$n_value = '<img class="ualp-tmb-img" src="' . wp_get_attachment_thumb_url( $value ) . '" />';
			return $n_value;
		} elseif ( '_product_image_gallery' == $key ) {
			$value = explode( ',', $value );
			if ( is_array( $value ) ) {
				foreach ( $value as $s_value ) {
					$n_value .= '<img class="ualp-tmb-img" src="' . wp_get_attachment_thumb_url( $s_value ) . '" />';
				}
			}
			return $n_value;
		} elseif ( 'rule' == $key ) {
			// For acf support.
			$value = maybe_unserialize( $value );
			if ( is_array( $value ) ) {
				$j = 0;
				foreach ( $value as $s_value ) {
					$s_value = maybe_unserialize( $s_value );
					if ( 0 != $j ) {
						$n_value .= '<br/>';
					}
					$i = 0;
					foreach ( $s_value as $ss_key => $ss_value ) {
						if ( 0 == $i ) {
							$n_value .= $ss_key . ':' . $ss_value;
						} else {
							$n_value .= ', ' . $ss_key . ':' . $ss_value;
						}
						$i++;
					}
					$j++;
				}
				return $n_value;
			}
		} elseif ( in_array( $key, $post_serialized_array ) ) {
			$value = maybe_unserialize( $value );
			if ( is_array( $value ) ) {
				$i = 0;
				foreach ( $value as $s_value ) {
					if ( 0 == $i ) {
						$n_value .= get_the_title( $s_value );
					} else {
						$n_value .= ', ' . get_the_title( $s_value );
					}
					$i++;
				}
			} else {
				$n_value = get_the_title( $value );
			}
			return $n_value;
		} elseif ( array_key_exists( $key, $cat_serialized_array ) ) {
			$value = maybe_unserialize( $value );
			if ( is_array( $value ) ) {
				$i = 0;
				foreach ( $value as $s_value ) {
					$term_data = get_term( $s_value, $cat_serialized_array[ $s_value ] );
					if ( ! empty( $term_data ) ) {
						if ( 0 == $i ) {
							$n_value .= $term_data->name;
						} else {
							$n_value .= ', ' . $term_data->name;
						}
					}
					$i++;
				}
			} else {
				$term_data = get_term( $value, $cat_serialized_array[ $value ] );
				if ( ! empty( $term_data ) ) {
					$n_value .= $term_data->name;
				}
			}
			return $n_value;
		} elseif ( ualp_term_in( $key ) ) {
			$value = maybe_unserialize( $value );
			if ( is_array( $value ) ) {
				$i = 0;
				foreach ( $value as $term_id ) {
					if ( ualp_term_in_old( $key ) ) {
						$taxname = ualp_str_after( $key, 'ualp_post_tax_old_' );
					} else {
						$taxname = ualp_str_after( $key, 'ualp_post_tax_new_' );
					}
					$term_data = get_term( $term_id, $taxname );
					if ( ! empty( $term_data ) ) {
						if ( 0 == $i ) {
							$n_value .= $term_data->name;
						} else {
							$n_value .= ', ' . $term_data->name;
						}
					}
					$i++;
				}
				return $n_value;
			}
		} elseif ( is_serialized( $value ) ) {
			$value = maybe_unserialize( $value );
			if ( is_array( $value ) ) {
				$i       = 0;
				$n_value = ualpserializeloop( $value );
				return $n_value;
			}
		}
		return $value;
	}
}

if ( ! function_exists( 'ualp_term_in' ) ) {
	/**
	 * Check if taxonomy term.
	 *
	 * @param string $tax_key Taxonomy Key.
	 */
	function ualp_term_in( $tax_key ) {
		if ( strpos( $tax_key, 'ualp_post_tax_' ) !== false ) {
			return true;
		}
		return false;
	}
}

if ( ! function_exists( 'ualp_term_in_old' ) ) {
	/**
	 * Check if taxonomy term old change.
	 *
	 * @param string $tax_key Taxonomy Key.
	 */
	function ualp_term_in_old( $tax_key ) {
		if ( strpos( $tax_key, 'ualp_post_tax_old_' ) !== false ) {
			return true;
		}
		return false;
	}
}

if ( ! function_exists( 'ualp_term_in_new' ) ) {
	/**
	 * Check if taxonomy term new change.
	 *
	 * @param string $tax_key Taxonomy Key.
	 */
	function ualp_term_in_new( $tax_key ) {
		if ( strpos( $tax_key, 'ualp_post_tax_new_' ) !== false ) {
			return true;
		}
		return false;
	}
}

if ( ! function_exists( 'ualp_option_in' ) ) {
	/**
	 * Check if option.
	 *
	 * @param string $option_key Option Key.
	 */
	function ualp_option_in( $option_key ) {
		if ( strpos( $option_key, 'ualp_option_' ) !== false ) {
			return true;
		}
		return false;
	}
}

if ( ! function_exists( 'ualp_option_in_old' ) ) {
	/**
	 * Check if option old change.
	 *
	 * @param string $option_key Option Key.
	 */
	function ualp_option_in_old( $option_key ) {
		if ( strpos( $option_key, 'ualp_option_old_' ) !== false ) {
			return true;
		}
		return false;
	}
}

if ( ! function_exists( 'ualp_option_in_new' ) ) {
	/**
	 * Check if option new change.
	 *
	 * @param string $option_key Option Key.
	 */
	function ualp_option_in_new( $option_key ) {
		if ( strpos( $option_key, 'ualp_option_new_' ) !== false ) {
			return true;
		}
		return false;
	}
}

if ( ! function_exists( 'ualp_str_after' ) ) {
	/**
	 * Get taxnonomy name.
	 *
	 * @param string $string String.
	 * @param string $substring Sub-string.
	 */
	function ualp_str_after( $string, $substring ) {
		$pos = strpos( $string, $substring );
		if ( false === $pos ) {
			return $string;
		} else {
			return( substr( $string, $pos + strlen( $substring ) ) );
		}
	}
}


if ( ! function_exists( 'ualp_posttypes' ) ) {
	/**
	 * Get all post types.
	 */
	function ualp_posttypes() {
		$ualp_post_types           = array();
		$post_types                = get_post_types( array(), 'objects', 'and' );
		$removable_post_type_array = array( 'nav_menu_item', 'revision', 'product_variation', 'shop_order_refund', 'user_request' );
		foreach ( $post_types as $post_type ) {
			if ( in_array( $post_type->name, $removable_post_type_array ) ) {
				unset( $post_types[ $post_type->name ] );
			}
		}
		return apply_filters( 'ualp_post_types', $post_types );
	}
}

if ( ! function_exists( 'ualp_taxonomies' ) ) {
	/**
	 * Get all taxonomies.
	 */
	function ualp_taxonomies() {
		$taxonomies               = get_taxonomies( array(), 'objects', 'and' );
		$removable_taxonomy_array = array( 'nav_menu', 'link_category', 'post_format', 'product_type' );
		foreach ( $taxonomies as $taxonomy ) {
			if ( in_array( $taxonomy->name, $removable_taxonomy_array ) ) {
				unset( $taxonomies[ $taxonomy->name ] );
			}
		}
		return apply_filters( 'ualp_taxonomies', $taxonomies );
	}
}

if ( ! function_exists( 'ualp_get_serialized_post_array' ) ) {
	/**
	 * Get all keys contain serialized array of post id.
	 */
	function ualp_get_serialized_post_array() {
		$serialized_array = array( '_upsell_ids', '_crosssell_ids', 'product_ids', 'exclude_product_ids', '_edd_discount_product_reqs', '_edd_discount_excluded_products' );
		$serialized_array = apply_filters( 'ualp_post_serialized_array', $serialized_array );
		return $serialized_array;
	}
}

if ( ! function_exists( 'ualp_get_serialized_cat_array' ) ) {
	/**
	 * Get all keys contain serialized array of Cat ids.
	 */
	function ualp_get_serialized_cat_array() {
		$serialized_array = array(
			'product_categories'         => 'product_cat',
			'exclude_product_categories' => 'product_cat',
		);
		$serialized_array = apply_filters( 'ualp_cat_serialized_array', $serialized_array );
		return $serialized_array;
	}
}

if ( ! function_exists( 'ualp_get_serialized_data_array' ) ) {
	/**
	 * Get all keys contain serialized array of data.
	 */
	function ualp_get_serialized_data_array() {
		$serialized_array = array( 'wp_capabilities', 'customer_email', 'field_', 'edd_variable_prices', 'edd_download_files' );
		$serialized_array = apply_filters( 'ualp_data_serialized_array', $serialized_array );
		return $serialized_array;
	}
}

if ( ! function_exists( 'ualpserializeloop' ) ) {
	/**
	 * Function to unserialoze content in loop.
	 *
	 * @param string $in_array In array.
	 * @param string $in_key In Key.
	 * @param string $n_value Value.
	 * @param string $j Count Variable.
	 */
	function ualpserializeloop( $in_array, $in_key = '', $n_value = '', $j = 0 ) {
		if ( is_array( $in_array ) && ! empty( $in_array ) ) {
			foreach ( $in_array as $in_key => $in_value ) {
				if ( '' !== $in_key && 'key' !== $in_key && 'field' !== $in_key ) {
					if ( 0 != $j ) {
						$n_value .= ', ';
					}
					if ( ! is_numeric( $in_key ) ) {
						$n_value .= $in_key . ' : ';
					}
					$j++;
					$n_value .= ualpserializeloop( $in_value, $in_key, '', $j );
				}
			}
		} else {
			if ( '' !== $in_key && 'key' !== $in_key ) {
				$n_value .= $in_array;
			}
		}
		return $n_value;
	}
}

if ( ! function_exists( 'ualp_get_primary_post_data' ) ) {
	/**
	 * Function to get all primary data key of post.
	 *
	 * @param string $post_type Post Type.
	 */
	function ualp_get_primary_post_data( $post_type = '' ) {
		$get_dafault_post_key_array = array( 'post_title', 'post_name', 'post_content', 'post_status', 'menu_order', 'post_date', 'post_date_gmt', 'post_excerpt', 'comment_status', 'ping_status', 'post_parent', 'post_author' );
		return apply_filters( 'ualp_post_primary_array', $get_dafault_post_key_array, $post_type );
	}
}

if ( ! function_exists( 'ualp_get_post_meta_data' ) ) {
	/**
	 * Function to get all meta key of post.
	 *
	 * @param string $post_type Post Type.
	 */
	function ualp_get_post_meta_data( $post_type = '' ) {
		$get_default_metakey_array = array( '_thumbnail_id', '_product_attributes', '_product_image_gallery', '_manage_stock', '_stock_status', '_stock', '_backorders', '_price', '_regular_price', '_sale_price', '_sale_price_dates_from', '_sale_price_dates_to', '_sold_individually', '_upsell_ids', '_crosssell_ids', '_tax_status', '_tax_class', '_weight', '_length', '_width', '_height', '_sku', '_featured', '_purchase_note', '_virtual', '_downloadable', '_visibility', 'discount_type', 'coupon_amount', 'individual_use', 'product_ids', 'exclude_product_ids', 'usage_limit', 'usage_limit_per_user', 'limit_usage_to_x_items', 'expiry_date', 'free_shipping', 'exclude_sale_items', 'product_categories', 'exclude_product_categories', 'minimum_amount', 'maximum_amount', 'customer_email', 'position', 'layout', 'hide_on_screen', 'rule', 'field_', '_edd_download_earnings', '_edd_download_sales', 'edd_price', 'edd_variable_prices', 'edd_download_files', '_edd_bundled_products', '_edd_bundled_products_conditions', '_variable_pricing', '_edd_default_price_id', '_edd_hide_purchase_link', '_edd_download_limit', '_edd_discount_code', '_edd_discount_name', '_edd_discount_status', '_edd_discount_max_uses', '_edd_discount_amount', '_edd_discount_start', '_edd_discount_expiration', '_edd_discount_type', '_edd_discount_min_price', '_edd_discount_product_reqs', '_edd_discount_product_condition', '_edd_discount_excluded_products', '_edd_discount_is_not_global', '_edd_discount_is_single_use', '_downloadable_files' );
		return apply_filters( 'ualp_post_meta_array', $get_default_metakey_array, $post_type );
	}
}

if ( ! function_exists( 'ualp_get_user_primary_data' ) ) {
	/**
	 * Function to get all user fields.
	 */
	function ualp_get_user_primary_data() {
		$user_primary_key_array = array(
			'email' => 'user_email',
			'url'   => 'user_url',
			'display_name',
		);
		return apply_filters( 'ualp_user_primary_array', $user_primary_key_array );
	}
}

if ( ! function_exists( 'ualp_get_user_meta_data' ) ) {
	/**
	 * Function to get all user fields.
	 */
	function ualp_get_user_meta_data() {
		$user_meta_key_array = array(
			'nickname',
			'first_name',
			'last_name',
			'description',
			'rich_editing',
			'admin_color',
			'admin_bar_front' => 'show_admin_bar_front',
			'billing_first_name',
			'billing_last_name',
			'billing_company',
			'billing_address_1',
			'billing_address_2',
			'billing_city',
			'billing_postcode',
			'billing_country',
			'billing_state',
			'billing_phone',
			'billing_email',
			'shipping_first_name',
			'shipping_last_name',
			'shipping_company',
			'shipping_address_1',
			'shipping_address_2',
			'shipping_city',
			'shipping_postcode',
			'shipping_country',
			'shipping_state',
			'googleplus',
			'facebook',
			'twitter',
			'linkedin',
		);
		return apply_filters( 'ualp_user_meta_array', $user_meta_key_array );
	}
}

if ( ! function_exists( 'ualploader' ) ) {
	/**
	 * Function for add loader.
	 */
	function ualploader() {
		?>
		<div class="loader">
			<span class="rotating"></span>
		</div>
		<?php
	}
}

add_action( 'admin_head', 'ualploader' );


if ( ! function_exists( 'ualp_get_options' ) ) {
	/**
	 * Function to get all whitelist options.
	 */
	function ualp_get_options() {
		$ualp_options_array = array(
			// General.
			'blogname',
			'blogdescription',
			'siteurl',
			'home',
			'admin_email',
			'users_can_register',
			'default_role',
			'timezone_string',
			'date_format',
			'time_format',
			'start_of_week',
			'gmt_offset',
			'WPLANG',
			'new_admin_email',
			// Writing.
			'use_smilies',
			'use_balanceTags',
			'default_category',
			'default_post_format',
			'mailserver_url',
			'mailserver_login',
			'mailserver_pass',
			'mailserver_port',
			'default_email_category',
			'default_link_category',
			'ping_sites',
			// Reading.
			'show_on_front',
			'page_on_front',
			'page_for_posts',
			'posts_per_page',
			'posts_per_rss',
			'rss_use_excerpt',
			'blog_public',
			// Discussion.
			'default_pingback_flag',
			'default_ping_status',
			'default_comment_status',
			'require_name_email',
			'comment_registration',
			'close_comments_for_old_posts',
			'close_comments_days_old',
			'thread_comments',
			'thread_comments_depth',
			'page_comments',
			'comments_per_page',
			'default_comments_page',
			'comment_order',
			'comments_notify',
			'moderation_notify',
			'comment_moderation',
			'comment_whitelist',
			'comment_max_links',
			'moderation_keys',
			'blacklist_keys',
			'show_avatars',
			'avatar_rating',
			'avatar_default',
			// Media.
			'thumbnail_size_w',
			'thumbnail_size_h',
			'thumbnail_crop',
			'medium_size_w',
			'medium_size_h',
			'large_size_w',
			'large_size_h',
			'uploads_use_yearmonth_folders',
			// Network.
			'registrationnotification',
			'registration',
			'add_new_users',
			'menu_items',
			'upload_space_check_disabled',
			'blog_upload_space',
			'upload_filetypes',
			'site_name',
			'first_post',
			'first_page',
			'first_comment',
			'first_comment_url',
			'first_comment_author',
			'welcome_email',
			'welcome_user_email',
			'fileupload_maxk',
			'global_terms_enabled',
			'illegal_names',
			'limited_email_domains',
			'banned_email_domains',
			'blog_count',
			'user_count',
			// permalink.
			'woocommerce_permalinks',
			'permalink_structure',
			'category_base',
			'tag_base',
		);
		return apply_filters( 'ualp_options_array', $ualp_options_array );
	}
}

if ( ! function_exists( 'ualp_permalinks' ) ) {
	/**
	 * Permalink array.
	 */
	function ualp_permalinks() {
		$permalink_array = apply_filters( 'ualp_permalink_array', array( 'woocommerce_permalinks', 'permalink_structure', 'category_base', 'tag_base' ) );
		return $permalink_array;
	}
}

if ( ! function_exists( 'ualp_latest_activity_logs' ) ) {
	/**
	 * Add Admin Dashboard Widget - Latest User Activity Logs.
	 */
	function ualp_latest_activity_logs() {
		// Register the new dashboard widget with the 'wp_dashboard_setup' action.
		add_action( 'wp_dashboard_setup', 'ualp_latest_logs' );
		if ( ! function_exists( 'ualp_latest_logs' ) ) {
			/**
			 * Latest Logs.
			 */
			function ualp_latest_logs() {
				add_screen_option(
					'layout_columns',
					array(
						'max'     => 3,
						'default' => 2,
					)
				);
				$ualp_role_access_list      = get_option( 'ualpRoleAccessList' );
				$ualp_user_access_list      = get_option( 'ualpUserAccessList' );
				$ualp_allow_dashbord_widget = get_option( 'ualpAllowDashbordWidget', '1' );
				if ( 1 == $ualp_allow_dashbord_widget ) {
					$current_user      = wp_get_current_user();
					$current_user_name = $current_user->ID;
					if ( isset( $current_user->roles[0] ) ) {
						$current_user_role = $current_user->roles[0];
					} else {
						if ( is_multisite() ) {
							$id       = get_current_user_id();
							$usermeta = get_user_meta( $id, 'wp_capabilities' );
							if ( is_array( $usermeta[0] ) ) {
								$current_userrolls = array_keys( $usermeta[0] );
							} else {
								$current_userrolls = $usermeta;
							}
							if ( isset( $current_userrolls[0] ) ) {
								$current_user_role = $current_userrolls[0];
							}
						}
					}
					$ualp_role = '';
					$ualp_user = '';
					if ( is_array( $ualp_role_access_list ) && ! empty( $ualp_role_access_list ) ) {
						if ( count( $ualp_role_access_list ) > 1 ) {
							foreach ( $ualp_role_access_list as $ualp_role ) {
								$ualp_role .= $ualp_role;
							}
						}
					}
					if ( is_array( $ualp_user_access_list ) && ! empty( $ualp_user_access_list ) ) {
						if ( count( $ualp_user_access_list ) > 1 ) {
							foreach ( $ualp_user_access_list as $ualp_user ) {
								$ualp_user .= $ualp_user;
							}
						}
					}
					if ( is_array( $ualp_role_access_list ) && is_array( $ualp_user_access_list ) && ! empty( $ualp_role_access_list ) && ! empty( $ualp_user_access_list ) && isset( $current_user_role ) && 'administrator' != $current_user_role ) {
						if ( ( isset( $current_user_role ) && $current_user_role == $ualp_role ) || ( isset( $current_user_name ) && $current_user_name == $ualp_user ) ) {
							add_meta_box( 'ualp_dashboard_widget', esc_html__( 'Latest User Activity Logs', 'user_activity_log_pro' ), 'ualp_dashboard_widget', 'dashboard', 'normal', 'high' );
						}
					} elseif ( is_array( $ualp_role_access_list ) && ! empty( $ualp_role_access_list ) && isset( $current_user_role ) && 'administrator' != $current_user_role ) {
						foreach ( $ualp_role_access_list as $ualp_role ) {
							if ( isset( $current_user_role ) && $current_user_role == $ualp_role ) {
								add_meta_box( 'ualp_dashboard_widget', esc_html__( 'Latest User Activity Logs', 'user_activity_log_pro' ), 'ualp_dashboard_widget', 'dashboard', 'normal', 'high' );
							}
						}
					} elseif ( is_array( $ualp_user_access_list ) && ! empty( $ualp_user_access_list ) && isset( $current_user_role ) && 'administrator' != $current_user_role ) {
						foreach ( $ualp_user_access_list as $ualp_user ) {
							if ( isset( $current_user_name ) && $current_user_name == $ualp_user ) {
								add_meta_box( 'ualp_dashboard_widget', esc_html__( 'Latest User Activity Logs', 'user_activity_log_pro' ), 'ualp_dashboard_widget', 'dashboard', 'normal', 'high' );
							}
						}
					} else {
						if ( isset( $current_user_role ) && 'administrator' == $current_user_role ) {
							add_meta_box( 'ualp_dashboard_widget', esc_html__( 'Latest User Activity Logs', 'user_activity_log_pro' ), 'ualp_dashboard_widget', 'dashboard', 'normal', 'high' );
						}
					}
				}
			}
		}
		if ( ! function_exists( 'ualp_dashboard_widget' ) ) {
			/**
			 * Dashboard Widget.
			 */
			function ualp_dashboard_widget() {
				?>
				<ul class="ualp_log_table">
					<li>
						<span class="ualp-column-date"><b><?php esc_html_e( 'Date', 'user_activity_log_pro' ); ?></b></span>
						<span class="ualp-column-author"><b><?php esc_html_e( 'Author', 'user_activity_log_pro' ); ?></b></span>
						<span class="ualp-column-activity"><b><?php esc_html_e( 'Activity', 'user_activity_log_pro' ); ?></b></span>
					</li>
					<?php
					global $wpdb, $externaldb;
					$where       = ' WHERE 1=1';
					$super_admin = is_super_admin( get_current_user_id() );
					if ( ualp_multi_site_support_return() ) {
						if ( 1 != $super_admin ) {
							$where .= " AND user_id = '" . get_current_user_id() . "'";
						}
					}
					if ( ualp_multi_site_support_return() ) {
						$where .= " AND site_id = '" . get_current_blog_id() . "'";
					}
					$number_of_widget_logs = 5;
					$select_query          = 'SELECT * from ' . UALP_TABLENAME . $where . ' ORDER BY modified_date desc LIMIT ' . $number_of_widget_logs;
					$get_data              = $externaldb->get_results( $select_query );
					if ( $get_data ) {
						$srno = 1;
						foreach ( $get_data as $data ) {
							?>
							<li>
								<span class="ualp-column-date">
								<?php
									$modified_date = strtotime( $data->modified_date );
									$date_format   = get_option( 'date_format' );
									$time_format   = get_option( 'time_format' );
									$date          = gmdate( $date_format, $modified_date );
									$time          = gmdate( $time_format, $modified_date );
									echo esc_html( $date ) . '<br/>' . esc_html( $time ) . '<br/>';
									$current_t = current_time( 'timestamp' );
									$t_diff    = human_time_diff( $modified_date, $current_t );
									echo '<b>' . esc_html( $t_diff ) . ' ' . esc_html__( 'ago', 'user_activity_log_pro' ) . '</b>'
								?>
								</span>
								<span class="ualp-column-author column-author">
								<?php
									global $wp_roles;
								if ( ! empty( $data->user_id ) && 0 !== (int) $data->user_id ) {
									$user = get_user_by( 'id', $data->user_id );
									if ( $user instanceof WP_User && 0 !== $user->ID ) {
										printf(
											'<a href="%s">%s <span class="author-name">%s</span></a><br /><small>%s</small><br/>%s',
											get_edit_user_link( $user->ID ),
											get_avatar( $user->ID, 40 ),
											ucfirst( $user->display_name ),
											isset( $user->roles[0] ) &&
												isset( $wp_roles->role_names[ $user->roles[0] ] ) ? esc_html( $wp_roles->role_names[ $user->roles[0] ] ) : '',
											esc_html( $user->user_email )
										);
									}
								} else {
									esc_html_e( 'Guest', 'user_activity_log_pro' );
								}
								?>
								</span>
								<span class="ualp-column-activity">
									<?php
									if ( 'Page' == $data->object_type && 'Viewed' == $data->action ) {
										echo wp_kses( ucfirst( $data->object_type ), ualp_args_kses() ) . ' ' . wp_kses( ucfirst( $data->action ), ualp_args_kses() ) . ' : ' . esc_html( $data->post_title );
									} else {
										echo wp_kses( $data->description, ualp_args_kses() );
									}
									?>
								</span>
							</li>
							<?php
						}
						if ( is_network_admin() && ualp_multi_site_support_return() ) {
							$adminurl = network_admin_url( 'admin.php?page=ual_pro' );
						} else {
							$adminurl = admin_url( 'admin.php?page=ual_pro' );
						}
						echo '<li>';
						echo '<p><a href="' . esc_url( $adminurl ) . '" target="_blank" class="button button-default">' . esc_html__( 'Read More', 'user_activity_log_pro' ) . ' →</a></p>';
						echo '</li>';
					} else {
						echo '<li>';
						echo '<div>' . esc_html__( 'No Activity found.', 'user_activity_log_pro' ) . '</div>';
						echo '</li>';
					}
					?>

				</ul>
				<?php
			}
		}
	}
}
add_action( 'plugins_loaded', 'ualp_latest_activity_logs' );

if ( ! function_exists( 'ualp_user_role_activity_logs' ) ) {
	/**
	 * Add Admin Dashboard Widget - User/Role Wise Activity Logs.
	 */
	function ualp_user_role_activity_logs() {
		// Register the new dashboard widget with the 'wp_dashboard_setup' action.
		add_action( 'wp_dashboard_setup', 'ualp_urlatest_logs' );
		if ( ! function_exists( 'ualp_urlatest_logs' ) ) {
			/**
			 * URL Latest Logs.
			 */
			function ualp_urlatest_logs() {
				add_screen_option(
					'layout_columns',
					array(
						'max'     => 3,
						'default' => 2,
					)
				);
				$ualp_role_access_list      = get_option( 'ualpRoleAccessList' );
				$ualp_user_access_list      = get_option( 'ualpUserAccessList' );
				$ualp_allow_dashbord_widget = get_option( 'ualpAllowDashbordWidget', '1' );
				if ( 1 == $ualp_allow_dashbord_widget ) {
					$current_user      = wp_get_current_user();
					$current_user_name = $current_user->ID;
					if ( isset( $current_user->roles[0] ) ) {
						$current_user_role = $current_user->roles[0];
					} else {
						if ( is_multisite() ) {
							$id       = get_current_user_id();
							$usermeta = get_user_meta( $id, 'wp_capabilities' );
							if ( is_array( $usermeta[0] ) ) {
								$current_userrolls = array_keys( $usermeta[0] );
							} else {
								$current_userrolls = $usermeta;
							}
							if ( isset( $current_userrolls[0] ) ) {
								$current_user_role = $current_userrolls[0];
							}
						}
					}
					$ualp_role = '';
					$ualp_user = '';
					if ( is_array( $ualp_role_access_list ) && ! empty( $ualp_role_access_list ) ) {
						if ( count( $ualp_role_access_list ) > 1 ) {
							foreach ( $ualp_role_access_list as $ualp_role ) {
								$ualp_role .= $ualp_role;
							}
						}
					}
					if ( is_array( $ualp_user_access_list ) && ! empty( $ualp_user_access_list ) ) {
						if ( count( $ualp_user_access_list ) > 1 ) {
							foreach ( $ualp_user_access_list as $ualp_user ) {
								$ualp_user .= $ualp_user;
							}
						}
					}
					if ( is_array( $ualp_role_access_list ) && is_array( $ualp_user_access_list ) && ! empty( $ualp_role_access_list ) && ! empty( $ualp_user_access_list ) && isset( $current_user_role ) && 'administrator' != $current_user_role ) {
						if ( ( isset( $current_user_role ) && $current_user_role == $ualp_role ) || ( isset( $current_user_name ) && $current_user_name == $ualp_user ) ) {
							add_meta_box( 'ualpro_dashboard_widget', esc_html__( 'User/Role Wise Activity Logs', 'user_activity_log_pro' ), 'ualp_dashboard_log_stats', 'dashboard', 'normal', 'high' );
						}
					} elseif ( is_array( $ualp_role_access_list ) && ! empty( $ualp_role_access_list ) && isset( $current_user_role ) && 'administrator' != $current_user_role ) {
						foreach ( $ualp_role_access_list as $ualp_role ) {
							if ( isset( $current_user_role ) && $current_user_role == $ualp_role ) {
								add_meta_box( 'ualpro_dashboard_widget', esc_html__( 'User/Role Wise Activity Logs', 'user_activity_log_pro' ), 'ualp_dashboard_log_stats', 'dashboard', 'normal', 'high' );
							}
						}
					} elseif ( is_array( $ualp_user_access_list ) && ! empty( $ualp_user_access_list ) && isset( $current_user_role ) && 'administrator' != $current_user_role ) {
						foreach ( $ualp_user_access_list as $ualp_user ) {
							if ( isset( $current_user_name ) && $current_user_name == $ualp_user ) {
								add_meta_box( 'ualpro_dashboard_widget', esc_html__( 'User/Role Wise Activity Logs', 'user_activity_log_pro' ), 'ualp_dashboard_log_stats', 'dashboard', 'normal', 'high' );
							}
						}
					} else {
						if ( isset( $current_user_role ) && 'administrator' == $current_user_role ) {
							add_meta_box( 'ualpro_dashboard_widget', esc_html__( 'User/Role Wise Activity Logs', 'user_activity_log_pro' ), 'ualp_dashboard_log_stats', 'dashboard', 'normal', 'high' );
						}
					}
				}
			}
		}
		/**
		 * Dashboard Log Stats.
		 */
		function ualp_dashboard_log_stats() {
			global $wpdb, $wp_roles;
			$all_roles                = $wp_roles->roles;
			$table_name               = UALP_TABLENAME;
			$stats_labels             = array();
			$stats_labels_to_datetime = array();
			$arr_dataset_data         = array();
			$num_days                 = 28;
			$user_role                = '';
			$user_role_data           = '';
			if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'ajax-nonce' ) ) {
				$user_role      = isset( $_POST['user_role'] ) ? sanitize_text_field( wp_unslash( $_POST['user_role'] ) ) : '';
				$user_role_data = isset( $_POST['user_role_data'] ) ? sanitize_text_field( wp_unslash( $_POST['user_role_data'] ) ) : '';
			}
			$stats_period_start_date = DateTime::createFromFormat( 'U', strtotime( "-$num_days days" ) );
			$stats_period_end_date   = DateTime::createFromFormat( 'U', time() );
			$interval                = DateInterval::createFromDateString( '1 day' );
			$period                  = new DatePeriod( $stats_period_start_date, $interval, $stats_period_end_date->add( date_interval_create_from_date_string( '1 days' ) ) );
			if ( 'role-wise' == $user_role ) {
				if ( ! empty( $user_role_data ) ) {
					foreach ( $period as $dt ) {
						$datef         = _x( 'M j', 'stats: date in rows per day chart', 'user-activity-log-pro' );
						$str_date      = date_i18n( $datef, $dt->getTimestamp() );
						$str_date_ymd  = gmdate( 'Y-m-d', $dt->getTimestamp() );
						$start_date    = $dt->format( 'Y-m-d 00:00:00' );
						$enddate       = $dt->format( 'Y-m-d 23:59:59' );
						$get_log_query = $wpdb->prepare( 'SELECT COUNT(*) FROM %s WHERE modified_date >= %s AND  modified_date < %s AND user_role = %s ORDER BY modified_date desc', $table_name, $start_date, $enddate, $user_role_data );

						$result_log_query = $wpdb->get_var( $wpdb->prepare( 'SELECT COUNT(*) FROM %s WHERE modified_date >= %s AND  modified_date < %s AND user_role = %s ORDER BY modified_date desc', $table_name, $start_date, $enddate, $user_role_data ) );
						$stats_labels[]   = $str_date;

						$stats_labels_to_datetime[] = array(
							'label' => $str_date,
							'date'  => $str_date_ymd,
						);
						if ( $result_log_query ) {
							$arr_dataset_data[] = $result_log_query;
						} else {
							$arr_dataset_data[] = 0;
						}
					}
				}
			} elseif ( 'user-wise' == $user_role ) {
				if ( ! empty( $user_role_data ) ) {
					foreach ( $period as $dt ) {
						$datef            = _x( 'M j', 'stats: date in rows per day chart', 'user-activity-log-pro' );
						$str_date         = date_i18n( $datef, $dt->getTimestamp() );
						$str_date_ymd     = gmdate( 'Y-m-d', $dt->getTimestamp() );
						$start_date       = $dt->format( 'Y-m-d 00:00:00' );
						$enddate          = $dt->format( 'Y-m-d 23:59:59' );
						$result_log_query = $wpdb->get_var( $wpdb->prepare( 'SELECT COUNT(*) FROM %s WHERE modified_date >= %s AND  modified_date < %s AND user_name = %s ORDER BY modified_date desc', $table_name, $start_date, $enddate, $user_role_data ) );
						$stats_labels[]   = $str_date;

						$stats_labels_to_datetime[] = array(
							'label' => $str_date,
							'date'  => $str_date_ymd,
						);
						if ( $result_log_query ) {
							$arr_dataset_data[] = $result_log_query;
						} else {
							$arr_dataset_data[] = 0;
						}
					}
				}
			} else {
				foreach ( $period as $dt ) {
					$datef            = _x( 'M j', 'stats: date in rows per day chart', 'user-activity-log-pro' );
					$str_date         = date_i18n( $datef, $dt->getTimestamp() );
					$str_date_ymd     = gmdate( 'Y-m-d', $dt->getTimestamp() );
					$start_date       = $dt->format( 'Y-m-d 00:00:00' );
					$enddate          = $dt->format( 'Y-m-d 23:59:59' );
					$result_log_query = $wpdb->get_var( $wpdb->prepare( " SELECT COUNT(*) FROM %1s WHERE modified_date >= '%1s' AND  modified_date < '%1s' ORDER BY modified_date desc", $table_name, $start_date, $enddate ) );
					$stats_labels[]   = $str_date;

					$stats_labels_to_datetime[] = array(
						'label' => $str_date,
						'date'  => $str_date_ymd,
					);
					if ( $result_log_query ) {
						$arr_dataset_data[] = $result_log_query;
					} else {
						$arr_dataset_data[] = 0;
					}
				}
			}
			$sum_of_count         = array_sum( $arr_dataset_data );
			$ualp_chat_label      = wp_json_encode( $stats_labels );
			$ualp_chat_date_value = wp_json_encode( $arr_dataset_data );
			?>
			<input type="hidden" class="ualp_chat_label" value="<?php echo esc_attr( $ualp_chat_label ); ?>" />
			<input type="hidden" class="ualp_chat_date_value" value="<?php echo esc_attr( $ualp_chat_date_value ); ?>" />
			<input type="hidden" class="user_role_wise_select" id="user_role_wise_select" value="" />
			<input type="hidden" class="user_role_select" id="user_role_select" value="" />
			<div>
				<select id="user-role-wise" class="user-role-wise">
					<option value="user-wise"><?php esc_html_e( 'User', 'user_activity_log_pro' ); ?></option>
					<option value="role-wise"><?php esc_html_e( 'Role', 'user_activity_log_pro' ); ?></option>
				</select>
				<?php
				if ( 'user-wise' == $user_role ) {
					$uact_user_role = $wpdb->get_results( 'SELECT * FROM ' . $wpdb->prefix . 'users' );
					?>
				<select name="user-role-data" class="user-role-data">
					<?php foreach ( $uact_user_role as $all_data ) { ?>
						<option value="<?php echo esc_attr( $all_data->display_name ); ?>"><?php echo esc_html( $all_data->display_name ); ?></option>
					<?php } ?>
				</select>
				<?php } elseif ( 'role-wise' == $user_role ) { ?>
					<select name="user-role-data" class="user-role-data">
						<?php foreach ( $all_roles as $all_data ) { ?>
							<option value="<?php echo esc_attr( $all_data['name'] ); ?>"><?php echo esc_html( $all_data['name'] ); ?></option>
						<?php } ?>
					</select>
				<?php } else { ?>
					<select name="user-role-data" class="user-role-data" style="display: none;">
					<option value="">Select any option</option>
					</select>
				<?php } ?>
			</div>
			<div style="margin-bottom:20px; font-size:14px; text-align:center;margin-top:20px;">
				<?php
				printf(
					// translators:.
					wp_kses( '<b>%1$s events</b> have been log the last <b>28 days</b>.', ualp_args_kses() ),
					esc_attr( $sum_of_count )
				);
				?>
			</div>
			<div style="position: relative; height: 0; overflow: hidden; padding-bottom: 50%;">
				<canvas style="position: absolute; left: 0; right: 0;" id="myChart" width="100" height="50"></canvas>
			</div>
				<script>
				var ctx = document.getElementById('myChart').getContext('2d');
				var chartLabels =  JSON.parse( jQuery(".ualp_chat_label").val() );
					var ualp_chat_date_value =  JSON.parse( jQuery(".ualp_chat_date_value").val() );
				var myChart = new Chart(ctx, {
					type: 'bar',
					data: {
						labels: chartLabels,
						datasets: [{
							label: '',
							data: ualp_chat_date_value,
							backgroundColor: "rgba(52, 64, 80, 1)",
							hoverBackgroundColor: "rgba(77, 92, 113, 1)",
							borderWidth: 1
						}]
					},
					options: {
						responsive: true,
						legend: {
								display: false
							},
						scales: {
							yAxes: [{
								ticks: {
									beginAtZero: true
								}
							}],
							xAxes: [{
									display: true
								}]
						}
					}
				});
				</script>
			<?php
		}
		if ( ! function_exists( 'ualp_dashboard_widget' ) ) {
			/**
			 * Dashboard Widget.
			 */
			function ualp_dashboard_widget() {
				?>
				<ul class="ualp_log_table">
					<li>
						<span class="ualp-column-date"><b><?php esc_html_e( 'Date', 'user_activity_log_pro' ); ?></b></span>
						<span class="ualp-column-author"><b><?php esc_html_e( 'Author', 'user_activity_log_pro' ); ?></b></span>
						<span class="ualp-column-activity"><b><?php esc_html_e( 'Activity', 'user_activity_log_pro' ); ?></b></span>
					</li>
					<?php
					global $wpdb, $externaldb;
					$where       = ' WHERE 1=1';
					$super_admin = is_super_admin( get_current_user_id() );
					if ( ualp_multi_site_support_return() ) {
						if ( 1 != $super_admin ) {
							$where .= " AND user_id = '" . get_current_user_id() . "'";
						}
					}
					if ( ualp_multi_site_support_return() ) {
						$where .= " AND site_id = '" . get_current_blog_id() . "'";
					}
					// $number_of_widget_logs   = get_option('ualp_number_of_widget_logs');.
					// if (!$number_of_widget_logs){.
						$number_of_widget_logs = 5;
					// }.
					$select_query = 'SELECT * from ' . UALP_TABLENAME . $where . ' ORDER BY modified_date desc LIMIT ' . $number_of_widget_logs;
					$get_data     = $externaldb->get_results( $select_query );
					if ( $get_data ) {
						$srno = 1;
						foreach ( $get_data as $data ) {
							?>
							<li>
								<span class="ualp-column-date">
								<?php
									$modified_date = strtotime( $data->modified_date );
									$date_format   = get_option( 'date_format' );
									$time_format   = get_option( 'time_format' );
									$date          = gmdate( $date_format, $modified_date );
									$time          = gmdate( $time_format, $modified_date );
									echo esc_html( $date ) . '<br/>' . esc_html( $time ) . '<br/>';
									$current_t = current_time( 'timestamp' );
									$t_diff    = human_time_diff( $modified_date, $current_t );
									echo '<b>' . esc_html( $t_diff ) . ' ' . esc_html__( 'ago', 'user_activity_log_pro' ) . '</b>'
								?>
								</span>
								<span class="ualp-column-author column-author">
								<?php
									global $wp_roles;
								if ( ! empty( $data->user_id ) && 0 !== (int) $data->user_id ) {
									$user = get_user_by( 'id', $data->user_id );
									if ( $user instanceof WP_User && 0 !== $user->ID ) {
										printf(
											'<a href="%s">%s <span class="author-name">%s</span></a><br /><small>%s</small><br/>%s',
											get_edit_user_link( $user->ID ),
											get_avatar( $user->ID, 40 ),
											ucfirst( $user->display_name ),
											isset( $user->roles[0] ) &&
												isset( $wp_roles->role_names[ $user->roles[0] ] ) ? esc_html( $wp_roles->role_names[ $user->roles[0] ] ) : '',
											esc_html( $user->user_email )
										);
									}
								} else {
									esc_html_e( 'Guest', 'user_activity_log_pro' );
								}
								?>
								</span>
								<span class="ualp-column-activity">
									<?php
									if ( 'Page' == $data->object_type && 'Viewed' == $data->action ) {
										echo wp_kses( ucfirst( $data->object_type ), ualp_args_kses() ) . ' ' . wp_kses( ucfirst( $data->action ), ualp_args_kses() ) . ' : ' . esc_html( $data->post_title );
									} else {
										echo wp_kses( $data->description, ualp_args_kses() );
									}
									?>
								</span>
							</li>
							<?php
						}
						if ( is_network_admin() && ualp_multi_site_support_return() ) {
							$adminurl = network_admin_url( 'admin.php?page=ual_pro' );
						} else {
							$adminurl = admin_url( 'admin.php?page=ual_pro' );
						}
						echo '<li>';
						echo '<p><a href="' . esc_url( $adminurl ) . '" target="_blank" class="button button-default">' . esc_html__( 'Read More', 'user_activity_log_pro' ) . ' →</a></p>';
						echo '</li>';
					} else {
						echo '<li>';
						echo '<div>' . esc_html__( 'No Activity found.', 'user_activity_log_pro' ) . '</div>';
						echo '</li>';
					}
					?>

				</ul>
				<?php
			}
		}
	}
}
add_action( 'plugins_loaded', 'ualp_user_role_activity_logs' );

add_action( 'admin_head', 'ualp_create_upload_folder' );

if ( ! function_exists( 'ualp_create_upload_folder' ) ) {
	/**
	 * Creates Upload Folder.
	 */
	function ualp_create_upload_folder() {
		if ( ! is_dir( WP_CONTENT_DIR . '/uploads' ) ) {
			mkdir( WP_CONTENT_DIR . '/uploads' );
		}
	}
}

if ( ! function_exists( 'ualp_external' ) ) {
	/**
	 * External DB.
	 */
	function ualp_external() {
		global $wpdb,$externaldb;

		$dbuser      = '';
		$dbpassword  = '';
		$dbname      = '';
		$dbhost      = '';
		$tableprefix = '';
		$externaldb  = $wpdb;
		$tableprefix = get_option( 'table_prefix' );
		$dbuser      = get_option( 'database_username' );
		$dbpassword  = get_option( 'database_password' );
		$dbname      = get_option( 'database_name' );
		$dbhost      = get_option( 'database_hostname' );

		if ( ( isset( $dbuser ) && '' != $dbuser ) && ( isset( $dbpassword ) ) && ( isset( $dbname ) && '' != $dbname ) && ( isset( $dbhost ) && '' != $dbhost ) ) {
			$externaldb = new wpdb( $dbuser, $dbpassword, $dbname, $dbhost );

			$plugin_data     = get_plugin_data( WP_PLUGIN_DIR . '/user-activity-log-pro/user_activity_log_pro.php', $markup = true, $translate = true );
			$current_version = $plugin_data['Version'];
			ualp_create_table();
			ualp_create_detail_table();
			ualp_create_external_table();
			ualp_create_detail_external_table();
			ualp_create_login_external_table();
			ualp_create_email_notification_table();
			ualp_create_sms_notification_table();
			ualp_create_email_notification_external_table();
			ualp_create_sms_notification_external_table();
			ualp_create_schedule_report();
			ualp_create_schedule_external_report();
			update_option( 'ualp_version', $current_version );
		}
		return $externaldb;
	}
}

if ( ! function_exists( 'ualp_migrate_current_to_external' ) ) {
	/**
	 * Migrate Current DB to External DB.
	 */
	function ualp_migrate_current_to_external() {
		global $externaldb,$wpdb;
		$tableprefix = '';
		$tableprefix = get_option( 'table_prefix' );
		$dbuser      = get_option( 'database_username' );
		$dbpassword  = get_option( 'database_password' );
		$dbname      = get_option( 'database_name' );
		$dbhost      = get_option( 'database_hostname' );
		if ( ( isset( $dbuser ) && '' != $dbuser ) && ( isset( $dbpassword ) ) && ( isset( $dbname ) && '' != $dbname ) && ( isset( $dbhost ) && '' != $dbhost ) ) {
			ualp_create_table();
			ualp_create_detail_table();
			ualp_create_external_table();
			ualp_create_detail_external_table();
			ualp_create_login_external_table();
			ualp_create_email_notification_table();
			ualp_create_sms_notification_table();
			ualp_create_email_notification_external_table();
			ualp_create_sms_notification_external_table();
			ualp_create_schedule_report();
			ualp_create_schedule_external_report();

			$results = $wpdb->get_results( "SELECT * FROM  {$wpdb->prefix}ualp_user_activity", OBJECT );

			foreach ( $results as $single_result ) {
				$uactid        = $single_result->uactid;
				$post_id       = $single_result->post_id;
				$favorite      = $single_result->favorite;
				$post_title    = $single_result->post_title;
				$user_id       = $single_result->user_id;
				$user_name     = $single_result->user_name;
				$user_role     = $single_result->user_role;
				$user_email    = $single_result->user_email;
				$ip_address    = $single_result->ip_address;
				$user_agent    = $single_result->user_agent;
				$country       = $single_result->country;
				$modified_date = $single_result->modified_date;
				$object_type   = $single_result->object_type;
				$action        = $single_result->action;
				$hook          = $single_result->hook;
				$description   = $single_result->description;
				$site_id       = $single_result->site_id;

				$insert_query = $externaldb->insert(
					UALP_TABLENAME,
					array(
						'post_id'       => $post_id,
						'favorite'      => '0',
						'post_title'    => $post_title,
						'user_id'       => $user_id,
						'user_name'     => $user_name,
						'user_role'     => $user_role,
						'user_email'    => $user_email,
						'ip_address'    => $ip_address,
						'user_agent'    => $user_agent,
						'country'       => $country,
						'modified_date' => $modified_date,
						'object_type'   => $object_type,
						'action'        => $action,
						'hook'          => $hook,
						'description'   => $description,
						'site_id'       => $site_id,
					)
				);

			}

			$detail_results = $wpdb->get_results( "SELECT * FROM  {$wpdb->prefix}ualp_user_activity_detail", OBJECT );

			foreach ( $detail_results as $single_result ) {
				$uactd_id    = $single_result->uactd_id;
				$uactid      = $single_result->uactid;
				$uactd_key   = $single_result->uactd_key;
				$uactd_value = $single_result->uactd_value;

				$externaldb->insert(
					$tableprefix . 'ualp_user_activity_detail',
					array(
						'uactd_id'    => '',
						'uactid'      => $uactid,
						'uactd_key'   => $uactd_key,
						'uactd_value' => $uactd_value,
					)
				);
			}
			$login_details_results = $externaldb->get_results( 'SELECT * FROM ' . $tableprefix . 'ualp_user_activity_login', OBJECT );
			if ( is_array( $login_details_results ) && ! empty( $login_details_results ) ) {
				foreach ( $login_details_results as $single_result ) {
					$uacltid        = $single_result->uacltid;
					$user_id        = $single_result->user_id;
					$user_name      = $single_result->user_name;
					$user_role      = $single_result->user_role;
					$user_email     = $single_result->user_email;
					$expiration     = $single_result->expiration;
					$session_token  = $single_result->session_token;
					$time_login     = $single_result->time_login;
					$time_logout    = $single_result->time_logout;
					$time_last_seen = $single_result->time_last_seen;
					$ip_address     = $single_result->ip_address;
					$country        = $single_result->country;
					$user_agent     = $single_result->user_agent;
					$login_status   = $single_result->login_status;
					$site_id        = $single_result->site_id;
					$wpdb->insert(
						$wpdb->prefix . 'ualp_user_activity_login',
						array(
							'uacltid'        => '',
							'user_id'        => $user_id,
							'user_name'      => $user_name,
							'user_role'      => $user_role,
							'user_email'     => $user_email,
							'expiration'     => $expiration,
							'session_token'  => $session_token,
							'time_login'     => $time_login,
							'time_logout'    => $time_logout,
							'time_last_seen' => $time_last_seen,
							'ip_address'     => $ip_address,
							'country'        => $country,
							'user_agent'     => $user_agent,
							'login_status'   => $login_status,
							'site_id'        => $site_id,
						)
					);
				}
			}
			$email_details_results = $externaldb->get_results( 'SELECT * FROM ' . $tableprefix . 'ualp_email_notification', OBJECT );
			if ( is_array( $email_details_results ) && ! empty( $email_details_results ) ) {
				foreach ( $email_details_results as $single_result ) {
					$uacteid             = $single_result->uacteid;
					$email_title         = $single_result->email_title;
					$email_content       = $single_result->email_content;
					$user_id             = $single_result->user_id;
					$user_name           = $single_result->user_name;
					$user_role           = $single_result->user_role;
					$user_email          = $single_result->user_email;
					$site_id             = $single_result->site_id;
					$custom_trigger_data = $single_result->custom_trigger_data;
					$email_address       = $single_result->email_address;
					$email_template      = $single_result->email_template;

					$wpdb->insert(
						$wpdb->prefix . 'ualp_email_notification',
						array(
							'uacteid'             => '',
							'email_title'         => $email_title,
							'email_content'       => $email_content,
							'user_id'             => $user_id,
							'user_name'           => $user_name,
							'user_role'           => $user_role,
							'user_email'          => $user_email,
							'site_id'             => $site_id,
							'custom_trigger_data' => $custom_trigger_data,
							'email_address'       => $email_address,
							'email_template'      => $email_template,
						)
					);
				}
			}
			$schedule_report_details_results = $externaldb->get_results( 'SELECT * FROM ' . $tableprefix . 'ualp_schedule_report', OBJECT );
			if ( is_array( $schedule_report_details_results ) && ! empty( $schedule_report_details_results ) ) {
				foreach ( $schedule_report_details_results as $single_result ) {
					$uactrid           = $single_result->uactrid;
					$report_title      = $single_result->report_title;
					$schedule_email_id = $single_result->schedule_email_id;
					$report_frequency  = $single_result->report_frequency;

					$wpdb->insert(
						$wpdb->prefix . 'ualp_schedule_report',
						array(
							'uactrid'           => '',
							'report_title'      => $report_title,
							'schedule_email_id' => $schedule_email_id,
							'report_frequency'  => $report_frequency,
						)
					);
				}
			}
		}
	}
}

if ( ! function_exists( 'ualp_migrate_external_to_current' ) ) {
	/**
	 * Migrate External DB to Current DB.
	 */
	function ualp_migrate_external_to_current() {
		global $externaldb,$wpdb;
		$tableprefix = '';
		$tableprefix = get_option( 'table_prefix' );
		$dbuser      = get_option( 'database_username' );
		$dbpassword  = get_option( 'database_password' );
		$dbname      = get_option( 'database_name' );
		$dbhost      = get_option( 'database_hostname' );
		if ( ( isset( $dbuser ) && '' != $dbuser ) && ( isset( $dbpassword ) ) && ( isset( $dbname ) && '' != $dbname ) && ( isset( $dbhost ) && '' != $dbhost ) ) {
			ualp_create_table();
			ualp_create_detail_table();
			ualp_create_external_table();
			ualp_create_detail_external_table();
			ualp_create_login_external_table();
			ualp_create_email_notification_table();
			ualp_create_sms_notification_table();
			ualp_create_email_notification_external_table();
			ualp_create_sms_notification_external_table();
			ualp_create_schedule_report();
			ualp_create_schedule_external_report();

			$results = $externaldb->get_results( 'SELECT * FROM ' . $tableprefix . 'ualp_user_activity', OBJECT );

			foreach ( $results as $single_result ) {
				$uactid        = $single_result->uactid;
				$post_id       = $single_result->post_id;
				$favorite      = $single_result->favorite;
				$post_title    = $single_result->post_title;
				$user_id       = $single_result->user_id;
				$user_name     = $single_result->user_name;
				$user_role     = $single_result->user_role;
				$user_email    = $single_result->user_email;
				$ip_address    = $single_result->ip_address;
				$user_agent    = $single_result->user_agent;
				$country       = $single_result->country;
				$modified_date = $single_result->modified_date;
				$object_type   = $single_result->object_type;
				$action        = $single_result->action;
				$hook          = $single_result->hook;
				$description   = $single_result->description;
				$site_id       = $single_result->site_id;
				$wpdb->insert(
					$wpdb->prefix . 'ualp_user_activity',
					array(
						'uactid'        => '',
						'post_id'       => $post_id,
						'favorite'      => $favorite,
						'post_title'    => $post_title,
						'user_id'       => $user_id,
						'user_name'     => $user_name,
						'user_role'     => $user_role,
						'user_email'    => $user_email,
						'ip_address'    => $ip_address,
						'user_agent'    => $user_agent,
						'country'       => $country,
						'modified_date' => $modified_date,
						'object_type'   => $object_type,
						'action'        => $action,
						'hook'          => $hook,
						'description'   => $description,
						'site_id'       => $site_id,
					),
					array(
						'%s',
						'%s',
						'%s',
						'%s',
						'%s',
						'%s',
						'%s',
						'%s',
						'%s',
						'%s',
						'%s',
						'%s',
						'%s',
						'%s',
						'%s',
					)
				);
			}

			$detail_results = $externaldb->get_results( 'SELECT * FROM ' . $tableprefix . 'ualp_user_activity_detail', OBJECT );

			foreach ( $detail_results as $single_result ) {
				$uactd_id    = $single_result->uactd_id;
				$uactid      = $single_result->uactid;
				$uactd_key   = $single_result->uactd_key;
				$uactd_value = $single_result->uactd_value;

				$wpdb->insert(
					$wpdb->prefix . 'ualp_user_activity_detail',
					array(
						'uactd_id'    => '',
						'uactid'      => $uactid,
						'uactd_key'   => $uactd_key,
						'uactd_value' => $uactd_value,
					)
				);
			}
			$login_details_results = $externaldb->get_results( 'SELECT * FROM ' . $tableprefix . 'ualp_user_activity_login', OBJECT );
			if ( is_array( $login_details_results ) && ! empty( $login_details_results ) ) {
				foreach ( $login_details_results as $single_result ) {
					$uacltid        = $single_result->uacltid;
					$user_id        = $single_result->user_id;
					$user_name      = $single_result->user_name;
					$user_role      = $single_result->user_role;
					$user_email     = $single_result->user_email;
					$expiration     = $single_result->expiration;
					$session_token  = $single_result->session_token;
					$time_login     = $single_result->time_login;
					$time_logout    = $single_result->time_logout;
					$time_last_seen = $single_result->time_last_seen;
					$ip_address     = $single_result->ip_address;
					$country        = $single_result->country;
					$user_agent     = $single_result->user_agent;
					$login_status   = $single_result->login_status;
					$site_id        = $single_result->site_id;
					$wpdb->insert(
						$wpdb->prefix . 'ualp_user_activity_login',
						array(
							'uacltid'        => '',
							'user_id'        => $user_id,
							'user_name'      => $user_name,
							'user_role'      => $user_role,
							'user_email'     => $user_email,
							'expiration'     => $expiration,
							'session_token'  => $session_token,
							'time_login'     => $time_login,
							'time_logout'    => $time_logout,
							'time_last_seen' => $time_last_seen,
							'ip_address'     => $ip_address,
							'country'        => $country,
							'user_agent'     => $user_agent,
							'login_status'   => $login_status,
							'site_id'        => $site_id,
						)
					);
				}
			}
			$email_details_results = $externaldb->get_results( 'SELECT * FROM ' . $tableprefix . 'ualp_email_notification', OBJECT );
			if ( is_array( $email_details_results ) && ! empty( $email_details_results ) ) {
				foreach ( $email_details_results as $single_result ) {
					$uacteid             = $single_result->uacteid;
					$email_title         = $single_result->email_title;
					$email_content       = $single_result->email_content;
					$user_id             = $single_result->user_id;
					$user_name           = $single_result->user_name;
					$user_role           = $single_result->user_role;
					$user_email          = $single_result->user_email;
					$site_id             = $single_result->site_id;
					$custom_trigger_data = $single_result->custom_trigger_data;
					$email_address       = $single_result->email_address;
					$email_template      = $single_result->email_template;

					$wpdb->insert(
						$wpdb->prefix . 'ualp_email_notification',
						array(
							'uacteid'             => '',
							'email_title'         => $email_title,
							'email_content'       => $email_content,
							'user_id'             => $user_id,
							'user_name'           => $user_name,
							'user_role'           => $user_role,
							'user_email'          => $user_email,
							'site_id'             => $site_id,
							'custom_trigger_data' => $custom_trigger_data,
							'email_address'       => $email_address,
							'email_template'      => $email_template,
						)
					);
				}
			}
			$schedule_report_details_results = $externaldb->get_results( 'SELECT * FROM ' . $tableprefix . 'ualp_schedule_report', OBJECT );
			if ( is_array( $schedule_report_details_results ) && ! empty( $schedule_report_details_results ) ) {
				foreach ( $schedule_report_details_results as $single_result ) {
					$uactrid           = $single_result->uactrid;
					$report_title      = $single_result->report_title;
					$schedule_email_id = $single_result->schedule_email_id;
					$report_frequency  = $single_result->report_frequency;

					$wpdb->insert(
						$wpdb->prefix . 'ualp_user_activity_detail',
						array(
							'uactd_id'          => '',
							'report_title'      => $report_title,
							'schedule_email_id' => $schedule_email_id,
							'report_frequency'  => $report_frequency,
						)
					);
				}
			}
		}
	}
}

if ( ! function_exists( 'ualp_allow_ip' ) ) {
	/**
	 * Function to check if Ipaddress is allowed.
	 */
	function ualp_allow_ip() {
		$ualp_allow_ip = get_option( 'ualp_allow_ip' );

		if ( 1 == $ualp_allow_ip ) {
			return true;
		}
		return false;
	}
}

if ( ! function_exists( 'ualp_allow_country' ) ) {
	/**
	 * Function to check if Country is allowed.
	 */
	function ualp_allow_country() {
		$ualp_allow_country = get_option( 'ualp_allow_country' );
		if ( 1 == $ualp_allow_country ) {
			return true;
		}
		return false;
	}
}

if ( ! function_exists( 'ualp_allow_user_agent' ) ) {
	/**
	 * Function to check if User Agent is allowed.
	 */
	function ualp_allow_user_agent() {
		$ualp_allow_user_agent = get_option( 'ualp_allow_user_agent' );
		if ( 1 == $ualp_allow_user_agent ) {
			return true;
		}
		return false;
	}
}

if ( ! function_exists( 'ualp_get_current_token' ) ) {
	/**
	 * Function to get current session Token.
	 *
	 * @param string $token Token.
	 */
	function ualp_get_current_token( $token = '' ) {
		if ( '' == $token ) {
			$token = wp_get_session_token();
		}
		if ( function_exists( 'hash' ) ) {
				return hash( 'sha256', $token );
		} else {
				return sha1( $token );
		}
	}
}

if ( ! function_exists( 'ualp_add_email_settings' ) ) {
	/**
	 * User activity email Settings.
	 */
	function ualp_add_email_settings() {
		$all_email_notification = '';
		$email_title            = '';
		$email_address          = '';
		$email_template         = 0;
		$custom_trigger_data    = array();
		if ( isset( $_GET['enotification'] ) ) {
			$enotification = sanitize_text_field( wp_unslash( $_GET['enotification'] ) );
			global $wpdb, $externaldb;
			$email_query                = 'SELECT * FROM ' . UALP_EMAIL_TABLENAME . ' where site_id = ' . get_current_blog_id() . ' AND uacteid =' . $enotification;
			$email_notification_content = $externaldb->get_results( $email_query );
			foreach ( $email_notification_content as $all_trigger_notification ) {
				$all_email_notification = $all_trigger_notification->email_content;
				$email_title            = $all_trigger_notification->email_title;
				$email_address          = $all_trigger_notification->email_address;
				$email_template         = $all_trigger_notification->email_template;
				$custom_trigger_data    = maybe_unserialize( $all_trigger_notification->custom_trigger_data );
			}
		}
		if ( isset( $_POST['saveTrigger'] ) && isset( $_POST['_wp_trigger_notify_nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wp_trigger_notify_nonce'] ) ), '_wp_trigger_notify_action' ) ) {
			$custom_trigger_operator_with = '';
			$custom_trigger_data          = array();
			$i                            = 0;
			$empty_count                  = 0;
			$j                            = 0;
			if ( ( isset( $_POST['custom_trigger_operator_with'] ) && '' != $_POST['custom_trigger_operator_with'] ) ||
				( isset( $_POST['custom_trigger_name'] ) && '' != $_POST['custom_trigger_name'] ) || ( isset( $_POST['custom_trigger_with'] ) && '' != $_POST['custom_trigger_with'] ) ) {
				$custom_trigger_operator_with = array_map( 'sanitize_text_field', wp_unslash( $_POST['custom_trigger_operator_with'] ) );
				$custom_trigger_name          = array_map( 'sanitize_text_field', wp_unslash( $_POST['custom_trigger_name'] ) );
				$custom_trigger_with          = array_map( 'sanitize_text_field', wp_unslash( $_POST['custom_trigger_with'] ) );
				foreach ( $custom_trigger_name as $custom_trigger_name_single ) {
					$j++;
					$custom_trigger_value[] = isset( $_POST[ 'custom_trigger_value_' . $j ] ) ? array_map( 'sanitize_text_field', wp_unslash( $_POST[ 'custom_trigger_value_' . $j ] ) ) : '';
				}
			}
			if ( is_array( $custom_trigger_operator_with ) && ! empty( $custom_trigger_operator_with ) ) {
				if ( count( $custom_trigger_operator_with ) > 0 ) {
					foreach ( $custom_trigger_operator_with as $custom_trigger_operator_with_single ) {
						if ( '' != $custom_trigger_operator_with_single ) {
							$custom_trigger_data[] = array(
								'custom_trigger_operator_with' => $custom_trigger_operator_with_single,
								'custom_trigger_name'  => $custom_trigger_name[ $i ],
								'custom_trigger_with'  => $custom_trigger_with[ $i ],
								'custom_trigger_value' => $custom_trigger_value[ $i ],
							);
							$i++;
						}
					}
				}
				if ( is_array( $custom_trigger_data ) && ! empty( $custom_trigger_data ) ) {
					if ( count( $custom_trigger_data ) >= 1 ) {
						foreach ( $custom_trigger_data as $custom_trigger_single_data ) {
							$triggeropratorwith = $custom_trigger_single_data['custom_trigger_operator_with'];
							$trigger_name       = $custom_trigger_single_data['custom_trigger_name'];
							$trigger_with       = $custom_trigger_single_data['custom_trigger_with'];
							$trigger_value      = $custom_trigger_single_data['custom_trigger_value'];
							if ( '' == $trigger_name || '' == $trigger_with || '' == $triggeropratorwith || '' == $trigger_value ) {
								$empty_count++;
							}
						}
					}
				}
			}
			if ( '0' == $empty_count && '' != $custom_trigger_data ) {
				$class   = 'updated';
				$message = esc_html__( 'Trigger Notification  is save successfully.', 'user_activity_log_pro' );
				do_action( 'ualp_custom_trigger_log_updated' );
			} elseif ( '0' == $empty_count && count( $custom_trigger_operator_with ) > 0 ) {
				$class   = 'updated';
				$message = esc_html__( 'Trigger Notification deleted successfully.', 'user_activity_log_pro' );
				do_action( 'ualp_custom_trigger_log_updated' );
			} else {
				$class   = 'error';
				$message = esc_html__( 'Trigger Notification is not save. Fill the empty field.', 'user_activity_log_pro' );
			}
			ualp_admin_notice_message( $class, $message );
			$all_email_notification = ( isset( $_POST['txtualpAddEmailMsg'] ) && '' != $_POST['txtualpAddEmailMsg'] ) ? sanitize_text_field( wp_unslash( $_POST['txtualpAddEmailMsg'] ) ) : '';
			$trigger_title          = ( isset( $_POST['trigger_title'] ) && '' != $_POST['trigger_title'] ) ? sanitize_text_field( wp_unslash( $_POST['trigger_title'] ) ) : '';
			$trigger_emailid        = ( isset( $_POST['trigger_emailid'] ) && '' != $_POST['trigger_emailid'] ) ? sanitize_text_field( wp_unslash( $_POST['trigger_emailid'] ) ) : '';
			$email_template         = ( isset( $_POST['email_template'] ) && '' != $_POST['email_template'] ) ? sanitize_text_field( wp_unslash( $_POST['email_template'] ) ) : 0;

			$current_user_id   = get_current_user_id();
			$current_user      = wp_get_current_user();
			$current_user_name = $current_user->user_login;
			if ( isset( $_GET['enotification'] ) ) {
				$enotification = sanitize_text_field( wp_unslash( $_GET['enotification'] ) );
				ualp_update_notification( $enotification, $all_email_notification, $custom_trigger_data, $trigger_title, $trigger_emailid, $email_template );
			} else {
				ualp_insert_email_notification( $current_user_name, $all_email_notification, $custom_trigger_data, $trigger_title, $trigger_emailid, $email_template );
			}
		}
		if ( is_network_admin() && ualp_multi_site_support_return() ) {
			$adminurl = network_admin_url( 'admin.php' );
		} else {
			$adminurl = admin_url( 'admin.php' );
		}
		if ( ! isset( $_GET['enotification'] ) ) {
			$current_user            = wp_get_current_user();
			$user_details            = '[user_details]';
			$all_email_notification  = esc_html__( 'Hi', 'user_activity_log_pro' ) . ' ';
			$all_email_notification .= $current_user->user_login . ',';
			$all_email_notification .= "\n\n" . __( 'Following user has done some activity in your site', 'user_activity_log_pro' ) . " \n$user_details";
			$all_email_notification .= "\n\n" . __( 'Thanks,', 'user_activity_log_pro' ) . " \n";
			$all_email_notification .= home_url();
		}
		$all_users = get_users();
		$qry_str   = isset( $_SERVER['QUERY_STRING'] ) ? sanitize_text_field( wp_unslash( $_SERVER['QUERY_STRING'] ) ) : '';
		?>
		<div class="add_new_email_notification_wrapper">
			<form class="ualpAddEmailNotication" id="triggervalue" action="<?php echo esc_attr( $adminurl ) . '?' . esc_attr( $qry_str ); ?>" method="POST">
				<div class="title"> 
					<h1><?php echo esc_html__( 'Add New Email Notification', 'user_activity_log_pro' ); ?></h1>
				</div>
				<div class="add_multiple_email_notification">
					<input type="text" name="trigger_title" value="<?php echo esc_attr( $email_title ); ?>" class="trigger_title" placeholder="Enter title here" required />
					<div class="display_note"><span><?php echo esc_html__( 'Trigger notification should be matched for a email notification to be sent in this section. email notifications rules by using the AND and OR operands', 'user_activity_log_pro' ); ?></span></div>
				<div class="add_custom_trigger_table">
				<div class="add-new-trigger-main-wrapp">
					<div class="add-new-trigger-title">
					<h3><?php echo esc_html__( 'Trigger', 'user_activity_log_pro' ); ?></h3>
					</div>
					<div class="add-new-trigger-wrapper">
						<div title="<?php esc_html_e( 'Add New Trigger', 'user_activity_log_pro' ); ?>" class="ualp_add_new_trigger_btn dashicons dashicons-plus-alt"></div>
					</div>
				</div>    
			   
					<table class="widefat custom_trigger_table" cellspacing="0">
						<?php
						$j                  = 1;
						$i                  = 0;
						$style              = '';
						$trigger_data_count = count( $custom_trigger_data );
						if ( is_array( $custom_trigger_data ) && ! empty( $custom_trigger_data ) && isset( $_GET['enotification'] ) ) {
							foreach ( $custom_trigger_data as $custom_trigger_data_single ) {
								$i++;
								?>
								<tr class="ualp_custom_trigger_tr">
									<td style="
									<?php
									if ( 1 == $j ) {
										?>
										display : none;<?php } ?>">
										<span class="trigger-lable"><?php esc_html_e( 'Trigger operator with', 'user_activity_log_pro' ); ?></span><br/>
										<select name="custom_trigger_operator_with[]" class="chosen-filter" required>
											<option value="AND" <?php selected( $custom_trigger_data_single['custom_trigger_operator_with'], 'AND' ); ?>>AND</option>
											<option value="OR" <?php selected( $custom_trigger_data_single['custom_trigger_operator_with'], 'OR' ); ?>  >OR</option>
										</select>
									</td>
									<td>
										<span class="trigger-lable">
										<?php
										esc_html_e( 'Trigger Name', 'user_activity_log_pro' );
										echo ' : ';
										?>
										</span><br/>
										<select name="custom_trigger_name[]" class="chosen-filter custom_trigger_name" required>
											<option value="user_id" <?php selected( $custom_trigger_data_single['custom_trigger_name'], 'user_id' ); ?>>User ID</option>
											<option value="user_name" <?php selected( $custom_trigger_data_single['custom_trigger_name'], 'user_name' ); ?>  >User Name</option>
											<?php if ( ualp_allow_ip() ) { ?>
												<option value="ip_address" <?php selected( $custom_trigger_data_single['custom_trigger_name'], 'ip_address' ); ?>  >Ip Address</option>
											<?php } ?>
											<option value="user_role" <?php selected( $custom_trigger_data_single['custom_trigger_name'], 'user_role' ); ?>  >User Role</option>
											<option value="hook" <?php selected( $custom_trigger_data_single['custom_trigger_name'], 'hook' ); ?>>Activity</option>
										</select>
									</td>
									<td>
										<span class="trigger-lable">
										<?php
										esc_html_e( 'Trigger with', 'user_activity_log_pro' );
										echo ' : ';
										?>
										</span><br/>
										<select name="custom_trigger_with[]" class="chosen-filter">
											<option value="equal" <?php selected( $custom_trigger_data_single['custom_trigger_with'], 'equal' ); ?>>Is Equal</option>
											<option value="not_equal" <?php selected( $custom_trigger_data_single['custom_trigger_with'], 'not_equal' ); ?>  >Not Equal</option>
										</select>
									</td>
									<td class="custom_trigger_value_tr">
										<span class="trigger-lable">
										<?php
										esc_html_e( 'Trigger Value', 'user_activity_log_pro' );
										echo ' : ';
										?>
										</span><br/>
										<div class="ualp_custom_trigger_value">
											<?php
											if ( 'user_id' == $custom_trigger_data_single['custom_trigger_name'] ) {
												?>
												<select name="custom_trigger_value_<?php echo esc_attr( $i ); ?>[]" class="chosen-filter" multiple required>
													<?php
													if ( isset( $all_users ) && ! empty( $all_users ) ) {
														foreach ( $all_users as $get_all_users ) {
															$user_id  = $get_all_users->ID;
															$selected = in_array( $user_id, $custom_trigger_data_single['custom_trigger_value'] ) ? 'selected="selected"' : '';
															echo '<option value="' . esc_attr( $user_id ) . '" ' . esc_attr( $selected ) . '>' . esc_html( $user_id ) . '</option>';
														}
													}
													?>
												</select>
												<?php
											}
											if ( 'user_name' == $custom_trigger_data_single['custom_trigger_name'] ) {
												?>
												<select name="custom_trigger_value_<?php echo esc_attr( $i ); ?>[]" class="chosen-filter" multiple required>
													<?php
													if ( isset( $all_users ) && ! empty( $all_users ) ) {
														foreach ( $all_users as $get_all_users ) {
															$user_name = $get_all_users->user_login;
															$selected  = in_array( $user_name, $custom_trigger_data_single['custom_trigger_value'] ) ? 'selected="selected"' : '';
															echo '<option value="' . esc_attr( $user_name ) . '" ' . esc_attr( $selected ) . '>' . esc_html( $user_name ) . '</option>';
														}
													}
													?>
												</select>
												<?php
											}
											if ( 'user_role' == $custom_trigger_data_single['custom_trigger_name'] ) {
												?>
												<select name="custom_trigger_value_<?php echo esc_attr( $i ); ?>[]" class="chosen-filter" multiple required>
														<?php
														$editable_roles = get_editable_roles();
														foreach ( $editable_roles as $role => $details ) {
															$name       = translate_user_role( $details['name'] );
															$role_label = esc_attr( $role );
															$selected   = in_array( $role_label, $custom_trigger_data_single['custom_trigger_value'] ) ? 'selected="selected"' : '';
															echo '<option value="' . esc_attr( $role_label ) . '" ' . esc_attr( $selected ) . '>' . esc_html( $role_label ) . '</option>';
														}
														?>
													</select>
												<?php
											}
											if ( 'hook' == $custom_trigger_data_single['custom_trigger_name'] ) {
												global $allhookdetails;
													$post_types = ualp_posttypes();
													$taxonomies = ualp_taxonomies();
													$all_hook   = array(
														'activated_plugin',
														'deactivated_plugin',
														'delete_plugin',
														'plugin_modify',
														'plugin_installed',
														'plugin_updated',
														'ualp_export_activity_log',
														'ualp_general_settings_updated',
														'ualp_hook_log_settings_updated',
														'ualp_manage_role',
														'ualp_set_password',
														'ualp_change_password',
														'ualp_email_settings_updated',
														'ualp_custom_event_log_updated',
														'ualp_exclude_log_updated',
														'ualp_insert_email_notification',
														'ualp_update_email_notification',
														'wp_login',
														'wp_login_failed',
														'wp_logout',
														'delete_user',
														'user_register',
														'insert_user_meta',
														'password_reset',
														'theme_file_modify',
														'switch_theme',
														'customize_save',
														'theme_updated',
														'theme_installed',
														'delete_site_transient_update_themes',
														'wp_insert_comment',
														'edit_comment',
														'delete_comment',
														'trash_comment',
														'spam_comment',
														'restore_comment',
														'comment_approve',
														'comment_unapprove',
														'wp_create_nav_menu',
														'delete_nav_menu',
														'admin_head',
														'wp_head',
														'404_page',
														'_core_updated_successfully',
														'import_end',
														'export_wp',
														'widget_update_callback',
														'updated_option',
														'dbdelta_queries',
														'wp_privacy_personal_data_exporters',
														'wp_privacy_personal_data_erasers',
														'wp_update_nav_menu',
														'pre_insert_term',
														'wp_update_term_parent',
														'delete_themes',
														'before_delete_post',
														'ualpSettingsUpdated',
														'pre_delete_term',
														'sidebar_admin_setup',
														'updated_post_meta',
													);
													foreach ( $post_types as $post_type ) {
														if ( 'wpcf7_contact_form' != $post_type->name && 'custom_css' != $post_type->name && 'customize_changeset' != $post_type->name && 'attachment' != $post_type->name && 'oembed_cache' != $post_type->name && 'wp_block' != $post_type->name ) {
															array_push( $all_hook, $post_type->name . '_create', $post_type->name . '_update', $post_type->name . '_delete' );
														}
														if ( 'attachment' == $post_type->name ) {
															array_push( $all_hook, 'add_' . $post_type->name, 'edit_' . $post_type->name, 'delete_' . $post_type->name );
														}
													}
													foreach ( $taxonomies as $taxonomy ) {
														if ( 'wpcf7_contact_form' != $taxonomy->name && 'custom_css' != $taxonomy->name && 'customize_changeset' != $taxonomy->name && 'product_visibility' != $taxonomy->name ) {
															array_push( $all_hook, $taxonomy->name . '_create', $taxonomy->name . '_update', $taxonomy->name . '_delete' );
														}
													}

													if ( class_exists( 'Woocommerce' ) ) {
														array_push( $all_hook, 'woocommerce_updated_option', 'woocommerce_new_product_variation', 'wp_ajax_woocommerce_remove_variations', 'woocommerce_save_product_variation' );
													}
													if ( is_plugin_active( 'buddypress/bp-loader.php' ) ) {
														array_push( $all_hook, 'groups_create_group', 'groups_update_group', 'groups_before_delete_group', 'groups_leave_group', 'groups_join_group', 'groups_promote_member', 'groups_demote_member', 'groups_ban_member', 'groups_unban_member', 'groups_remove_member', 'xprofile_field_after_save', 'xprofile_fields_deleted_field', 'xprofile_group_after_save', 'xprofile_groups_deleted_group' );
													}
													if ( is_plugin_active( 'easy-digital-downloads/easy-digital-downloads.php' ) ) {
														array_push( $all_hook, 'edd_activate_discount', 'edd_deactivate_discount', 'edd_add_discount', 'edd_delete_discount', 'transition_post_status' );
													}
													if ( is_plugin_active( 'gravityforms/gravityforms.php' ) ) {
														array_push( $all_hook, 'gf_export_forms', 'notification_duplicated', 'gform_entry_created', 'gf_resend_notifications', 'gform_pre_note_deleted', 'gform_post_export_entries', 'gform_post_note_added', 'gform_delete_entries', 'gform_post_form_views_deleted', 'gform_post_form_activated', 'gform_post_form_deactivated', 'gform_before_delete_form', 'gform_post_form_trashed', 'gform_post_form_restored', 'gform_post_form_duplicated', 'gform_after_save_form', 'gform_forms_post_import', 'gform_pre_notification_deleted', 'gform_pre_confirmation_deleted', 'wp_ajax_rg_update_notification_active', 'wp_ajax_rg_update_confirmation_active', 'gform_pre_confirmation_save', 'gform_pre_notification_save', 'gform_delete_lead', 'gform_update_status', 'gform_update_is_read', 'gform_update_is_starred' );
													}
													if ( is_plugin_active( 'wordfence/wordfence.php' ) ) {
														array_push( $all_hook, 'wordfence_updateIssueStatus', 'wordfence_saveWAFConfig', 'wordfence_wafConfigureAutoPrepend', 'wordfence_whitelistBulkEnable', 'wordfence_whitelistBulkDisable', 'wordfence_whitelistBulkDelete', 'wordfence_updateConfig', 'wordfence_loadLiveTraffic', 'wordfence_loadCacheExclusions', 'wordfence_saveCacheConfig', 'wordfence_cache_clear', 'wordfence_reverseLookup', 'wordfence_whois', 'wordfence_clearPageCache', 'wordfence_saveConfig', 'wordfence_blockIP', 'wordfence_addTwoFactor', 'wordfence_exportSettings', 'wordfence_clearAllBlocked', 'wordfence_blockIPUARange', 'wordfence_unblockRange', 'wordfence_saveCountryBlocking', 'wordfence_saveDebuggingConfig', 'wordfence_startPasswdAudit', 'wordfence_saveScanSchedule', 'wordfence_updateWAFRules', 'wordfence_sendTestEmail', 'wordfence_sendDiagnostic', 'wordfence_updateAllIssues', 'wordfence_sendActivityLog', 'wordfence_addCacheExclusion', 'wordfence_daily_autoUpdate', 'wordfence_update_blocked_IPs', 'wordfence_start_scheduled_scan', 'wordfence_doScan', 'wordfence_daily_cron', 'wordfence_hourly_cron', 'wordfence_email_activity_report', 'wordfence_scan', 'wordfence_updateAlertEmail', 'wordfence_restoreFile', 'wordfence_deletePasswdAudit', 'wordfence_weakPasswordsFix', 'wordfence_killPasswdAudit', 'wordfence_importSettings', 'wordfence_bulkOperation', 'wordfence_deleteFile', 'wordfence_deleteDatabaseOption', 'wordfence_removeExclusion', 'wordfence_deleteIssue', 'wordfence_unlockOutIP', 'wordfence_unblockIP', 'wordfence_permBlockIP', 'wordfence_downloadHtaccess', 'wordfence_checkFalconHtaccess', 'wordfence_removeFromCache', 'wordfence_autoUpdateChoice', 'wordfence_saveCacheOptions', 'wordfence_adminEmailChoice', 'wordfence_getCacheStats', 'wordfence_killScan', 'wordfence_tourClosed', 'wordfence_removeCacheExclusion', 'wordfence_unblockNetwork', 'wordfence_permanentlyBlockAllIPs', 'wordfence_deleteAdminUser', 'wordfence_revokeAdminUser', 'wordfence_hideFileHtaccess', 'wordfence_loadBlockRanges' );
													}
													if ( is_plugin_active( 'wordpress-seo/wp-seo.php' ) ) {
														array_push( $all_hook, 'wpseo_handle_import', 'wpseo_import', 'wpseo_save_auth_code', 'seo_check_admin_referer', 'updated_post_meta' );
													}
													if ( is_plugin_active( 'contact-form-7/wp-contact-form-7.php' ) ) {
														array_push( $all_hook, 'wpcf7_after_create' );
														array_push( $all_hook, 'wpcf7_after_update' );
														array_push( $all_hook, 'wpcf7_admin_notices' );
														array_push( $all_hook, 'wpcf7-recaptcha-setup' );
													}
													if ( is_plugin_active( 'blog-designer-pro/blog-designer-pro.php' ) ) {
														array_push( $all_hook, 'bdp_update_shortcode', 'bdp_reset_shortcode', 'bdp_reset_archive_page', 'bdp_add_shortcode', 'bdp_update_single_file', 'bdp_add_archive_layout', 'bdp_delete_shortcode', 'bdp_delete_single_template', 'bdp_copy_single_template', 'bdp_duplicate_layout_settings', 'bdp_import_blog_layout_settings', 'bdp_import_archive_layout_settings', 'bdp_export_blog_layout_settings', 'bdp_export_archive_layout_settings', 'bdp_update_archive_page', 'bdp_import_single_layout_settings', 'bdp_export_single_layout_settings', 'bdp_delete_archive', 'bdp_delete_single_layout', 'bdp_update_single_page', 'bdp_reset_single_page', 'bdp_add_single_layout', 'bdp_duplicate_single_layout_settings' );
													}
													if ( is_plugin_active( 'portfolio-designer/portfolio-designer.php' ) ) {
														array_push( $all_hook, 'import_sample_portfolio_data', 'create_portfolio_layout', 'duplicate_portfolio_layout', 'delete_portfolio_layout', 'update_portfolio_layout', 'reset_portfolio_layout', 'create_post_type', 'update_post_type', 'export_portfolio_layout', 'delete_post_type' );
													}
													if ( is_plugin_active( 'all-in-one-reservation/all-in-one-reservation.php' ) ) {
														array_push( $all_hook, 'save_reservation_settings', 'rest_update_reservation_status', 'reset_reservation_settings' );
													}
													if ( is_plugin_active( 'wc-checkout-fields-editor/wc-checkout-fields-editor.php' ) ) {
														array_push( $all_hook, 'save_wcfe_settings', 'wp_ajax_wcfe_get_checkout_required_fields' );
													}
													if ( is_plugin_active( 'easy-digital-downloads/easy-digital-downloads.php' ) ) {
														array_push( $all_hook, 'edd_activate_discount', 'edd_settings', 'edd_deactivate_discount', 'edd_add_discount', 'edd_delete_discount' );
													}
													if ( is_plugin_active( 'bbpress/bbpress.php' ) ) {
														array_push( $all_hook, 'bbpress_settings' );
													}
													if ( is_plugin_active( 'autoptimize/autoptimize.php' ) ) {
														array_push( $all_hook, 'wp_ajax_autoptimize_delete_cache', 'autoptimizer_settings' );
													}
													if ( is_plugin_active( 'users-ultra/xoousers.php' ) ) {
														array_push( $all_hook, 'wp_ajax_save_fields_settings', 'wp_ajax_delete_profile_field', 'wp_ajax_sort_fileds_list', 'wp_ajax_custom_fields_reset', 'wp_ajax_create_uploader_folder', 'wp_ajax_package_add_new', 'wp_ajax_package_delete', 'wp_ajax_package_edit_confirm', 'wp_ajax_sync_users', 'wp_ajax_user_resend_activation_link', 'wp_ajax_delete_uultra_transtient', 'wp_ajax_delete_uultra_stats', 'wp_ajax_delete_uultra_reviews', 'wp_ajax_edit_photo_cate_conf', 'wp_ajax_edit_photo_cate_del', 'wp_ajax_edit_photo_cate', 'userultra_admin_menu_hook' );
													}
													if ( is_plugin_active( 'user-switching/user-switching.php' ) ) {
														array_push( $all_hook, 'switch_to_user', 'switch_back_user', 'switch_off_user' );
													}
													if ( is_plugin_active( 'advanced-custom-fields/acf.php' ) ) {
														array_push( $all_hook, 'export_acf' );
													}
													?>
													<select name="custom_trigger_value_<?php echo esc_attr( $i ); ?>[]" class="chosen-filter" multiple>
														<?php
														foreach ( $all_hook as $hook ) {
															$label = $hook;
															foreach ( $allhookdetails as $key => $val ) {
																if ( $hook == $key ) {
																	$label = $val;
																}
															}
															$selected = in_array( $hook, $custom_trigger_data_single['custom_trigger_value'] ) ? 'selected="selected"' : '';
															echo '<option value="' . esc_attr( $hook ) . '" ' . esc_attr( $selected ) . '>' . esc_html( $label ) . '</option>';
														}
														?>
														</select>
													<?php
											}
											if ( 'ip_address' == $custom_trigger_data_single['custom_trigger_name'] ) {
												?>
												<input class="ualp_event_medium" type="text" name="custom_trigger_value_<?php echo esc_attr( $i ); ?>[]" placeholder="" value="<?php echo esc_attr( $custom_trigger_data_single['custom_trigger_value'][0] ); ?>" required /> 
												<?php
											}
											?>
										</div>
									</td>
									<?php if ( 1 != $j ) { ?>
									<td class="ulap_last_column">
										<span title="<?php esc_html_e( 'Remove', 'user_activity_log_pro' ); ?>" class="ualp_remove_trigger_btn dashicons dashicons-trash"></span>
									</td>
									<?php } ?>
								</tr>
								<?php
								$j++;
							}
						} else {
							?>
							<tr class="ualp_custom_trigger_tr">
								<td style="display: none;">
									<span class="trigger-lable"><?php esc_html_e( 'Trigger With', 'user_activity_log_pro' ); ?></span><br/>
										<select name="custom_trigger_operator_with[]" class="chosen-filter" required>
											<option value="AND">AND</option>
											<option value="OR">OR</option>
										</select>
								</td>
								<td>
									<span class="trigger-lable">
									<?php
									esc_html_e( 'Trigger Name', 'user_activity_log_pro' );
									echo ' : ';
									?>
									</span><br/>
									<select name="custom_trigger_name[]" class="chosen-filter custom_trigger_name" required>
										<option value="user_id">User ID</option>
										<option value="user_name">User Name</option>
										<?php if ( ualp_allow_ip() ) { ?>
										<option value="ip_address">IP address</option>
										<?php } ?>
										<option value="user_role">User Role</option>
										<option value="hook">Activity</option>
									</select>
								</td>
								<td>
									<span class="trigger-lable">
									<?php
									esc_html_e( 'Trigger with', 'user_activity_log_pro' );
									echo ' : ';
									?>
									</span><br/>
									<select name="custom_trigger_with[]" class="chosen-filter">
										<option value="equal">IS Equal</option>
										<option value="not_equal">Not Equal</option>
									</select>
								</td>
								<td class="custom_trigger_value_tr">
										<span class="trigger-lable">
										<?php
										esc_html_e( 'Trigger Value', 'user_activity_log_pro' );
										echo ' : ';
										?>
										</span><br/>
										<div class="ualp_custom_trigger_value">
											<select name="custom_trigger_value_1[]" class="chosen-filter" multiple="multiple" required>
												<?php
												if ( isset( $all_users ) && ! empty( $all_users ) ) {
													foreach ( $all_users as $get_all_users ) {
														$user_id   = $get_all_users->ID;
														$user_name = $get_all_users->user_login;
														echo '<option value="' . esc_attr( $user_id ) . '">' . esc_html( $user_id ) . '</option>';
													}
												}
												?>
											</select>
										</div>
									</td>
							</tr>
						<?php } ?>
					</table>
				</div>
				<div class="add-trigger-email-wrapper">
					<span class="trigger-lable">
					<?php
					esc_html_e( 'Email Address', 'user_activity_log_pro' );
					echo ' : ';
					?>
					</span><input type="name" name="trigger_emailid" value="<?php echo esc_attr( $email_address ); ?>" class="trigger_emailid" placeholder="Ex: test@mail.com" required>
					<div class="email_display_note"><b><?php echo esc_html__( 'Note', 'user_activity_log_pro' ); ?></b>&nbsp;:&nbsp;<?php echo esc_html__( 'The Email address notifications will be sent to. For multiple email address separate them with a comma. Ex. example@gmail.com,test123@gmail.com', 'user_activity_log_pro' ); ?></div>
				</div>
				<div class="add-trigger-email-template">
					<div class="email-template-lable">
					<span class="trigger-lable">
					<?php
					esc_html_e( 'Email Template', 'user_activity_log_pro' );
					echo ' : ';
					?>
					</span>
					</div>
					<div class="email-template-value">   
						<div class="email_template_wrap">
						<input type="radio" name="email_template" value="0"  
						<?php
						if ( 0 == $email_template ) {
							echo 'checked'; }
						?>
						><label><?php echo esc_html__( 'Use default email template', 'user_activity_log_pro' ); ?></label><br>
						</div>
						<div class="email_template_wrap">
						<input type="radio" name="email_template" value="1" 
						<?php
						if ( 1 == $email_template ) {
							echo 'checked'; }
						?>
						><label><?php echo esc_html__( 'Use event specific email template', 'user_activity_log_pro' ); ?></label><br>
						</div>
					</div>
				</div>
				<div class="event-spacific-wrapp">
				<?php
					wp_editor( $all_email_notification, 'txtualpAddEmailMsg', array( 'editor_height' => 250 ) );
				?>
				</div>
				<?php
					wp_nonce_field( '_wp_trigger_notify_action', '_wp_trigger_notify_nonce' );
				?>
				<p class="submit">
					<input id="submit2" class="button button-primary" type="submit" value="
					<?php
					if ( isset( $_GET['enotification'] ) ) {
						esc_attr_e( 'Update Notification', 'user_activity_log_pro' );
					} else {
						esc_attr_e( 'Add Notification', 'user_activity_log_pro' ); }
					?>
					" name="saveTrigger">
				</p>
				</div>
			</form>
		</div>
		<?php
	}
}

/*-----------------------------------------------------------*/
add_action( 'ualp_report_schedule', 'ualp_daily_schedule_report' );
if ( ! wp_next_scheduled( 'ualp_report_schedule' ) ) {
	wp_schedule_event( time(), 'daily', 'ualp_report_schedule' );
}
add_action( 'admin_init', 'ualp_create_upload_directory' );
if ( ! function_exists( 'ualp_create_upload_directory' ) ) {
	/**
	 * Create Upload Directory.
	 */
	function ualp_create_upload_directory() {
		$upload_dir = WP_CONTENT_DIR . '/uploads/ualp/';
		if ( ! is_dir( $upload_dir ) ) {
			mkdir( $upload_dir, 0777 );
		}
	}
}
if ( ! function_exists( 'ualp_daily_schedule_report' ) ) {
	/**
	 * Daily Schedule Report.
	 */
	function ualp_daily_schedule_report() {
		$result_log = array();
		global $wpdb, $externaldb;
		$display_current_time                = gmdate( 'M,d,Y h:i:s A' );
		$current_time                        = current_time( 'timestamp' );
		$ualp_generate_report_username_list  = array();
		$ualp_generate_report_userrole_list  = array();
		$ualp_generate_report_ipaddress_list = array();
		$ualp_generate_report_hook_list      = array();
		$ualp_generate_report_posttype_list  = array();
		$ualp_generate_report_allpost_list   = array();
		$ualp_generate_report_country_list   = array();
		$ualp_generate_report_logtype_list   = array();
		$csv_content                         = array();
		$ualp_generate_report_start_date     = '';
		$ualp_generate_report_end_date       = '';
		$ualp_generate_report_format         = '';
		$logged_head                         = '';
		$where                               = ' where 1 = 1';
		$ualp_generate_report_username_list  = get_option( 'ualp_generate_report_username_list' );
		$ualp_generate_report_userrole_list  = get_option( 'ualp_generate_report_userrole_list' );
		$ualp_generate_report_ipaddress_list = get_option( 'ualp_generate_report_ipaddress_list' );
		$ualp_generate_report_country_list   = get_option( 'ualp_generate_report_country_list' );
		$ualp_generate_report_logtype_list   = get_option( 'ualp_generate_report_logtype_list' );
		$ualp_generate_report_hook_list      = get_option( 'ualp_generate_report_hook_list' );
		$ualp_generate_report_posttype_list  = get_option( 'ualp_generate_report_posttype_list' );
		$ualp_generate_report_allpost_list   = get_option( 'ualp_generate_report_allpost_list' );
		$ualp_generate_report_start_date     = get_option( 'ualp_generate_report_start_date' );
		$ualp_generate_report_end_date       = get_option( 'ualp_generate_report_end_date' );
		$ualp_generate_report_format         = get_option( 'ualp_generate_report_format' );

		if ( isset( $ualp_generate_report_username_list ) && ! empty( $ualp_generate_report_username_list ) ) {
			$ualp_generated_username_list = maybe_unserialize( $ualp_generate_report_username_list );
			$ualp_generated_username_list = "'" . implode( "','", $ualp_generated_username_list ) . "'";
			$where                       .= " AND user_name IN ($ualp_generated_username_list)";
		}
		if ( isset( $ualp_generate_report_userrole_list ) && ! empty( $ualp_generate_report_userrole_list ) ) {
			$ualp_generate_userrole_list = maybe_unserialize( $ualp_generate_report_userrole_list );
			$ualp_generate_userrole_list = '' . implode( "%' OR user_role like '%", $ualp_generate_userrole_list ) . '';
			$where                      .= " AND (user_role like '%$ualp_generate_userrole_list%')";
		}
		if ( isset( $ualp_generate_report_ipaddress_list ) && ! empty( $ualp_generate_report_ipaddress_list ) ) {
			$ualp_generate_ipaddress_list = maybe_unserialize( $ualp_generate_report_ipaddress_list );
			$ualp_generate_ipaddress_list = "'" . implode( "','", $ualp_generate_ipaddress_list ) . "'";
			$where                       .= " AND ip_address IN ($ualp_generate_ipaddress_list)";
		}
		if ( isset( $ualp_generate_report_logtype_list ) && ! empty( $ualp_generate_report_logtype_list ) ) {
			$ualp_generate_logtype_list = maybe_unserialize( $ualp_generate_report_logtype_list );
			$ualp_generate_logtype_list = "'" . implode( "','", $ualp_generate_logtype_list ) . "'";
			$where                     .= " AND object_type IN ($ualp_generate_logtype_list)";
		}
		if ( wp_get_schedule( 'ualp_report_schedule' ) || wp_get_schedule( 'ualp_weekly_report_sent' ) || wp_get_schedule( 'ualp_montly_report_sent' ) || wp_get_schedule( 'ualp_quarterly_report_sent' ) ) {
			if ( wp_get_schedule( 'ualp_report_schedule' ) ) {
				$start_time = $current_time;
				$end_time   = strtotime( '+1 day', $current_time );
				$start_time = gmdate( 'Y-m-d H:i:s', $start_time );
				$end_time   = gmdate( 'Y-m-d H:i:s', $end_time );
				$where     .= $externaldb->prepare( ' AND modified_date > "%s" AND modified_date < "%s"', $start_time, $end_time );
			} elseif ( wp_get_schedule( 'ualp_weekly_report_sent' ) ) {
				$start_time = strtotime( '-1 week', $start_time );
				$end_time   = strtotime( '+1 day', $current_time );

				$start_time = gmdate( 'Y-m-d H:i:s', $start_time );
				$end_time   = gmdate( 'Y-m-d H:i:s', $end_time );
				$where     .= $externaldb->prepare( ' AND modified_date > "%s" AND modified_date < "%s"', $start_time, $end_time );
			} elseif ( wp_get_schedule( 'ualp_montly_report_sent' ) ) {
				$start_time = strtotime( '-1 month', $start_time );
				$end_time   = strtotime( '+1 day', $current_time );
				$start_time = gmdate( 'Y-m-d H:i:s', $start_time );
				$end_time   = gmdate( 'Y-m-d H:i:s', $end_time );
				$where     .= $externaldb->prepare( ' AND modified_date > "%s" AND modified_date < "%s"', $start_time, $end_time );
			} elseif ( wp_get_schedule( 'ualp_quarterly_report_sent' ) ) {
				$start_time = strtotime( '-3 month', $start_time );
				$end_time   = strtotime( '+1 day', $current_time );
				$start_time = gmdate( 'Y-m-d H:i:s', $start_time );
				$end_time   = gmdate( 'Y-m-d H:i:s', $end_time );
				$where     .= $externaldb->prepare( ' AND modified_date > "%s" AND modified_date < "%s"', $start_time, $end_time );
			}
		} else {
			if ( isset( $ualp_generate_report_start_date ) && isset( $ualp_generate_report_end_date ) && $ualp_generate_report_start_date != $ualp_generate_report_end_date ) {
				if ( isset( $ualp_generate_report_start_date ) && '' != $ualp_generate_report_start_date ) {
					$start_time = strtotime( $ualp_generate_report_start_date );
					$start_time = strtotime( '-1 day', $start_time );
					if ( '' != $start_time ) {
						$start_time = gmdate( 'Y-m-d H:i:s', $start_time );
						$where     .= " AND modified_date > '$start_time' ";
					}
				}
				if ( isset( $ualp_generate_report_end_date ) && '' != $ualp_generate_report_end_date ) {
					$end_time = strtotime( $ualp_generate_report_end_date );
					$end_time = strtotime( '+1 day', $end_time );
					if ( '' != $end_time ) {
						$end_time = gmdate( 'Y-m-d H:i:s', $end_time );
						$where   .= " AND modified_date < '$end_time'";
					}
				}
			} elseif ( isset( $ualp_generate_report_start_date ) && isset( $ualp_generate_report_end_date ) && $ualp_generate_report_start_date == $ualp_generate_report_end_date ) {
				if ( isset( $ualp_generate_report_start_date ) && '' != $ualp_generate_report_start_date ) {
					$start_time = strtotime( $ualp_generate_report_start_date );
					if ( '' != $start_time ) {
						$start_time = gmdate( 'Y-m-d H:i:s', $start_time );
						$where     .= " AND modified_date >= '$start_time' ";
					}
				}
				if ( isset( $ualp_generate_report_end_date ) && '' != $ualp_generate_report_end_date ) {
					$end_time = strtotime( $ualp_generate_report_end_date );
					$end_time = strtotime( '+1 day', $end_time );
					if ( '' != $end_time ) {
						$end_time = gmdate( 'Y-m-d H:i:s', $end_time );
						$where   .= " AND modified_date < '$end_time'";
					}
				}
			}
		}
		$get_log_query    = 'SELECT * FROM ' . UALP_TABLENAME . " $where ORDER BY modified_date desc";
		$result_log_query = $externaldb->get_results( $get_log_query );
		$c                = 0;
		$date_format      = get_option( 'date_format' );
		$time_format      = get_option( 'time_format' );
		$export_delimiter = ',';
		$random_id        = wp_rand();
		$header           = array();
		if ( ! empty( $result_log_query ) ) {
			foreach ( $result_log_query as $result_log ) {
				$c++;
				$result_log = (array) $result_log;
				if ( ! ualp_allow_ip() ) {
					if ( isset( $result_log['ip_address'] ) ) {
						unset( $result_log['ip_address'] );
					}
				}
				if ( ! ualp_allow_country() ) {
					if ( isset( $result_log['country'] ) ) {
						unset( $result_log['country'] );
					}
				}
				if ( ! ualp_allow_user_agent() ) {
					if ( isset( $result_log['user_agent'] ) ) {
						unset( $result_log['user_agent'] );
					}
				}
				if ( isset( $result_log['uactid'] ) ) {
					$result_log['uactid'] = $c;
				}
				if ( isset( $result_log['favorite'] ) ) {
					if ( 1 == $result_log['favorite'] ) {
						$result_log['favorite'] = 'Yes';
					} elseif ( 0 == $result_log['favorite'] ) {
						$result_log['favorite'] = 'No';
					}
				}
				if ( isset( $result_log['modified_date'] ) ) {
					$get_date                    = strtotime( $result_log['modified_date'] );
					$date                        = gmdate( $date_format, $get_date );
					$time                        = gmdate( $time_format, $get_date );
					$result_log['modified_date'] = $date . ' ' . $time;
				}
				if ( isset( $result_log['description'] ) ) {
					if ( '' == $result_log['description'] ) {
						$action                    = '';
						$post_title                = '';
						$action                    = $result_log['action'];
						$post_title                = $result_log['post_title'];
						$result_log['description'] = $action . ' : ' . $post_title;
					} else {
						$result_log['description'] = html_entity_decode( $result_log['description'] );
					}
				}
				unset( $result_log['site_id'] );
				unset( $result_log['post_id'] );
				unset( $result_log['user_id'] );
				unset( $result_log['post_title'] );
				unset( $result_log['action'] );
				unset( $result_log['post_id'] );

				if ( 'yes' != $logged_head ) {
					foreach ( $result_log as $result_log_key => $result_log_val ) {
						if ( ! in_array( $result_log_key, $header ) ) {
							$header[] = $result_log_key;
						}
					}
					$result_log[ $result_log_key ] = html_entity_decode( $result_log_val );
				}
				$logged_head = 'yes';
				if ( isset( $ualp_generate_report_format ) && ( 'html' == $ualp_generate_report_format || 'pdf' == $ualp_generate_report_format ) ) {
					$htmlresult_log[] = $result_log;
				} else {
					$csv_content[] = $result_log;
				}
			}
		}
		$upload_dir       = WP_CONTENT_DIR . '/uploads/ualp/';
		$export_delimiter = ',';

		require_once ABSPATH . 'wp-admin/includes/file.php';
		if ( isset( $ualp_generate_report_format ) && 'csv' == $ualp_generate_report_format ) {
			$filename = $upload_dir . 'export_user_report_' . gmdate( 'Y-m-d_H-i-s', time() ) . '.csv';
			$csv_file = ualp_schedule_report_csv_output( $csv_file_name, $csv_content, $header, $export_delimiter );
			$savefile = fopen( $filename, 'w' );
			fwrite( $savefile, $csv_file );
			fclose( $savefile );
		}
		if ( isset( $ualp_generate_report_format ) && 'json' == $ualp_generate_report_format ) {
			$filename = $upload_dir . 'export_user_report_' . gmdate( 'Y-m-d_H-i-s', time() ) . '.json';
			$savefile = fopen( $filename, 'w' );
			fwrite( $savefile, wp_json_encode( $csv_content ) );
			fclose( $savefile );
		}
		$html_content = '';
		if ( isset( $ualp_generate_report_format ) && 'html' == $ualp_generate_report_format ) {
			$html_content  = '';
			$html_content .= '<div class="ualp_generat_report_wrap">';
			$html_content .= '<div class="report_header_title_wrap">';
			$html_content .= '<p class="header_description">Report generated with</p>';
			$html_content .= ' <h1>User Activity Log Pro</h1>';
			$html_content .= '</div>';
			$html_content .= '<div class="ualp_report_content_wrap">';
			$html_content .= '<div class="report_details_cover">';
			$html_content .= '<span class="report_details_title"><b>Report details:</b></span>';
			$html_content .= '<span class="report_generated_span"><b>Generated On</b> : ' . $display_current_time . '</span>';
			$html_content .= '</div>';

			$html_content .= '<div>';
			$html_content .= '<table>';
			$html_content .= '<thead>';
			$html_content .= '<tr>';
			$header        = array_values( $header );
			$count_header  = ( $header );
			for ( $h = 0;$h < $count_header;$h++ ) {
				$html_content .= '<th>';
				if ( 'uactid' == $header[ $h ] ) {
					$html_content .= esc_html__( 'Sr No.', 'user_activity_log_pro' );
				}
				if ( 'user_name' == $header[ $h ] ) {
					$html_content .= esc_html__( 'Author Name', 'user_activity_log_pro' );
				}
				if ( 'user_role' == $header[ $h ] ) {
					$html_content .= esc_html__( 'Author Role', 'user_activity_log_pro' );
				}
				if ( 'user_email' == $header[ $h ] ) {
					$html_content .= esc_html__( 'Author E-mail', 'user_activity_log_pro' );
				}
				if ( ualp_allow_ip() ) {
					if ( 'ip_address' == $header[ $h ] ) {
						$html_content .= esc_html__( 'IP Address', 'user_activity_log_pro' );
					}
				}
				if ( ualp_allow_country() ) {
					if ( 'country' == $header[ $h ] ) {
						$html_content .= esc_html__( 'Country', 'user_activity_log_pro' );
					}
				}
				if ( ualp_allow_user_agent() ) {
					if ( 'user_agent' == $header[ $h ] ) {
						$html_content .= esc_html__( 'User Agent', 'user_activity_log_pro' );
					}
				}
				if ( 'modified_date' == $header[ $h ] ) {
					$html_content .= esc_html__( 'Date', 'user_activity_log_pro' );
				}
				if ( 'object_type' == $header[ $h ] ) {
					$html_content .= esc_html__( 'Activity Type', 'user_activity_log_pro' );
				}
				if ( 'hook' == $header[ $h ] ) {
					$html_content .= esc_html__( 'Activity Hook', 'user_activity_log_pro' );
				}
				if ( 'description' == $header[ $h ] ) {
					$html_content .= esc_html__( 'Description', 'user_activity_log_pro' );
				}
				if ( 'favorite' == $header[ $h ] ) {
					$html_content .= esc_html__( 'Favorite?', 'user_activity_log_pro' );
				}
				if ( 't_session' == $header[ $h ] ) {
					$html_content .= esc_html__( 'T Session', 'user_activity_log_pro' );
				}
				$html_content .= '</th>';
			}
			$html_content .= '</tr>';
			$html_content .= '</thead>';
			$html_content .= '<tbody>';
			$c             = 0;
			if ( $htmlresult_log ) {
				foreach ( $htmlresult_log as $htmlvalue ) {
					$html_content .= '<tr>';
					foreach ( $htmlvalue as $htmlvalue1 ) {
						$html_content .= '<th>';
						$html_content .= $htmlvalue1;
						$html_content .= '</th>';
					}
					$html_content .= '</tr>';
					$c++;
				}
			}
			$html_content .= '</tbody>';
			$html_content .= '</table>';
			$html_content .= '</div>';
			$html_content .= '</div>';
			$html_content .= '</div>';
			$upload_dir    = WP_CONTENT_DIR . '/uploads/ualp/';
			$filename      = $upload_dir . 'export_user_report_' . gmdate( 'Y-m-d_H-i-s', time() ) . '.html';
			$html_content .= '<style>
                .report_header_title_wrap {display: inline-block;width: 100%;text-align: center;}
                .report_details_cover .report_details_title {display: inline-block;width: 100%;border-bottom: 1px solid #000;padding-bottom: 10px;}
                .report_generated_span {display: inline-block;width: 100%;padding: 10px 0;}
                table {font-family: arial, sans-serif;border-collapse: collapse;width: 100%;}
                td, th {border:0;text-align: left;padding: 20px 10px;}
                thead th {background: #000;color: #fff;opacity: 0.6;}
                tr:nth-child(even) {background-color: #f1f1f1;}
                </style>';
				$savefile  = fopen( $filename, 'w' );
				fwrite( $savefile, $html_content );
				fclose( $savefile );
		}
		$pdf_content = '';
		if ( isset( $ualp_generate_report_format ) && 'pdf' == $ualp_generate_report_format ) {
			$pdf_content  = '<table>';
			$pdf_content .= '<tr>';
			$header       = array_values( $header );
			$count_header = count( $header );
			for ( $h = 0;$h < $count_header;$h++ ) {
				$pdf_content .= '<td>';
				if ( 'uactid' == $header[ $h ] ) {
					$pdf_content .= esc_html__( 'Sr No.', 'user_activity_log_pro' );
				}
				if ( 'user_name' == $header[ $h ] ) {
					$pdf_content .= esc_html__( 'Author Name', 'user_activity_log_pro' );
				}
				if ( 'user_role' == $header[ $h ] ) {
					$pdf_content .= esc_html__( 'Author Role', 'user_activity_log_pro' );
				}
				if ( 'user_email' == $header[ $h ] ) {
					$pdf_content .= esc_html__( 'Author E-mail', 'user_activity_log_pro' );
				}
				if ( ualp_allow_ip() ) {
					if ( 'ip_address' == $header[ $h ] ) {
						$pdf_content .= esc_html__( 'IP Address', 'user_activity_log_pro' );
					}
				}
				if ( ualp_allow_country() ) {
					if ( 'country' == $header[ $h ] ) {
						$pdf_content .= esc_html__( 'Country', 'user_activity_log_pro' );
					}
				}
				if ( ualp_allow_user_agent() ) {
					if ( 'user_agent' == $header[ $h ] ) {
						$pdf_content .= esc_html__( 'User Agent', 'user_activity_log_pro' );
					}
				}
				if ( 'modified_date' == $header[ $h ] ) {
					$pdf_content .= esc_html__( 'Date', 'user_activity_log_pro' );
				}
				if ( 'object_type' == $header[ $h ] ) {
					$pdf_content .= esc_html__( 'Activity Type', 'user_activity_log_pro' );
				}
				if ( 'hook' == $header[ $h ] ) {
					$pdf_content .= esc_html__( 'Activity Hook', 'user_activity_log_pro' );
				}
				if ( 'description' == $header[ $h ] ) {
					$pdf_content .= esc_html__( 'Description', 'user_activity_log_pro' );
				}
				if ( 'favorite' == $header[ $h ] ) {
					$pdf_content .= esc_html__( 'Favorite?', 'user_activity_log_pro' );
				}
				if ( 't_session' == $header[ $h ] ) {
					$pdf_content .= esc_html__( 'T Session', 'user_activity_log_pro' );
				}
				$pdf_content .= '</td>';
			}
			$pdf_content .= '</tr>';
			$c            = 0;
			if ( ! empty( $htmlresult_log ) ) {
				foreach ( $htmlresult_log as $htmldatavalue ) {
					$pdf_content .= '<tr>';
					foreach ( $htmldatavalue as $html_content ) {
						$pdf_content .= '<td>';
						$pdf_content .= $html_content;
						$pdf_content .= '</td>';
					}
					$pdf_content .= '</tr>';
					$c++;
				}
			}
			$pdf_content .= '</table>';
			$upload_dir   = WP_CONTENT_DIR . '/uploads/ualp/';
			ob_start();
			$filename = $upload_dir . 'export_user_report_' . gmdate( 'Y-m-d_H-i-s', time() ) . '.pdf';
			require 'html2pdf/html2pdf.php';
			$pdf = new PDF_HTML_Table();
			$pdf->AddPage();
			$pdf->SetFont( 'Arial', '', 10 );
			$pdf->WriteHTML( "User Activity Log Pro<br>$pdf_content" );
			$pdf->Output( $filename, 'F' );
			ob_end_flush();
		}
		if ( isset( $filename ) && '' != $filename ) {
			$schedule_where = ' where 1 = 1';
			if ( wp_get_schedule( 'ualp_report_schedule' ) ) {
				$schedule_report_content = $wpdb->get_results( $wpdb->prepare( 'SELECT * FROM %s AND report_frequency like daily ', UALP_SCHEDULE_TABLENAME ) );
			} elseif ( wp_get_schedule( 'ualp_weekly_report_sent' ) ) {
				$schedule_report_content = $wpdb->get_results( $wpdb->prepare( 'SELECT * FROM %s AND report_frequency like weekly ', UALP_SCHEDULE_TABLENAME ) );
			} elseif ( wp_get_schedule( 'ualp_montly_report_sent' ) ) {
				$schedule_report_content = $wpdb->get_results( $wpdb->prepare( 'SELECT * FROM %s AND report_frequency like monthly ', UALP_SCHEDULE_TABLENAME ) );
			} elseif ( wp_get_schedule( 'ualp_quarterly_report_sent' ) ) {
				$schedule_report_content = $wpdb->get_results( $wpdb->prepare( 'SELECT * FROM %s AND report_frequency like quarterly ', UALP_SCHEDULE_TABLENAME ) );
			}
			// $schedule_report_content = $wpdb->get_results( $schedule_query );
			$s            = 0;
			$report_title = esc_html__( 'User activity Notification', 'user_activity_log_pro' );

			$from_email = get_option( 'ualpFromEmail' );
			$to_email   = '';

			foreach ( $schedule_report_content as $all_schedule_report_content ) {
				$report_title = $all_schedule_report_content->report_title;
				$to_email     = $all_schedule_report_content->schedule_email_id;
			}
			$msg             = 'User Activity Log Report';
			$headers         = 'From: ' . wp_strip_all_tags( $from_email ) . "\r\n";
			$headers        .= 'Reply-To: ' . wp_strip_all_tags( $from_email ) . "\r\n";
			$headers        .= "MIME-Version: 1.0\r\n";
			$headers        .= "Content-Type: text/html; charset=ISO-8859-1\r\n";
			$to_email_array  = explode( ',', $to_email );
			$mail_attachment = array( $filename );
			$count_email     = count( $to_email_array );
			for ( $i = 0; $i < $count_email;$i++ ) {
				$to_mail = trim( $to_email_array[ $i ], ' ' );
				wp_mail( $to_mail, $report_title, $msg, $headers, $mail_attachment );
			}
		}
	}
}
if ( ! function_exists( 'ualp_custom_cron_schedule_report' ) ) {
	/**
	 * Custom Cron Schedule Report.
	 *
	 * @param string $schedules Schedules.
	 */
	function ualp_custom_cron_schedule_report( $schedules ) {
		$schedules['weekly']    = array(
			'interval' => 604800, // Weekly.
			'display'  => __( 'Once Weekly', 'user_activity_log_pro' ),
		);
		$schedules['monthly']   = array(
			'interval' => 2592000, // monthly.
			'display'  => __( 'Once monthly', 'user_activity_log_pro' ),
		);
		$schedules['quarterly'] = array(
			'interval' => 7776000, // Weekly.
			'display'  => __( 'Once Quarterly', 'user_activity_log_pro' ),
		);
		return $schedules;
	}
}

add_filter( 'cron_schedules', 'ualp_custom_cron_schedule_report' );

add_action( 'ualp_weekly_report_sent', 'ualp_daily_schedule_report' );
if ( ! wp_next_scheduled( 'ualp_weekly_report_sent' ) ) {
	wp_schedule_event( time(), 'weekly', 'ualp_weekly_report_sent' );
}
add_action( 'ualp_montly_report_sent', 'ualp_daily_schedule_report' );

if ( ! wp_next_scheduled( 'ualp_montly_report_sent' ) ) {
	wp_schedule_event( time(), 'monthly', 'ualp_montly_report_sent' );
}
add_action( 'ualp_quarterly_report_sent', 'ualp_daily_schedule_report' );

if ( ! wp_next_scheduled( 'ualp_quarterly_report_sent' ) ) {
	wp_schedule_event( time(), 'quarterly', 'ualp_quarterly_report_sent' );
}

if ( ! function_exists( 'ualp_get_schedule_report_data' ) ) {

	/**
	 * Ajax callback function to get row by row id.
	 *
	 * @global type $wpdb
	 */
	function ualp_get_schedule_report_data() {
		global $wpdb, $externaldb;
		$tabel_name = UALP_SCHEDULE_TABLENAME;
		if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'ajax-nonce' ) ) {
			if ( isset( $_REQUEST['editid'] ) ) {
				$edit_id = sanitize_text_field( wp_unslash( $_REQUEST['editid'] ) );
			}
		}
		$thepost  = $externaldb->get_row( $externaldb->prepare( 'SELECT * FROM ' . $tabel_name . " WHERE uactrid = '%d'", $edit_id ), ARRAY_A );
		$mixarray = array(
			'data' => isset( $thepost ) ? $thepost : array(),
		);
		print wp_json_encode( $mixarray, JSON_UNESCAPED_SLASHES );
		wp_die();
	}
}
add_action( 'wp_ajax_ualp_get_schedule_report_data', 'ualp_get_schedule_report_data' );
add_action( 'wp_ajax_nopriv_ualp_get_schedule_report_data', 'ualp_get_schedule_report_data' );

if ( ! function_exists( 'ualp_delete_schedule_data' ) ) {
	/**
	 * Insert record when user logout.
	 */
	function ualp_delete_schedule_data() {
		global $wpdb, $externaldb;
		if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'ajax-nonce' ) ) {
			if ( isset( $_REQUEST['deleteid'] ) ) {
				$deleteid = sanitize_text_field( wp_unslash( $_REQUEST['deleteid'] ) );
				do_action( 'ualp_before_update_notification' );
				$table_names = UALP_SCHEDULE_TABLENAME;
				$externaldb->query(
					'DELETE  FROM ' . $table_names . ' WHERE uactrid = "' . $deleteid . '"'
				);
				do_action( 'ualp_after_update_notification' );
			}
		}
	}
}
add_action( 'wp_ajax_ualp_delete_schedule_report_data', 'ualp_delete_schedule_data' );
add_action( 'wp_ajax_nopriv_ualp_delete_schedule_report_data', 'ualp_delete_schedule_data' );

add_action( 'wp_ajax_ualp_sendnow_schedule_report_data', 'ualp_sendnow_schedule_report_data' );
add_action( 'wp_ajax_nopriv_ualp_sendnow_schedule_report_data', 'ualp_sendnow_schedule_report_data' );

if ( ! function_exists( 'ualp_sendnow_schedule_report_data' ) ) {
	/**
	 * Send schedule report data.
	 */
	function ualp_sendnow_schedule_report_data() {

		$result_log1                  = array();
		$display_current_time         = gmdate( 'M,d,Y h:i:s A' );
		$current_time                 = current_time( 'timestamp' );
		$ualp_generated_username      = '';
		$ualp_generated_userrole      = '';
		$ualp_generated_ipaddress     = '';
		$ualp_generated_logtype       = '';
		$ualp_generated_hook          = '';
		$ualp_generated_end_date      = '';
		$ualp_generated_start_date    = '';
		$ualp_generated_report_format = '';
		$ualp_generated_username_list = array();
		$ualp_generated_userrole_list = array();
		$ualp_generated_ip_list       = array();
		$ualp_generated_logtype_list  = array();
		$ualp_generated_hook_list     = array();
		global $wpdb, $query, $externaldb;
		$header                              = array();
		$user_role                           = '';
		$username                            = '';
		$type                                = '';
		$ip                                  = '';
		$time                                = '';
		$search                              = '';
		$start_date                          = '';
		$end_date                            = '';
		$user_agent                          = '';
		$t_session                           = '';
		$site_id                             = '';
		$country                             = '';
		$hook                                = '';
		$favorite                            = '';
		$where                               = ' where 1 = 1';
		$ualp_generate_report_username       = get_option( 'ualp_generate_report_username', 'all' );
		$ualp_generate_report_userrole       = get_option( 'ualp_generate_report_userrole', 'all' );
		$ualp_generate_report_ipaddress      = get_option( 'ualp_generate_report_ipaddress', 'all' );
		$ualp_generate_report_logtype        = get_option( 'ualp_generate_report_logtype', 'all' );
		$ualp_generate_report_hook           = get_option( 'ualp_generate_report_hook', 'all' );
		$ualp_generate_report_username_list  = get_option( 'ualp_generate_report_username_list' );
		$ualp_generate_report_userrole_list  = get_option( 'ualp_generate_report_userrole_list' );
		$ualp_generate_report_ipaddress_list = get_option( 'ualp_generate_report_ipaddress_list' );
		$ualp_generate_report_logtype_list   = get_option( 'ualp_generate_report_logtype_list' );
		$ualp_generate_report_hook_list      = get_option( 'ualp_generate_report_hook_list' );
		$ualp_generate_report_start_date     = get_option( 'ualp_generate_report_start_date', '' );
		$ualp_generate_report_end_date       = get_option( 'ualp_generate_report_end_date', '' );
		$ualp_generate_report_format         = get_option( 'ualp_generate_report_format' );

		$where = ' where 1 = 1';
		if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'ajax-nonce' ) ) {
			if ( isset( $_REQUEST['sendid'] ) && ! empty( $_REQUEST['sendid'] ) ) {
				$sendid = sanitize_text_field( wp_unslash( $_REQUEST['sendid'] ) );
			}
		}
		$report_frequency        = '';
		$schedule_query          = 'SELECT * FROM ' . UALP_SCHEDULE_TABLENAME . ' where uactrid=' . $sendid;
		$schedule_report_content = $externaldb->get_results( $externaldb->prepare( 'SELECT * FROM ' . UALP_SCHEDULE_TABLENAME . ' where uactrid = %d', $sendid ) );

		foreach ( $schedule_report_content as $all_schedule_report_content ) {
			$report_title     = $all_schedule_report_content->report_title;
			$to_email         = $all_schedule_report_content->schedule_email_id;
			$report_frequency = $all_schedule_report_content->report_frequency;

		}

		if ( isset( $ualp_generate_report_username ) && '' != $ualp_generate_report_username ) {
			if ( 'all' != $ualp_generate_report_username ) {
				if ( isset( $ualp_generate_report_username_list ) && '' != $ualp_generate_report_username_list ) {
					$ualp_generated_username_list = maybe_unserialize( $ualp_generate_report_username_list );
					$ualp_generated_username_list = "'" . implode( "','", $ualp_generated_username_list ) . "'";
					$where                       .= " AND user_name IN ($ualp_generated_username_list)";
				}
			}
		}
		if ( isset( $ualp_generate_report_userrole ) && '' != $ualp_generate_report_userrole ) {
			if ( 'all' != $ualp_generate_report_userrole ) {
				if ( isset( $ualp_generate_report_userrole_list ) && '' != $ualp_generate_report_userrole_list ) {
					$ualp_generate_userrole_list = maybe_unserialize( $ualp_generate_report_userrole_list );
					$ualp_generate_userrole_list = '' . implode( "%' OR user_role like '%", $ualp_generate_userrole_list ) . '';
					$where                      .= " AND (user_role like '%$ualp_generate_userrole_list%')";
				}
			}
		}
		if ( isset( $ualp_generate_report_ipaddress ) && '' != $ualp_generate_report_ipaddress ) {
			if ( 'all' != $ualp_generate_report_ipaddress ) {
				if ( isset( $ualp_generate_report_ipaddress_list ) && '' != $ualp_generate_report_ipaddress_list ) {
					$ualp_generate_report_ipaddress_list = maybe_unserialize( $ualp_generate_report_ipaddress_list );
					$ualp_generate_report_ipaddress_list = "'" . implode( "','", $ualp_generate_report_ipaddress_list ) . "'";
					$where                              .= " AND ip_address IN ($ualp_generate_report_ipaddress_list)";
				}
			}
		}
		if ( isset( $ualp_generate_report_logtype ) && '' != $ualp_generate_report_logtype ) {
			if ( 'all' != $ualp_generate_report_logtype ) {
				if ( isset( $ualp_generate_report_logtype_list ) && '' != $ualp_generate_report_logtype_list ) {
					$ualp_generate_report_logtype_list = maybe_unserialize( $ualp_generate_report_logtype_list );
					$ualp_generate_report_logtype_list = "'" . implode( "','", $ualp_generate_report_logtype_list ) . "'";
					$where                            .= " AND object_type IN ($ualp_generate_report_logtype_list)";
				}
			}
		}
		if ( isset( $ualp_generate_report_hook ) && '' != $ualp_generate_report_hook ) {
			if ( 'all' != $ualp_generate_report_hook ) {
				if ( isset( $ualp_generate_report_hook_list ) && '' != $ualp_generate_report_hook_list ) {
					$ualp_generate_report_hook_list = maybe_unserialize( $ualp_generate_report_hook_list );
					$ualp_generate_report_hook_list = "'" . implode( "','", $ualp_generate_report_hook_list ) . "'";
					$where                         .= " AND hook IN ($ualp_generate_report_hook_list)";
				}
			}
		}
		if ( ! isset( $report_frequency ) && empty( $report_frequency ) ) {
			if ( isset( $ualp_generate_report_start_date ) && '' != $ualp_generate_report_start_date ) {
				$start_time = strtotime( $ualp_generate_report_start_date );
				$start_time = strtotime( '-1 day', $start_time );
				if ( '' != $start_time ) {
					$start_time = gmdate( 'Y-m-d H:i:s', $start_time );
					$where     .= " AND modified_date > '$start_time' ";
				}
			}
			if ( isset( $ualp_generate_report_end_date ) && '' != $ualp_generate_report_end_date ) {
				$end_time = strtotime( $ualp_generate_report_end_date );
				$end_time = strtotime( '+1 day', $end_time );
				if ( '' != $end_time ) {
					$end_time = gmdate( 'Y-m-d H:i:s', $end_time );
					$where   .= " AND modified_date < '$end_time'";
				}
			}
		}
		if ( isset( $report_frequency ) && ! empty( $report_frequency ) && 'daily' == $report_frequency ) {
			$start_time = $current_time;
			$end_time   = strtotime( '+1 day', $current_time );
			$start_time = gmdate( 'Y-m-d 00:00:00', $start_time );
			$end_time   = gmdate( 'Y-m-d 00:00:00', $end_time );
			$where     .= $externaldb->prepare( ' AND modified_date > "%s" AND modified_date < "%s"', $start_time, $end_time );
		} elseif ( isset( $report_frequency ) && ! empty( $report_frequency ) && 'weekly' == $report_frequency ) {
			$start_time = $current_time;
			$start_time = strtotime( '-1 week', $start_time );
			$end_time   = strtotime( '+1 day', $current_time );
			$start_time = gmdate( 'Y-m-d 23:59:59', $start_time );
			$end_time   = gmdate( 'Y-m-d 00:00:00', $end_time );
			$where     .= $externaldb->prepare( ' AND modified_date > "%s" AND modified_date < "%s"', $start_time, $end_time );
		} elseif ( isset( $report_frequency ) && ! empty( $report_frequency ) && 'monthly' == $report_frequency ) {
			$start_time = $current_time;
			$start_time = strtotime( '-1 month', $start_time );
			$end_time   = strtotime( '+1 day', $current_time );
			$start_time = gmdate( 'Y-m-d 23:59:59', $start_time );
			$end_time   = gmdate( 'Y-m-d 00:00:00', $end_time );
			$where     .= $externaldb->prepare( ' AND modified_date > "%s" AND modified_date < "%s"', $start_time, $end_time );
		} elseif ( isset( $report_frequency ) && ! empty( $report_frequency ) && 'quarterly' == $report_frequency ) {
			$start_time = $current_time;
			$start_time = strtotime( '-3 month', $start_time );
			$end_time   = strtotime( '+1 day', $current_time );
			$start_time = gmdate( 'Y-m-d 23:59:59', $start_time );
			$end_time   = gmdate( 'Y-m-d 00:00:00', $end_time );
			$where     .= $externaldb->prepare( ' AND modified_date > "%s" AND modified_date < "%s"', $start_time, $end_time );
		}
		$get_log_query    = 'SELECT * FROM ' . UALP_TABLENAME . " $where ORDER BY modified_date desc";
		$result_log_query = $externaldb->get_results( $get_log_query );
		$csv_content      = array();
		$c                = 0;
		$date_format      = get_option( 'date_format' );
		$time_format      = get_option( 'time_format' );
		$logged_head      = '';
		$export_delimiter = ',';
		$random_id        = wp_rand();
		if ( isset( $ualp_generate_report_format ) && 'html' == $ualp_generate_report_format ) {
			if ( ! empty( $result_log_query ) ) {
				foreach ( $result_log_query as $result_log ) {

					$c++;
					$result_log = (array) $result_log;

					if ( ! ualp_allow_ip() ) {
						if ( isset( $result_log['ip_address'] ) ) {
							unset( $result_log['ip_address'] );
						}
					}
					if ( ! ualp_allow_country() ) {
						if ( isset( $result_log['country'] ) ) {
							unset( $result_log['country'] );
						}
					}
					if ( ! ualp_allow_user_agent() ) {
						if ( isset( $result_log['user_agent'] ) ) {
							unset( $result_log['user_agent'] );
						}
					}
					if ( isset( $result_log['uactid'] ) ) {
						$result_log['uactid'] = $c;
					}
					if ( isset( $result_log['favorite'] ) ) {
						if ( 1 == $result_log['favorite'] ) {
							$result_log['favorite'] = 'Yes';
						} elseif ( 0 == $result_log['favorite'] ) {
							$result_log['favorite'] = 'No';
						}
					}
					if ( isset( $result_log['modified_date'] ) ) {
						$get_date                    = strtotime( $result_log['modified_date'] );
						$date                        = gmdate( $date_format, $get_date );
						$time                        = gmdate( $time_format, $get_date );
						$result_log['modified_date'] = $date . ' ' . $time;
					}
					if ( isset( $result_log['description'] ) ) {
						if ( '' == $result_log['description'] ) {
							$action                    = '';
							$post_title                = '';
							$action                    = $result_log['action'];
							$post_title                = $result_log['post_title'];
							$result_log['description'] = $action . ' : ' . $post_title;
						} else {
							$result_log['description'] = html_entity_decode( $result_log['description'] );
						}
					}
					unset( $result_log['site_id'] );
					unset( $result_log['post_id'] );
					unset( $result_log['user_id'] );
					unset( $result_log['post_title'] );
					unset( $result_log['action'] );
					unset( $result_log['t_session'] );
					unset( $result_log['favorite'] );

					if ( 'yes' != $logged_head ) {
						foreach ( $result_log as $result_log_key => $result_log_val ) {
							if ( ! in_array( $result_log_key, $header ) ) {
								$header[] = $result_log_key;
							}
						}
						$result_log[ $result_log_key ] = html_entity_decode( $result_log_val );
					}
					$logged_head   = 'yes';
					$result_log1[] = $result_log;
				}
			}
			$csv_content  = '';
			$csv_content .= '<div class="ualp_generat_report_wrap">';
			$csv_content .= '<div class="report_header_title_wrap">';
			$csv_content .= '<p class="header_description">Report generated with</p>';
			$csv_content .= ' <h1>User Activity Log Pro</h1>';
			$csv_content .= '</div>';
			$csv_content .= '<div class="ualp_report_content_wrap">';
			$csv_content .= '<div class="report_details_cover">';
			$csv_content .= '<span class="report_details_title"><b>Report details:</b></span>';
			$csv_content .= '<span class="report_generated_span"><b>Generated On:</b>' . $display_current_time . '</span>';
			$csv_content .= '</div>';
			$csv_content .= '<div>';
			$csv_content .= '<table>';
			$csv_content .= '<thead>';
			$csv_content .= '<tr>';
			$header       = array_values( $header );
			$count_header = count( $header );
			for ( $h = 0;$h < $count_header;$h++ ) {
				$csv_content .= '<th>';
				if ( 'uactid' == $header[ $h ] ) {
					$csv_content .= esc_html__( 'Sr No.', 'user_activity_log_pro' );
				}
				if ( 'user_name' == $header[ $h ] ) {
					$csv_content .= esc_html__( 'Author Name', 'user_activity_log_pro' );
				}
				if ( 'user_role' == $header[ $h ] ) {
					$csv_content .= esc_html__( 'Author Role', 'user_activity_log_pro' );
				}
				if ( 'user_email' == $header[ $h ] ) {
					$csv_content .= esc_html__( 'Author E-mail', 'user_activity_log_pro' );
				}
				if ( ualp_allow_ip() ) {
					if ( 'ip_address' == $header[ $h ] ) {
						$csv_content .= esc_html__( 'IP Address', 'user_activity_log_pro' );
					}
				}
				if ( ualp_allow_country() ) {
					if ( 'country' == $header[ $h ] ) {
						$csv_content .= esc_html__( 'Country', 'user_activity_log_pro' );
					}
				}
				if ( ualp_allow_user_agent() ) {
					if ( 'user_agent' == $header[ $h ] ) {
						$csv_content .= esc_html__( 'User Agent', 'user_activity_log_pro' );
					}
				}
				if ( 'modified_date' == $header[ $h ] ) {
					$csv_content .= esc_html__( 'Date', 'user_activity_log_pro' );
				}
				if ( 'object_type' == $header[ $h ] ) {
					$csv_content .= esc_html__( 'Activity Type', 'user_activity_log_pro' );
				}
				if ( 'hook' == $header[ $h ] ) {
					$csv_content .= esc_html__( 'Activity Hook', 'user_activity_log_pro' );
				}
				if ( 'description' == $header[ $h ] ) {
					$csv_content .= esc_html__( 'Description', 'user_activity_log_pro' );
				}
				if ( 'favorite' == $header[ $h ] ) {
					$csv_content .= esc_html__( 'Favorite?', 'user_activity_log_pro' );
				}
				if ( 't_session' == $header[ $h ] ) {
					$csv_content .= esc_html__( 'T Session', 'user_activity_log_pro' );
				}
				if ( 'site_id' == $header[ $h ] ) {
					$csv_content .= esc_html__( 'Site Name', 'user_activity_log_pro' );
				}
				$csv_content .= '</th>';
			}
			$csv_content .= '</tr>';

			$csv_content .= '</thead>';
			$csv_content .= '<tbody>';
			$c            = 0;
			foreach ( $result_log1 as $htmlvalue ) {
				$csv_content .= '<tr>';
				foreach ( $htmlvalue as $htmlvalue1 ) {
					$csv_content .= '<th>';
					$csv_content .= $htmlvalue1;
					$csv_content .= '</th>';
				}
				$csv_content .= '</tr>';
				$c++;
			}

			$csv_content .= '</tbody>';

			$csv_content .= '</table>';
			$csv_content .= '</div>';

			$csv_content .= '</div>';
			$csv_content .= '</div>';
			$upload_dir   = WP_CONTENT_DIR . '/uploads/ualp/';
			$filename     = $upload_dir . 'export_user_report_' . gmdate( 'Y-m-d_H-i-s', time() ) . '.html';
			$csv_content .= '<style>
                .report_header_title_wrap {
                    display: inline-block;
                    width: 100%;
                    text-align: center;
                }
                .report_details_cover .report_details_title {
                    display: inline-block;
                    width: 100%;
                    border-bottom: 1px solid #000;
                    padding-bottom: 10px;
                }
                .report_generated_span {
                    display: inline-block;
                    width: 100%;
                    padding: 10px 0;
                }
                table {
                    font-family: arial, sans-serif;
                    border-collapse: collapse;
                    width: 100%;
                }
        
                td, th {
                    border:0;
                    text-align: left;
                    padding: 20px 10px;
                }
                thead th {
                    background: #000;
                    color: #fff;
                    opacity: 0.6;
                }
                tr:nth-child(even) {
                    background-color: #f1f1f1;
                }
                </style>';
			$savefile     = fopen( $filename, 'w' );
			fwrite( $savefile, $csv_content );
			fclose( $savefile );
			$report_title = esc_html__( 'User activity Notification', 'user_activity_log_pro' );
			$from_email   = get_option( 'ualpFromEmail' );

			foreach ( $schedule_report_content as $all_schedule_report_content ) {
				$report_title = $all_schedule_report_content->report_title;
				$to_email     = $all_schedule_report_content->schedule_email_id;
			}
			$headers         = 'From: ' . wp_strip_all_tags( $from_email ) . "\r\n";
			$headers         = 'Reply-To: ' . wp_strip_all_tags( $to_email ) . " \r\n";
			$headers        .= "MIME-Version: 1.0\r\n";
			$headers        .= "Content-Type: text/html; charset=ISO-8859-1\r\n";
			$subject         = $report_title;
			$message         = '<p>UALP Report</p>';
			$mail_result     = false;
			$mail_attachment = array( $filename );
			$mail_result     = wp_mail( $to_email, $subject, $message, $headers, $mail_attachment );
			echo esc_html( $mail_result );
		}
		if ( isset( $ualp_generate_report_format ) && 'csv' == $ualp_generate_report_format ) {
			if ( ! empty( $result_log_query ) ) {
				foreach ( $result_log_query as $result_log ) {
					$c++;
					$result_log = (array) $result_log;
					if ( ! ualp_allow_ip() ) {
						if ( isset( $result_log['ip_address'] ) ) {
							unset( $result_log['ip_address'] );
						}
					}
					if ( ! ualp_allow_country() ) {
						if ( isset( $result_log['country'] ) ) {
							unset( $result_log['country'] );
						}
					}
					if ( ! ualp_allow_user_agent() ) {
						if ( isset( $result_log['user_agent'] ) ) {
							unset( $result_log['user_agent'] );
						}
					}
					if ( isset( $result_log['uactid'] ) ) {
						$result_log['uactid'] = $c;
					}
					if ( isset( $result_log['favorite'] ) ) {
						if ( 1 == $result_log['favorite'] ) {
							$result_log['favorite'] = 'Yes';
						} elseif ( 0 == $result_log['favorite'] ) {
							$result_log['favorite'] = 'No';
						}
					}
					if ( isset( $result_log['modified_date'] ) ) {
						$get_date                    = strtotime( $result_log['modified_date'] );
						$date                        = gmdate( $date_format, $get_date );
						$time                        = gmdate( $time_format, $get_date );
						$result_log['modified_date'] = $date . ' ' . $time;
					}
					if ( isset( $result_log['description'] ) ) {
						if ( '' == $result_log['description'] ) {
							$action                    = '';
							$post_title                = '';
							$action                    = $result_log['action'];
							$post_title                = $result_log['post_title'];
							$result_log['description'] = $action . ' : ' . $post_title;
						} else {
							$result_log['description'] = html_entity_decode( $result_log['description'] );
						}
					}
					unset( $result_log['site_id'] );
					unset( $result_log['post_id'] );
					unset( $result_log['user_id'] );
					unset( $result_log['post_title'] );
					unset( $result_log['action'] );
					unset( $result_log['t_session'] );
					unset( $result_log['favorite'] );

					if ( 'yes' != $logged_head ) {
						foreach ( $result_log as $result_log_key => $result_log_val ) {
							if ( ! in_array( $result_log_key, $header ) ) {
								$header[] = $result_log_key;
							}
						}
						$result_log[ $result_log_key ] = html_entity_decode( $result_log_val );
					}
					$logged_head   = 'yes';
					$csv_content[] = $result_log;
				}
			}
			$header       = array_values( $header );
			$count_header = count( $header );
			for ( $h = 0;$h < $count_header;$h++ ) {
				if ( 'uactid' == $header[ $h ] ) {
					$header[ $h ] = esc_html__( 'Sr No.', 'user_activity_log_pro' );
				}
				if ( 'user_name' == $header[ $h ] ) {
					$header[ $h ] = esc_html__( 'Author Name', 'user_activity_log_pro' );
				}
				if ( 'user_role' == $header[ $h ] ) {
					$header[ $h ] = esc_html__( 'Author Role', 'user_activity_log_pro' );
				}
				if ( 'user_email' == $header[ $h ] ) {
					$header[ $h ] = esc_html__( 'Author E-mail', 'user_activity_log_pro' );
				}
				if ( ualp_allow_ip() ) {
					if ( 'ip_address' == $header[ $h ] ) {
						$header[ $h ] = esc_html__( 'IP Address', 'user_activity_log_pro' );
					}
				}
				if ( ualp_allow_country() ) {
					if ( 'country' == $header[ $h ] ) {
						$header[ $h ] = esc_html__( 'Country', 'user_activity_log_pro' );
					}
				}
				if ( ualp_allow_user_agent() ) {
					if ( 'user_agent' == $header[ $h ] ) {
						$header[ $h ] = esc_html__( 'User Agent', 'user_activity_log_pro' );
					}
				}
				if ( 'modified_date' == $header[ $h ] ) {
					$header[ $h ] = esc_html__( 'Date', 'user_activity_log_pro' );
				}
				if ( 'object_type' == $header[ $h ] ) {
					$header[ $h ] = esc_html__( 'Activity Type', 'user_activity_log_pro' );
				}
				if ( 'hook' == $header[ $h ] ) {
					$header[ $h ] = esc_html__( 'Activity Hook', 'user_activity_log_pro' );
				}
				if ( 'description' == $header[ $h ] ) {
					$header[ $h ] = esc_html__( 'Description', 'user_activity_log_pro' );
				}
				if ( 'favorite' == $header[ $h ] ) {
					$header[ $h ] = esc_html__( 'Favorite?', 'user_activity_log_pro' );
				}
				if ( 't_session' == $header[ $h ] ) {
					$header[ $h ] = esc_html__( 'T Session', 'user_activity_log_pro' );
				}
				if ( 'site_id' == $header[ $h ] ) {
					$header[ $h ] = esc_html__( 'Site Name', 'user_activity_log_pro' );
				}
			}
			$upload_dir = WP_CONTENT_DIR . '/uploads/ualp/';
			$filename   = $upload_dir . 'export_user_report_' . gmdate( 'Y-m-d_H-i-s', time() ) . '.csv';
			$savefile   = fopen( $filename, 'w' );
			fwrite( $savefile, $csv_content );
			fclose( $savefile );
			$report_title = esc_html__( 'User activity Notification', 'user_activity_log_pro' );
			$from_email   = get_option( 'ualpFromEmail' );

			foreach ( $schedule_report_content as $all_schedule_report_content ) {
				$report_title = $all_schedule_report_content->report_title;
				$to_email     = $all_schedule_report_content->schedule_email_id;
			}
			$headers         = 'From: ' . wp_strip_all_tags( $from_email ) . "\r\n";
			$headers         = 'Reply-To: ' . wp_strip_all_tags( $to_email ) . " \r\n";
			$headers        .= "MIME-Version: 1.0\r\n";
			$headers        .= "Content-Type: text/html; charset=ISO-8859-1\r\n";
			$subject         = $report_title;
			$message         = '<p>UALP Report</p>';
			$mail_result     = false;
			$mail_attachment = array( $filename );
			$mail_result     = wp_mail( $to_email, $subject, $message, $headers, $mail_attachment );
			echo esc_html( $mail_result );
		}
		if ( isset( $ualp_generate_report_format ) && 'pdf' == $ualp_generate_report_format ) {
			// $csv_content = '';.
			if ( ! empty( $result_log_query ) ) {
				foreach ( $result_log_query as $result_log ) {
					$c++;
					$result_log = (array) $result_log;
					if ( ! ualp_allow_ip() ) {
						if ( isset( $result_log['ip_address'] ) ) {
							unset( $result_log['ip_address'] );
						}
					}
					if ( ! ualp_allow_country() ) {
						if ( isset( $result_log['country'] ) ) {
							unset( $result_log['country'] );
						}
					}
					if ( ! ualp_allow_user_agent() ) {
						if ( isset( $result_log['user_agent'] ) ) {
							unset( $result_log['user_agent'] );
						}
					}
					if ( isset( $result_log['uactid'] ) ) {
						$result_log['uactid'] = $c;
					}
					if ( isset( $result_log['favorite'] ) ) {
						if ( 1 == $result_log['favorite'] ) {
							$result_log['favorite'] = 'Yes';
						} elseif ( 0 == $result_log['favorite'] ) {
							$result_log['favorite'] = 'No';
						}
					}
					if ( isset( $result_log['modified_date'] ) ) {
						$get_date                    = strtotime( $result_log['modified_date'] );
						$date                        = gmdate( $date_format, $get_date );
						$time                        = gmdate( $time_format, $get_date );
						$result_log['modified_date'] = $date . ' ' . $time;
					}
					if ( isset( $result_log['description'] ) ) {
						if ( '' == $result_log['description'] ) {
							$action                    = '';
							$post_title                = '';
							$action                    = $result_log['action'];
							$post_title                = $result_log['post_title'];
							$result_log['description'] = $action . ' : ' . $post_title;
						} else {
							$result_log['description'] = html_entity_decode( $result_log['description'] );
						}
					}
					unset( $result_log['site_id'] );
					unset( $result_log['post_id'] );
					unset( $result_log['user_id'] );
					unset( $result_log['post_title'] );
					unset( $result_log['action'] );
					unset( $result_log['t_session'] );

					if ( 'yes' != $logged_head ) {
						foreach ( $result_log as $result_log_key => $result_log_val ) {
							if ( ! in_array( $result_log_key, $header ) ) {
								$header[] = $result_log_key;
							}
						}
						$result_log[ $result_log_key ] = html_entity_decode( $result_log_val );
					}
					$logged_head      = 'yes';
					$htmlresult_log[] = $result_log;

				}
			}
			$csv_content  = '<table>';
			$csv_content .= '<tr>';
			$header       = array_values( $header );
			$count_header = count( $header );
			for ( $h = 0;$h < $count_header;$h++ ) {
				$csv_content .= '<td>';
				if ( 'uactid' == $header[ $h ] ) {
					$csv_content .= esc_html__( 'Sr No.', 'user_activity_log_pro' );
				}
				if ( 'user_name' == $header[ $h ] ) {
					$csv_content .= esc_html__( 'Author Name', 'user_activity_log_pro' );
				}
				if ( 'user_role' == $header[ $h ] ) {
					$csv_content .= esc_html__( 'Author Role', 'user_activity_log_pro' );
				}
				if ( 'user_email' == $header[ $h ] ) {
					$csv_content .= esc_html__( 'Author E-mail', 'user_activity_log_pro' );
				}
				if ( ualp_allow_ip() ) {
					if ( 'ip_address' == $header[ $h ] ) {
						$csv_content .= esc_html__( 'IP Address', 'user_activity_log_pro' );
					}
				}
				if ( ualp_allow_country() ) {
					if ( 'country' == $header[ $h ] ) {
						$csv_content .= esc_html__( 'Country', 'user_activity_log_pro' );
					}
				}
				if ( ualp_allow_user_agent() ) {
					if ( 'user_agent' == $header[ $h ] ) {
						$csv_content .= esc_html__( 'User Agent', 'user_activity_log_pro' );
					}
				}
				if ( 'modified_date' == $header[ $h ] ) {
					$csv_content .= esc_html__( 'Date', 'user_activity_log_pro' );
				}
				if ( 'object_type' == $header[ $h ] ) {
					$csv_content .= esc_html__( 'Activity Type', 'user_activity_log_pro' );
				}
				if ( 'hook' == $header[ $h ] ) {
					$csv_content .= esc_html__( 'Activity Hook', 'user_activity_log_pro' );
				}
				if ( 'description' == $header[ $h ] ) {
					$csv_content .= esc_html__( 'Description', 'user_activity_log_pro' );
				}
				if ( 'favorite' == $header[ $h ] ) {
					$csv_content .= esc_html__( 'Favorite?', 'user_activity_log_pro' );
				}
				if ( 't_session' == $header[ $h ] ) {
					$csv_content .= esc_html__( 'T Session', 'user_activity_log_pro' );
				}
				$csv_content .= '</td>';
			}
			$csv_content .= '</tr>';
			$c            = 0;
			if ( ! empty( $htmlresult_log ) ) {
				foreach ( $htmlresult_log as $htmldatavalue ) {
					$csv_content .= '<tr>';
					foreach ( $htmldatavalue as $html_content ) {
						$csv_content .= '<td>';
						$csv_content .= $html_content;
						$csv_content .= '</td>';
					}
					$csv_content .= '</tr>';
					$c++;
				}
			}
			$csv_content .= '</table>';
			$upload_dir   = WP_CONTENT_DIR . '/uploads/ualp/';
			ob_start();
			$filename = $upload_dir . 'export_user_report_' . gmdate( 'Y-m-d_H-i-s', time() ) . '.pdf';
			require 'html2pdf/html2pdf.php';
			$pdf = new PDF_HTML_Table();
			$pdf->AddPage();
			$pdf->SetFont( 'Arial', '', 10 );
			$pdf->WriteHTML( "User Activity Log Pro<br>$csv_content" );
			$pdf->Output( $filename, 'F' );
			ob_end_flush();
			$report_title = esc_html__( 'User activity Notification', 'user_activity_log_pro' );
			$from_email   = get_option( 'ualpFromEmail' );

			foreach ( $schedule_report_content as $all_schedule_report_content ) {
				$report_title = $all_schedule_report_content->report_title;
				$to_email     = $all_schedule_report_content->schedule_email_id;
			}
			$headers         = 'From: ' . wp_strip_all_tags( $from_email ) . "\r\n";
			$headers         = 'Reply-To: ' . wp_strip_all_tags( $to_email ) . " \r\n";
			$headers        .= "MIME-Version: 1.0\r\n";
			$headers        .= "Content-Type: text/html; charset=ISO-8859-1\r\n";
			$subject         = $report_title;
			$message         = '<p>UALP Report</p>';
			$mail_result     = false;
			$mail_attachment = array( $filename );
			$mail_result     = wp_mail( $to_email, $subject, $message, $headers, $mail_attachment );
			echo esc_html( $mail_result );
		}
	}
}


add_action( 'show_user_profile', 'ualp_view_user_activity_page', 999 );
add_action( 'edit_user_profile', 'ualp_view_user_activity_page', 999 );

if ( ! function_exists( 'ualp_view_user_activity_page' ) ) {
	/**
	 * View user activity page.
	 *
	 * @param string $user File name.
	 */
	function ualp_view_user_activity_page( $user ) {
		$user_login = $user->display_name;
		if ( is_network_admin() && ualp_multi_site_support_return() ) {
			$ualp_user_activity_link = network_admin_url( 'admin.php?page=ual_pro&usershow=' . $user_login );
		} else {
			$ualp_user_activity_link = admin_url( 'admin.php?page=ual_pro&usershow=' . $user_login );
		}
		?>
		<style>
			.ualp_user_activity_page {
				position: relative;
				float: left;
				top: 0;
				left: 0;
			}
			.ualp_user_activity {
				display: inline-block;
			}
		</style>
		<div class="ualp_user_activity"><a class="button-primary ualp_user_activity_page" target="_blank" href="<?php echo esc_url( $ualp_user_activity_link ); ?>"><?php esc_html_e( 'View Activity', 'user_activity_log_pro' ); ?></a></div>
		<?php
	}
}

add_action( 'add_meta_boxes', 'ualp_view_button_function' );
if ( ! function_exists( 'ualp_view_button_function' ) ) {
	/**
	 * View Button.
	 */
	function ualp_view_button_function() {
		global $pagenow ,$post;
		$post_type                 = $post->post_type;
		$log_hook_list             = get_option( 'enableLogHookList' );
		$activity_display_posttype = array( 'post', 'page' );
		$activity_posttype         = array( 'post_create', 'post_update', 'post_delete', 'page_create', 'page_update', 'page_delete' );
		if ( in_array( $post_type . '_create', $log_hook_list ) || in_array( $post_type . '_update', $log_hook_list ) || in_array( $post_type . '_delete', $log_hook_list ) ) {
			if ( 'post.php' == $pagenow && isset( $post->post_type ) && ( in_array( $post->post_type . '_create', $activity_posttype ) || in_array( $post->post_type . '_update', $activity_posttype ) || in_array( $post->post_type . '_delete', $activity_posttype ) ) && isset( $_GET['action'] ) && 'edit' == $_GET['action'] ) {
				$args = array();
				add_meta_box( 'post_activity_id', __( 'Activity Block', 'user_activity_log_pro' ), 'ualp_post_activity', $activity_display_posttype, 'side', 'high', $args );
			}
		}
	}
}

if ( ! function_exists( 'ualp_post_activity' ) ) {
	/**
	 * Post Activity.
	 *
	 * @param string $args Arguments.
	 */
	function ualp_post_activity( $args ) {
		global $post;
		$post_ID = $post->ID;
		if ( is_network_admin() && ualp_multi_site_support_return() ) {
			$ualp_user_activity_link = network_admin_url( 'admin.php?page=ual_pro&filterposts=' . $post_ID );
		} else {
			$ualp_user_activity_link = admin_url( 'admin.php?page=ual_pro&filterposts=' . $post_ID );
		}
		?>
		<style>#post_activity_id .ualp_post_activity_page{margin-top: 10px;} </style>
		<a class="button-primary ualp_post_activity_page" target="_blank" href="<?php echo esc_url( $ualp_user_activity_link ); ?>"><?php esc_html_e( 'View History', 'user_activity_log_pro' ); ?></a>
		<?php
	}
}
add_action( 'wp_ajax_nopriv_get_post_type_post_list', 'ualp_get_post_type_post_list', 16 );
add_action( 'wp_ajax_get_post_type_post_list', 'ualp_get_post_type_post_list', 16 );
/**
 * Get Post Type Lists.
 */
function ualp_get_post_type_post_list() {
	global $wpdb, $externaldb;
	$transition_where = '1 = 1';
	if ( ualp_multi_site_support_return() ) {
		$transition_where .= " AND site_id = '" . get_current_blog_id() . "'";
	}
		$transition_post_status = $externaldb->get_results(
			'SELECT * FROM ' . UALP_TABLENAME . '
                    WHERE ' . $transition_where . ' and hook="transition_post_status" and object_type IN ("page","post")
                    ORDER BY hook
            '
		);
	if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'ajax-nonce' ) ) {
		$posttypeval          = isset( $_POST['posttype'] ) ? sanitize_text_field( wp_unslash( $_POST['posttype'] ) ) : '';
		$transition_posttype  = array();
		$transition_posttitle = array();
		foreach ( $transition_post_status as $all_posts ) {
			$post_id_type  = $all_posts->object_type;
			$post_id_title = $all_posts->post_title;
			if ( '' != $all_posts->post_id ) {
				if ( $post_id_type == $posttypeval ) {
					$transition_posttitle[ $all_posts->post_id ] = $post_id_title;
				} elseif ( '' == $posttypeval ) {
					$transition_posttitle[ $all_posts->post_id ] = $post_id_title;
				}
				$transition_posttitle = array_unique( $transition_posttitle );
			}
		}
	}
	?>
	<select class="chosen-filter" name="showallpost" id="ualp-filter-showallpost">
		<?php
		printf( '<option value="">' . esc_html__( 'All Posts', 'user_activity_log_pro' ) . '</option>' );
		foreach ( $transition_posttitle as $key => $val ) {
			echo '<option value="' . esc_attr( $key ) . '">' . esc_html( $val ) . '</option>';
		}
		?>
	</select>
	<?php
	die();
}
add_action( 'wp_ajax_noriv_get_report_post_type_post_list', 'get_report_post_type_post_list', 17 );
add_action( 'wp_ajax_get_report_post_type_post_list', 'get_report_post_type_post_list', 17 );
/**
 * Get Report for Post Type Lists.
 */
function get_report_post_type_post_list() {
	global $wpdb, $externaldb;
	$transition_where = '1 = 1';
	if ( ualp_multi_site_support_return() ) {
		$transition_where .= " AND site_id = '" . get_current_blog_id() . "'";
	}
		$transition_post_status = $externaldb->get_results(
			'SELECT * FROM ' . UALP_TABLENAME . '
                    WHERE ' . $transition_where . ' and hook="transition_post_status" 
                    ORDER BY hook
            '
		);
	if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'ajax-nonce' ) ) {
		$posttypeval          = isset( $_POST['posttype'] ) ? sanitize_text_field( wp_unslash( $_POST['posttype'] ) ) : '';
		$transition_posttype  = array();
		$transition_posttitle = array();
		foreach ( $transition_post_status as $all_posts ) {
			$post_id_type  = $all_posts->object_type;
			$post_id_title = $all_posts->post_title;
			if ( '' != $all_posts->post_id ) {
				if ( in_array( $post_id_type, $posttypeval ) ) {
					$transition_posttitle[ $all_posts->post_id ] = $post_id_title;
				} elseif ( empty( $posttypeval ) ) {
					$transition_posttitle[ $all_posts->post_id ] = $post_id_title;
				}
					$transition_posttitle = array_unique( $transition_posttitle );
			}
		}
	}
	?>
		<select class="chosen-filter" name="showallpost" id="ualp-filter-showallpost "data-placeholder="<?php esc_html_e( 'Select Posts', 'user_activity_log_pro' ); ?>" multiple>
			<?php
			foreach ( $transition_posttitle as $key => $val ) {
				echo '<option value="' . esc_attr( $key ) . '">' . esc_html( $val ) . '</option>';
			}
			?>
		</select>
		<?php
		echo '<span class="report-note"><b>' . esc_html__( 'Note', 'user_activity_log_pro' ) . '&nbsp;:&nbsp;</b>' . esc_html__( 'Default All Posts Selected', 'user_activity_log_pro' ) . '</span>';
		die();
}
/**
 * Schedule Report for CSV Output.
 *
 * @param string $filename File name.
 * @param array  $data Data.
 * @param array  $fields Fields.
 * @param string $delimiter Delimiter.
 */
function ualp_schedule_report_csv_output( $filename = null, $data = array(), $fields = array(), $delimiter = null ) {
	if ( null === $delimiter ) {
		$delimiter = ',';
	}
	$random_id = wp_rand();
	$filename  = 'export_user_logged_' . gmdate( 'Y-m-d_H-i-s', time() ) . '.csv';
	if ( $filename ) {
		$data = ualp_unparse( $data, $fields, null, null, $delimiter );
	}
	return $data;
}
add_action( 'wp_ajax_ualp_get_trigger_name_data', 'ualp_get_trigger_name_data' );
add_action( 'wp_ajax_nopriv_ualp_get_trigger_name_data', 'ualp_get_trigger_name_data' );
/**
 * Get Trigger Name Data.
 */
function ualp_get_trigger_name_data() {
	if ( isset( $_POST['nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['nonce'] ) ), 'ajax-nonce' ) ) {
		$selectrow = 1;
		if ( isset( $_REQUEST['ualp_trigger_name'] ) ) {
			$ualp_trigger_name = sanitize_text_field( wp_unslash( $_REQUEST['ualp_trigger_name'] ) );
		}
		if ( isset( $_REQUEST['selectrow'] ) ) {
			$selectrow = sanitize_text_field( wp_unslash( $_REQUEST['selectrow'] ) );
		}
		$all_users = get_users();
		if ( 'user_name' == $ualp_trigger_name ) {
			?>
			<select name="custom_trigger_value_<?php echo esc_attr( $selectrow ); ?>[]" class="chosen-filter" multiple required>
				<?php
				if ( isset( $all_users ) && ! empty( $all_users ) ) {
					foreach ( $all_users as $get_all_users ) {
						$user_id   = $get_all_users->ID;
						$user_name = $get_all_users->user_login;
						echo '<option value="' . esc_attr( $user_name ) . '">' . esc_html( $user_name ) . '</option>';
					}
				}
				?>
			</select>
			<?php
		}
		if ( 'user_id' == $ualp_trigger_name ) {
			?>
			<select name="custom_trigger_value_<?php echo esc_attr( $selectrow ); ?>[]" class="chosen-filter" multiple required>
				<?php
				if ( isset( $all_users ) && ! empty( $all_users ) ) {
					foreach ( $all_users as $get_all_users ) {
						$user_id   = $get_all_users->ID;
						$user_name = $get_all_users->user_login;
						echo '<option value="' . esc_attr( $user_id ) . '">' . esc_html( $user_id ) . '</option>';
					}
				}
				?>
			</select>
			<?php
		}
		if ( 'user_role' == $ualp_trigger_name ) {
			?>
			<select name="custom_trigger_value_<?php echo esc_attr( $selectrow ); ?>[]" class="chosen-filter" multiple required>
				<?php
				$editable_roles = get_editable_roles();
				foreach ( $editable_roles as $role => $details ) {
					$name       = translate_user_role( $details['name'] );
					$role_label = esc_attr( $role );
					echo '<option value="' . esc_attr( $role_label ) . '">' . esc_html( $role_label ) . '</option>';
				}
				?>
			</select>
			<?php
		}
		if ( 'hook' == $ualp_trigger_name ) {
			global $wpdb,$allhookdetails;
			$post_types = ualp_posttypes();
			$taxonomies = ualp_taxonomies();
			$all_hook   = array( 'activated_plugin', 'deactivated_plugin', 'delete_plugin', 'plugin_modify', 'plugin_installed', 'plugin_updated', 'ualp_export_activity_log', 'ualp_general_settings_updated', 'ualp_hook_log_settings_updated', 'ualp_manage_role', 'ualp_set_password', 'ualp_change_password', 'ualp_email_settings_updated', 'ualp_custom_event_log_updated', 'ualp_exclude_log_updated', 'ualp_insert_email_notification', 'ualp_update_email_notification', 'wp_login', 'wp_login_failed', 'wp_logout', 'delete_user', 'user_register', 'insert_user_meta', 'password_reset', 'theme_file_modify', 'switch_theme', 'customize_save', 'theme_updated', 'theme_installed', 'delete_site_transient_update_themes', 'wp_insert_comment', 'edit_comment', 'delete_comment', 'trash_comment', 'spam_comment', 'restore_comment', 'comment_approve', 'comment_unapprove', 'wp_create_nav_menu', 'delete_nav_menu', 'admin_head', 'wp_head', '404_page', '_core_updated_successfully', 'import_end', 'export_wp', 'widget_update_callback', 'updated_option', 'dbdelta_queries', 'wp_privacy_personal_data_exporters', 'wp_privacy_personal_data_erasers', 'wp_update_nav_menu', 'pre_insert_term', 'wp_update_term_parent', 'delete_themes', 'before_delete_post', 'ualpSettingsUpdated', 'pre_delete_term', 'sidebar_admin_setup', 'updated_post_meta' );
			foreach ( $post_types as $post_type ) {
				if ( 'wpcf7_contact_form' != $post_type->name && 'custom_css' != $post_type->name && 'customize_changeset' != $post_type->name && 'attachment' != $post_type->name && 'oembed_cache' != $post_type->name && 'wp_block' != $post_type->name ) {
					array_push( $all_hook, $post_type->name . '_create', $post_type->name . '_update', $post_type->name . '_delete' );
				}
				if ( 'attachment' == $post_type->name ) {
					array_push( $all_hook, 'add_' . $post_type->name, 'edit_' . $post_type->name, 'delete_' . $post_type->name );
				}
			}
			foreach ( $taxonomies as $taxonomy ) {
				if ( 'wpcf7_contact_form' != $taxonomy->name && 'custom_css' != $taxonomy->name && 'customize_changeset' != $taxonomy->name && 'product_visibility' != $taxonomy->name ) {
					array_push( $all_hook, $taxonomy->name . '_create', $taxonomy->name . '_update', $taxonomy->name . '_delete' );
				}
			}
			if ( class_exists( 'Woocommerce' ) ) {
				array_push( $all_hook, 'woocommerce_updated_option', 'woocommerce_new_product_variation', 'wp_ajax_woocommerce_remove_variations', 'woocommerce_save_product_variation' );
			}
			if ( is_plugin_active( 'buddypress/bp-loader.php' ) ) {
				array_push( $all_hook, 'groups_create_group', 'groups_update_group', 'groups_before_delete_group', 'groups_leave_group', 'groups_join_group', 'groups_promote_member', 'groups_demote_member', 'groups_ban_member', 'groups_unban_member', 'groups_remove_member', 'xprofile_field_after_save', 'xprofile_fields_deleted_field', 'xprofile_group_after_save', 'xprofile_groups_deleted_group' );
			}
			if ( is_plugin_active( 'easy-digital-downloads/easy-digital-downloads.php' ) ) {
				array_push( $all_hook, 'edd_activate_discount', 'edd_deactivate_discount', 'edd_add_discount', 'edd_delete_discount', 'transition_post_status' );
			}
			if ( is_plugin_active( 'gravityforms/gravityforms.php' ) ) {
				array_push( $all_hook, 'gf_export_forms', 'notification_duplicated', 'gform_entry_created', 'gf_resend_notifications', 'gform_pre_note_deleted', 'gform_post_export_entries', 'gform_post_note_added', 'gform_delete_entries', 'gform_post_form_views_deleted', 'gform_post_form_activated', 'gform_post_form_deactivated', 'gform_before_delete_form', 'gform_post_form_trashed', 'gform_post_form_restored', 'gform_post_form_duplicated', 'gform_after_save_form', 'gform_forms_post_import', 'gform_pre_notification_deleted', 'gform_pre_confirmation_deleted', 'wp_ajax_rg_update_notification_active', 'wp_ajax_rg_update_confirmation_active', 'gform_pre_confirmation_save', 'gform_pre_notification_save', 'gform_delete_lead', 'gform_update_status', 'gform_update_is_read', 'gform_update_is_starred' );
			}
			if ( is_plugin_active( 'wordfence/wordfence.php' ) ) {
				array_push( $all_hook, 'wordfence_updateIssueStatus', 'wordfence_saveWAFConfig', 'wordfence_wafConfigureAutoPrepend', 'wordfence_whitelistBulkEnable', 'wordfence_whitelistBulkDisable', 'wordfence_whitelistBulkDelete', 'wordfence_updateConfig', 'wordfence_loadLiveTraffic', 'wordfence_loadCacheExclusions', 'wordfence_saveCacheConfig', 'wordfence_cache_clear', 'wordfence_reverseLookup', 'wordfence_whois', 'wordfence_clearPageCache', 'wordfence_saveConfig', 'wordfence_blockIP', 'wordfence_addTwoFactor', 'wordfence_exportSettings', 'wordfence_clearAllBlocked', 'wordfence_blockIPUARange', 'wordfence_unblockRange', 'wordfence_saveCountryBlocking', 'wordfence_saveDebuggingConfig', 'wordfence_startPasswdAudit', 'wordfence_saveScanSchedule', 'wordfence_updateWAFRules', 'wordfence_sendTestEmail', 'wordfence_sendDiagnostic', 'wordfence_updateAllIssues', 'wordfence_sendActivityLog', 'wordfence_addCacheExclusion', 'wordfence_daily_autoUpdate', 'wordfence_update_blocked_IPs', 'wordfence_start_scheduled_scan', 'wordfence_doScan', 'wordfence_daily_cron', 'wordfence_hourly_cron', 'wordfence_email_activity_report', 'wordfence_scan', 'wordfence_updateAlertEmail', 'wordfence_restoreFile', 'wordfence_deletePasswdAudit', 'wordfence_weakPasswordsFix', 'wordfence_killPasswdAudit', 'wordfence_importSettings', 'wordfence_bulkOperation', 'wordfence_deleteFile', 'wordfence_deleteDatabaseOption', 'wordfence_removeExclusion', 'wordfence_deleteIssue', 'wordfence_unlockOutIP', 'wordfence_unblockIP', 'wordfence_permBlockIP', 'wordfence_downloadHtaccess', 'wordfence_checkFalconHtaccess', 'wordfence_removeFromCache', 'wordfence_autoUpdateChoice', 'wordfence_saveCacheOptions', 'wordfence_adminEmailChoice', 'wordfence_getCacheStats', 'wordfence_killScan', 'wordfence_tourClosed', 'wordfence_removeCacheExclusion', 'wordfence_unblockNetwork', 'wordfence_permanentlyBlockAllIPs', 'wordfence_deleteAdminUser', 'wordfence_revokeAdminUser', 'wordfence_hideFileHtaccess', 'wordfence_loadBlockRanges' );
			}
			if ( is_plugin_active( 'wordpress-seo/wp-seo.php' ) ) {
				array_push( $all_hook, 'wpseo_handle_import', 'wpseo_import', 'wpseo_save_auth_code', 'seo_check_admin_referer', 'updated_post_meta' );
			}
			if ( is_plugin_active( 'contact-form-7/wp-contact-form-7.php' ) ) {
				array_push( $all_hook, 'wpcf7_after_create' );
				array_push( $all_hook, 'wpcf7_after_update' );
				array_push( $all_hook, 'wpcf7_admin_notices' );
				array_push( $all_hook, 'wpcf7-recaptcha-setup' );
			}
			if ( is_plugin_active( 'blog-designer-pro/blog-designer-pro.php' ) ) {
				array_push( $all_hook, 'bdp_update_shortcode', 'bdp_reset_shortcode', 'bdp_reset_archive_page', 'bdp_add_shortcode', 'bdp_update_single_file', 'bdp_add_archive_layout', 'bdp_delete_shortcode', 'bdp_delete_single_template', 'bdp_copy_single_template', 'bdp_duplicate_layout_settings', 'bdp_import_blog_layout_settings', 'bdp_import_archive_layout_settings', 'bdp_export_blog_layout_settings', 'bdp_export_archive_layout_settings', 'bdp_update_archive_page', 'bdp_import_single_layout_settings', 'bdp_export_single_layout_settings', 'bdp_delete_archive', 'bdp_delete_single_layout', 'bdp_update_single_page', 'bdp_reset_single_page', 'bdp_add_single_layout', 'bdp_duplicate_single_layout_settings' );
			}
			if ( is_plugin_active( 'portfolio-designer/portfolio-designer.php' ) ) {
				array_push( $all_hook, 'import_sample_portfolio_data', 'create_portfolio_layout', 'duplicate_portfolio_layout', 'delete_portfolio_layout', 'update_portfolio_layout', 'reset_portfolio_layout', 'create_post_type', 'update_post_type', 'export_portfolio_layout', 'delete_post_type' );
			}
			if ( is_plugin_active( 'all-in-one-reservation/all-in-one-reservation.php' ) ) {
				array_push( $all_hook, 'save_reservation_settings', 'rest_update_reservation_status', 'reset_reservation_settings' );
			}
			if ( is_plugin_active( 'wc-checkout-fields-editor/wc-checkout-fields-editor.php' ) ) {
				array_push( $all_hook, 'save_wcfe_settings', 'wp_ajax_wcfe_get_checkout_required_fields' );
			}
			if ( is_plugin_active( 'easy-digital-downloads/easy-digital-downloads.php' ) ) {
				array_push( $all_hook, 'edd_activate_discount', 'edd_settings', 'edd_deactivate_discount', 'edd_add_discount', 'edd_delete_discount' );
			}
			if ( is_plugin_active( 'bbpress/bbpress.php' ) ) {
				array_push( $all_hook, 'bbpress_settings' );
			}
			if ( is_plugin_active( 'autoptimize/autoptimize.php' ) ) {
				array_push( $all_hook, 'wp_ajax_autoptimize_delete_cache', 'autoptimizer_settings' );
			}
			if ( is_plugin_active( 'users-ultra/xoousers.php' ) ) {
				array_push( $all_hook, 'wp_ajax_save_fields_settings', 'wp_ajax_delete_profile_field', 'wp_ajax_sort_fileds_list', 'wp_ajax_custom_fields_reset', 'wp_ajax_create_uploader_folder', 'wp_ajax_package_add_new', 'wp_ajax_package_delete', 'wp_ajax_package_edit_confirm', 'wp_ajax_sync_users', 'wp_ajax_user_resend_activation_link', 'wp_ajax_delete_uultra_transtient', 'wp_ajax_delete_uultra_stats', 'wp_ajax_delete_uultra_reviews', 'wp_ajax_edit_photo_cate_conf', 'wp_ajax_edit_photo_cate_del', 'wp_ajax_edit_photo_cate', 'userultra_admin_menu_hook' );
			}
			if ( is_plugin_active( 'user-switching/user-switching.php' ) ) {
				array_push( $all_hook, 'switch_to_user', 'switch_back_user', 'switch_off_user' );
			}
			if ( is_plugin_active( 'advanced-custom-fields/acf.php' ) ) {
				array_push( $all_hook, 'export_acf' );
			}

			?>
			<select name="custom_trigger_value_<?php echo esc_attr( $selectrow ); ?>[]" class="chosen-filter" multiple required>
				<?php
				foreach ( $all_hook as $hook ) {
					$label = $hook;
					foreach ( $allhookdetails as $key => $val ) {
						if ( $hook == $key ) {
							$label = $val;
						}
					}
					echo '<option value="' . esc_attr( $hook ) . '">' . esc_html( $label ) . '</option>';
				}
				?>
				</select>
			<?php
		}
		if ( 'ip_address' == $ualp_trigger_name ) {
			?>
			<input class="ualp_event_medium" type="text" name="custom_trigger_value_<?php echo esc_attr( $selectrow ); ?>[]" placeholder="" value="" required /> 
			<?php
		}
		die();
	}
}

if ( ! function_exists( 'ualp_terminate_all_sessions' ) ) {
	/**
	 * Terminate all session.
	 */
	function ualp_terminate_all_sessions() {
		$users = get_users( array( 'fields' => array( 'ID' ) ) );
		global $wpdb;
		if ( is_multisite() ) {
			$logged_in_users = $wpdb->get_results( "Select * from {$wpdb->base_prefix}ualp_user_activity_login order by time_login desc" );
		} else {
			$logged_in_users = $wpdb->get_results( "Select * from {$wpdb->prefix}ualp_user_activity_login order by time_login desc" );
		}
		foreach ( $logged_in_users as $logged_in_user ) {
			$user_id           = $logged_in_user->user_id;
			$user_login_status = $logged_in_user->login_status;
			$session_token     = $logged_in_user->session_token;
			$login_status      = 'logout';
			$time_logout       = current_time( 'mysql' );
			if ( 'login' == $user_login_status && 'logout' == $login_status ) {
				$userloginstatus = $wpdb->query( $wpdb->prepare( "UPDATE {$wpdb->base_prefix}ualp_user_activity_login SET login_status = %s,time_logout = %s WHERE session_token = %s", $login_status, $time_logout, $session_token ) );
			}
		}
		foreach ( $users as $user_id ) {
			$all_user_id = $user_id->ID;
			$sessions    = WP_Session_Tokens::get_instance( $all_user_id );
			$sessions->destroy_all();
		}
		wp_logout();
		wp_login_url( get_permalink() );
	}
}

if ( ! function_exists( 'ualp_is_gutenberg_active' ) ) {
	/**
	 * Check if Gutenberg is enabled.
	 */
	function ualp_is_gutenberg_active() {
		$gutenberg    = false;
		$block_editor = false;
		if ( has_filter( 'replace_editor', 'gutenberg_init' ) ) {
			// Gutenberg is installed and activated.
			$gutenberg = true;
		}
		if ( version_compare( $GLOBALS['wp_version'], '5.0-beta', '>' ) ) {
			// Block editor.
			$block_editor = true;
		}
		if ( ! $gutenberg && ! $block_editor ) {
			return false;
		}
		include_once ABSPATH . 'wp-admin/includes/plugin.php';
		if ( ! is_plugin_active( 'classic-editor/classic-editor.php' ) && ! is_plugin_active( 'disable-gutenberg/disable-gutenberg.php' ) ) {
			return true;
		}
		$use_block_editor = ( get_option( 'classic-editor-replace' ) === 'no-replace' );
		return $use_block_editor;
	}
}

/**
 * Html Keses default.
 *
 * @since 1.0
 * @return array
 */
function ualp_html_kses() {
	$allowed_atts = array(
		'align'      => array(),
		'class'      => array(),
		'type'       => array(),
		'id'         => array(),
		'dir'        => array(),
		'lang'       => array(),
		'style'      => array(),
		'xml:lang'   => array(),
		'src'        => array(),
		'alt'        => array(),
		'href'       => array(),
		'rel'        => array(),
		'rev'        => array(),
		'target'     => array(),
		'novalidate' => array(),
		'type'       => array(),
		'value'      => array(),
		'name'       => array(),
		'tabindex'   => array(),
		'action'     => array(),
		'method'     => array(),
		'for'        => array(),
		'width'      => array(),
		'height'     => array(),
		'data'       => array(),
		'title'      => array(),
	);
	$allowed_tags = wp_kses_allowed_html( 'post' );
	return $allowed_tags;
}

/**
 * Argument for Kses.
 *
 * @since    1.0.0
 * @return  array
 */
function ualp_args_kses() {
	$args_kses = array(
		'div'    => array(
			'class'  => true,
			'id'     => true,
			'style'  => true,
			'script' => true,
		),
		'script' => array(
			'type'    => true,
			'charset' => true,
		),
		'style'  => array(
			'type' => true,
		),
		'iframe' => array(
			'src'          => true,
			'style'        => true,
			'marginwidth'  => true,
			'marginheight' => true,
			'scrolling'    => true,
			'frameborder'  => true,
		),
		'img'    => array(
			'src'      => true,
			'class'    => true,
			'alt'      => true,
			'style'    => true,
			'script'   => true,
			'srcset'   => true,
			'height'   => true,
			'width'    => true,
			'loading'  => true,
			'decoding' => true,
		),
		'a'      => array(
			'href'   => true,
			'class'  => true,
			'script' => true,
			'rel'    => true,
		),
		'ul'     => array(
			'class' => true,
			'id'    => true,
			'style' => true,
		),
		'li'     => array(
			'class' => true,
			'id'    => true,
			'style' => true,
		),
		'b'      => array(),
		'br'     => array(),
		'p'      => array(),
		'span'   => array(),
		'small'  => array(),
		'input'  => array(
			'class'       => true,
			'type'        => true,
			'name'        => true,
			'value'       => true,
			'placeholder' => true,
		),
		'label'  => array(),
		'h2'     => array(),
		'form'   => array(
			'name' => true,
			'id'   => true,
		),
		'option' => array(
			'value'           => true,
			'selected'        => true,
			'data-select2-id' => true,
		),

	);
	return $args_kses;
}
