<?php
/**
 * User Activity Settings Tab.
 *
 * @package User Activity Log PRO
 */

/**
 * Exit if accessed directly.
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! function_exists( 'ualp_general_settings' ) ) {
	/**
	 * User activity general Settings.
	 */
	function ualp_general_settings() {
		global $wpdb;
		if ( isset( $_POST['btnResetGeneralData'] ) ) {
			update_option( 'ualpKeepLogsDay', '' );
			update_option( 'ualp_allow_ip', '' );
			update_option( 'ualpAllowDashbordWidget', '' );
			update_option( 'ualp_allow_country', '' );
			update_option( 'ualp_allow_user_agent', '' );
			update_option( 'ualpDeleteAllData', '' );
			update_site_option( 'ualpMultisiteMenu', '' );
			update_option( 'ualpLogPasswordProtect', '1' );

			$message = esc_html__( 'General Settings reset successfully.', 'user_activity_log_pro' );

			ualp_admin_notice_message( 'updated', $message );
		}
		if ( isset( $_POST['btnSubmitGeneralData'] ) && isset( $_POST['_wp_general_log_nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wp_general_log_nonce'] ) ), '_wp_general_log_action' ) ) {
			// Log days settings.
			$logdel = isset( $_POST['logdel'] ) ? sanitize_text_field( wp_unslash( $_POST['logdel'] ) ) : '';
			update_option( 'ualpKeepLogsDay', $logdel );
			if ( isset( $_POST['ualpAllowIp'] ) ) {
				update_option( 'ualp_allow_ip', '1' );
			} else {
				update_option( 'ualp_allow_ip', '0' );
			}
			if ( isset( $_POST['ualpIPtypes'] ) ) {
				update_option( 'ualpIPtypes', sanitize_text_field( wp_unslash( $_POST['ualpIPtypes'] ) ) );
			} else {
				update_option( 'ualpIPtypes', 'REMOTE_ADDR' );
			}
			if ( isset( $_POST['ualpAllowDashbordWidget'] ) ) {
				update_option( 'ualpAllowDashbordWidget', '1' );
			} else {
				update_option( 'ualpAllowDashbordWidget', '0' );
			}
			if ( isset( $_POST['ualpAllowCountry'] ) ) {
				update_option( 'ualp_allow_country', '1' );
			} else {
				update_option( 'ualp_allow_country', '0' );
			}
			if ( isset( $_POST['ualpAllowUserAgent'] ) ) {
				update_option( 'ualp_allow_user_agent', '1' );
			} else {
				update_option( 'ualp_allow_user_agent', '0' );
			}
			if ( isset( $_POST['ualpDeleteAllData'] ) ) {
				update_option( 'ualpDeleteAllData', '1' );
			} else {
				update_option( 'ualpDeleteAllData', '0' );
			}
			if ( isset( $_POST['ualpMultisiteMenu'] ) ) {
				update_site_option( 'ualpMultisiteMenu', '1' );
			} else {
				update_site_option( 'ualpMultisiteMenu', '0' );
			}
			if ( isset( $_POST['ualpLogPasswordProtect'] ) ) {
				update_option( 'ualpLogPasswordProtect', '1' );
			} else {
				update_option( 'ualpLogPasswordProtect', '0' );
			}

			if ( isset( $_POST['logs_failed_login'] ) && ! empty( $_POST['logs_failed_login'] ) ) {
				update_option( 'logs_failed_login', sanitize_text_field( wp_unslash( $_POST['logs_failed_login'] ) ) );
			}
			if ( isset( $_POST['login_failed_existing_user'] ) && ! empty( $_POST['login_failed_existing_user'] ) ) {
				update_option( 'login_failed_existing_user', sanitize_text_field( wp_unslash( $_POST['login_failed_existing_user'] ) ) );
			}
			if ( isset( $_POST['login_failed_non_existing_user'] ) && ! empty( $_POST['login_failed_non_existing_user'] ) ) {
				update_option( 'login_failed_non_existing_user', sanitize_text_field( wp_unslash( $_POST['login_failed_non_existing_user'] ) ) );
			}
			if ( isset( $_POST['ualpSpreadsheetAndWorksheet'] ) && ! empty( $_POST['ualpSpreadsheetAndWorksheet'] ) ) {
				update_option( 'ualpSpreadsheetAndWorksheet', sanitize_text_field( wp_unslash( $_POST['ualpSpreadsheetAndWorksheet'] ) ) );
			}
			$message = esc_html__( 'General Settings updated successfully.', 'user_activity_log_pro' );
			ualp_admin_notice_message( 'updated', $message );
			do_action( 'ualp_general_settings_updated' );
		}
		$log_day                        = get_option( 'ualpKeepLogsDay' );
		$ualp_allow_ip                  = get_option( 'ualp_allow_ip' );
		$ualp_ip_types                  = get_option( 'ualpIPtypes', 'REMOTE_ADDR' );
		$ualp_allow_dashbord_widget     = get_option( 'ualpAllowDashbordWidget', true );
		$ualp_allow_country             = get_option( 'ualp_allow_country' );
		$ualp_allow_user_agent          = get_option( 'ualp_allow_user_agent' );
		$ualp_delete_all_data           = get_option( 'ualpDeleteAllData', 0 );
		$ualp_multisite_menu            = get_site_option( 'ualpMultisiteMenu', 0 );
		$logs_failed_login              = get_option( 'logs_failed_login' );
		$ualp_log_password_protect      = get_option( 'ualpLogPasswordProtect', true );
		$login_failed_non_existing_user = get_option( 'login_failed_non_existing_user' );
		$login_failed_existing_user     = get_option( 'login_failed_existing_user' );
		$ualp_spreadsheet_and_worksheet = get_option( 'ualpSpreadsheetAndWorksheet' );

		if ( is_network_admin() && ualp_multi_site_support_return() ) {
			$adminurl = network_admin_url( 'admin.php' );
		} else {
			$adminurl = admin_url( 'admin.php' );
		}
		$qry_str = isset( $_SERVER['QUERY_STRING'] ) ? sanitize_text_field( wp_unslash( $_SERVER['QUERY_STRING'] ) ) : '';
		?>
		<form class="ualpSettingsForm" action="<?php echo esc_attr( $adminurl ) . '?' . esc_attr( $qry_str ); ?>" method="POST">
			<div class="padding_left_right_15">
				<p class="margin_bottom_30"></p>
				<table>
					<tr>
						<th><?php esc_html_e( 'Enable Dashboard Widget', 'user_activity_log_pro' ); ?></th>
						<td>
							<input id="ualpAllowDashbordWidget" type="checkbox" value="1" <?php checked( '1', $ualp_allow_dashbord_widget ); ?> name="ualpAllowDashbordWidget">&nbsp;<label for="ualpAllowDashbordWidget"><?php esc_html_e( 'Display Dashboard Widget.', 'user_activity_log_pro' ); ?></label>
						</td>
					</tr>
					<tr>
						<th><?php esc_html_e( 'Enable IP Address For Log', 'user_activity_log_pro' ); ?></th>
						<td>
							<input id="ualpAllowIp" type="checkbox" value="1" <?php checked( '1', $ualp_allow_ip ); ?> name="ualpAllowIp">&nbsp;<label for="ualpAllowIp"><?php esc_html_e( 'Allow IP Address of users to log.', 'user_activity_log_pro' ); ?></label>
						</td>
					</tr>
					<tr>
						<th><?php esc_html_e( 'How does User Activity Log get IPs', 'user_activity_log_pro' ); ?></th>
						<td>
							<input id="ualpIPtypes-default" type="radio" value="REMOTE_ADDR" <?php checked( 'REMOTE_ADDR', $ualp_ip_types ); ?> name="ualpIPtypes">&nbsp;<label for="ualpIPtypes-default"><?php esc_html_e( 'Let Plugin use the most secure method to get visitor IP addresses. Prevents spoofing and works with most sites. (Recommended)', 'user_activity_log_pro' ); ?></label><br>
							<input id="ualpIPtypes-HTTP_X_FORWARDED_FOR" type="radio" value="HTTP_X_FORWARDED_FOR" <?php checked( 'HTTP_X_FORWARDED_FOR', $ualp_ip_types ); ?> name="ualpIPtypes">&nbsp;<label for="ualpIPtypes-HTTP_X_FORWARDED_FOR"><?php esc_html_e( 'Use the X-Forwarded-For HTTP header. Only use if you have a front-end proxy or spoofing may result.', 'user_activity_log_pro' ); ?></label><br>
							<input id="ualpIPtypes-HTTP_X_REAL_IP" type="radio" value="HTTP_X_REAL_IP" <?php checked( 'HTTP_X_REAL_IP', $ualp_ip_types ); ?> name="ualpIPtypes">&nbsp;<label for="ualpIPtypes-HTTP_X_REAL_IP"><?php esc_html_e( 'Use the X-Real-IP HTTP header. Only use if you have a front-end proxy or spoofing may result.', 'user_activity_log_pro' ); ?></label><br>
							<div class="ual-left"><?php esc_html_e( 'Detected IP(s):', 'user_activity_log_pro' ); ?> <span id="howGetIPs-preview-all"><strong><?php echo ualp_ip_info(); ?></strong></span></div>
							<div class="ual-left"><?php esc_html_e( 'Your IP with this setting:', 'user_activity_log_pro' ); ?> <span id="howGetIPs-preview-single"><?php echo esc_html( ualp_ip_info() ); ?></span></div>
						</td>
					</tr>
					<tr>
						<th><?php esc_html_e( 'Enable Country For Log', 'user_activity_log_pro' ); ?></th>
						<td>
							<input id="ualpAllowCountry" type="checkbox" value="1" <?php checked( '1', $ualp_allow_country ); ?> name="ualpAllowCountry">&nbsp;<label for="ualpAllowCountry"><?php esc_html_e( 'Allow Country of users to log.', 'user_activity_log_pro' ); ?></label>
						</td>
					</tr>
					<tr>
						<th><?php esc_html_e( 'Enable User Agent For Log', 'user_activity_log_pro' ); ?></th>
						<td>
							<input id="ualpAllowUserAgent" type="checkbox" value="1" <?php checked( '1', $ualp_allow_user_agent ); ?> name="ualpAllowUserAgent">&nbsp;<label for="ualpAllowUserAgent"><?php esc_html_e( 'Allow User agent of users to log.', 'user_activity_log_pro' ); ?></label>
						</td>
					</tr>
					<tr>
						<th><?php esc_html_e( 'Enable Protect password when deleting log?', 'user_activity_log_pro' ); ?></th>
						<td>
							<input id="ualpLogPasswordProtect" type="checkbox" value="1" <?php checked( '1', $ualp_log_password_protect ); ?> name="ualpLogPasswordProtect">&nbsp;<label for="ualpLogPasswordProtect"><?php esc_html_e( 'Enable Protect password when deleting log.', 'user_activity_log_pro' ); ?></label>
						</td>
					</tr>
					<tr>
						<th><?php esc_html_e( 'Keep Failed Login Logs', 'user_activity_log_pro' ); ?></th>
						<td>
							<select name="logs_failed_login">
								<option value="yes" 
								<?php
								if ( 'yes' == $logs_failed_login ) {
									echo 'selected="selected"'; }
								?>
								><?php esc_html_e( 'Keep', 'user_activity_log_pro' ); ?></option>
								<option value="no" 
								<?php
								if ( 'no' == $logs_failed_login ) {
									echo 'selected="selected"'; }
								?>
								><?php esc_html_e( "Don't Keep", 'user_activity_log_pro' ); ?> (<?php esc_html_e( 'Not recommended', 'user_activity_log_pro' ); ?>)</option>
							</select>
						</td>
					</tr>
					<tr class="no_of_failed_login">
							<th><?php esc_html_e( 'Number of failed login for existing user', 'user_activity_log_pro' ); ?></th>
							<td>
								<input type="number" step="1" min="0" placeholder="0" value="<?php echo esc_attr( $login_failed_existing_user ); ?>" name="login_failed_existing_user">
								<p><?php esc_html_e( 'Number of login attempts to log. Enter 0 to log all failed login attempts.', 'user_activity_log_pro' ); ?></p>
							</td>
						</tr>
						<tr class="no_of_failed_login">
							<th><?php esc_html_e( 'Number of failed login for non existing user', 'user_activity_log_pro' ); ?></th>
							<td>
								<input type="number" step="1" min="0" placeholder="0" value="<?php echo esc_attr( $login_failed_non_existing_user ); ?>" name="login_failed_non_existing_user">
								<p><?php esc_html_e( 'Number of login attempts to log. Enter 0 to log all failed login attempts.', 'user_activity_log_pro' ); ?></p>
							</td>
						</tr>
					<tr>
						<th><?php esc_html_e( 'Keep logs for', 'user_activity_log_pro' ); ?></th>
						<td>
							<input type="number" step="1" min="1" placeholder="30" value="<?php echo esc_attr( $log_day ); ?>" name="logdel">&nbsp;<?php esc_html_e( 'Days', 'user_activity_log_pro' ); ?>
							<p>
								<?php
									esc_html_e( 'Maximum number of days to keep activity log. Leave blank to keep activity log forever', 'user_activity_log_pro' );
									echo '(';
									esc_html_e( 'not recommended', 'user_activity_log_pro' );
									echo ').';
								?>
							</p>
						</td>
					</tr>
					<tr>
						<th><?php esc_html_e( 'Delete data on deletion of plugin', 'user_activity_log_pro' ); ?></th>
						<td>
							<input id="ualpDeleteAllData" type="checkbox" value="1" <?php checked( '1', $ualp_delete_all_data ); ?> name="ualpDeleteAllData">&nbsp;<label for="ualpDeleteAllData"><?php esc_html_e( 'Delete data on deletion of plugin', 'user_activity_log_pro' ); ?></label>
						</td>
					</tr>
					<?php if ( is_network_admin() && ualp_multi_site_support_return() ) { ?>
						<tr>
							<th><?php esc_html_e( 'Enable Settings Menu in every site (Multisite Feature)', 'user_activity_log_pro' ); ?></th>
							<td>
								<input id="ualpMultisiteMenu" type="checkbox" value="1" <?php checked( '1', $ualp_multisite_menu ); ?> name="ualpMultisiteMenu">&nbsp;<label for="ualpMultisiteMenu"><?php esc_html_e( 'Display Settings menu in every site.', 'user_activity_log_pro' ); ?></label>
							</td>
						</tr>
					<?php } ?>
					<tr>
						<th><?php esc_html_e( 'Delete Log Activities', 'user_activity_log_pro' ); ?></th>
						<td>
							<?php
							$nonce               = wp_create_nonce( 'my-nonce' );
							$ualp_delete_log_pwd = get_option( 'ualp_delete_log_pwd' );
							if ( ! $ualp_delete_log_pwd || '' == $ualp_delete_log_pwd ) {
								$class = 'setDeleteDataPassword';
							} else {
								$class = 'deleteLogData';
							}
							if ( is_network_admin() && ualp_multi_site_support_return() ) {
								$adminurlsetting = network_admin_url( 'admin.php?page=ualp_settings' );
							} else {
								$adminurlsetting = admin_url( 'admin.php?page=ualp_settings' );
							}
							?>
							<a id="<?php echo esc_attr( $class ); ?>" href="<?php echo esc_url( $adminurlsetting ); ?>&_wpnonce=<?php echo esc_url( $nonce ); ?>">
								<?php esc_html_e( 'Reset Database', 'user_activity_log_pro' ); ?>
							</a>
							<p>
							<?php
								echo '<b class="red">' . esc_html__( 'Warning', 'user_activity_log_pro' ) . ' : </b>';
								esc_html_e( 'Clicking this will delete all activities from the database.', 'user_activity_log_pro' );
							?>
							</p>
						</td>
					</tr>
					<?php
					$ualp_add_json_credentials = get_option( 'ualpAddJsonCredentials' );
					$json_credentials          = json_decode( $ualp_add_json_credentials );
					if ( isset( $json_credentials->client_email ) ) {
						$sheets = Ualp_Google_Sheets_API::ualp_spreadsheets_and_worksheets();
						?>
						<tr>
							<th><?php esc_html_e( 'Spreadsheets and Worksheets', 'user_activity_log_pro' ); ?></th>
							<td>
								<select name='ualpSpreadsheetAndWorksheet' id='ualpSpreadsheetAndWorksheet' class="regular-text" >
									<option value=""  > <?php esc_html_e( 'Select Spreadsheets and Worksheets', 'user_activity_log_pro' ); ?> </option>
									<?php
									if ( isset( $sheets ) ) {
										foreach ( $sheets as $sheet ) {
											foreach ( $sheet[1] as $sub_sheet ) {
												?>
											<option value="<?php echo esc_attr( $sub_sheet ); ?>" 
												<?php
												if ( $sub_sheet == $ualp_spreadsheet_and_worksheet ) {
													echo 'selected="selected"'; }
												?>
											><?php echo esc_html( $sheet[0] ) . ' :: ' . esc_html( $sub_sheet ); ?></option>
												<?php
											}
										}
									}
									?>
								</select>
							</td>
						</tr>
					<?php } ?>
				</table>
				<?php
				wp_nonce_field( '_wp_general_log_action', '_wp_general_log_nonce' );
				?>
				<p class="submit">
					<input type="submit" name="btnSubmitGeneralData" class="button button-primary" id="submit" value="<?php esc_attr_e( 'Save Changes', 'user_activity_log_pro' ); ?>" >
					<input type="submit" name="btnResetGeneralData" class="button-secondary" value="<?php esc_html_e( 'Reset Changes', 'user_activity_log_pro' ); ?>"  onclick="return confirm('<?php esc_js( esc_html_e( 'Are you sure to reset data?', 'user_activity_log_pro' ) ); ?>')" >
				</p>
			</div>
		</form>
		<?php
	}
}

if ( ! function_exists( 'ualp_logged_in_users_settings' ) ) {
	/**
	 * User activity logged in users Settings.
	 */
	function ualp_logged_in_users_settings() {
		$table = new UALP_User_Logged_Activity_Log_List_Table();
		$table->prepare_items();
		$message   = '';
		$args_kses = ualp_args_kses();
		if ( isset( $_REQUEST['wp_logged_in_users_nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_REQUEST['wp_logged_in_users_nonce'] ) ), 'wp_logged_in_users_action' ) ) {
			if ( isset( $_REQUEST['action'] ) == 'delete' || isset( $_REQUEST['bulk_action'] ) == 'Apply' ) {
				$message = '<div class="updated below-h2" id="message"><p>' . __( 'Logged deleted successfully', 'user_activity_log_pro' ) . '</p></div>';
				echo wp_kses( $message, $args_kses );
			}
		}
		$host        = isset( $_SERVER['HTTP_HOST'] ) ? esc_url_raw( wp_unslash( $_SERVER['HTTP_HOST'] ) ) : '';
		$request_url = isset( $_SERVER['REQUEST_URI'] ) ? esc_url_raw( wp_unslash( $_SERVER['REQUEST_URI'] ) ) : '';
		$current_url = ( isset( $_SERVER['HTTPS'] ) ? 'https://' : 'http://' ) . $host . $request_url;
		$req_page    = isset( $_REQUEST['page'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['page'] ) ) : '';
		?>
		<h1><?php esc_html_e( 'Logged In/Logged Out Users', 'user_activity_log_pro' ); ?></h1>
		<form class="ualploginSettingsForm"  method="get">
			<input type="hidden" name="page" value="<?php echo esc_attr( $req_page ); ?>" />
			<div class="padding_right_15">
				<?php $table->display(); ?>
				<p class="margin_bottom_30"></p>
			</div>
			<?php wp_nonce_field( 'wp_logged_in_users_action', 'wp_logged_in_users_nonce' ); ?>
		</form>
		<?php
	}
}


if ( ! function_exists( 'ualp_log_settings' ) ) {
	/**
	 * User activity log Settings.
	 */
	function ualp_log_settings() {
		global $allhookdetails;
		global $wpdb;
		$post_types = ualp_posttypes();
		$taxonomies = ualp_taxonomies();
		$tax_count  = count( $taxonomies ) - 1;
		if ( isset( $_POST['btnSubmitLog'] ) && isset( $_POST['_wp_log_hook_nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wp_log_hook_nonce'] ) ), '_wp_log_hook_action' ) ) {
			$log_hook_list = ( isset( $_POST['hook'] ) && ( '' != $_POST['hook'] ) ) ? array_map( 'sanitize_text_field', wp_unslash( $_POST['hook'] ) ) : array();
			update_option( 'enableLogHookList', $log_hook_list );
			$message = esc_html__( 'Log Hook Settings updated successfully.', 'user_activity_log_pro' );
			ualp_admin_notice_message( 'updated', $message );
			do_action( 'ualp_hook_log_settings_updated' );
		}
		// Set default enable log for action type.
		if ( isset( $_POST['btnResetLogData'] ) ) {
			$log_hook = array();
			foreach ( $post_types as $post_type ) {
				if ( is_multisite() ) {
					$post_type_hook[] = $post_type->name;
				}
				$post_type_hook = !is_array( $post_type_hook ) ? array() : $post_type_hook;
				if ( 'attachment' != $post_type->name ) {
					array_push( $post_type_hook, $post_type->name . '_create' );
					array_push( $post_type_hook, $post_type->name . '_update' );
					array_push( $post_type_hook, $post_type->name . '_delete' );
				}
				if ( 'attachment' == $post_type->name ) {
					array_push( $post_type_hook, 'add_' . $post_type->name );
					array_push( $post_type_hook, 'edit_' . $post_type->name );
					array_push( $post_type_hook, 'delete_' . $post_type->name );
				}
			}
			$log_hook = array_merge( $log_hook, $post_type_hook );
			foreach ( $taxonomies as $taxonomy ) {
				if ( is_multisite() ) {
					$taxonomy_hook[] = $taxonomy->name;
				}
				$taxonomy_hook = !is_array( $taxonomy_hook ) ? array() : $taxonomy_hook;
				if ( 'wpcf7_contact_form' != $taxonomy->name && 'custom_css' != $taxonomy->name && 'customize_changeset' != $taxonomy->name && 'product_visibility' != $taxonomy->name ) {
					array_push( $taxonomy_hook, $taxonomy->name . '_create' );
					array_push( $taxonomy_hook, $taxonomy->name . '_update' );
					array_push( $taxonomy_hook, $taxonomy->name . '_delete' );
				}
			}
			$log_hook              = array_merge( $log_hook, $taxonomy_hook );
			$plugin_hook           = array( 'activated_plugin', 'deactivated_plugin', 'delete_plugin', 'plugin_modify', 'plugin_updated', 'plugin_installed' );
			$log_hook              = array_merge( $plugin_hook, $log_hook );
			$contact_hook          = array( 'wpcf7_after_create', 'wpcf7_after_update', 'wpcf7_admin_notices', 'wpcf7-recaptcha-setup' );
			$log_hook              = array_merge( $contact_hook, $log_hook );
			$ualp_hook             = array( 'ualp_export_activity_log', 'ualp_general_settings_updated', 'ualp_hook_log_settings_updated', 'ualp_manage_role', 'ualp_set_password', 'ualp_change_password', 'ualp_email_settings_updated' );
			$log_hook              = array_merge( $ualp_hook, $log_hook );
			$user_hook             = array( 'wp_login', 'wp_login_failed', 'wp_logout', 'delete_user', 'user_register', 'insert_user_meta' );
			$log_hook              = array_merge( $user_hook, $log_hook );
			$theme_hook            = array( 'theme_file_modify', 'switch_theme', 'customize_save', 'theme_updated', 'theme_installed', 'delete_site_transient_update_themes' );
			$log_hook              = array_merge( $theme_hook, $log_hook );
			$comment_hook          = array( 'wp_insert_comment', 'edit_comment', 'trash_comment', 'spam_comment', 'restore_comment', 'delete_comment', 'comment_unapprove', 'comment_approve' );
			$log_hook              = array_merge( $comment_hook, $log_hook );
			$menuview_hook         = array( 'wp_create_nav_menu', 'wp_update_nav_menu', 'delete_nav_menu' );
			$log_hook              = array_merge( $menuview_hook, $log_hook );
			$other_hook            = array( '_core_updated_successfully', 'import_end', 'export_wp', 'widget_update_callback', 'updated_option', 'woocommerce_updated_option', 'dbdelta_queries', 'wp_privacy_personal_data_exporters', 'wp_privacy_personal_data_erasers' );
			$log_hook              = array_merge( $other_hook, $log_hook );
			$supported_plugin_hook = array( 'woocommerce_new_product_variation', 'wp_ajax_woocommerce_remove_variations', 'woocommerce_save_product_variation', 'groups_create_group', 'groups_update_group', 'groups_before_delete_group', 'groups_leave_group', 'groups_join_group', 'groups_promote_member', 'groups_demote_member', 'groups_ban_member', 'groups_unban_member', 'groups_remove_member', 'xprofile_field_after_save', 'xprofile_fields_deleted_field', 'xprofile_group_after_save', 'xprofile_groups_deleted_group', 'wordfence_updateIssueStatus', 'wordfence_saveWAFConfig', 'wordfence_wafConfigureAutoPrepend', 'wordfence_whitelistBulkEnable', 'wordfence_whitelistBulkDisable', 'wordfence_whitelistBulkDelete', 'wordfence_updateConfig', 'wordfence_loadLiveTraffic', 'wordfence_loadCacheExclusions', 'wordfence_saveCacheConfig', 'wordfence_cache_clear', 'wordfence_reverseLookup', 'wordfence_whois', 'wordfence_clearPageCache', 'wordfence_saveConfig', 'wordfence_blockIP', 'wordfence_addTwoFactor', 'wordfence_exportSettings', 'wordfence_clearAllBlocked', 'wordfence_blockIPUARange', 'wordfence_unblockRange', 'wordfence_saveCountryBlocking', 'wordfence_saveDebuggingConfig', 'wordfence_startPasswdAudit', 'wordfence_saveScanSchedule', 'wordfence_updateWAFRules', 'wordfence_sendTestEmail', 'wordfence_sendDiagnostic', 'wordfence_updateAllIssues', 'wordfence_sendActivityLog', 'wordfence_addCacheExclusion', 'wordfence_daily_autoUpdate', 'wordfence_update_blocked_IPs', 'wordfence_start_scheduled_scan', 'wordfence_doScan', 'wordfence_daily_cron', 'wordfence_hourly_cron', 'wordfence_email_activity_report', 'wordfence_scan', 'wordfence_updateAlertEmail', 'wordfence_restoreFile', 'wordfence_deletePasswdAudit', 'wordfence_weakPasswordsFix', 'wordfence_killPasswdAudit', 'wordfence_importSettings', 'wordfence_bulkOperation', 'wordfence_deleteFile', 'wordfence_deleteDatabaseOption', 'wordfence_removeExclusion', 'wordfence_deleteIssue', 'wordfence_unlockOutIP', 'wordfence_loadBlockRanges', 'wordfence_unblockIP', 'wordfence_permBlockIP', 'wordfence_downloadHtaccess', 'wordfence_checkFalconHtaccess', 'wordfence_removeFromCache', 'wordfence_autoUpdateChoice', 'wordfence_saveCacheOptions', 'wordfence_adminEmailChoice', 'wordfence_getCacheStats', 'wordfence_killScan', 'wordfence_tourClosed', 'wordfence_removeCacheExclusion', 'wordfence_unblockNetwork', 'wordfence_permanentlyBlockAllIPs', 'wordfence_deleteAdminUser', 'wordfence_revokeAdminUser', 'wordfence_hideFileHtaccess', 'wpseo_handle_import', 'wpseo_import', 'wpseo_save_auth_code', 'seo_check_admin_referer', 'updated_post_meta', 'bdp_update_shortcode', 'bdp_reset_shortcode', 'bdp_reset_archive_page', 'bdp_add_shortcode', 'bdp_update_single_file', 'bdp_add_archive_layout', 'bdp_delete_shortcode', 'bdp_delete_single_template', 'bdp_copy_single_template', 'bdp_duplicate_layout_settings', 'bdp_import_blog_layout_settings', 'bdp_import_archive_layout_settings', 'bdp_export_archive_layout_settings', 'bdp_export_blog_layout_settings', 'bdp_update_archive_page', 'bdp_import_single_layout_settings', 'bdp_export_single_layout_settings', 'bdp_delete_archive', 'bdp_delete_single_layout', 'bdp_update_single_page', 'bdp_reset_single_page', 'bdp_add_single_layout', 'bdp_duplicate_single_layout_settings', 'import_sample_portfolio_data', 'create_portfolio_layout', 'duplicate_portfolio_layout', 'delete_portfolio_layout', 'update_portfolio_layout', 'reset_portfolio_layout', 'import_portfolio_layout', 'export_portfolio_layout', 'create_post_type', 'update_post_type', 'delete_post_type', 'autoptimizer_settings', 'wp_ajax_autoptimize_delete_cache', 'switch_to_user', 'switch_back_user', 'switch_off_user', 'reset_reservation_settings', 'rest_update_reservation_status', 'save_reservation_settings', 'edd_activate_discount', 'edd_deactivate_discount', 'edd_add_discount', 'edd_delete_discount', 'transition_post_status', 'gf_export_forms', 'notification_duplicated', 'gform_entry_created', 'gf_resend_notifications', 'gform_pre_note_deleted', 'gravityforms_updated_option', 'gform_post_export_entries', 'gform_post_note_added', 'gform_delete_entries', 'gform_post_form_views_deleted', 'gform_post_form_activated', 'gform_post_form_deactivated', 'gform_before_delete_form', 'gform_post_form_trashed', 'gform_post_form_restored', 'gform_post_form_duplicated', 'gform_after_save_form', 'gform_forms_post_import', 'gform_pre_notification_deleted', 'gform_pre_confirmation_deleted', 'wp_ajax_rg_update_notification_active', 'wp_ajax_rg_update_confirmation_active', 'gform_pre_confirmation_save', 'gform_pre_notification_save', 'gform_delete_lead', 'gform_update_status', 'gform_update_is_read', 'gform_update_is_starred', 'wp_ajax_add_new_custom_profile_field', 'wp_ajax_save_fields_settings', 'wp_ajax_delete_profile_field', 'wp_ajax_sort_fileds_list', 'wp_ajax_custom_fields_reset', 'wp_ajax_create_uploader_folder', 'user_ultra_settings', 'wp_ajax_package_add_new', 'wp_ajax_package_delete', 'wp_ajax_package_edit_confirm', 'wp_ajax_sync_users', 'wp_ajax_user_resend_activation_link', 'wp_ajax_delete_uultra_transtient', 'wp_ajax_delete_uultra_stats', 'wp_ajax_delete_uultra_reviews', 'wp_ajax_edit_photo_cate_conf', 'wp_ajax_edit_photo_cate_del', 'wp_ajax_edit_photo_cate', 'userultra_admin_menu_hook', 'user_ultra_settings', 'user_ultra_settings', 'user_ultra_settings', 'save_wcfe_settings', 'wp_ajax_wcfe_get_checkout_required_fields' );
			$log_hook              = array_merge( $supported_plugin_hook, $log_hook );
			update_option( 'enableLogHookList', $log_hook );

			$message = esc_html__( 'Log hook settings reset successfully.', 'user_activity_log_pro' );
			ualp_admin_notice_message( 'updated', $message );
		}
		$log_hook_list = get_option( 'enableLogHookList' );
		if ( ! is_array( $log_hook_list ) ) {
			update_option( 'enableLogHookList', array() );
		}
		if ( is_network_admin() && ualp_multi_site_support_return() ) {
			$adminurl = network_admin_url( 'admin.php' );
		} else {
			$adminurl = admin_url( 'admin.php' );
		}
		$qry_str = isset( $_SERVER['QUERY_STRING'] ) ? sanitize_text_field( wp_unslash( $_SERVER['QUERY_STRING'] ) ) : '';
		?>
		<form class="ualpSettingsForm" action="<?php echo esc_attr( $adminurl ) . '?' . esc_attr( $qry_str ); ?>" method="POST">
			<div class="padding_left_right_15">
				<h3 class="ualpHeaderTitle"><?php esc_html_e( 'Display Option', 'user_activity_log_pro' ); ?></h3>
				<table class="widefat post">
					<tbody>
						<tr class="groupHeader">
							<th class="check-column" scope="row">
								<input type="checkbox" name="" data-hookGroup="post_type">
							</th>
							<td colspan="2"><?php esc_html_e( 'Post Types', 'user_activity_log_pro' ); ?></td>
						</tr>
						<?php
						foreach ( $post_types as $post_type ) {
							if ( 'wpcf7_contact_form' != $post_type->name && 'custom_css' != $post_type->name && 'customize_changeset' != $post_type->name && 'attachment' != $post_type->name && 'oembed_cache' != $post_type->name && 'wp_block' != $post_type->name ) {
								?>
								<tr class="post_type">
									<th class="check-column" scope="row">
										<input value="<?php echo esc_attr( $post_type->name ); ?>_create" id="<?php echo esc_attr( $post_type->name ); ?>_create" <?php checked( in_array( $post_type->name . '_create', $log_hook_list ), 1 ); ?> data-type="<?php echo esc_attr( $post_type->label ); ?>" name="hook[]" type="checkbox">
									</th>
									<td><label for="<?php echo esc_attr( $post_type->name ); ?>_create"><?php echo esc_attr( $post_type->label ); ?> Create <?php echo '( ' . esc_attr( $post_type->name ) . '_create )'; ?></label></td>
									<td>
									<?php
									echo esc_html( $post_type->label ) . ' : ';
									esc_html_e( 'When', 'user_activity_log_pro' );
									echo ' ' . esc_html( $post_type->name ) . ' ';
									esc_html_e( 'is created.', 'user_activity_log_pro' );
									?>
									</td>
								</tr>
								<tr class="post_type">
									<th class="check-column" scope="row">
										<input value="<?php echo esc_attr( $post_type->name ); ?>_update" id="<?php echo esc_attr( $post_type->name ); ?>_update" <?php checked( in_array( $post_type->name . '_update', $log_hook_list ), 1 ); ?> data-type="<?php echo esc_attr( $post_type->label ); ?>" name="hook[]" type="checkbox">
									</th>
									<td><label for="<?php echo esc_attr( $post_type->name ); ?>_update"><?php echo esc_attr( $post_type->label ); ?> Update <?php echo '( ' . esc_attr( $post_type->name ) . '_update )'; ?></label></td>
									<td>
									<?php
									echo esc_html( $post_type->label ) . ' : ';
									esc_html_e( 'When', 'user_activity_log_pro' );
									echo ' ' . esc_html( $post_type->name ) . ' ';
									esc_html_e( 'is updated.', 'user_activity_log_pro' );
									?>
									</td>
								</tr>
								<tr class="post_type">
									<th class="check-column" scope="row">
										<input value="<?php echo esc_attr( $post_type->name ); ?>_delete" id="<?php echo esc_attr( $post_type->name ); ?>_delete" <?php checked( in_array( $post_type->name . '_delete', $log_hook_list ), 1 ); ?> data-type="<?php echo esc_attr( $post_type->label ); ?>" name="hook[]" type="checkbox">
									</th>
									<td><label for="<?php echo esc_attr( $post_type->name ); ?>_delete"><?php echo esc_attr( $post_type->label ); ?> Delete <?php echo '( ' . esc_attr( $post_type->name ) . '_delete )'; ?></label></td>
									<td>
									<?php
									echo esc_html( $post_type->label ) . ' : ';
									esc_html_e( 'When', 'user_activity_log_pro' );
									echo ' ' . esc_html( $post_type->name ) . ' ';
									esc_html_e( 'is deleted.', 'user_activity_log_pro' );
									?>
									</td>
								</tr>
								<?php
							}
							if ( 'attachment' == $post_type->name ) {
								?>
								<tr class="post_type">
									<th class="check-column" scope="row">
										<input value="<?php echo 'add_' . esc_attr( $post_type->name ); ?>" id="<?php echo 'add_' . esc_attr( $post_type->name ); ?>" <?php checked( in_array( 'add_' . $post_type->name, $log_hook_list ), 1 ); ?> data-type="<?php echo esc_attr( $post_type->label ); ?>" name="hook[]" type="checkbox">
									</th>
									<td><label for="<?php echo 'add_' . esc_attr( $post_type->name ); ?>"><?php echo esc_attr( $post_type->label ); ?> Create <?php echo '( add_' . esc_attr( $post_type->name ) . ' )'; ?></label></td>
									<td>
									<?php
									echo esc_html( $post_type->label ) . ' : ';
									esc_html_e( 'When', 'user_activity_log_pro' );
									echo ' ' . esc_html( $post_type->name ) . ' ';
									esc_html_e( 'is created.', 'user_activity_log_pro' );
									?>
									</td>
								</tr>
								<tr class="post_type">
									<th class="check-column" scope="row">
										<input value="<?php echo 'edit_' . esc_attr( $post_type->name ); ?>" id="<?php echo 'edit_' . esc_attr( $post_type->name ); ?>" <?php checked( in_array( 'edit_' . $post_type->name, $log_hook_list ), 1 ); ?> data-type="<?php echo esc_attr( $post_type->label ); ?>" name="hook[]" type="checkbox">
									</th>
									<td><label for="<?php echo 'edit_' . esc_attr( $post_type->name ); ?>"><?php echo esc_attr( $post_type->label ); ?> Update <?php echo '( edit_' . esc_attr( $post_type->name ) . ' )'; ?></label></td>
									<td>
									<?php
									echo esc_html( $post_type->label ) . ' : ';
									esc_html_e( 'When', 'user_activity_log_pro' );
									echo ' ' . esc_html( $post_type->name ) . ' ';
									esc_html_e( 'is updated.', 'user_activity_log_pro' );
									?>
									</td>
								</tr>
								<tr class="post_type">
									<th class="check-column" scope="row">
										<input value="<?php echo 'delete_' . esc_attr( $post_type->name ); ?>" id="<?php echo 'delete_' . esc_attr( $post_type->name ); ?>" <?php checked( in_array( 'delete_' . $post_type->name, $log_hook_list ), 1 ); ?> data-type="<?php echo esc_attr( $post_type->label ); ?>" name="hook[]" type="checkbox">
									</th>
									<td><label for="<?php echo 'delete_' . esc_attr( $post_type->name ); ?>"><?php echo esc_attr( $post_type->label ); ?> Delete <?php echo '( delete_' . esc_attr( $post_type->name ) . ' )'; ?></label></td>
									<td>
									<?php
									echo esc_html( $post_type->label ) . ' : ';
									esc_html_e( 'When', 'user_activity_log_pro' );
									echo ' ' . esc_html( $post_type->name ) . ' ';
									esc_html_e( 'is deleted.', 'user_activity_log_pro' );
									?>
									</td>
								</tr>
								<?php
							}
						}
						?>
						<tr class="groupHeader">
							<th class="check-column" scope="row">
								<input type="checkbox" name="" data-hookGroup="taxonomy">
							</th>
							<td colspan="2"><?php esc_html_e( 'Taxonomies', 'user_activity_log_pro' ); ?></td>
						</tr>
						<?php
						foreach ( $taxonomies as $taxonomy ) {
							if ( 'wpcf7_contact_form' != $taxonomy->name && 'custom_css' != $taxonomy->name && 'customize_changeset' != $taxonomy->name && 'product_visibility' != $taxonomy->name ) {
								?>
								<tr class="taxonomy">
									<th class="check-column" scope="row">
										<input value="<?php echo esc_attr( $taxonomy->name ); ?>_create" id="<?php echo esc_attr( $taxonomy->name ); ?>_create" <?php checked( in_array( $taxonomy->name . '_create', $log_hook_list ), 1 ); ?> data-type="<?php echo esc_attr( $taxonomy->label ); ?>" name="hook[]" type="checkbox" >
									</th>
									<td><label for="<?php echo esc_attr( $taxonomy->name ); ?>_create">
									<?php
									echo esc_html( $taxonomy->label ) . ' Create';
									echo ' ( ' . esc_html( $taxonomy->name ) . '_create )';
									?>
									</label></td>
									<td>
									<?php
									echo esc_html( $taxonomy->label ) . ' : ';
									esc_html_e( 'When', 'user_activity_log_pro' );
									echo ' ' . esc_html( $taxonomy->name ) . ' ';
									esc_html_e( 'is created.', 'user_activity_log_pro' );
									?>
									</td>
								</tr>
								<tr class="taxonomy">
									<th class="check-column" scope="row">
										<input value="<?php echo esc_attr( $taxonomy->name ); ?>_update" id="<?php echo esc_attr( $taxonomy->name ); ?>_update" <?php checked( in_array( $taxonomy->name . '_update', $log_hook_list ), 1 ); ?> data-type="<?php echo esc_attr( $taxonomy->label ); ?>" name="hook[]" type="checkbox" >
									</th>
									<td><label for="<?php echo esc_attr( $taxonomy->name ); ?>_update">
										<?php
										echo esc_html( $taxonomy->label ) . ' Update';
										echo ' ( ' . esc_html( $taxonomy->name ) . '_update )';
										?>
									</label></td>
									<td>
									<?php
									echo esc_html( $taxonomy->label ) . ' : ';
									esc_html_e( 'When', 'user_activity_log_pro' );
									echo ' ' . esc_html( $taxonomy->name ) . ' ';
									esc_html_e( 'is updated.', 'user_activity_log_pro' );
									?>
									</td>
								</tr>
								<tr class="taxonomy">
									<th class="check-column" scope="row">
										<input value="<?php echo esc_attr( $taxonomy->name ); ?>_delete" id="<?php echo esc_attr( $taxonomy->name ); ?>_delete" <?php checked( in_array( $taxonomy->name . '_delete', $log_hook_list ), 1 ); ?> data-type="<?php echo esc_attr( $taxonomy->label ); ?>" name="hook[]" type="checkbox" >
									</th>
									<td><label for="<?php echo esc_attr( $taxonomy->name ); ?>_delete">
										<?php
										echo esc_html( $taxonomy->label ) . ' Delete';
										echo ' ( ' . esc_html( $taxonomy->name ) . '_delete )';
										?>
									</label></td>
									<td>
									<?php
									echo esc_html( $taxonomy->label ) . ' : ';
									esc_html_e( 'When', 'user_activity_log_pro' );
									echo ' ' . esc_html( $taxonomy->name ) . ' ';
									esc_html_e( 'is deleted.', 'user_activity_log_pro' );
									?>
									</td>
								</tr>
								<?php
							}
						}
						?>
						<tr class="groupHeader">
							<th class="check-column" scope="row">
								<input type="checkbox" name="" data-hookGroup="plugin">
							</th>
							<td colspan="2"><?php esc_html_e( 'Plugin', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="plugin">
							<th class="check-column" scope="row">
								<input value="activated_plugin" id="activated_plugin" <?php checked( in_array( 'activated_plugin', $log_hook_list ), 1 ); ?> data-type="plugin activated" name="hook[]" type="checkbox">
							</th>
							<td><label for="activated_plugin"><?php echo 'activated_plugin'; ?></label></td>
							<td><?php esc_html_e( 'Plugin activated', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="plugin">
							<th class="check-column" scope="row">
								<input value="deactivated_plugin" id="deactivated_plugin" <?php checked( in_array( 'deactivated_plugin', $log_hook_list ), 1 ); ?> data-type="plugin deactivated" name="hook[]" type="checkbox">
							</th>
							<td><label for="deactivated_plugin"><?php echo 'deactivated_plugin'; ?></label></td>
							<td><?php esc_html_e( 'Plugin deactivated', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="plugin">
							<th class="check-column" scope="row">
								<input value="delete_plugin" id="delete_plugin" <?php checked( in_array( 'delete_plugin', $log_hook_list ), 1 ); ?> data-type="plugin deleted" name="hook[]" type="checkbox" >
							</th>
							<td><label for="delete_plugin"><?php echo 'delete_plugin'; ?></label></td>
							<td><?php esc_html_e( 'Plugin deleted', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="plugin">
							<th class="check-column" scope="row">
								<input value="plugin_modify" id="plugin_modify" <?php checked( in_array( 'plugin_modify', $log_hook_list ), 1 ); ?> data-type="Plugin file modified" name="hook[]" type="checkbox" >
							</th>
							<td><label for="plugin_modify"><?php echo 'wp_redirect'; ?></label></td>
							<td><?php esc_html_e( 'Plugin file modified', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="plugin">
							<th class="check-column" scope="row">
								<input value="plugin_installed" id="plugin_installed" <?php checked( in_array( 'plugin_installed', $log_hook_list ), 1 ); ?> data-type="Plugin Installed" name="hook[]" type="checkbox" >
							</th>
							<td><label for="plugin_installed"><?php echo 'plugin_installed'; ?></label></td>
							<td><?php esc_html_e( 'Plugin installed', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="plugin">
							<th class="check-column" scope="row">
								<input value="plugin_updated" id="plugin_updated" <?php checked( in_array( 'plugin_updated', $log_hook_list ), 1 ); ?> data-type="Plugin updated" name="hook[]" type="checkbox" >
							</th>
							<td><label for="plugin_updated"><?php echo 'plugin_updated'; ?></label></td>
							<td><?php esc_html_e( 'Plugin updated', 'user_activity_log_pro' ); ?></td>
						</tr>
						<?php if ( is_plugin_active( 'contact-form-7/wp-contact-form-7.php' ) ) { ?>
						<tr class="groupHeader">
							<th class="check-column" scope="row">
								<input type="checkbox" name="" data-hookGroup="contactForm">
							</th>
							<td colspan="2"><?php esc_html_e( 'Contact Form 7', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="contactForm">
							<th class="check-column" scope="row">
								<input value="wpcf7_after_create" id="wpcf7_after_create" <?php checked( in_array( 'wpcf7_after_create', $log_hook_list ), 1 ); ?> data-type="Contact Form created" name="hook[]" type="checkbox" >
							</th>
							<td><label for="wpcf7_after_create"><?php echo 'wpcf7_after_create'; ?></label></td>
							<td><?php esc_html_e( 'Contact Form created', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="contactForm">
							<th class="check-column" scope="row">
								<input value="wpcf7_after_update" id="wpcf7_after_update" <?php checked( in_array( 'wpcf7_after_update', $log_hook_list ), 1 ); ?> data-type="Contact Form updated" name="hook[]" type="checkbox" >
							</th>
							<td><label for="wpcf7_after_update"><?php echo 'wpcf7_after_update'; ?></label></td>
							<td><?php esc_html_e( 'Contact Form updated', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="contactForm">
							<th class="check-column" scope="row">
								<input value="wpcf7_admin_notices" id="wpcf7_admin_notices" <?php checked( in_array( 'wpcf7_admin_notices', $log_hook_list ), 1 ); ?> data-type="Contact Form deleted" name="hook[]" type="checkbox" >
							</th>
							<td><label for="wpcf7_admin_notices"><?php echo 'wpcf7_admin_notices'; ?></label></td>
							<td><?php esc_html_e( 'Contact Form deleted', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="contactForm">
							<th class="check-column" scope="row">
								<input value="wpcf7-recaptcha-setup" id="wpcf7-recaptcha-setup" <?php checked( in_array( 'wpcf7-recaptcha-setup', $log_hook_list ), 1 ); ?> data-type="Contact Form Recaptcha setting updated" name="hook[]" type="checkbox" >
							</th>
							<td><label for="wpcf7-recaptcha-setup"><?php echo 'wpcf7-recaptcha-setup'; ?></label></td>
							<td><?php esc_html_e( 'Contact Form Recaptcha Settings updated', 'user_activity_log_pro' ); ?></td>
						</tr>
						<?php } ?>
						<tr class="groupHeader">
							<th class="check-column" scope="row">
								<input type="checkbox" name="" data-hookGroup="ualp">
							</th>
							<td colspan="2"><?php esc_html_e( 'User Activity Log Settings', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="ualp">
							<th class="check-column" scope="row">
								<input value="ualp_export_activity_log" id="ualp_export_activity_log" <?php checked( in_array( 'ualp_export_activity_log', $log_hook_list ), 1 ); ?> data-type="User activity log exported" name="hook[]" type="checkbox" >
							</th>
							<td><label for="ualp_export_activity_log"><?php echo 'ualp_export_activity_log'; ?></label></td>
							<td><?php esc_html_e( 'User activity log exported', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="ualp">
							<th class="check-column" scope="row">
								<input value="ualp_general_settings_updated" id="ualp_general_settings_updated" <?php checked( in_array( 'ualp_general_settings_updated', $log_hook_list ), 1 ); ?> data-type="General settings updated" name="hook[]" type="checkbox" >
							</th>
							<td><label for="ualp_general_settings_updated"><?php echo 'ualp_general_settings_updated'; ?></label></td>
							<td><?php esc_html_e( 'General settings updated', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="ualp">
							<th class="check-column" scope="row">
								<input value="ualp_hook_log_settings_updated" id="ualp_hook_log_settings_updated" <?php checked( in_array( 'ualp_hook_log_settings_updated', $log_hook_list ), 1 ); ?> data-type="Log Hook settings updated" name="hook[]" type="checkbox" >
							</th>
							<td><label for="ualp_hook_log_settings_updated"><?php echo 'ualp_hook_log_settings_updated'; ?></label></td>
							<td><?php esc_html_e( 'Log Hook settings updated', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="ualp">
							<th class="check-column" scope="row">
								<input value="ualp_manage_role" id="ualp_manage_role" <?php checked( in_array( 'ualp_manage_role', $log_hook_list ), 1 ); ?> data-type="Manage Role settings" name="hook[]" type="checkbox" >
							</th>
							<td><label for="ualp_manage_role"><?php echo 'ualp_manage_role'; ?></label></td>
							<td><?php esc_html_e( 'Manage Role settings', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="ualp">
							<th class="check-column" scope="row">
								<input value="ualp_set_password" id="ualp_set_password" <?php checked( in_array( 'ualp_set_password', $log_hook_list ), 1 ); ?> data-type="Set password" name="hook[]" type="checkbox" >
							</th>
							<td><label for="ualp_set_password"><?php echo 'ualp_set_password'; ?></label></td>
							<td><?php esc_html_e( 'Set password', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="ualp">
							<th class="check-column" scope="row">
								<input value="ualp_change_password" id="ualp_change_password" <?php checked( in_array( 'ualp_change_password', $log_hook_list ), 1 ); ?> data-type="Change password" name="hook[]" type="checkbox" >
							</th>
							<td><label for="ualp_change_password"><?php echo 'ualp_change_password'; ?></label></td>
							<td><?php esc_html_e( 'Change password', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="ualp">
							<th class="check-column" scope="row">
								<input value="ualp_email_settings_updated" id="ualp_email_settings_updated" <?php checked( in_array( 'ualp_email_settings_updated', $log_hook_list ), 1 ); ?> data-type="Email settings updated" name="hook[]" type="checkbox" >
							</th>
							<td><label for="ualp_email_settings_updated"><?php echo 'ualp_email_settings_updated'; ?></label></td>
							<td><?php esc_html_e( 'Email settings updated', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="ualp">
							<th class="check-column" scope="row">
								<input value="ualp_custom_event_log_updated" id="ualp_custom_event_log_updated" <?php checked( in_array( 'ualp_custom_event_log_updated', $log_hook_list ), 1 ); ?> data-type="Custom event log updated" name="hook[]" type="checkbox" >
							</th>
							<td><label for="ualp_custom_event_log_updated"><?php echo 'ualp_custom_event_log_updated'; ?></label></td>
							<td><?php esc_html_e( 'Custom event log updated', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="ualp">
							<th class="check-column" scope="row">
								<input value="ualp_exclude_log_updated" id="ualp_exclude_log_updated" <?php checked( in_array( 'ualp_exclude_log_updated', $log_hook_list ), 1 ); ?> data-type="Exclude log updated" name="hook[]" type="checkbox" >
							</th>
							<td><label for="ualp_exclude_log_updated"><?php echo 'ualp_exclude_log_updated'; ?></label></td>
							<td><?php esc_html_e( 'Exclude log updated', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="ualp">
							<th class="check-column" scope="row">
								<input value="ualp_insert_email_notification" id="ualp_insert_email_notification" <?php checked( in_array( 'ualp_insert_email_notification', $log_hook_list ), 1 ); ?> data-type="Exclude log updated" name="hook[]" type="checkbox" >
							</th>
							<td><label for="ualp_insert_email_notification"><?php echo 'ualp_insert_email_notification'; ?></label></td>
							<td><?php esc_html_e( 'Email trigger nottification created', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="ualp">
							<th class="check-column" scope="row">
								<input value="ualp_update_email_notification" id="ualp_update_email_notification" <?php checked( in_array( 'ualp_update_email_notification', $log_hook_list ), 1 ); ?> data-type="Exclude log updated" name="hook[]" type="checkbox" >
							</th>
							<td><label for="ualp_update_email_notification"><?php echo 'ualp_update_email_notification'; ?></label></td>
							<td><?php esc_html_e( 'Email trigger nottification updated', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="groupHeader">
							<th class="check-column" scope="row">
								<input type="checkbox" name="" data-hookGroup="users">
							</th>
							<td colspan="2"><?php esc_html_e( 'User', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="users">
							<th class="check-column" scope="row">
								<input value="wp_login" id="wp_login" <?php checked( in_array( 'wp_login', $log_hook_list ), 1 ); ?> data-type="User login" name="hook[]" type="checkbox" >
							</th>
							<td><label for="wp_login"><?php echo 'wp_login'; ?></label></td>
							<td><?php esc_html_e( 'User login', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="users">
							<th class="check-column" scope="row">
								<input value="wp_login_failed" id="wp_login_failed" <?php checked( in_array( 'wp_login_failed', $log_hook_list ), 1 ); ?> data-type="User login failed" name="hook[]" type="checkbox" >
							</th>
							<td><label for="wp_login_failed"><?php echo 'wp_login_failed'; ?></label></td>
							<td><?php esc_html_e( 'User login failed', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="users">
							<th class="check-column" scope="row">
								<input value="wp_logout" id="wp_logout" <?php checked( in_array( 'wp_logout', $log_hook_list ), 1 ); ?> data-type="User logout" name="hook[]" type="checkbox" >
							</th>
							<td><label for="wp_logout"><?php echo 'wp_logout'; ?></label></td>
							<td><?php esc_html_e( 'User logout', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="users">
							<th class="check-column" scope="row">
								<input value="delete_user" id="delete_user" <?php checked( in_array( 'delete_user', $log_hook_list ), 1 ); ?> data-type="User deleted" name="hook[]" type="checkbox" >
							</th>
							<td><label for="delete_user"><?php echo 'delete_user'; ?></label></td>
							<td><?php esc_html_e( 'User deleted', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="users">
							<th class="check-column" scope="row">
								<input value="user_register" id="user_register" <?php checked( in_array( 'user_register', $log_hook_list ), 1 ); ?> data-type="User registerd" name="hook[]" type="checkbox" >
							</th>
							<td><label for="user_register"><?php echo 'user_register'; ?></label></td>
							<td><?php esc_html_e( 'User registerd', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="users">
							<th class="check-column" scope="row">
								<input value="insert_user_meta" id="insert_user_meta" <?php checked( in_array( 'insert_user_meta', $log_hook_list ), 1 ); ?> data-type="User profile updated" name="hook[]" type="checkbox" >
							</th>
							<td><label for="insert_user_meta"><?php echo 'insert_user_meta'; ?></label></td>
							<td><?php esc_html_e( 'User profile updated', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="groupHeader">
							<th class="check-column" scope="row">
								<input type="checkbox" name="" data-hookGroup="theme">
							</th>
							<td colspan="2"><?php esc_html_e( 'Theme', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="theme">
							<th class="check-column" scope="row">
								<input value="theme_file_modify" id="theme_file_modify" <?php checked( in_array( 'theme_file_modify', $log_hook_list ), 1 ); ?> data-type="Theme file updated" name="hook[]" type="checkbox" >
							</th>
							<td><label for="theme_file_modify"><?php echo 'theme_file_modify'; ?></label></td>
							<td><?php esc_html_e( 'Theme file updated', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="theme">
							<th class="check-column" scope="row">
								<input value="switch_theme" id="switch_theme" <?php checked( in_array( 'switch_theme', $log_hook_list ), 1 ); ?> data-type="Theme Switched" name="hook[]" type="checkbox" >
							</th>
							<td><label for="switch_theme"><?php echo 'switch_theme'; ?></label></td>
							<td><?php esc_html_e( 'Theme Switched', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="theme">
							<th class="check-column" scope="row">
								<input value="customize_save" id="customize_save" <?php checked( in_array( 'customize_save', $log_hook_list ), 1 ); ?> data-type="Theme customizer updated" name="hook[]" type="checkbox" >
							</th>
							<td><label for="customize_save"><?php echo 'customize_save'; ?></label</td>
							<td><?php esc_html_e( 'Theme customizer updated', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="theme">
							<th class="check-column" scope="row">
								<input value="theme_updated" id="theme_updated" <?php checked( in_array( 'theme_updated', $log_hook_list ), 1 ); ?> data-type="Theme updated" name="hook[]" type="checkbox" >
							</th>
							<td><label for="theme_updated"><?php echo 'theme_updated'; ?></label></td>
							<td><?php esc_html_e( 'Theme updated', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="theme">
							<th class="check-column" scope="row">
								<input value="theme_installed" id="theme_installed" <?php checked( in_array( 'theme_installed', $log_hook_list ), 1 ); ?> data-type="Theme installed" name="hook[]" type="checkbox" >
							</th>
							<td><label for="theme_installed"><?php echo 'theme_installed'; ?></label></td>
							<td><?php esc_html_e( 'Theme installed', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="theme">
							<th class="check-column" scope="row">
								<input value="delete_site_transient_update_themes" id="delete_site_transient_update_themes" <?php checked( in_array( 'delete_site_transient_update_themes', $log_hook_list ), 1 ); ?> data-type="Theme deleted" name="hook[]" type="checkbox" >
							</th>
							<td><label for="delete_site_transient_update_themes"><?php echo 'delete_themes'; ?></label></td>
							<td><?php esc_html_e( 'Theme deleted', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="groupHeader">
							<th class="check-column" scope="row">
								<input type="checkbox" name="" data-hookGroup="comment">
							</th>
							<td colspan="2"><?php esc_html_e( 'Comments', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="comment">
							<th class="check-column" scope="row">
								<input value="wp_insert_comment" id="wp_insert_comment" <?php checked( in_array( 'wp_insert_comment', $log_hook_list ), 1 ); ?> data-type="Comment inserted" name="hook[]" type="checkbox" >
							</th>
							<td><label for="wp_insert_comment"><?php echo 'wp_insert_comment'; ?></label></td>
							<td><?php esc_html_e( 'Comment inserted', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="comment">
							<th class="check-column" scope="row">
								<input value="edit_comment" id="edit_comment" <?php checked( in_array( 'edit_comment', $log_hook_list ), 1 ); ?> data-type="Comment updated" name="hook[]" type="checkbox" >
							</th>
							<td><label for="edit_comment"><?php echo 'edit_comment'; ?></label></td>
							<td><?php esc_html_e( 'Comment updated', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="comment">
							<th class="check-column" scope="row">
								<input value="delete_comment" id="delete_comment" <?php checked( in_array( 'delete_comment', $log_hook_list ), 1 ); ?> data-type="Comment deleted" name="hook[]" type="checkbox" >
							</th>
							<td><label for="delete_comment"><?php echo 'delete_comment'; ?></label></td>
							<td><?php esc_html_e( 'Comment deleted', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="comment">
							<th class="check-column" scope="row">
								<input value="trash_comment" id="trash_comment" <?php checked( in_array( 'trash_comment', $log_hook_list ), 1 ); ?> data-type="Comment trashed" name="hook[]" type="checkbox" >
							</th>
							<td><label for="trash_comment"><?php echo 'trash_comment'; ?></label></td>
							<td><?php esc_html_e( 'Comment trashed', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="comment">
							<th class="check-column" scope="row">
								<input value="spam_comment" id="spam_comment" <?php checked( in_array( 'spam_comment', $log_hook_list ), 1 ); ?> data-type="Comment spammed" name="hook[]" type="checkbox" >
							</th>
							<td><label for="spam_comment"><?php echo 'spam_comment'; ?></label></td>
							<td><?php esc_html_e( 'Comment spammed', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="comment">
							<th class="check-column" scope="row">
								<input value="restore_comment" id="restore_comment" <?php checked( in_array( 'restore_comment', $log_hook_list ), 1 ); ?> data-type="Comment restore" name="hook[]" type="checkbox" >
							</th>
							<td><label for="restore_comment"><?php echo 'restore_comment'; ?></label></td>
							<td><?php esc_html_e( 'Comment restore', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="comment">
							<th class="check-column" scope="row">
								<input value="comment_approve" id="comment_approve" <?php checked( in_array( 'comment_approve', $log_hook_list ), 1 ); ?> data-type="Comment approved" name="hook[]" type="checkbox" >
							</th>
							<td><label for="comment_approve"><?php echo 'comment_approve'; ?></label></td>
							<td><?php esc_html_e( 'Comment approved', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="comment">
							<th class="check-column" scope="row">
								<input value="comment_unapprove" id="comment_unapprove" <?php checked( in_array( 'comment_unapprove', $log_hook_list ), 1 ); ?> data-type="Comment unapproved" name="hook[]" type="checkbox" >
							</th>
							<td><label for="comment_unapprove"><?php echo 'comment_unapprove'; ?></label></td>
							<td><?php esc_html_e( 'Comment unapproved', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="groupHeader">
							<th class="check-column" scope="row">
								<input type="checkbox" name="" data-hookGroup="menu">
							</th>
							<td colspan="2"><?php esc_html_e( 'Menu', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="menu">
							<th class="check-column" scope="row">
								<input value="wp_create_nav_menu" id="wp_create_nav_menu" <?php checked( in_array( 'wp_create_nav_menu', $log_hook_list ), 1 ); ?> data-type="Menu created" name="hook[]" type="checkbox" >
							</th>
							<td><label for="wp_create_nav_menu"><?php echo 'wp_create_nav_menu'; ?></label></td>
							<td><?php esc_html_e( 'Menu created', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="menu">
							<th class="check-column" scope="row">
								<input value="wp_update_nav_menu" id="wp_update_nav_menu" <?php checked( in_array( 'wp_update_nav_menu', $log_hook_list ), 1 ); ?> data-type="Menu updated" name="hook[]" type="checkbox" >
							</th>
							<td><label for="wp_update_nav_menu"><?php echo 'wp_update_nav_menu'; ?></label></td>
							<td><?php esc_html_e( 'Menu updated', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="menu">
							<th class="check-column" scope="row">
								<input value="delete_nav_menu" id="delete_nav_menu" <?php checked( in_array( 'delete_nav_menu', $log_hook_list ), 1 ); ?> data-type="Menu deleted" name="hook[]" type="checkbox" >
							</th>
							<td><label for="delete_nav_menu"><?php echo 'delete_nav_menu'; ?></label></td>
							<td><?php esc_html_e( 'Menu deleted', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="groupHeader">
							<th class="check-column" scope="row">
								<input type="checkbox" name="" data-hookGroup="view">
							</th>
							<td colspan="2"><?php esc_html_e( 'Page Views', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="view">
							<th class="check-column" scope="row">
								<input value="admin_head" id="admin_head" <?php checked( in_array( 'admin_head', $log_hook_list ), 1 ); ?> data-type="Admin page viewed" name="hook[]" type="checkbox" >
							</th>
							<td><label for="admin_head"><?php echo 'admin_head'; ?></label></td>
							<td><?php esc_html_e( 'Admin page viewed', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="view">
							<th class="check-column" scope="row">
								<input value="wp_head" id="wp_head" <?php checked( in_array( 'wp_head', $log_hook_list ), 1 ); ?> data-type="Front page viewed" name="hook[]" type="checkbox" >
							</th>
							<td><label for="wp_head"><?php echo 'wp_head'; ?></label></td>
							<td><?php esc_html_e( 'Front page viewed', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="view">
							<th class="check-column" scope="row">
								<input value="404_page" id="404_page" <?php checked( in_array( '404_page', $log_hook_list ), 1 ); ?> data-type="404 page viewed" name="hook[]" type="checkbox" >
							</th>
							<td><label for="404_page"><?php echo '404 page'; ?></label></td>
							<td><?php esc_html_e( '404 page viewed', 'user_activity_log_pro' ); ?></td>
						</tr>
						<?php
							$c = 6;
						?>
						<tr class="groupHeader">
							<th class="check-column" scope="row">
								<input type="checkbox" name="" data-hookGroup="other">
							</th>
							<td colspan="2"><?php esc_html_e( 'Other', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="other">
							<th class="check-column" scope="row">
								<input value="_core_updated_successfully" id="_core_updated_successfully" <?php checked( in_array( '_core_updated_successfully', $log_hook_list ), 1 ); ?> data-type="Core Update" name="hook[]" type="checkbox" >
							</th>
							<td><label for="_core_updated_successfully"><?php echo '_core_updated_successfully'; ?></label></td>
							<td><?php esc_html_e( 'Core Update', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="other">
							<th class="check-column" scope="row">
								<input value="import_end" id="import_end" <?php checked( in_array( 'import_end', $log_hook_list ), 1 ); ?> data-type="Import data" name="hook[]" type="checkbox" >
							</th>
							<td><label for="import_end"><?php echo 'import_end'; ?></label></td>
							<td><?php esc_html_e( 'Import WordPress data', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="other">
							<th class="check-column" scope="row">
								<input value="export_wp" id="export_wp" <?php checked( in_array( 'export_wp', $log_hook_list ), 1 ); ?> data-type="Export" name="hook[]" type="checkbox" >
							</th>
							<td><label for="export_wp"><?php echo 'export_wp'; ?></label></td>
							<td><?php esc_html_e( 'Export WordPress data', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="other">
							<th class="check-column" scope="row">
								<input value="widget_update_callback" id="widget_update_callback" <?php checked( in_array( 'widget_update_callback', $log_hook_list ), 1 ); ?> data-type="Widget updated" name="hook[]" type="checkbox" >
							</th>
							<td><label for="widget_update_callback"><?php echo 'widget_update_callback'; ?></label></td>
							<td><?php esc_html_e( 'Widget updated', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="other">
							<th class="check-column" scope="row">
								<input value="updated_option" id="updated_option" <?php checked( in_array( 'updated_option', $log_hook_list ), 1 ); ?> data-type="Settings Option" name="hook[]" type="checkbox" >
							</th>
							<td><label for="updated_option"><?php echo 'updated_option'; ?></label></td>
							<td><?php esc_html_e( 'Settings option updated', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="other">
							<th class="check-column" scope="row">
								<input value="dbdelta_queries" id="dbdelta_queries" <?php checked( in_array( 'dbdelta_queries', $log_hook_list ), 1 ); ?> data-type="Table create, update or delete from the database by plugin or theme." name="hook[]" type="checkbox" >
							</th>
							<td><label for="dbdelta_queries"><?php echo 'dbdelta_queries'; ?></label></td>
							<td><?php esc_html_e( 'Table create, update or delete from the database by plugin or theme.', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="other">
							<th class="check-column" scope="row">
								<input value="wp_privacy_personal_data_exporters" id="wp_privacy_personal_data_exporters" <?php checked( in_array( 'wp_privacy_personal_data_exporters', $log_hook_list ), 1 ); ?> data-type="Export the Personal Data in ZIP file." name="hook[]" type="checkbox" >
							</th>
							<td><label for="wp_privacy_personal_data_exporters"><?php echo 'wp_privacy_personal_data_exporters'; ?></label></td>
							<td><?php esc_html_e( 'Export the Personal Data in ZIP file.', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="other">
							<th class="check-column" scope="row">
								<input value="wp_privacy_personal_data_erasers" id="wp_privacy_personal_data_erasers" <?php checked( in_array( 'wp_privacy_personal_data_erasers', $log_hook_list ), 1 ); ?> data-type="Erase the Personal Data." name="hook[]" type="checkbox" >
							</th>
							<td><label for="wp_privacy_personal_data_erasers"><?php echo 'wp_privacy_personal_data_erasers'; ?></label></td>
							<td><?php esc_html_e( 'Erase the Personal Data.', 'user_activity_log_pro' ); ?></td>
						</tr>
						<?php if ( class_exists( 'Woocommerce' ) ) { ?>
						<tr class="groupHeader">
							<th class="check-column" scope="row">
								<input type="checkbox" name="" data-hookGroup="woocommerce">
							</th>
							<td colspan="2"><?php esc_html_e( 'Woocommerce', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="woocommerce">
							<th class="check-column" scope="row">
								<input value="woocommerce_updated_option" id="woocommerce_updated_option" <?php checked( in_array( 'woocommerce_updated_option', $log_hook_list ), 1 ); ?> data-type="Woocommerce Settings" name="hook[]" type="checkbox" >
							</th>
							<td class="woocommerce_option"><label for="woocommerce_updated_option"><?php echo 'woocommerce_updated_option'; ?></label></td>
							<td><?php esc_html_e( 'Woocommerce settings updated', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="woocommerce">
							<th class="check-column" scope="row">
								<input value="woocommerce_new_product_variation" id="woocommerce_new_product_variation" <?php checked( in_array( 'woocommerce_new_product_variation', $log_hook_list ), 1 ); ?> data-type="Woocommerce new variation create" name="hook[]" type="checkbox" >
							</th>
							<td><label for="woocommerce_new_product_variation"><?php echo 'woocommerce_new_product_variation'; ?></label></td>
							<td><?php esc_html_e( 'Woocommerce variation created', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="woocommerce">
							<th class="check-column" scope="row">
								<input value="wp_ajax_woocommerce_remove_variations" id="wp_ajax_woocommerce_remove_variations" <?php checked( in_array( 'wp_ajax_woocommerce_remove_variations', $log_hook_list ), 1 ); ?> data-type="Woocommerce delete variation" name="hook[]" type="checkbox" >
							</th>
							<td><label for="wp_ajax_woocommerce_remove_variations"><?php echo 'wp_ajax_woocommerce_remove_variations'; ?></label></td>
							<td><?php esc_html_e( 'Woocommerce variation deleted', 'user_activity_log_pro' ); ?></td>
						</tr>
						<tr class="woocommerce">
							<th class="check-column" scope="row">
								<input value="woocommerce_save_product_variation" id="woocommerce_save_product_variation" <?php checked( in_array( 'woocommerce_save_product_variation', $log_hook_list ), 1 ); ?> data-type="Woocommerce save variation" name="hook[]" type="checkbox" >
							</th>
							<td><label for="woocommerce_save_product_variation"><?php echo 'woocommerce_save_product_variation'; ?></label></td>
							<td><?php esc_html_e( 'Woocommerce save product variation', 'user_activity_log_pro' ); ?></td>
						</tr>
							<?php
						}
						if ( is_plugin_active( 'buddypress/bp-loader.php' ) ) {
							?>
							<tr class="groupHeader">
								<th class="check-column" scope="row">
									<input type="checkbox" name="" data-hookGroup="buddypress">
								</th>
								<td colspan="2"><?php esc_html_e( 'BuddyPress', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="buddypress">
								<th class="check-column" scope="row">
									<input value="groups_create_group" id="groups_create_group" <?php checked( in_array( 'groups_create_group', $log_hook_list ), 1 ); ?> data-type="BuddyPress Group Create" name="hook[]" type="checkbox" >
								</th>
								<td><label for="groups_create_group"><?php echo 'groups_create_group'; ?></label></td>
								<td><?php esc_html_e( 'Group created', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="buddypress">
								<th class="check-column" scope="row">
									<input value="groups_update_group" id="groups_update_group" <?php checked( in_array( 'groups_update_group', $log_hook_list ), 1 ); ?> data-type="BuddyPress Group Update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="groups_update_group"><?php echo 'groups_update_group'; ?></label></td>
								<td><?php esc_html_e( 'Group updated.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="buddypress">
								<th class="check-column" scope="row">
									<input value="groups_before_delete_group" id="groups_before_delete_group" <?php checked( in_array( 'groups_before_delete_group', $log_hook_list ), 1 ); ?> data-type="BuddyPress Group Delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="groups_before_delete_group"><?php echo 'groups_before_delete_group'; ?></label></td>
								<td><?php esc_html_e( 'Group deleted.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="buddypress">
								<th class="check-column" scope="row">
									<input value="groups_leave_group" id="groups_leave_group" <?php checked( in_array( 'groups_leave_group', $log_hook_list ), 1 ); ?> data-type="BuddyPress Leave Group" name="hook[]" type="checkbox" >
								</th>
								<td><label for="groups_leave_group"><?php echo 'groups_leave_group'; ?></label></td>
								<td><?php esc_html_e( 'Group leave.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="buddypress">
								<th class="check-column" scope="row">
									<input value="groups_join_group" id="groups_join_group" <?php checked( in_array( 'groups_join_group', $log_hook_list ), 1 ); ?> data-type="BuddyPress Join Group" name="hook[]" type="checkbox" >
								</th>
								<td><label for="groups_join_group"><?php echo 'groups_join_group'; ?></label></td>
								<td><?php esc_html_e( 'Group join.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="buddypress">
								<th class="check-column" scope="row">
									<input value="groups_promote_member" id="groups_promote_member" <?php checked( in_array( 'groups_promote_member', $log_hook_list ), 1 ); ?> data-type="BuddyPress Promote Group" name="hook[]" type="checkbox" >
								</th>
								<td><label for="groups_promote_member"><?php echo 'groups_promote_member'; ?></label></td>
								<td><?php esc_html_e( 'Group promoted.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="buddypress">
								<th class="check-column" scope="row">
									<input value="groups_demote_member" id="groups_demote_member" <?php checked( in_array( 'groups_demote_member', $log_hook_list ), 1 ); ?> data-type="BuddyPress Demote Group" name="hook[]" type="checkbox" >
								</th>
								<td><label for="groups_demote_member"><?php echo 'groups_demote_member'; ?></label></td>
								<td><?php esc_html_e( 'Group demoted.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="buddypress">
								<th class="check-column" scope="row">
									<input value="groups_ban_member" id="groups_ban_member" <?php checked( in_array( 'groups_ban_member', $log_hook_list ), 1 ); ?> data-type="BuddyPress Ban Group" name="hook[]" type="checkbox" >
								</th>
								<td><label for="groups_ban_member"><?php echo 'groups_ban_member'; ?></label></td>
								<td><?php esc_html_e( 'Group ban.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="buddypress">
								<th class="check-column" scope="row">
									<input value="groups_unban_member" id="groups_unban_member" <?php checked( in_array( 'groups_unban_member', $log_hook_list ), 1 ); ?> data-type="BuddyPress Unban Group" name="hook[]" type="checkbox" >
								</th>
								<td><label for="groups_unban_member"><?php echo 'groups_unban_member'; ?></label></td>
								<td><?php esc_html_e( 'Group unband.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="buddypress">
								<th class="check-column" scope="row">
									<input value="groups_remove_member" id="groups_remove_member" <?php checked( in_array( 'groups_remove_member', $log_hook_list ), 1 ); ?> data-type="BuddyPress Remove Group" name="hook[]" type="checkbox" >
								</th>
								<td><label for="groups_remove_member"><?php echo 'groups_remove_member'; ?></label></td>
								<td><?php esc_html_e( 'Group remove.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="buddypress">
								<th class="check-column" scope="row">
									<input value="xprofile_field_after_save" id="xprofile_field_after_save" <?php checked( in_array( 'xprofile_field_after_save', $log_hook_list ), 1 ); ?> data-type="BuddyPress Group Profile Field Save" name="hook[]" type="checkbox" >
								</th>
								<td><label for="xprofile_field_after_save"><?php echo 'xprofile_field_after_save'; ?></label></td>
								<td><?php esc_html_e( 'Group profile field save.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="buddypress">
								<th class="check-column" scope="row">
									<input value="xprofile_fields_deleted_field" id="xprofile_fields_deleted_field" <?php checked( in_array( 'xprofile_fields_deleted_field', $log_hook_list ), 1 ); ?> data-type="BuddyPress Group Profile Field Delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="xprofile_fields_deleted_field"><?php echo 'xprofile_fields_deleted_field'; ?></label></td>
								<td><?php esc_html_e( 'Group profile field delete.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="buddypress">
								<th class="check-column" scope="row">
									<input value="xprofile_group_after_save" id="xprofile_group_after_save" <?php checked( in_array( 'xprofile_group_after_save', $log_hook_list ), 1 ); ?> data-type="BuddyPress Create/Update Group Profile Field" name="hook[]" type="checkbox" >
								</th>
								<td><label for="xprofile_group_after_save"><?php echo 'xprofile_group_after_save'; ?></label></td>
								<td><?php esc_html_e( 'Create/Update group profile field.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="buddypress">
								<th class="check-column" scope="row">
									<input value="xprofile_groups_deleted_group" id="xprofile_groups_deleted_group" <?php checked( in_array( 'xprofile_groups_deleted_group', $log_hook_list ), 1 ); ?> data-type="BuddyPress Delete Group Profile Field" name="hook[]" type="checkbox" >
								</th>
								<td><label for="xprofile_groups_deleted_group"><?php echo 'xprofile_groups_deleted_group'; ?></label></td>
								<td><?php esc_html_e( 'Delete group profile field.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<?php
						}
						if ( is_plugin_active( 'gravityforms/gravityforms.php' ) ) {
							?>
							<tr class="groupHeader">
								<th class="check-column" scope="row">
									<input type="checkbox" name="" data-hookGroup="gravityforms">
								</th>
								<td colspan="2"><?php esc_html_e( 'Gravity Forms', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gravityforms_updated_option" id="gravityforms_updated_option" <?php checked( in_array( 'gravityforms_updated_option', $log_hook_list ), 1 ); ?> data-type="Gravity Forms Setting Update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gravityforms_updated_option"><?php echo 'gravityforms_updated_option'; ?></label></td>
								<td><?php esc_html_e( 'Gravity forms settings updated.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gf_export_forms" id="gf_export_forms" <?php checked( in_array( 'gf_export_forms', $log_hook_list ), 1 ); ?> data-type="Gravity Form Export" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gf_export_forms"><?php echo 'gf_export_forms'; ?></label></td>
								<td><?php esc_html_e( 'Form exported.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="notification_duplicated" id="notification_duplicated" <?php checked( in_array( 'notification_duplicated', $log_hook_list ), 1 ); ?> data-type="Gravity Form Notification Duplicate." name="hook[]" type="checkbox" >
								</th>
								<td><label for="notification_duplicated"><?php echo 'notification_duplicated'; ?></label></td>
								<td><?php esc_html_e( 'Notification duplicated.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gform_entry_created" id="gform_entry_created" <?php checked( in_array( 'gform_entry_created', $log_hook_list ), 1 ); ?> data-type="Gravity Form Entry Create" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gform_entry_created"><?php echo 'gform_entry_created'; ?></label></td>
								<td><?php esc_html_e( 'Entry is created.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gf_resend_notifications" id="gf_resend_notifications" <?php checked( in_array( 'gf_resend_notifications', $log_hook_list ), 1 ); ?> data-type="Gravity Form Resend Email Notifications" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gf_resend_notifications"><?php echo 'gf_resend_notifications'; ?></label></td>
								<td><?php esc_html_e( 'Resend email notifications.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gform_pre_note_deleted" id="gform_pre_note_deleted" <?php checked( in_array( 'gform_pre_note_deleted', $log_hook_list ), 1 ); ?> data-type="Gravity Form Resend Email Notifications" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gform_pre_note_deleted"><?php echo 'gform_pre_note_deleted'; ?></label></td>
								<td><?php esc_html_e( 'Note deleted.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gform_post_export_entries" id="gform_post_export_entries" <?php checked( in_array( 'gform_post_export_entries', $log_hook_list ), 1 ); ?> data-type="Gravity Form Exporting all the entries in form" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gform_post_export_entries"><?php echo 'gform_post_export_entries'; ?></label></td>
								<td><?php esc_html_e( 'Exporting all the entries in form.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gform_post_note_added" id="gform_post_note_added" <?php checked( in_array( 'gform_post_note_added', $log_hook_list ), 1 ); ?> data-type="Gravity Form note has been added to an entry" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gform_post_note_added"><?php echo 'gform_post_note_added'; ?></label></td>
								<td><?php esc_html_e( 'Note has been added to an entry.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gform_delete_entries" id="gform_delete_entries" <?php checked( in_array( 'gform_delete_entries', $log_hook_list ), 1 ); ?> data-type="Gravity Form entries delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gform_delete_entries"><?php echo 'gform_delete_entries'; ?></label></td>
								<td><?php esc_html_e( 'Form entries delete.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gform_post_form_views_deleted" id="gform_post_form_views_deleted" <?php checked( in_array( 'gform_post_form_views_deleted', $log_hook_list ), 1 ); ?> data-type="Gravity Forms Views Delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gform_post_form_views_deleted"><?php echo 'gform_post_form_views_deleted'; ?></label></td>
								<td><?php esc_html_e( 'Forms views deleted.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gform_post_form_activated" id="gform_post_form_activated" <?php checked( in_array( 'gform_post_form_activated', $log_hook_list ), 1 ); ?> data-type="Gravity forms activat" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gform_post_form_activated"><?php echo 'gform_post_form_activated'; ?></label></td>
								<td><?php esc_html_e( 'Gravity forms activated.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gform_post_form_deactivated" id="gform_post_form_deactivated" <?php checked( in_array( 'gform_post_form_deactivated', $log_hook_list ), 1 ); ?> data-type="Gravity forms deactivat" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gform_post_form_deactivated"><?php echo 'gform_post_form_deactivated'; ?></label></td>
								<td><?php esc_html_e( 'Gravity forms deactivated.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gform_before_delete_form" id="gform_before_delete_form" <?php checked( in_array( 'gform_before_delete_form', $log_hook_list ), 1 ); ?> data-type="Gravity form delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gform_before_delete_form"><?php echo 'gform_before_delete_form'; ?></label></td>
								<td><?php esc_html_e( 'Gravity form deleted.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gform_post_form_trashed" id="gform_post_form_trashed" <?php checked( in_array( 'gform_post_form_trashed', $log_hook_list ), 1 ); ?> data-type="Gravity form move to trash" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gform_post_form_trashed"><?php echo 'gform_post_form_trashed'; ?></label></td>
								<td><?php esc_html_e( 'Gravity form trashed.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gform_post_form_restored" id="gform_post_form_restored" <?php checked( in_array( 'gform_post_form_restored', $log_hook_list ), 1 ); ?> data-type="Gravity form restor" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gform_post_form_restored"><?php echo 'gform_post_form_restored'; ?></label></td>
								<td><?php esc_html_e( 'Gravity form restored.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gform_post_form_duplicated" id="gform_post_form_duplicated" <?php checked( in_array( 'gform_post_form_duplicated', $log_hook_list ), 1 ); ?> data-type="Gravity form duplicate" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gform_post_form_duplicated"><?php echo 'gform_post_form_duplicated'; ?></label></td>
								<td><?php esc_html_e( 'Gravity form duplicated.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gform_after_save_form" id="gform_after_save_form" <?php checked( in_array( 'gform_after_save_form', $log_hook_list ), 1 ); ?> data-type="Gravity form save" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gform_after_save_form"><?php echo 'gform_after_save_form'; ?></label></td>
								<td><?php esc_html_e( 'Gravity form saved.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gform_forms_post_import" id="gform_forms_post_import" <?php checked( in_array( 'gform_forms_post_import', $log_hook_list ), 1 ); ?> data-type="Gravity form import" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gform_forms_post_import"><?php echo 'gform_forms_post_import'; ?></label></td>
								<td><?php esc_html_e( 'Gravity form import.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gform_pre_notification_deleted" id="gform_pre_notification_deleted" <?php checked( in_array( 'gform_pre_notification_deleted', $log_hook_list ), 1 ); ?> data-type="Gravity Forms Notification Delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gform_pre_notification_deleted"><?php echo 'gform_pre_notification_deleted'; ?></label></td>
								<td><?php esc_html_e( 'Notification deleted.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gform_pre_confirmation_deleted" id="gform_pre_confirmation_deleted" <?php checked( in_array( 'gform_pre_confirmation_deleted', $log_hook_list ), 1 ); ?> data-type="Gravity Forms Confirmation delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gform_pre_confirmation_deleted"><?php echo 'gform_pre_confirmation_deleted'; ?></label></td>
								<td><?php esc_html_e( 'Confirmation deleted.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="wp_ajax_rg_update_notification_active" id="wp_ajax_rg_update_notification_active" <?php checked( in_array( 'wp_ajax_rg_update_notification_active', $log_hook_list ), 1 ); ?> data-type="Gravity Forms Notification Update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wp_ajax_rg_update_notification_active"><?php echo 'wp_ajax_rg_update_notification_active'; ?></label></td>
								<td><?php esc_html_e( 'Notification updated.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="wp_ajax_rg_update_confirmation_active" id="wp_ajax_rg_update_confirmation_active" <?php checked( in_array( 'wp_ajax_rg_update_confirmation_active', $log_hook_list ), 1 ); ?> data-type="Gravity Forms Confirmation activate" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wp_ajax_rg_update_confirmation_active"><?php echo 'wp_ajax_rg_update_confirmation_active'; ?></label></td>
								<td><?php esc_html_e( 'Confirmation activated.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gform_pre_confirmation_save" id="gform_pre_confirmation_save" <?php checked( in_array( 'gform_pre_confirmation_save', $log_hook_list ), 1 ); ?> data-type="Gravity Forms Confirmations Save" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gform_pre_confirmation_save"><?php echo 'gform_pre_confirmation_save'; ?></label></td>
								<td><?php esc_html_e( 'Confirmations saved.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gform_pre_notification_save" id="gform_pre_notification_save" <?php checked( in_array( 'gform_pre_notification_save', $log_hook_list ), 1 ); ?> data-type="Gravity Forms Notification Save" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gform_pre_notification_save"><?php echo 'gform_pre_notification_save'; ?></label></td>
								<td><?php esc_html_e( 'Notification saved.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gform_delete_lead" id="gform_delete_lead" <?php checked( in_array( 'gform_delete_lead', $log_hook_list ), 1 ); ?> data-type="Gravity Forms Lead delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gform_delete_lead"><?php echo 'gform_delete_lead'; ?></label></td>
								<td><?php esc_html_e( 'Lead deleted', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gform_update_status" id="gform_update_status" <?php checked( in_array( 'gform_update_status', $log_hook_list ), 1 ); ?> data-type="Gravity Forms Status Update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gform_update_status"><?php echo 'gform_update_status'; ?></label></td>
								<td><?php esc_html_e( 'Form status updated', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gform_update_is_read" id="gform_update_is_read" <?php checked( in_array( 'gform_update_is_read', $log_hook_list ), 1 ); ?> data-type="Gravity Forms read entry" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gform_update_is_read"><?php echo 'gform_update_is_read'; ?></label></td>
								<td><?php esc_html_e( 'Form read entry.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="gravityforms">
								<th class="check-column" scope="row">
									<input value="gform_update_is_starred" id="gform_update_is_starred" <?php checked( in_array( 'gform_update_is_starred', $log_hook_list ), 1 ); ?> data-type="Gravity Forms Entry marked as starred" name="hook[]" type="checkbox" >
								</th>
								<td><label for="gform_update_is_starred"><?php echo 'gform_update_is_starred'; ?></label></td>
								<td><?php esc_html_e( 'Entry marked as starred.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<?php
						}
						if ( is_plugin_active( 'wordfence/wordfence.php' ) ) {
							?>
							<tr class="groupHeader">
								<th class="check-column" scope="row">
									<input type="checkbox" name="" data-hookGroup="wordfence">
								</th>
								<td colspan="2"><?php esc_html_e( 'WordFence', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_updateIssueStatus" id="wordfence_updateIssueStatus" <?php checked( in_array( 'wordfence_updateIssueStatus', $log_hook_list ), 1 ); ?> data-type="WordFence Issue status update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_updateIssueStatus"><?php echo 'wordfence_updateIssueStatus'; ?></label></td>
								<td><?php esc_html_e( 'Issue status updated.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_saveWAFConfig" id="wordfence_saveWAFConfig" <?php checked( in_array( 'wordfence_saveWAFConfig', $log_hook_list ), 1 ); ?> data-type="WordFence" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_saveWAFConfig"><?php echo 'wordfence_saveWAFConfig'; ?></label></td>
								<td><?php esc_html_e( 'WAF config updated.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_wafConfigureAutoPrepend" id="wordfence_wafConfigureAutoPrepend" <?php checked( in_array( 'wordfence_wafConfigureAutoPrepend', $log_hook_list ), 1 ); ?> data-type="WordFence Optimize WAF firewall" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_wafConfigureAutoPrepend"><?php echo 'wordfence_wafConfigureAutoPrepend'; ?></label></td>
								<td><?php esc_html_e( 'Optimize WAF firewall.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_whitelistBulkEnable" id="wordfence_whitelistBulkEnable" <?php checked( in_array( 'wordfence_whitelistBulkEnable', $log_hook_list ), 1 ); ?> data-type="WordFence bulk enable whitelist" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_whitelistBulkEnable"><?php echo 'wordfence_whitelistBulkEnable'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence bulk enable whitelist.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_whitelistBulkDisable" id="wordfence_whitelistBulkDisable" <?php checked( in_array( 'wordfence_whitelistBulkDisable', $log_hook_list ), 1 ); ?> data-type="WordFence bulk disable whitelist" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_whitelistBulkDisable"><?php echo 'wordfence_whitelistBulkDisable'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence bulk disable whitelist.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_whitelistBulkDelete" id="wordfence_whitelistBulkDelete" <?php checked( in_array( 'wordfence_whitelistBulkDelete', $log_hook_list ), 1 ); ?> data-type="WordFence bulk delete whitelist" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_whitelistBulkDelete"><?php echo 'wordfence_whitelistBulkDelete'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence bulk delete whitelist.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_updateConfig" id="wordfence_updateConfig" <?php checked( in_array( 'wordfence_updateConfig', $log_hook_list ), 1 ); ?> data-type="WordFence config updated" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_updateConfig"><?php echo 'wordfence_updateConfig'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence config updated.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_loadLiveTraffic" id="wordfence_loadLiveTraffic" <?php checked( in_array( 'wordfence_loadLiveTraffic', $log_hook_list ), 1 ); ?> data-type="WordFence load live traffic" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_loadLiveTraffic"><?php echo 'wordfence_loadLiveTraffic'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence load live traffic.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_loadBlockRanges" id="wordfence_loadBlockRanges" <?php checked( in_array( 'wordfence_loadBlockRanges', $log_hook_list ), 1 ); ?> data-type="WordFence load block ranges" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_loadBlockRanges"><?php echo 'wordfence_loadBlockRanges'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence load block ranges.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_loadCacheExclusions" id="wordfence_loadCacheExclusions" <?php checked( in_array( 'wordfence_loadCacheExclusions', $log_hook_list ), 1 ); ?> data-type="WordFence load cache exclusions" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_loadCacheExclusions"><?php echo 'wordfence_loadCacheExclusions'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence load cache exclusions.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_saveCacheConfig" id="wordfence_saveCacheConfig" <?php checked( in_array( 'wordfence_saveCacheConfig', $log_hook_list ), 1 ); ?> data-type="WordFence cache config save" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_saveCacheConfig"><?php echo 'wordfence_saveCacheConfig'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence cache config save.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_cache_clear" id="wordfence_cache_clear" <?php checked( in_array( 'wordfence_cache_clear', $log_hook_list ), 1 ); ?> data-type="WordFence clear cache" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_cache_clear"><?php echo 'wordfence_cache_clear'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence clear cache.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_reverseLookup" id="wordfence_reverseLookup" <?php checked( in_array( 'wordfence_reverseLookup', $log_hook_list ), 1 ); ?> data-type="WordFence" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_reverseLookup"><?php echo 'wordfence_reverseLookup'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence reverse look up.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_whois" id="wordfence_whois" <?php checked( in_array( 'wordfence_whois', $log_hook_list ), 1 ); ?> data-type="WordFence WHOIS lookup" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_whois"><?php echo 'wordfence_whois'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence WHOIS lookup.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_clearPageCache" id="wordfence_clearPageCache" <?php checked( in_array( 'wordfence_clearPageCache', $log_hook_list ), 1 ); ?> data-type="WordFence clear page cache" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_clearPageCache"><?php echo 'wordfence_clearPageCache'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence clear page cache.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_saveConfig" id="wordfence_saveConfig" <?php checked( in_array( 'wordfence_saveConfig', $log_hook_list ), 1 ); ?> data-type="WordFence configuration save" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_saveConfig"><?php echo 'wordfence_saveConfig'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence configuration save.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_blockIP" id="wordfence_blockIP" <?php checked( in_array( 'wordfence_blockIP', $log_hook_list ), 1 ); ?> data-type="WordFence block ip address" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_blockIP"><?php echo 'wordfence_blockIP'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence block ip address.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_addTwoFactor" id="wordfence_addTwoFactor" <?php checked( in_array( 'wordfence_addTwoFactor', $log_hook_list ), 1 ); ?> data-type="WordFence add two factor - enable cellphone sign in" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_addTwoFactor"><?php echo 'wordfence_addTwoFactor'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence add two factor - enable cellphone sign in.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_exportSettings" id="wordfence_exportSettings" <?php checked( in_array( 'wordfence_exportSettings', $log_hook_list ), 1 ); ?> data-type="WordFence export settings" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_exportSettings"><?php echo 'wordfence_exportSettings'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence export settings.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_clearAllBlocked" id="wordfence_clearAllBlocked" <?php checked( in_array( 'wordfence_clearAllBlocked', $log_hook_list ), 1 ); ?> data-type="WordFence clear all ip blocked" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_clearAllBlocked"><?php echo 'wordfence_clearAllBlocked'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence clear all ip blocked.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_blockIPUARange" id="wordfence_blockIPUARange" <?php checked( in_array( 'wordfence_blockIPUARange', $log_hook_list ), 1 ); ?> data-type="WordFence set block ip UA range" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_blockIPUARange"><?php echo 'wordfence_blockIPUARange'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence set block ip UA range.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_unblockRange" id="wordfence_unblockRange" <?php checked( in_array( 'wordfence_unblockRange', $log_hook_list ), 1 ); ?> data-type="WordFence unblock ip rang" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_unblockRange"><?php echo 'wordfence_unblockRange'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence unblock ip rang', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_saveCountryBlocking" id="wordfence_saveCountryBlocking" <?php checked( in_array( 'wordfence_saveCountryBlocking', $log_hook_list ), 1 ); ?> data-type="WordFence country blocking save" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_saveCountryBlocking"><?php echo 'wordfence_saveCountryBlocking'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence country blocking save.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_saveDebuggingConfig" id="wordfence_saveDebuggingConfig" <?php checked( in_array( 'wordfence_saveDebuggingConfig', $log_hook_list ), 1 ); ?> data-type="WordFence debugging config file save" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_saveDebuggingConfig"><?php echo 'wordfence_saveDebuggingConfig'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence debugging config file save.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_startPasswdAudit" id="wordfence_startPasswdAudit" <?php checked( in_array( 'wordfence_startPasswdAudit', $log_hook_list ), 1 ); ?> data-type="WordFence password audit start" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_startPasswdAudit"><?php echo 'wordfence_startPasswdAudit'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence password audit started.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_saveScanSchedule" id="wordfence_saveScanSchedule" <?php checked( in_array( 'wordfence_saveScanSchedule', $log_hook_list ), 1 ); ?> data-type="WordFence scan schedule save" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_saveScanSchedule"><?php echo 'wordfence_saveScanSchedule'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence scan schedule save.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_updateWAFRules" id="wordfence_updateWAFRules" <?php checked( in_array( 'wordfence_updateWAFRules', $log_hook_list ), 1 ); ?> data-type="WordFence WAF rules update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_updateWAFRules"><?php echo 'wordfence_updateWAFRules'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence WAF rules updated.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_sendTestEmail" id="wordfence_sendTestEmail" <?php checked( in_array( 'wordfence_sendTestEmail', $log_hook_list ), 1 ); ?> data-type="WordFence send test email" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_sendTestEmail"><?php echo 'wordfence_sendTestEmail'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence send test email.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_sendDiagnostic" id="wordfence_sendDiagnostic" <?php checked( in_array( 'wordfence_sendDiagnostic', $log_hook_list ), 1 ); ?> data-type="WordFence send diagnostic" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_sendDiagnostic"><?php echo 'wordfence_sendDiagnostic'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence send diagnostic.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_updateAllIssues" id="wordfence_updateAllIssues" <?php checked( in_array( 'wordfence_updateAllIssues', $log_hook_list ), 1 ); ?> data-type="WordFence all issue update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_updateAllIssues"><?php echo 'wordfence_updateAllIssues'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence all issue update.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_sendActivityLog" id="wordfence_sendActivityLog" <?php checked( in_array( 'wordfence_sendActivityLog', $log_hook_list ), 1 ); ?> data-type="WordFence activity log update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_sendActivityLog"><?php echo 'wordfence_sendActivityLog'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence activity log update.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_addCacheExclusion" id="wordfence_addCacheExclusion" <?php checked( in_array( 'wordfence_addCacheExclusion', $log_hook_list ), 1 ); ?> data-type="WordFence cache exclusion added" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_addCacheExclusion"><?php echo 'wordfence_addCacheExclusion'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence cache exclusion added.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_daily_autoUpdate" id="wordfence_daily_autoUpdate" <?php checked( in_array( 'wordfence_daily_autoUpdate', $log_hook_list ), 1 ); ?> data-type="WordFence daily auto upadate" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_daily_autoUpdate"><?php echo 'wordfence_daily_autoUpdate'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence daily auto upadate', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_update_blocked_IPs" id="wordfence_update_blocked_IPs" <?php checked( in_array( 'wordfence_update_blocked_IPs', $log_hook_list ), 1 ); ?> data-type="WordFence block ips update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_update_blocked_IPs"><?php echo 'wordfence_update_blocked_IPs'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence block ips update.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_start_scheduled_scan" id="wordfence_start_scheduled_scan" <?php checked( in_array( 'wordfence_start_scheduled_scan', $log_hook_list ), 1 ); ?> data-type="WordFence schedule scan start" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_start_scheduled_scan"><?php echo 'wordfence_start_scheduled_scan'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence schedule scan started.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_doScan" id="wordfence_doScan" <?php checked( in_array( 'wordfence_doScan', $log_hook_list ), 1 ); ?> data-type="WordFence do scan" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_doScan"><?php echo 'wordfence_doScan'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence do scan.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_daily_cron" id="wordfence_daily_cron" <?php checked( in_array( 'wordfence_daily_cron', $log_hook_list ), 1 ); ?> data-type="WordFence do daily cron" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_daily_cron"><?php echo 'wordfence_daily_cron'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence do daily cron.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_hourly_cron" id="wordfence_hourly_cron" <?php checked( in_array( 'wordfence_hourly_cron', $log_hook_list ), 1 ); ?> data-type="WordFence do hourly cron" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_hourly_cron"><?php echo 'wordfence_hourly_cron'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence do hourly cron.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_email_activity_report" id="wordfence_email_activity_report" <?php checked( in_array( 'wordfence_email_activity_report', $log_hook_list ), 1 ); ?> data-type="WordFence email activity report" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_email_activity_report"><?php echo 'wordfence_email_activity_report'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence email activity report.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_scan" id="wordfence_scan" <?php checked( in_array( 'wordfence_scan', $log_hook_list ), 1 ); ?> data-type="WordFence scan start" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_scan"><?php echo 'wordfence_scan'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence scan start.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_updateAlertEmail" id="wordfence_updateAlertEmail" <?php checked( in_array( 'wordfence_updateAlertEmail', $log_hook_list ), 1 ); ?> data-type="WordFence email alert update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_updateAlertEmail"><?php echo 'wordfence_updateAlertEmail'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence email alert update.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_restoreFile" id="wordfence_restoreFile" <?php checked( in_array( 'wordfence_restoreFile', $log_hook_list ), 1 ); ?> data-type="WordFence file restore" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_restoreFile"><?php echo 'wordfence_restoreFile'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence file restore.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_deletePasswdAudit" id="wordfence_deletePasswdAudit" <?php checked( in_array( 'wordfence_deletePasswdAudit', $log_hook_list ), 1 ); ?> data-type="WordFence password audit delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_deletePasswdAudit"><?php echo 'wordfence_deletePasswdAudit'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence password audit deleted.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_weakPasswordsFix" id="wordfence_weakPasswordsFix" <?php checked( in_array( 'wordfence_weakPasswordsFix', $log_hook_list ), 1 ); ?> data-type="WordFence weak password fixed" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_weakPasswordsFix"><?php echo 'wordfence_weakPasswordsFix'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence weak password fixed.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_killPasswdAudit" id="wordfence_killPasswdAudit" <?php checked( in_array( 'wordfence_killPasswdAudit', $log_hook_list ), 1 ); ?> data-type="WordFence password audit kill" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_killPasswdAudit"><?php echo 'wordfence_killPasswdAudit'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence password audit kill.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_importSettings" id="wordfence_importSettings" <?php checked( in_array( 'wordfence_importSettings', $log_hook_list ), 1 ); ?> data-type="WordFence settings import" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_importSettings"><?php echo 'wordfence_importSettings'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence settings import.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_bulkOperation" id="wordfence_bulkOperation" <?php checked( in_array( 'wordfence_bulkOperation', $log_hook_list ), 1 ); ?> data-type="WordFence bulk operation" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_bulkOperation"><?php echo 'wordfence_bulkOperation'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence bulk operation.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_deleteFile" id="wordfence_deleteFile" <?php checked( in_array( 'wordfence_deleteFile', $log_hook_list ), 1 ); ?> data-type="WordFence file delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_deleteFile"><?php echo 'wordfence_deleteFile'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence file deleted.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_deleteDatabaseOption" id="wordfence_deleteDatabaseOption" <?php checked( in_array( 'wordfence_deleteDatabaseOption', $log_hook_list ), 1 ); ?> data-type="WordFence database option delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_deleteDatabaseOption"><?php echo 'wordfence_deleteDatabaseOption'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence database option deleted.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_removeExclusion" id="wordfence_removeExclusion" <?php checked( in_array( 'wordfence_removeExclusion', $log_hook_list ), 1 ); ?> data-type="WordFence exclusion remove" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_removeExclusion"><?php echo 'wordfence_removeExclusion'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence exclusion remove.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_deleteIssue" id="wordfence_deleteIssue" <?php checked( in_array( 'wordfence_deleteIssue', $log_hook_list ), 1 ); ?> data-type="WordFence issue delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_deleteIssue"><?php echo 'wordfence_deleteIssue'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence issue deleted.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_unlockOutIP" id="wordfence_unlockOutIP" <?php checked( in_array( 'wordfence_unlockOutIP', $log_hook_list ), 1 ); ?> data-type="WordFence unblock out ip" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_unlockOutIP"><?php echo 'wordfence_unlockOutIP'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence unblock out ip.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_unblockIP" id="wordfence_unblockIP" <?php checked( in_array( 'wordfence_unblockIP', $log_hook_list ), 1 ); ?> data-type="WordFence unblock ip" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_unblockIP"><?php echo 'wordfence_unblockIP'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence unblock ip.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_permBlockIP" id="wordfence_permBlockIP" <?php checked( in_array( 'wordfence_permBlockIP', $log_hook_list ), 1 ); ?> data-type="WordFence permanently block ip" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_permBlockIP"><?php echo 'wordfence_permBlockIP'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence permanently block ip.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_downloadHtaccess" id="wordfence_downloadHtaccess" <?php checked( in_array( 'wordfence_downloadHtaccess', $log_hook_list ), 1 ); ?> data-type="WordFence htaccess file download" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_downloadHtaccess"><?php echo 'wordfence_downloadHtaccess'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence htaccess file download.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_checkFalconHtaccess" id="wordfence_checkFalconHtaccess" <?php checked( in_array( 'wordfence_checkFalconHtaccess', $log_hook_list ), 1 ); ?> data-type="WordFence Falcon htaccess file check" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_checkFalconHtaccess"><?php echo 'wordfence_checkFalconHtaccess'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence Falcon htaccess file check.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_removeFromCache" id="wordfence_removeFromCache" <?php checked( in_array( 'wordfence_removeFromCache', $log_hook_list ), 1 ); ?> data-type="WordFence remmove from cache" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_removeFromCache"><?php echo 'wordfence_removeFromCache'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence remmove from cache.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_autoUpdateChoice" id="wordfence_autoUpdateChoice" <?php checked( in_array( 'wordfence_autoUpdateChoice', $log_hook_list ), 1 ); ?> data-type="WordFence auto update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_autoUpdateChoice"><?php echo 'wordfence_autoUpdateChoice'; ?></label></td>
								<td><?php esc_html_e( 'Enable auto update.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_saveCacheOptions" id="wordfence_saveCacheOptions" <?php checked( in_array( 'wordfence_saveCacheOptions', $log_hook_list ), 1 ); ?> data-type="WordFence cache options save" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_saveCacheOptions"><?php echo 'wordfence_saveCacheOptions'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence cache options save.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_adminEmailChoice" id="wordfence_adminEmailChoice" <?php checked( in_array( 'wordfence_adminEmailChoice', $log_hook_list ), 1 ); ?> data-type="WordFence admin email choice" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_adminEmailChoice"><?php echo 'wordfence_adminEmailChoice'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence admin email choice.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_getCacheStats" id="wordfence_getCacheStats" <?php checked( in_array( 'wordfence_getCacheStats', $log_hook_list ), 1 ); ?> data-type="WordFence get cache stats" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_getCacheStats"><?php echo 'wordfence_getCacheStats'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence get cache stats.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_killScan" id="wordfence_killScan" <?php checked( in_array( 'wordfence_killScan', $log_hook_list ), 1 ); ?> data-type="WordFence kill scan" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_killScan"><?php echo 'wordfence_killScan'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence kill scan.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_tourClosed" id="wordfence_tourClosed" <?php checked( in_array( 'wordfence_tourClosed', $log_hook_list ), 1 ); ?> data-type="WordFence plugin tour closed" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_tourClosed"><?php echo 'wordfence_tourClosed'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence plugin tour closed.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_removeCacheExclusion" id="wordfence_removeCacheExclusion" <?php checked( in_array( 'wordfence_removeCacheExclusion', $log_hook_list ), 1 ); ?> data-type="WordFence cache exclusion remove" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_removeCacheExclusion"><?php echo 'wordfence_removeCacheExclusion'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence cache exclusion remove.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_unblockNetwork" id="wordfence_unblockNetwork" <?php checked( in_array( 'wordfence_unblockNetwork', $log_hook_list ), 1 ); ?> data-type="WordFence unblock network" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_unblockNetwork"><?php echo 'wordfence_unblockNetwork'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence unblock network.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_permanentlyBlockAllIPs" id="wordfence_permanentlyBlockAllIPs" <?php checked( in_array( 'wordfence_permanentlyBlockAllIPs', $log_hook_list ), 1 ); ?> data-type="WordFence permanetly block all ips" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_permanentlyBlockAllIPs"><?php echo 'wordfence_permanentlyBlockAllIPs'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence permanetly block all ips.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_deleteAdminUser" id="wordfence_deleteAdminUser" <?php checked( in_array( 'wordfence_deleteAdminUser', $log_hook_list ), 1 ); ?> data-type="WordFence admin user delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_deleteAdminUser"><?php echo 'wordfence_deleteAdminUser'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence admin user delete.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_revokeAdminUser" id="wordfence_revokeAdminUser" <?php checked( in_array( 'wordfence_revokeAdminUser', $log_hook_list ), 1 ); ?> data-type="WordFence admin user revoke" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_revokeAdminUser"><?php echo 'wordfence_revokeAdminUser'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence admin user revoke.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordfence">
								<th class="check-column" scope="row">
									<input value="wordfence_hideFileHtaccess" id="wordfence_hideFileHtaccess" <?php checked( in_array( 'wordfence_hideFileHtaccess', $log_hook_list ), 1 ); ?> data-type="WordFence htaccess file hide" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wordfence_hideFileHtaccess"><?php echo 'wordfence_hideFileHtaccess'; ?></label></td>
								<td><?php esc_html_e( 'Wordfence htaccess file hide.', 'user_activity_log_pro' ); ?></td>
							</tr>

							<?php
						}
						if ( is_plugin_active( 'wordpress-seo/wp-seo.php' ) ) {
							?>
							<tr class="groupHeader">
								<th class="check-column" scope="row">
									<input type="checkbox" name="" data-hookGroup="wordpressseo">
								</th>
								<td colspan="2"><?php esc_html_e( 'Yoast SEO', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordpressseo">
								<th class="check-column" scope="row">
									<input value="wpseo_handle_import" id="wpseo_handle_import" <?php checked( in_array( 'wpseo_handle_import', $log_hook_list ), 1 ); ?> data-type="Yoast SEO import data from other SEO plugins" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wpseo_handle_import"><?php echo 'wpseo_handle_import'; ?></label></td>
								<td><?php esc_html_e( 'Yoast SEO import data from other SEO plugins.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordpressseo">
								<th class="check-column" scope="row">
									<input value="wpseo_import" id="wpseo_import" <?php checked( in_array( 'wpseo_import', $log_hook_list ), 1 ); ?> data-type="Yoast SEO import data" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wpseo_import"><?php echo 'wpseo_import'; ?></label></td>
								<td><?php esc_html_e( 'Yoast SEO import data.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordpressseo">
								<th class="check-column" scope="row">
									<input value="wpseo_save_auth_code" id="wpseo_save_auth_code" <?php checked( in_array( 'wpseo_save_auth_code', $log_hook_list ), 1 ); ?> data-type="Yoast SEO authentication code save" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wpseo_save_auth_code"><?php echo 'wpseo_save_auth_code'; ?></label></td>
								<td><?php esc_html_e( 'Yoast SEO authentication code save.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordpressseo">
								<th class="check-column" scope="row">
									<input value="seo_check_admin_referer" id="seo_check_admin_referer" <?php checked( in_array( 'seo_check_admin_referer', $log_hook_list ), 1 ); ?> data-type="Yoast SEO robots.txt update, htaccess file update, import file" name="hook[]" type="checkbox" >
								</th>
								<td><label for="seo_check_admin_referer"><?php echo 'seo_check_admin_referer'; ?></label></td>
								<td><?php esc_html_e( 'Yoast SEO robots.txt update, htaccess file update, import file.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wordpressseo">
								<th class="check-column" scope="row">
									<input value="seo_updated_post_meta" id="updated_post_meta" <?php checked( in_array( 'seo_updated_post_meta', $log_hook_list ), 1 ); ?> data-type="Yoast SEO post meta update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="seo_updated_post_meta"><?php echo 'seo_updated_post_meta'; ?></label></td>
								<td><?php esc_html_e( 'Yoast SEO post meta update.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<?php
						}
						if ( is_plugin_active( 'blog-designer-pro/blog-designer-pro.php' ) ) {
							?>
							<tr class="groupHeader">
								<th class="check-column" scope="row">
									<input type="checkbox" name="" data-hookGroup="blog-designer-pro">
								</th>
								<td colspan="2"><?php esc_html_e( 'Blog Designer PRO', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="blog-designer-pro">
								<th class="check-column" scope="row">
									<input value="bdp_update_shortcode" id="bdp_update_shortcode" <?php checked( in_array( 'bdp_update_shortcode', $log_hook_list ), 1 ); ?> data-type="Blog layout is update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="bdp_update_shortcode"><?php echo 'bdp_update_shortcode'; ?></label></td>
								<td><?php esc_html_e( 'Blog layout is update.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="blog-designer-pro">
								<th class="check-column" scope="row">
									<input value="bdp_reset_shortcode" id="bdp_reset_shortcode" <?php checked( in_array( 'bdp_reset_shortcode', $log_hook_list ), 1 ); ?> data-type="Blog layout reset" name="hook[]" type="checkbox" >
								</th>
								<td><label for="bdp_reset_shortcode"><?php echo 'bdp_reset_shortcode'; ?></label></td>
								<td><?php esc_html_e( 'Blog layout reset.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="blog-designer-pro">
								<th class="check-column" scope="row">
									<input value="bdp_reset_archive_page" id="bdp_reset_archive_page" <?php checked( in_array( 'bdp_reset_archive_page', $log_hook_list ), 1 ); ?> data-type="Archive layout reset" name="hook[]" type="checkbox" >
								</th>
								<td><label for="bdp_reset_archive_page"><?php echo 'bdp_reset_archive_page'; ?></label></td>
								<td><?php esc_html_e( 'Archive layout reset.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="blog-designer-pro">
								<th class="check-column" scope="row">
									<input value="bdp_add_shortcode" id="bdp_add_shortcode" <?php checked( in_array( 'bdp_add_shortcode', $log_hook_list ), 1 ); ?> data-type="Blog layout added" name="hook[]" type="checkbox" >
								</th>
								<td><label for="bdp_add_shortcode"><?php echo 'bdp_add_shortcode'; ?></label></td>
								<td><?php esc_html_e( 'Blog layout added.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="blog-designer-pro">
								<th class="check-column" scope="row">
									<input value="bdp_update_single_file" id="bdp_update_single_file" <?php checked( in_array( 'bdp_update_single_file', $log_hook_list ), 1 ); ?> data-type="Single file updated" name="hook[]" type="checkbox" >
								</th>
								<td><label for="bdp_update_single_file"><?php echo 'bdp_update_single_file'; ?></label></td>
								<td><?php esc_html_e( 'Single file updated.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="blog-designer-pro">
								<th class="check-column" scope="row">
									<input value="bdp_add_archive_layout" id="bdp_add_archive_layout" <?php checked( in_array( 'bdp_add_archive_layout', $log_hook_list ), 1 ); ?> data-type="Archive layout create" name="hook[]" type="checkbox" >
								</th>
								<td><label for="bdp_add_archive_layout"><?php echo 'bdp_add_archive_layout'; ?></label></td>
								<td><?php esc_html_e( 'Archive layout created.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="blog-designer-pro">
								<th class="check-column" scope="row">
									<input value="bdp_delete_shortcode" id="bdp_delete_shortcode" <?php checked( in_array( 'bdp_delete_shortcode', $log_hook_list ), 1 ); ?> data-type="Blog layout delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="bdp_delete_shortcode"><?php echo 'bdp_delete_shortcode'; ?></label></td>
								<td><?php esc_html_e( 'Blog layout deleted.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="blog-designer-pro">
								<th class="check-column" scope="row">
									<input value="bdp_delete_single_template" id="bdp_delete_single_template" <?php checked( in_array( 'bdp_delete_single_template', $log_hook_list ), 1 ); ?> data-type="Single template delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="bdp_delete_single_template"><?php echo 'bdp_delete_single_template'; ?></label></td>
								<td><?php esc_html_e( 'Single template deleted', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="blog-designer-pro">
								<th class="check-column" scope="row">
									<input value="bdp_copy_single_template" id="bdp_copy_single_template" <?php checked( in_array( 'bdp_copy_single_template', $log_hook_list ), 1 ); ?> data-type="Single template copy" name="hook[]" type="checkbox" >
								</th>
								<td><label for="bdp_copy_single_template"><?php echo 'bdp_copy_single_template'; ?></label></td>
								<td><?php esc_html_e( 'Single template copy.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="blog-designer-pro">
								<th class="check-column" scope="row">
									<input value="bdp_duplicate_layout_settings" id="bdp_duplicate_layout_settings" <?php checked( in_array( 'bdp_duplicate_layout_settings', $log_hook_list ), 1 ); ?> data-type="Blog Designer Pro Duplicate layout" name="hook[]" type="checkbox" >
								</th>
								<td><label for="bdp_duplicate_layout_settings"><?php echo 'bdp_duplicate_layout_settings'; ?></label></td>
								<td><?php esc_html_e( 'Duplicate layout.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="blog-designer-pro">
								<th class="check-column" scope="row">
									<input value="bdp_import_blog_layout_settings" id="bdp_import_blog_layout_settings" <?php checked( in_array( 'bdp_import_blog_layout_settings', $log_hook_list ), 1 ); ?> data-type="Blog layout import" name="hook[]" type="checkbox" >
								</th>
								<td><label for="bdp_import_blog_layout_settings"><?php echo 'bdp_import_blog_layout_settings'; ?></label></td>
								<td><?php esc_html_e( 'Blog layout import.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="blog-designer-pro">
								<th class="check-column" scope="row">
									<input value="bdp_import_archive_layout_settings" id="bdp_import_archive_layout_settings" <?php checked( in_array( 'bdp_import_archive_layout_settings', $log_hook_list ), 1 ); ?> data-type="Archive layout import" name="hook[]" type="checkbox" >
								</th>
								<td><label for="bdp_import_archive_layout_settings"><?php echo 'bdp_import_archive_layout_settings'; ?></label></td>
								<td><?php esc_html_e( 'Archive layout import.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="blog-designer-pro">
								<th class="check-column" scope="row">
									<input value="bdp_export_blog_layout_settings" id="bdp_export_blog_layout_settings" <?php checked( in_array( 'bdp_export_blog_layout_settings', $log_hook_list ), 1 ); ?> data-type="Blog layout export" name="hook[]" type="checkbox" >
								</th>
								<td><label for="bdp_export_blog_layout_settings"><?php echo 'bdp_export_blog_layout_settings'; ?></label></td>
								<td><?php esc_html_e( 'Blog layout export.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="blog-designer-pro">
								<th class="check-column" scope="row">
									<input value="bdp_export_archive_layout_settings" id="bdp_export_archive_layout_settings" <?php checked( in_array( 'bdp_export_archive_layout_settings', $log_hook_list ), 1 ); ?> data-type="Archive layout export" name="hook[]" type="checkbox" >
								</th>
								<td><label for="bdp_export_archive_layout_settings"><?php echo 'bdp_export_archive_layout_settings'; ?></label></td>
								<td><?php esc_html_e( 'Archive layout export.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="blog-designer-pro">
								<th class="check-column" scope="row">
									<input value="bdp_update_archive_page" id="bdp_update_archive_page" <?php checked( in_array( 'bdp_update_archive_page', $log_hook_list ), 1 ); ?> data-type="Archive layout update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="bdp_update_archive_page"><?php echo 'bdp_update_archive_page'; ?></label></td>
								<td><?php esc_html_e( 'Archive layout update.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="blog-designer-pro">
								<th class="check-column" scope="row">
									<input value="bdp_import_single_layout_settings" id="bdp_import_single_layout_settings" <?php checked( in_array( 'bdp_import_single_layout_settings', $log_hook_list ), 1 ); ?> data-type="Single layout import" name="hook[]" type="checkbox" >
								</th>
								<td><label for="bdp_import_single_layout_settings"><?php echo 'bdp_import_single_layout_settings'; ?></label></td>
								<td><?php esc_html_e( 'Single layout import.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="blog-designer-pro">
								<th class="check-column" scope="row">
									<input value="bdp_export_single_layout_settings" id="bdp_export_single_layout_settings" <?php checked( in_array( 'bdp_export_single_layout_settings', $log_hook_list ), 1 ); ?> data-type="Single layout export" name="hook[]" type="checkbox" >
								</th>
								<td><label for="bdp_export_single_layout_settings"><?php echo 'bdp_export_single_layout_settings'; ?></label></td>
								<td><?php esc_html_e( 'Single layout export.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="blog-designer-pro">
								<th class="check-column" scope="row">
									<input value="bdp_delete_archive" id="bdp_delete_archive" <?php checked( in_array( 'bdp_delete_archive', $log_hook_list ), 1 ); ?> data-type="Archive layout delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="bdp_delete_archive"><?php echo 'bdp_delete_archive'; ?></label></td>
								<td><?php esc_html_e( 'Archive layout deleted.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="blog-designer-pro">
								<th class="check-column" scope="row">
									<input value="bdp_delete_single_layout" id="bdp_delete_single_layout" <?php checked( in_array( 'bdp_delete_single_layout', $log_hook_list ), 1 ); ?> data-type="Single layout delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="bdp_delete_single_layout"><?php echo 'bdp_delete_single_layout'; ?></label></td>
								<td><?php esc_html_e( 'Single layout deleted.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="blog-designer-pro">
								<th class="check-column" scope="row">
									<input value="bdp_update_single_page" id="bdp_update_single_page" <?php checked( in_array( 'bdp_update_single_page', $log_hook_list ), 1 ); ?> data-type="Single layout update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="bdp_update_single_page"><?php echo 'bdp_update_single_page'; ?></label></td>
								<td><?php esc_html_e( 'Single layout updated.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="blog-designer-pro">
								<th class="check-column" scope="row">
									<input value="bdp_reset_single_page" id="bdp_reset_single_page" <?php checked( in_array( 'bdp_reset_single_page', $log_hook_list ), 1 ); ?> data-type="Single layout reset" name="hook[]" type="checkbox" >
								</th>
								<td><label for="bdp_reset_single_page"><?php echo 'bdp_reset_single_page'; ?></label></td>
								<td><?php esc_html_e( 'Single layout reset.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="blog-designer-pro">
								<th class="check-column" scope="row">
									<input value="bdp_add_single_layout" id="bdp_add_single_layout" <?php checked( in_array( 'bdp_add_single_layout', $log_hook_list ), 1 ); ?> data-type="Single layout create" name="hook[]" type="checkbox" >
								</th>
								<td><label for="bdp_add_single_layout"><?php echo 'bdp_add_single_layout'; ?></label></td>
								<td><?php esc_html_e( 'Single layout created.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="blog-designer-pro">
								<th class="check-column" scope="row">
									<input value="bdp_duplicate_single_layout_settings" id="bdp_duplicate_single_layout_settings" <?php checked( in_array( 'bdp_duplicate_single_layout_settings', $log_hook_list ), 1 ); ?> data-type="Duplicate single layout" name="hook[]" type="checkbox" >
								</th>
								<td><label for="bdp_duplicate_single_layout_settings"><?php echo 'bdp_duplicate_single_layout_settings'; ?></label></td>
								<td><?php esc_html_e( 'Duplicate single layout.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<?php
						}
						if ( is_plugin_active( 'portfolio-designer/portfolio-designer.php' ) ) {
							?>
							<tr class="groupHeader">
								<th class="check-column" scope="row">
									<input type="checkbox" name="" data-hookGroup="portfolio_designer">
								</th>
								<td colspan="2"><?php esc_html_e( 'Portfolio Designer', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="portfolio_designer">
								<th class="check-column" scope="row">
									<input value="import_sample_portfolio_data" id="import_sample_portfolio_data" <?php checked( in_array( 'import_sample_portfolio_data', $log_hook_list ), 1 ); ?> data-type="Portfolio Designer Import sample data" name="hook[]" type="checkbox" >
								</th>
								<td><label for="import_sample_portfolio_data"><?php echo 'import_sample_portfolio_data'; ?></label></td>
								<td><?php esc_html_e( 'Import sample data.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="portfolio_designer">
								<th class="check-column" scope="row">
									<input value="create_portfolio_layout" id="create_portfolio_layout" <?php checked( in_array( 'create_portfolio_layout', $log_hook_list ), 1 ); ?> data-type="Portfolio Designer layout create" name="hook[]" type="checkbox" >
								</th>
								<td><label for="create_portfolio_layout"><?php echo 'create_portfolio_layout'; ?></label></td>
								<td><?php esc_html_e( 'Portfolio Designer layout created.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="portfolio_designer">
								<th class="check-column" scope="row">
									<input value="duplicate_portfolio_layout" id="duplicate_portfolio_layout" <?php checked( in_array( 'duplicate_portfolio_layout', $log_hook_list ), 1 ); ?> data-type="Duplicate portfolio layout" name="hook[]" type="checkbox" >
								</th>
								<td><label for="duplicate_portfolio_layout"><?php echo 'duplicate_portfolio_layout'; ?></label></td>
								<td><?php esc_html_e( 'Duplicate portfolio layout.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="portfolio_designer">
								<th class="check-column" scope="row">
									<input value="delete_portfolio_layout" id="delete_portfolio_layout" <?php checked( in_array( 'delete_portfolio_layout', $log_hook_list ), 1 ); ?> data-type="Portfolio Designer layout delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="delete_portfolio_layout"><?php echo 'delete_portfolio_layout'; ?></label></td>
								<td><?php esc_html_e( 'Portfolio Designer layout deleted.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="portfolio_designer">
								<th class="check-column" scope="row">
									<input value="update_portfolio_layout" id="update_portfolio_layout" <?php checked( in_array( 'update_portfolio_layout', $log_hook_list ), 1 ); ?> data-type="Portfolio Designer layout update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="update_portfolio_layout"><?php echo 'update_portfolio_layout'; ?></label></td>
								<td><?php esc_html_e( 'Portfolio Designer layout updated.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="portfolio_designer">
								<th class="check-column" scope="row">
									<input value="reset_portfolio_layout" id="reset_portfolio_layout" <?php checked( in_array( 'reset_portfolio_layout', $log_hook_list ), 1 ); ?> data-type="Reset portfolio designer layout" name="hook[]" type="checkbox" >
								</th>
								<td><label for="reset_portfolio_layout"><?php echo 'reset_portfolio_layout'; ?></label></td>
								<td><?php esc_html_e( 'Reset portfolio designer layout.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="portfolio_designer">
								<th class="check-column" scope="row">
									<input value="create_post_type" id="create_post_type" <?php checked( in_array( 'create_post_type', $log_hook_list ), 1 ); ?> data-type="Portfolio Designer Post type create" name="hook[]" type="checkbox" >
								</th>
								<td><label for="create_post_type"><?php echo 'create_post_type'; ?></label></td>
								<td><?php esc_html_e( 'Post type created.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="portfolio_designer">
								<th class="check-column" scope="row">
									<input value="update_post_type" id="update_post_type" <?php checked( in_array( 'update_post_type', $log_hook_list ), 1 ); ?> data-type="Portfolio Designer Post type update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="update_post_type"><?php echo 'update_post_type'; ?></label></td>
								<td><?php esc_html_e( 'Post type updated.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="portfolio_designer">
								<th class="check-column" scope="row">
									<input value="export_portfolio_layout" id="export_portfolio_layout" <?php checked( in_array( 'export_portfolio_layout', $log_hook_list ), 1 ); ?> data-type="Portfolio Designer Export portfolio layout" name="hook[]" type="checkbox" >
								</th>
								<td><label for="export_portfolio_layout"><?php echo 'export_portfolio_layout'; ?></label></td>
								<td><?php esc_html_e( 'Export portfolio layout.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="portfolio_designer">
								<th class="check-column" scope="row">
									<input value="delete_post_type" id="delete_post_type" <?php checked( in_array( 'delete_post_type', $log_hook_list ), 1 ); ?> data-type="Portfolio Designer Post type delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="delete_post_type"><?php echo 'delete_post_type'; ?></label></td>
								<td><?php esc_html_e( 'Post type deleted.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<?php
						}
						if ( is_plugin_active( 'all-in-one-reservation/all-in-one-reservation.php' ) ) {
							?>
							<tr class="groupHeader">
								<th class="check-column" scope="row">
									<input type="checkbox" name="" data-hookGroup="all_in_one_reservation">
								</th>
								<td colspan="2"><?php esc_html_e( 'All In One Reservation', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="all_in_one_reservation">
								<th class="check-column" scope="row">
									<input value="save_reservation_settings" id="save_reservation_settings" <?php checked( in_array( 'save_reservation_settings', $log_hook_list ), 1 ); ?> data-type="All in one reservation Setting Update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="save_reservation_settings"><?php echo 'save_reservation_settings'; ?></label></td>
								<td><?php esc_html_e( 'Save reservation settings.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="all_in_one_reservation">
								<th class="check-column" scope="row">
									<input value="rest_update_reservation_status" id="rest_update_reservation_status" <?php checked( in_array( 'rest_update_reservation_status', $log_hook_list ), 1 ); ?> data-type="All in one reservation reservation status update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="rest_update_reservation_status"><?php echo 'rest_update_reservation_status'; ?></label></td>
								<td><?php esc_html_e( 'Update reservation status.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="all_in_one_reservation">
								<th class="check-column" scope="row">
									<input value="reset_reservation_settings" id="reset_reservation_settings" <?php checked( in_array( 'reset_reservation_settings', $log_hook_list ), 1 ); ?> data-type="All in one reservation Reset reservation settings" name="hook[]" type="checkbox" >
								</th>
								<td><label for="reset_reservation_settings"><?php echo 'reset_reservation_settings'; ?></label></td>
								<td><?php esc_html_e( 'Reset reservation settings.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<?php
						}
						if ( is_plugin_active( 'wc-checkout-fields-editor/wc-checkout-fields-editor.php' ) ) {
							?>
							<tr class="groupHeader">
								<th class="check-column" scope="row">
									<input type="checkbox" name="" data-hookGroup="wc_checkout_fields_editor">
								</th>
								<td colspan="2"><?php esc_html_e( 'WC Checkout Fields Editor', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wc_checkout_fields_editor">
								<th class="check-column" scope="row">
									<input value="save_wcfe_settings" id="save_wcfe_settings" <?php checked( in_array( 'save_wcfe_settings', $log_hook_list ), 1 ); ?> data-type="WC checkout fields settings updated" name="hook[]" type="checkbox" >
								</th>
								<td><label for="save_wcfe_settings"><?php echo 'save_wcfe_settings'; ?></label></td>
								<td><?php esc_html_e( 'WC checkout fields settings updated.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="wc_checkout_fields_editor">
								<th class="check-column" scope="row">
									<input value="wp_ajax_wcfe_get_checkout_required_fields" id="wp_ajax_wcfe_get_checkout_required_fields" <?php checked( in_array( 'wp_ajax_wcfe_get_checkout_required_fields', $log_hook_list ), 1 ); ?> data-type="WC checkout fields editor setting restore" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wp_ajax_wcfe_get_checkout_required_fields"><?php echo 'wp_ajax_wcfe_get_checkout_required_fields'; ?></label></td>
								<td><?php esc_html_e( 'Default setting restore.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<?php
						}
						if ( is_plugin_active( 'easy-digital-downloads/easy-digital-downloads.php' ) ) {
							?>
							<tr class="groupHeader">
								<th class="check-column" scope="row">
									<input type="checkbox" name="" data-hookGroup="edd">
								</th>
								<td colspan="2"><?php esc_html_e( 'Easy Digital Downloads', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="edd">
								<th class="check-column" scope="row">
									<input value="edd_settings" id="edd_settings" <?php checked( in_array( 'edd_settings', $log_hook_list ), 1 ); ?> data-type="Easy Digital Download setting Update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="edd_settings"><?php echo 'edd_settings'; ?></label></td>
								<td><?php esc_html_e( 'Easy Digital Download settings updated.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="edd">
								<th class="check-column" scope="row">
									<input value="edd_activate_discount" id="edd_activate_discount" <?php checked( in_array( 'edd_activate_discount', $log_hook_list ), 1 ); ?> data-type="Easy Digital Download discount code activate" name="hook[]" type="checkbox" >
								</th>
								<td><label for="edd_activate_discount"><?php echo 'edd_activate_discount'; ?></label></td>
								<td><?php esc_html_e( 'Activate discount code.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="edd">
								<th class="check-column" scope="row">
									<input value="edd_deactivate_discount" id="edd_deactivate_discount" <?php checked( in_array( 'edd_deactivate_discount', $log_hook_list ), 1 ); ?> data-type="Easy Digital Download discount code deactive" name="hook[]" type="checkbox" >
								</th>
								<td><label for="edd_deactivate_discount"><?php echo 'edd_deactivate_discount'; ?></label></td>
								<td><?php esc_html_e( 'Deactivate discount code.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="edd">
								<th class="check-column" scope="row">
									<input value="edd_add_discount" id="edd_add_discount" <?php checked( in_array( 'edd_add_discount', $log_hook_list ), 1 ); ?> data-type="Easy Digital Download discount code add" name="hook[]" type="checkbox" >
								</th>
								<td><label for="edd_add_discount"><?php echo 'edd_add_discount'; ?></label></td>
								<td><?php esc_html_e( 'Add discount code.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="edd">
								<th class="check-column" scope="row">
									<input value="edd_delete_discount" id="edd_delete_discount" <?php checked( in_array( 'edd_delete_discount', $log_hook_list ), 1 ); ?> data-type="Easy Digital Download discount code delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="edd_delete_discount"><?php echo 'edd_delete_discount'; ?></label></td>
								<td><?php esc_html_e( 'Delete discount code.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="edd">
								<th class="check-column" scope="row">
									<input value="edd_discount" id="edd_discount" <?php checked( in_array( 'edd_discount', $log_hook_list ), 1 ); ?> data-type="Easy Digital Download discount code update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="edd_discount"><?php echo 'edd_discount'; ?></label></td>
								<td><?php esc_html_e( 'Edit discount code.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<?php
						}
						if ( is_plugin_active( 'bbpress/bbpress.php' ) ) {
							?>
							<tr class="groupHeader">
								<th class="check-column" scope="row">
									<input type="checkbox" name="" data-hookGroup="bbpress">
								</th>
								<td colspan="2"><?php esc_html_e( 'bbPress', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="bbpress">
								<th class="check-column" scope="row">
									<input value="bbpress_settings" id="bbpress_settings" <?php checked( in_array( 'bbpress_settings', $log_hook_list ), 1 ); ?> data-type="bbPress Forum setting update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="bbpress_settings"><?php esc_html_e( 'bbpress_settings', 'user_activity_log_pro' ); ?></label></td>
								<td><?php esc_html_e( 'bbPress Forum settings updated', 'user_activity_log_pro' ); ?></td>
							</tr>
							<?php
						}
						if ( is_plugin_active( 'autoptimize/autoptimize.php' ) ) {
							?>
							<tr class="groupHeader">
								<th class="check-column" scope="row">
									<input type="checkbox" name="" data-hookGroup="autoptimize">
								</th>
								<td colspan="2"><?php esc_html_e( 'Autoptimize', 'user_activity_log_pro' ); ?>(2)</td>
							</tr>
							<tr class="autoptimize">
								<th class="check-column" scope="row">
									<input value="autoptimizer_settings" id="autoptimizer_settings" <?php checked( in_array( 'autoptimizer_settings', $log_hook_list ), 1 ); ?> data-type="Autoptimize settings update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="autoptimizer_settings"><?php echo 'autoptimizer_settings'; ?></label></td>
								<td><?php esc_html_e( 'Autoptimize settings updated.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="autoptimize">
								<th class="check-column" scope="row">
									<input value="wp_ajax_autoptimize_delete_cache" id="wp_ajax_autoptimize_delete_cache" <?php checked( in_array( 'wp_ajax_autoptimize_delete_cache', $log_hook_list ), 1 ); ?> data-type="Autoptimize Cache delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wp_ajax_autoptimize_delete_cache"><?php echo 'wp_ajax_autoptimize_delete_cache'; ?></label></td>
								<td><?php esc_html_e( 'Cache deleted.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<?php
						}
						if ( is_plugin_active( 'users-ultra/xoousers.php' ) ) {
							?>
							<tr class="groupHeader">
								<th class="check-column" scope="row">
									<input type="checkbox" name="" data-hookGroup="xoousers">
								</th>
								<td colspan="2"><?php esc_html_e( 'Users Ultra Membership', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="xoousers">
								<th class="check-column" scope="row">
									<input value="wp_ajax_add_new_custom_profile_field" id="wp_ajax_add_new_custom_profile_field" <?php checked( in_array( 'wp_ajax_add_new_custom_profile_field', $log_hook_list ), 1 ); ?> data-type="Custom profile field create" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wp_ajax_add_new_custom_profile_field"><?php echo 'wp_ajax_add_new_custom_profile_field'; ?></label></td>
								<td><?php esc_html_e( 'Custom profile field created.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="xoousers">
								<th class="check-column" scope="row">
									<input value="user_ultra_settings" id="user_ultra_settings" <?php checked( in_array( 'user_ultra_settings', $log_hook_list ), 1 ); ?> data-type="User Ultra membership settings updated" name="hook[]" type="checkbox" >
								</th>
								<td><label for="user_ultra_settings"><?php echo 'user_ultra_settings'; ?></label></td>
								<td><?php esc_html_e( 'User ultra membership settings updated', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="xoousers">
								<th class="check-column" scope="row">
									<input value="wp_ajax_save_fields_settings" id="wp_ajax_save_fields_settings" <?php checked( in_array( 'wp_ajax_save_fields_settings', $log_hook_list ), 1 ); ?> data-type="User profile fields save" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wp_ajax_save_fields_settings"><?php echo 'wp_ajax_save_fields_settings'; ?></label></td>
								<td><?php esc_html_e( 'User profile fields saved.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="xoousers">
								<th class="check-column" scope="row">
									<input value="wp_ajax_delete_profile_field" id="wp_ajax_delete_profile_field" <?php checked( in_array( 'wp_ajax_delete_profile_field', $log_hook_list ), 1 ); ?> data-type="User profile field delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wp_ajax_delete_profile_field"><?php echo 'wp_ajax_delete_profile_field'; ?></label></td>
								<td><?php esc_html_e( 'User profile field deleted.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="xoousers">
								<th class="check-column" scope="row">
									<input value="wp_ajax_sort_fileds_list" id="wp_ajax_sort_fileds_list" <?php checked( in_array( 'wp_ajax_sort_fileds_list', $log_hook_list ), 1 ); ?> data-type="User Ultra Fields order update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wp_ajax_sort_fileds_list"><?php echo 'wp_ajax_sort_fileds_list'; ?></label></td>
								<td><?php esc_html_e( 'Fields order updated.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="xoousers">
								<th class="check-column" scope="row">
									<input value="wp_ajax_custom_fields_reset" id="wp_ajax_custom_fields_reset" <?php checked( in_array( 'wp_ajax_custom_fields_reset', $log_hook_list ), 1 ); ?> data-type="User Ultra Custom Profile fields reset" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wp_ajax_custom_fields_reset"><?php echo 'wp_ajax_custom_fields_reset'; ?></label></td>
								<td><?php esc_html_e( 'Custom Profile fields reset.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="xoousers">
								<th class="check-column" scope="row">
									<input value="wp_ajax_create_uploader_folder" id="wp_ajax_create_uploader_folder" <?php checked( in_array( 'wp_ajax_create_uploader_folder', $log_hook_list ), 1 ); ?> data-type="User Ultra Uploader folder create" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wp_ajax_create_uploader_folder"><?php echo 'wp_ajax_create_uploader_folder'; ?></label></td>
								<td><?php esc_html_e( 'Uploader folder created.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="xoousers">
								<th class="check-column" scope="row">
									<input value="wp_ajax_package_add_new" id="wp_ajax_package_add_new" <?php checked( in_array( 'wp_ajax_package_add_new', $log_hook_list ), 1 ); ?> data-type="User Ultra Membership package create" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wp_ajax_package_add_new"><?php echo 'wp_ajax_package_add_new'; ?></label></td>
								<td><?php esc_html_e( 'Membership package created.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="xoousers">
								<th class="check-column" scope="row">
									<input value="wp_ajax_package_delete" id="wp_ajax_package_delete" <?php checked( in_array( 'wp_ajax_package_delete', $log_hook_list ), 1 ); ?> data-type="User Ultra Membership package delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wp_ajax_package_delete"><?php echo 'wp_ajax_package_delete'; ?></label></td>
								<td><?php esc_html_e( 'Membership package deleted.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="xoousers">
								<th class="check-column" scope="row">
									<input value="wp_ajax_package_edit_confirm" id="wp_ajax_package_edit_confirm" <?php checked( in_array( 'wp_ajax_package_edit_confirm', $log_hook_list ), 1 ); ?> data-type="User Ultra Membership package update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wp_ajax_package_edit_confirm"><?php echo 'wp_ajax_package_edit_confirm'; ?></label></td>
								<td><?php esc_html_e( 'Membership package updated.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="xoousers">
								<th class="check-column" scope="row">
									<input value="wp_ajax_sync_users" id="wp_ajax_sync_users" <?php checked( in_array( 'wp_ajax_sync_users', $log_hook_list ), 1 ); ?> data-type="User Ultra Sync user" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wp_ajax_sync_users"><?php echo 'wp_ajax_sync_users'; ?></label></td>
								<td><?php esc_html_e( 'Sync users.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="xoousers">
								<th class="check-column" scope="row">
									<input value="wp_ajax_user_resend_activation_link" id="wp_ajax_user_resend_activation_link" <?php checked( in_array( 'wp_ajax_user_resend_activation_link', $log_hook_list ), 1 ); ?> data-type="User Ultra" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wp_ajax_user_resend_activation_link"><?php echo 'wp_ajax_user_resend_activation_link'; ?></label></td>
								<td><?php esc_html_e( 'Resend activation link', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="xoousers">
								<th class="check-column" scope="row">
									<input value="wp_ajax_delete_uultra_transtient" id="wp_ajax_delete_uultra_transtient" <?php checked( in_array( 'wp_ajax_delete_uultra_transtient', $log_hook_list ), 1 ); ?> data-type="User ultra transients delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wp_ajax_delete_uultra_transtient"><?php echo 'wp_ajax_delete_uultra_transtient'; ?></label></td>
								<td><?php esc_html_e( 'User ultra transients deleted', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="xoousers">
								<th class="check-column" scope="row">
									<input value="wp_ajax_delete_uultra_stats" id="wp_ajax_delete_uultra_stats" <?php checked( in_array( 'wp_ajax_delete_uultra_stats', $log_hook_list ), 1 ); ?> data-type="User Ultra Clean the Users ultra stats" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wp_ajax_delete_uultra_stats"><?php echo 'wp_ajax_delete_uultra_stats'; ?></label></td>
								<td><?php esc_html_e( 'Clean the Users ultra stats.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="xoousers">
								<th class="check-column" scope="row">
									<input value="wp_ajax_delete_uultra_reviews" id="wp_ajax_delete_uultra_reviews" <?php checked( in_array( 'wp_ajax_delete_uultra_reviews', $log_hook_list ), 1 ); ?> data-type="User Ultra Clean the users ultra ratings" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wp_ajax_delete_uultra_reviews"><?php echo 'wp_ajax_delete_uultra_reviews'; ?></label></td>
								<td><?php esc_html_e( 'Clean the users ultra ratings.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="xoousers">
								<th class="check-column" scope="row">
									<input value="wp_ajax_edit_photo_cate_conf" id="wp_ajax_edit_photo_cate_conf" <?php checked( in_array( 'wp_ajax_edit_photo_cate_conf', $log_hook_list ), 1 ); ?> data-type="User Ultra Photo category update" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wp_ajax_edit_photo_cate_conf"><?php echo 'wp_ajax_edit_photo_cate_conf'; ?></label></td>
								<td><?php esc_html_e( 'Photo category updated.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="xoousers">
								<th class="check-column" scope="row">
									<input value="wp_ajax_edit_photo_cate_del" id="wp_ajax_edit_photo_cate_del" <?php checked( in_array( 'wp_ajax_edit_photo_cate_del', $log_hook_list ), 1 ); ?> data-type="User Ultra Photo category delete" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wp_ajax_edit_photo_cate_del"><?php echo 'wp_ajax_edit_photo_cate_del'; ?></label></td>
								<td><?php esc_html_e( 'Photo category deleted.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="xoousers">
								<th class="check-column" scope="row">
									<input value="wp_ajax_edit_photo_cate" id="wp_ajax_edit_photo_cate" <?php checked( in_array( 'wp_ajax_edit_photo_cate', $log_hook_list ), 1 ); ?> data-type="User Ultra Photo category edit" name="hook[]" type="checkbox" >
								</th>
								<td><label for="wp_ajax_edit_photo_cate"><?php echo 'wp_ajax_edit_photo_cate'; ?></label></td>
								<td><?php esc_html_e( 'Photo category edited.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="xoousers">
								<th class="check-column" scope="row">
									<input value="userultra_admin_menu_hook" id="userultra_admin_menu_hook" <?php checked( in_array( 'userultra_admin_menu_hook', $log_hook_list ), 1 ); ?> data-type="User Ultra Photo category create" name="hook[]" type="checkbox" >
								</th>
								<td><label for="userultra_admin_menu_hook"><?php echo 'userultra_admin_menu_hook'; ?></label></td>
								<td><?php esc_html_e( 'Photo category created.', 'user_activity_log_pro' ); ?></td>
							</tr>
							<?php
						}
						if ( is_plugin_active( 'user-switching/user-switching.php' ) ) {
							?>
							<tr class="groupHeader">
								<th class="check-column" scope="row">
									<input type="checkbox" name="" data-hookGroup="user_switching">
								</th>
								<td colspan="2"><?php esc_html_e( 'User Switching', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="user_switching">
								<th class="check-column" scope="row">
									<input value="switch_to_user" id="switch_to_user" <?php checked( in_array( 'switch_to_user', $log_hook_list ), 1 ); ?> data-type="User switches to another user account" name="hook[]" type="checkbox" >
								</th>
								<td><label for="switch_to_user"><?php echo 'switch_to_user'; ?></label></td>
								<td><?php esc_html_e( 'User switches to another user account', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="user_switching">
								<th class="check-column" scope="row">
									<input value="switch_back_user" id="switch_back_user" <?php checked( in_array( 'switch_back_user', $log_hook_list ), 1 ); ?> data-type="User switches back to their originating account" name="hook[]" type="checkbox" >
								</th>
								<td><label for="switch_back_user"><?php echo 'switch_back_user'; ?></label></td>
								<td><?php esc_html_e( 'User switches back to their originating account', 'user_activity_log_pro' ); ?></td>
							</tr>
							<tr class="user_switching">
								<th class="check-column" scope="row">
									<input value="switch_off_user" id="switch_off_user" <?php checked( in_array( 'switch_off_user', $log_hook_list ), 1 ); ?> data-type="User switches off" name="hook[]" type="checkbox" >
								</th>
								<td><label for="switch_off_user"><?php echo 'switch_off_user'; ?></label></td>
								<td><?php esc_html_e( 'User switches off', 'user_activity_log_pro' ); ?></td>
							</tr>
							<?php
						}
						?>
					</tbody>
					<thead>
						<tr>
							<th class="column-cb check-column">
								<input type="checkbox" id="cb_select" title="<?php esc_html_e( 'Select All', 'user_activity_log_pro' ); ?>">
							</th>
							<td scope="col"><?php esc_html_e( 'Activity', 'user_activity_log_pro' ); ?></td>
							<td scope="col"><?php esc_html_e( 'Description', 'user_activity_log_pro' ); ?></td>
						</tr>
					</thead>
					<tfoot>
						<tr>
							<th class="column-cb check-column">
								<input type="checkbox" id="cb_select1" title="<?php esc_html_e( 'Select All', 'user_activity_log_pro' ); ?>">
							</th>
							<td scope="col"><?php esc_html_e( 'Activity', 'user_activity_log_pro' ); ?></td>
							<td scope="col"><?php esc_html_e( 'Description', 'user_activity_log_pro' ); ?></td>
						</tr>
					</tfoot>
				</table>
				<?php
				wp_nonce_field( '_wp_log_hook_action', '_wp_log_hook_nonce' );
				?>
				<p class="submit">
					<input type="submit" name="btnSubmitLog" class="button button-primary" id="submit1" value="<?php esc_attr_e( 'Save Changes', 'user_activity_log_pro' ); ?>" >
					<input type="submit" name="btnResetLogData" class="button-secondary" value="<?php esc_html_e( 'Reset Changes', 'user_activity_log_pro' ); ?>"  onclick="return confirm('<?php esc_js( esc_html_e( 'Are you sure to reset data?', 'user_activity_log_pro' ) ); ?>')" >
				</p>
			</div>
		</form>
		<?php
	}
}

if ( ! function_exists( 'ualp_email_settings' ) ) {
	/**
	 * User activity email Settings.
	 */
	function ualp_email_settings() {
		?>
		<div class="wrap">
			<h1><?php esc_html_e( 'Email Settings', 'user_activity_log_pro' ); ?></h1>
			<?php
			if ( isset( $_SESSION['success_msg'] ) ) {
				?>
				<div class="success_msg">
				<?php
					ualp_admin_notice_message( 'updated', $_SESSION['success_msg'] );
					unset( $_SESSION['success_msg'] );
				?>
				</div>
				<?php
			}
			?>
			<div class="tab_parent_parent ualpParentTabs">
				<h2 class="nav-tab-wrapper nav-tab-wrapper">
					<a class="nav-tab nav-tab-active ualpTriggerSettings" data-href="ualpTriggerSettings" href="javascript:void(0)">
						<?php esc_html_e( 'Trigger Notification Settings', 'user_activity_log_pro' ); ?>
					</a>
					<a class="nav-tab ualpEmailNotificationSettings" data-href="ualpEmailNotificationSettings" href="javascript:void(0)" >
						<?php esc_html_e( 'Email Notification Settings', 'user_activity_log_pro' ); ?>
					</a>
				   
				</h2>
			</div>
			<div class="ualpTabContentWrap">
				<div id="ualpTriggerSettings" style="display: none" class="ualpContentDiv"><?php ualp_trigger_notification_settings(); ?></div>
				<div id="ualpEmailNotificationSettings" style="display: none" class="ualpContentDiv"><?php ualp_email_notification_settings(); ?></div>
			</div>
		</div>
		<?php
	}
}
if ( ! function_exists( 'ualp_email_notification_settings' ) ) {
	/**
	 * User activity email notification settings.
	 */
	function ualp_email_notification_settings() {
		global $wpdb, $current_user;
			$search        = '';
			$get_user_data = '';
			$get_role_data = '';
			$paged         = 1;
			$total_pages   = 1;
			$srno          = 0;
			$recordperpage = 10;
			$where         = 'where 1=1';
			$display       = 'roles';
			$user_details  = '[user_details]';
			wp_get_current_user();

		if ( isset( $_POST['saveLogin'] ) && isset( $_POST['_wp_email_notify_nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wp_email_notify_nonce'] ) ), '_wp_email_notify_action' ) ) {
			$from_email                         = ( isset( $_POST['txtualpFromEmail'] ) && '' != $_POST['txtualpFromEmail'] ) ? sanitize_text_field( wp_unslash( $_POST['txtualpFromEmail'] ) ) : '';
			$ualp_to_failed_login_email         = ( isset( $_POST['ualpToFaildLoginEmail'] ) && '' != $_POST['ualpToFaildLoginEmail'] ) ? sanitize_text_field( wp_unslash( $_POST['ualpToFaildLoginEmail'] ) ) : '';
			$ualp_to_fail_login_not_user_email  = ( isset( $_POST['ualpFailLoginNotUserEmail'] ) && '' != $_POST['ualpFailLoginNotUserEmail'] ) ? sanitize_text_field( wp_unslash( $_POST['ualpFailLoginNotUserEmail'] ) ) : '';
			$ualp_fail_login_number             = ( isset( $_POST['ualpFailLoginNumber'] ) && '' != $_POST['ualpFailLoginNumber'] ) ? sanitize_text_field( wp_unslash( $_POST['ualpFailLoginNumber'] ) ) : '';
			$ualp_to_fail_login_not_user_number = ( isset( $_POST['ualpFailLoginNotUserNumber'] ) && '' != $_POST['ualpFailLoginNotUserNumber'] ) ? sanitize_text_field( wp_unslash( $_POST['ualpFailLoginNotUserNumber'] ) ) : '';
			$ualp404_page_access_number         = ( isset( $_POST['ualp404PageAccessNumber'] ) && '' != $_POST['ualp404PageAccessNumber'] ) ? sanitize_text_field( wp_unslash( $_POST['ualp404PageAccessNumber'] ) ) : '';
			$ualp_to404_login_email             = ( isset( $_POST['ualp404LoginEmail'] ) && '' != $_POST['ualp404LoginEmail'] ) ? sanitize_text_field( wp_unslash( $_POST['ualp404LoginEmail'] ) ) : '';
			$mail_msg                           = ( isset( $_POST['txtualpEmailMsg'] ) && '' != $_POST['txtualpEmailMsg'] ) ? sanitize_text_field( wp_unslash( $_POST['txtualpEmailMsg'] ) ) : '';
			$ualp_user_email_notify             = ( isset( $_POST['ualpUserEmailNotify'] ) && '' != $_POST['ualpUserEmailNotify'] ) ? array_map( 'sanitize_text_field', wp_unslash( $_POST['ualpUserEmailNotify'] ) ) : array();
			if ( '' == $mail_msg ) {
				$message = esc_html__( 'Please enter message', 'user_activity_log_pro' );
				ualp_admin_notice_message( 'error', $message );
			} elseif ( '' == $from_email ) {
				$message = esc_html__( 'Please enter the email address.', 'user_activity_log_pro' );
				ualp_admin_notice_message( 'error', $message );
			} elseif ( ! filter_var( $from_email, FILTER_VALIDATE_EMAIL ) || ! is_email( $from_email ) ) {
				$message = esc_html__( 'Please enter valid email address.', 'user_activity_log_pro' );
				ualp_admin_notice_message( 'error', $message );
			} else {
				update_option( 'ualpFromEmail', $from_email );
				update_option( 'ualpToFaildLoginEmail', $ualp_to_failed_login_email );
				update_option( 'ualpFailLoginNotUserEmail', $ualp_to_fail_login_not_user_email );
				update_option( 'ualpFailLoginNumber', $ualp_fail_login_number );
				update_option( 'ualpFailLoginNotUserNumber', $ualp_to_fail_login_not_user_number );
				update_option( 'ualp404PageAccessNumber', $ualp404_page_access_number );
				update_option( 'ualp404LoginEmail', $ualp_to404_login_email );
				update_option( 'ualpEmailMessage', $mail_msg );
				update_option( 'ualpEmailNotification', $ualp_user_email_notify );
				$message = esc_html__( 'Email Settings saved successfully.', 'user_activity_log_pro' );
				ualp_admin_notice_message( 'updated', $message );
				do_action( 'ualp_email_settings_updated' );
			}
		}

		if ( isset( $_POST['btnResetEmailData'] ) ) {
			$ualp_to_failed_login_email         = '';
			$ualp_to_fail_login_not_user_email  = '';
			$ualp_fail_login_number             = '';
			$ualp_to_fail_login_not_user_number = '';
			$ualp404_page_access_number         = '';
			$ualp_to404_login_email             = '';
			$ualp_user_email_notify             = '';
			$enable_user_login                  = '';
			$mail_msg                           = esc_html__( 'Hi', 'user_activity_log_pro' ) . ' ';
			$mail_msg                          .= $current_user->display_name . ',';
			$mail_msg                          .= "\n\n" . __( 'Following user has done some activity in your site', 'user_activity_log_pro' ) . " \n$user_details";
			$mail_msg                          .= "\n\n" . __( 'Thanks,', 'user_activity_log_pro' ) . " \n";
			$mail_msg                          .= home_url();
			$admin_email                        = get_option( 'admin_email' );
			update_option( 'ualpToFaildLoginEmail', $ualp_to_failed_login_email );
			update_option( 'ualpFailLoginNotUserEmail', $ualp_to_fail_login_not_user_email );
			update_option( 'ualpFailLoginNumber', $ualp_fail_login_number );
			update_option( 'ualpFailLoginNotUserNumber', $ualp_to_fail_login_not_user_number );
			update_option( 'ualp404PageAccessNumber', $ualp404_page_access_number );
			update_option( 'ualp404LoginEmail', $ualp_to404_login_email );
			update_option( 'ualpFromEmail', $admin_email );
			update_option( 'ualpEmailMessage', $mail_msg );
			update_option( 'ualpEmailNotification', $ualp_user_email_notify );
			$message = esc_html__( 'Email settings reset successfully.', 'user_activity_log_pro' );
			ualp_admin_notice_message( 'updated', $message );
		}
		if ( isset( $_GET['paged'] ) ) {
			$paged = sanitize_text_field( wp_unslash( $_GET['paged'] ) );
		}
			$offset = ( $paged - 1 ) * $recordperpage;
		if ( isset( $_GET['display'] ) ) {
			$display = sanitize_text_field( wp_unslash( $_GET['display'] ) );
		}
		if ( isset( $_GET['txtsearch'] ) && '' != $_GET['txtsearch'] && 'users' == $display ) {
			$search = isset( $_GET['txtsearch'] ) ? sanitize_text_field( wp_unslash( $_GET['txtsearch'] ) ) : '';
			$where .= " and user_login like '%$search%' or user_email like '%$search%' or display_name like '%$search%'";
		}
			$ualp_from_email                   = get_option( 'ualpFromEmail' );
			$ualp_to_failed_login_email        = get_option( 'ualpToFaildLoginEmail' );
			$ualp_to_fail_login_not_user_email = get_option( 'ualpFailLoginNotUserEmail' );
			$ualp_fail_login_number            = get_option( 'ualpFailLoginNumber' );
			$ualp_fail_login_not_user_number   = get_option( 'ualpFailLoginNotUserNumber' );
			$ualp404_page_access_number        = get_option( 'ualp404PageAccessNumber' );
			$ualp404_login_email               = get_option( 'ualp404LoginEmail' );
			$ualp_email_msg                    = get_option( 'ualpEmailMessage' );
			$ualp_user_email_notify            = get_option( 'ualpEmailNotification' );

			$mail_message  = esc_html__( 'Hi', 'user_activity_log_pro' ) . ' ';
			$mail_message .= $current_user->display_name . ',';
			$mail_message .= "\n\n" . __( 'Following user has done some activity in your site', 'user_activity_log_pro' ) . " \n$user_details";
			$mail_message .= "\n\n" . __( 'Thanks,', 'user_activity_log_pro' ) . " \n";
			$mail_message .= home_url();

		if ( ! isset( $ualp_user_email_notify ) || ! is_array( $ualp_user_email_notify ) || empty( $ualp_user_email_notify ) ) {
			$ualp_user_email_notify = array();
		}
			$admin_email                    = get_option( 'admin_email' );
			$from_email                     = ( isset( $ualp_from_email ) && '' != $ualp_from_email ) ? $ualp_from_email : $admin_email;
			$ualp_faild_login_email         = ( isset( $ualp_to_failed_login_email ) && '' != $ualp_to_failed_login_email ) ? $ualp_to_failed_login_email : '';
			$ualp_fail_login_not_user_email = ( isset( $ualp_to_fail_login_not_user_email ) && '' != $ualp_to_fail_login_not_user_email ) ? $ualp_to_fail_login_not_user_email : '';

			$ualp_fail_login_number          = ( isset( $ualp_fail_login_number ) && '' != $ualp_fail_login_number ) ? $ualp_fail_login_number : '';
			$ualp_fail_login_not_user_number = ( isset( $ualp_fail_login_not_user_number ) && $ualp_fail_login_not_user_number ) ? $ualp_fail_login_not_user_number : '';
			$ualp404_page_access_number      = ( isset( $ualp404_page_access_number ) && '' != $ualp404_page_access_number ) ? $ualp404_page_access_number : '';
			$ualp404_login_email             = ( isset( $ualp404_login_email ) && '' != $ualp404_login_email ) ? $ualp404_login_email : '';
			$mail_msg                        = ( isset( $ualp_email_msg ) && '' != $ualp_email_msg ) ? $ualp_email_msg : $mail_message;
		if ( is_network_admin() && ualp_multi_site_support_return() ) {
			$adminurl = network_admin_url( 'admin.php' );
		} else {
			$adminurl = admin_url( 'admin.php' );
		}
		?>
			<div class="wrap">
				<?php
				if ( isset( $_SESSION['success_msg'] ) ) {
					?>
					<div class="success_msg">
					<?php
						ualp_admin_notice_message( 'updated', $_SESSION['success_msg'] );
						unset( $_SESSION['success_msg'] );
					?>
					</div>
					<?php
				}
				$qry_str  = isset( $_SERVER['QUERY_STRING'] ) ? sanitize_text_field( wp_unslash( $_SERVER['QUERY_STRING'] ) ) : '';
				$req_page = isset( $_REQUEST['page'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['page'] ) ) : '';
				?>
			<form method="POST" class="ualpEmailSettingsForm" action="<?php echo esc_attr( $adminurl ) . '?' . esc_attr( $qry_str ); ?>">
			<input type="hidden" name="page" value="<?php echo esc_attr( $req_page ); ?>" />
				<div class="padding_left_right_15">
					<div class="emailNotificationCover">
						<h3 class="ualpHeaderTitle"><?php esc_html_e( 'Email Settings', 'user_activity_log_pro' ); ?></h3>
						<table class="widefat" cellspacing="0">
							<tr class="fromEmailTr">
								<th><?php esc_html_e( 'From Email', 'user_activity_log_pro' ); ?></th>
								<td>
									<input type="email" name="txtualpFromEmail" placeholder="example@gmail.com" value="<?php echo esc_attr( $from_email ); ?>">
									<p class="description"><?php esc_html_e( 'The source Email address', 'user_activity_log_pro' ); ?></p>
								</td>
							</tr>
							<tr class="toEmailnotification">
								<th><?php esc_html_e( 'Alert Me', 'user_activity_log_pro' ); ?></th>
								<td class="suspicious_tr">
								<p>
									<input type="checkbox" id="ualpFailLogin" <?php checked( in_array( 'ualpFailLogin', $ualp_user_email_notify ), 1 ); ?> value="ualpFailLogin" name="ualpUserEmailNotify[]" title="User Fail Login for a WordPress User" class="ualpFailLogin">
									<label for="ualpFailLogin"><?php esc_html_e( 'Alert me on email', 'user_activity_log_pro' ); ?></label>
									<input type="name" name="ualpToFaildLoginEmail" placeholder="example@gmail.com" value="<?php echo esc_attr( $ualp_faild_login_email ); ?>">
									<label for="ualpFailLogin"><?php esc_html_e( 'when WordPress user failed logins more than', 'user_activity_log_pro' ); ?></label> 
									<input type="number" value="<?php echo esc_attr( $ualp_fail_login_number ); ?>" name="ualpFailLoginNumber" placeholder="10" min="1">
									<label for="ualpFailLogin"><?php esc_html_e( 'times.', 'user_activity_log_pro' ); ?></label>
								</p>
								<p>
									<input type="checkbox" id="ualpFailLoginNotUser" <?php checked( in_array( 'ualpFailLoginNotUser', $ualp_user_email_notify ), 1 ); ?> value="ualpFailLoginNotUser" name="ualpUserEmailNotify[]" title="User Fail user Login of Not Existing user" class="ualpFailLoginNotUser">
									<label for="ualpFailLoginNotUser"><?php esc_html_e( 'Alert me on email', 'user_activity_log_pro' ); ?></label>
									<input type="name" name="ualpFailLoginNotUserEmail" placeholder="example@gmail.com" value="<?php echo esc_attr( $ualp_fail_login_not_user_email ); ?>">
									<label for="ualpFailLoginNotUser"><?php esc_html_e( 'when non existing users failed logins more than', 'user_activity_log_pro' ); ?></label>
									<input type="number" value="<?php echo esc_attr( $ualp_fail_login_not_user_number ); ?>" name="ualpFailLoginNotUserNumber" placeholder="10" min="1">
									<label for="ualpFailLoginNotUser"><?php esc_html_e( 'times.', 'user_activity_log_pro' ); ?></label>
								</p>
								<p>
									<input type="checkbox" id="ualpUser404PageAccess" <?php checked( in_array( 'ualpUser404PageAccess', $ualp_user_email_notify ), 1 ); ?> value="ualpUser404PageAccess" name="ualpUserEmailNotify[]" title="User Generates 404 Page" class="ualpUser404PageAccess">
									<label for="ualpUser404PageAccess"><?php esc_html_e( 'Alert me on email', 'user_activity_log_pro' ); ?></label>
									<input type="name" name="ualp404LoginEmail" placeholder="example@gmail.com" value="<?php echo esc_attr( $ualp404_login_email ); ?>">
									<label class="ualpUser404PageAccess"><?php esc_html_e( 'when a user generates 404 HTTP errors more than', 'user_activity_log_pro' ); ?></label>
									<input type="number" value="<?php echo esc_attr( $ualp404_page_access_number ); ?>" name="ualp404PageAccessNumber" placeholder="10" min="1"> 
									<label class="ualpUser404PageAccess"><?php esc_html_e( 'times.', 'user_activity_log_pro' ); ?></label>
								</p>
									<p class="description"><?php esc_html_e( 'The Email address notifications will be sent to. For multiple email address separate them with a comma.', 'user_activity_log_pro' ); ?>&nbsp;<small><?php echo 'Ex. example@gmail.com,test123@gmail.com'; ?></small></p>
								</td>
							</tr>
							<tr class="messageTr">
								<th><?php esc_html_e( 'Message', 'user_activity_log_pro' ); ?></th>
								<td>
									<textarea name="txtualpEmailMsg" rows="10" cols="100"><?php echo esc_html( $mail_msg ); ?></textarea>
									<p class="description"><?php esc_html_e( 'Customize the message as per your requirement', 'user_activity_log_pro' ); ?></p>
								</td>
							</tr>
						</table>
					</div>
					<?php
					wp_nonce_field( '_wp_email_notify_action', '_wp_email_notify_nonce' );
					?>
					<p class="submit">
						<input id="submit2" class="button button-primary" type="submit" value="<?php esc_attr_e( 'Save Changes', 'user_activity_log_pro' ); ?>" name="saveLogin">
						<input class="button button-secondary" type="submit" value="<?php esc_attr_e( 'Reset Changes', 'user_activity_log_pro' ); ?>" name="btnResetEmailData" onclick="return confirm('<?php esc_js( esc_html_e( 'Are you sure to reset data?', 'user_activity_log_pro' ) ); ?>')">
					</p>
				</div>
			</form>
			</div>
		<?php
	}
}

if ( ! function_exists( 'ualp_trigger_notification_settings' ) ) {
	/**
	 * Trigger Notification Settings.
	 */
	function ualp_trigger_notification_settings() {

		?>
		<div class="all_email_notification_wrapp">
			<div class="all_trigger_notification_title">
				<h3 class="ualpHeaderTitle"><?php esc_html_e( 'List Of Trigger Notification', 'user_activity_log_pro' ); ?></h3>
				<div class="add_new_trigger_wrapp">
				<?php
				if ( is_network_admin() && ualp_multi_site_support_return() ) {
					$adminurl = network_admin_url( 'admin.php?page=ualp_add_email_notification' );
				} else {
					$adminurl = admin_url( 'admin.php?page=ualp_add_email_notification' );
				}
				?>
				<a class="add_new_trigger" href="<?php echo esc_url( $adminurl ); ?>"><?php esc_attr_e( 'ADD TRIGGER', 'user_activity_log_pro' ); ?></a>
				</div>
			</div>
			<?php
				$table = new Ualp_Email_Notification_List_Table();
				$table->prepare_items();
				$message   = '';
				$args_kses = ualp_args_kses();
			if ( isset( $_REQUEST['wp_email_trigger_nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_REQUEST['wp_email_trigger_nonce'] ) ), 'wp_email_trigger_action' ) ) {
				if ( isset( $_REQUEST['action'] ) == 'delete' || isset( $_REQUEST['bulk_action'] ) == 'Apply' ) {
					$message = '<div class="updated below-h2" id="message"><p>' . __( 'Trigger Notification deleted successfully', 'user_activity_log_pro' ) . '</p></div>';
					echo wp_kses( $message, $args_kses );
				}
			}
				$host        = isset( $_SERVER['HTTP_HOST'] ) ? esc_url_raw( wp_unslash( $_SERVER['HTTP_HOST'] ) ) : '';
				$request_url = isset( $_SERVER['REQUEST_URI'] ) ? esc_url_raw( wp_unslash( $_SERVER['REQUEST_URI'] ) ) : '';
				$current_url = ( isset( $_SERVER['HTTPS'] ) ? 'https://' : 'http://' ) . $host . $request_url;
				$req_page    = isset( $_REQUEST['page'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['page'] ) ) : '';
			?>
			<form class="ualploginSettingsForm" method="get">
			<input type="hidden" name="page" value="<?php echo esc_attr( $req_page ); ?>" />
			<div class="padding_right_15">
				<?php $table->display(); ?>
				<p class="margin_bottom_30"></p>
			</div>
			<?php wp_nonce_field( 'wp_email_trigger_action', 'wp_email_trigger_nonce' ); ?>
		</form>
		</div>
		<?php
	}
}

if ( ! function_exists( 'ualp_role_setting' ) ) {
	/**
	 * User activity Role Manager settings.
	 */
	function ualp_role_setting() {
		global $wpdb ,$wp_roles;
		$users_list = get_users();
		if ( isset( $_POST['btnRoleManager'] ) && isset( $_POST['_wp_role_manage_nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wp_role_manage_nonce'] ) ), '_wp_role_manage_action' ) ) {
			if ( isset( $_POST['rolesManage'] ) && '' != $_POST['rolesManage'] ) {
				update_option( 'ualpRoleAccessList', array_map( 'sanitize_text_field', wp_unslash( $_POST['rolesManage'] ) ) );
			} else {
				update_option( 'ualpRoleAccessList', array( 0 => 'administrator' ) );
			}
			if ( isset( $_POST['usersIDManage'] ) && '' != $_POST['usersIDManage'] ) {
				update_option( 'ualpUserAccessList', array_map( 'sanitize_text_field', wp_unslash( $_POST['usersIDManage'] ) ) );
			}
			do_action( 'ualp_manage_role' );
			$message = esc_html__( 'User Activity Log Pro panel access saved successfully.', 'user_activity_log_pro' );
			ualp_admin_notice_message( 'updated', $message );
		}
		if ( isset( $_POST['btnResetRoleData'] ) ) {
			update_option(
				'ualpRoleAccessList',
				array(
					0 => 'administrator',
					1 => 'editor',
					2 => 'author',
					3 => 'contributor',
					4 => 'subscriber',
				)
			);
			$message = esc_html__( 'User Activity Log Pro panel access reset successfully.', 'user_activity_log_pro' );
			ualp_admin_notice_message( 'updated', $message );
		}
		$ualp_role_access_list = get_option( 'ualpRoleAccessList' );
		$ualp_user_access_list = get_option( 'ualpUserAccessList' );
		if ( is_network_admin() && ualp_multi_site_support_return() ) {
			$adminurl = network_admin_url( 'admin.php' );
		} else {
			$adminurl = admin_url( 'admin.php' );
		}
		$super_admins = get_super_admins();
		$qry_str      = isset( $_SERVER['QUERY_STRING'] ) ? sanitize_text_field( wp_unslash( $_SERVER['QUERY_STRING'] ) ) : '';
		?>
		<form class="ualpSettingsForm" action="<?php echo esc_attr( $adminurl ) . '?' . esc_attr( $qry_str ); ?>" method="POST">
			<div class="padding_left_right_15">
				<h3 class="ualpHeaderTitle"><?php esc_html_e( 'Select Role', 'user_activity_log_pro' ); ?></h3>
				<p class="margin_bottom_30"><?php esc_html_e( 'Who can access the User Activity Log Pro panel? There is one "Role Manager" tab option available under settings page and you can assign that stuff to Contributor, Editor or one of your custom role.', 'user_activity_log_pro' ); ?></p>
				<p class="margin_bottom_30">
					<label><?php esc_html_e( 'Select display types', 'user_activity_log_pro' ); ?></label>
					<select class="ualpRoleSelectDisplayCat" name="ualpRoleSelectDisplayCat" onchange="ualpRoleDisplayUsersBy();">
						<option value="roles"><?php esc_html_e( 'Roles', 'user_activity_log_pro' ); ?></option>
						<option value="users"><?php esc_html_e( 'Users', 'user_activity_log_pro' ); ?></option>
					</select>
				</p>
				<table class="widefat striped rolemanagerrolesList" cellspacing="0">
					<thead>
						<tr>
							<th class="column-cb check-column"><input type="checkbox" id="cb-select-all-roles"/></th>
							<td scope="col"><?php esc_html_e( 'Role', 'user_activity_log_pro' ); ?></td>
						</tr>
					</thead>
					<tfoot>
						<tr>
							<th class="column-cb check-column"><input type="checkbox" id="cb-select-all-roles1"/></th>
							<td scope="col"><?php esc_html_e( 'Role', 'user_activity_log_pro' ); ?></td>
						</tr>
					</tfoot>
					<tbody>
					<?php
						$editable_roles = get_editable_roles();
					foreach ( $editable_roles as $role => $details ) {
						$name       = translate_user_role( $details['name'] );
						$role_label = esc_attr( $role );
						?>
							<tr>
								<th class="
								<?php
								if ( 'administrator' != $role_label ) {
									echo 'check-column';
								} else {
									echo 'margin_0 ualp-not-allowed';}
								?>
								">
									<input id='<?php echo esc_attr( $role_label ) . 'role'; ?>' type="checkbox" name="rolesManage[]" value="<?php echo esc_attr( $role_label ); ?>" 
										<?php
										if ( 'administrator' == $role_label ) {
											echo "disabled=''";
											echo " checked=''";}
										?>
										<?php
										if ( in_array( $role_label, $ualp_role_access_list ) ) {
											$checked = 'checked=checked';
											echo esc_attr( $checked );
										}
										?>
>
								</th>
								<td>
									<label for='<?php echo esc_attr( $role_label ) . 'role'; ?>'><?php echo esc_html( ucfirst( $name ) ); ?></label>
								</td>
							</tr>
							<?php
					}
					?>
					</tbody>
				</table>
				<table class="widefat striped rolemanageruserList" style="display: none" cellspacing="0">
					<thead>
						<tr>
							<th scope="col" class="column-cb check-column"><input type="checkbox" id="cb-select-all-user2"/></th>
							<td scope="col"><?php esc_html_e( 'Username', 'user_activity_log_pro' ); ?></td>
							<td scope="col"><?php esc_html_e( 'Name', 'user_activity_log_pro' ); ?></td>
							<td scope="col"><?php esc_html_e( 'Email address', 'user_activity_log_pro' ); ?></td>
							<td scope="col"><?php esc_html_e( 'Roles', 'user_activity_log_pro' ); ?></td>
						</tr>
					</thead>
					<tfoot>
						<tr>
							<th scope="col" class="column-cb check-column"><input type="checkbox" id="cb-select-all-user3"/></th>
							<td scope="col"><?php esc_html_e( 'Username', 'user_activity_log_pro' ); ?></td>
							<td scope="col"><?php esc_html_e( 'Name', 'user_activity_log_pro' ); ?></td>
							<td scope="col"><?php esc_html_e( 'Email address', 'user_activity_log_pro' ); ?></td>
							<td scope="col"><?php esc_html_e( 'Roles', 'user_activity_log_pro' ); ?></td>
						</tr>
					</tfoot>
					<tbody>
						<?php
						foreach ( $users_list as $data ) {
							$role_name  = '';
							$u_d        = get_userdata( $data->ID );
							$first_name = ( isset( $u_d->user_firstname ) && ! empty( $u_d->user_firstname ) ) ? $u_d->user_firstname : ' - ';
							$last_name  = isset( $u_d->user_lastname ) ? $u_d->user_lastname : '-';
							$user       = get_user_by( 'id', $data->ID );
							if ( is_array( $data->roles ) ) {
								$data_roles_count = count( $data->roles );
								for ( $i = 0;$i < $data_roles_count;$i++ ) {
									$user_role = $data->roles[ $i ];
									if ( '' == $role_name ) {
										$role_name = $wp_roles->role_names[ $user_role ];
									} else {
										$role_name = $role_name . ', ' . $wp_roles->role_names[ $user_role ];
									}
								}
								$user_roles = $role_name;
							}
							?>
							<tr>
								<th class="
								<?php
								if ( 'Administrator' == $user_roles ) {
									echo 'margin_0 ualp-not-allowed';
								} elseif ( in_array( $user->user_login, $super_admins ) ) {
									echo 'margin_0 ualp-not-allowed'; } else {
									echo 'column-cb check-column';
									}
									?>
								" scope="row">
									<input type="checkbox" name="usersIDManage[]" value="<?php echo esc_attr( $data->ID ); ?>" 
										<?php
										if ( '' != $ualp_user_access_list && in_array( $data->ID, $ualp_user_access_list ) ) {
											$checked = 'checked=checked';
											echo esc_attr( $checked );
										}
										if ( 'Administrator' == $user_roles ) {
											echo "disabled=''";
											echo " checked=''";
										} elseif ( in_array( $user->user_login, $super_admins ) ) {
											echo "disabled=''";
											echo " checked=''"; }
										?>
									/>
								</th>
								<td class="author column-author">
									<?php
									if ( ! empty( $data->ID ) && 0 !== (int) $data->ID ) {
										if ( $user instanceof WP_User && 0 !== $user->ID ) {
											$args_kses = ualp_args_kses();
											?>
											<a href="<?php echo esc_url( get_edit_user_link( $user->ID ) ); ?>">
												<?php echo wp_kses( get_avatar( $user->ID, 40 ), $args_kses ); ?>
												<span><?php echo esc_html( ucfirst( $user->user_login ) ); ?></span>
											</a>
											<?php
										}
									}
									?>
								</td>
								<td><?php echo esc_html( ucfirst( $first_name ) ) . ' ' . esc_html( ucfirst( $last_name ) ); ?>
								</td>
								<td><?php echo esc_html( $data->user_email ); ?></td>
								<td><?php echo esc_html( $user_roles ); ?></td>
							</tr>
						<?php } ?>
					</tbody>
				</table>

				<?php
				wp_nonce_field( '_wp_role_manage_action', '_wp_role_manage_nonce' );
				?>
				<p class="submit">
					<input id="roleManager" class="button button-primary" type="submit" name="btnRoleManager" value="<?php esc_attr_e( 'Save Changes', 'user_activity_log_pro' ); ?>">
					<input class="button button-secondary" type="submit" name="btnResetRoleData" value="<?php esc_html_e( 'Reset Changes', 'user_activity_log_pro' ); ?>" onclick="return confirm('<?php esc_js( esc_html_e( 'Are you sure to reset data?', 'user_activity_log_pro' ) ); ?>')">
				</p>
			</div>
		</form>
		<?php
	}
}

if ( ! function_exists( 'ualp_pwd_settings' ) ) {
	/**
	 * User activity Set password for delete log.
	 */
	function ualp_pwd_settings() {
		$admin_pass          = '';
		$password            = '';
		$repeat_password     = '';
		$old_password        = '';
		$new_password        = '';
		$repeat_new_password = '';
		$old_password_md5    = '';
		$password_md5        = '';
		$new_password_md5    = '';
		$matchadminpwd       = '';
		$matchadminnewpwd    = '';
		$current_user        = wp_get_current_user();
		$admin_pass          = $current_user->user_pass;
		$current_password    = get_option( 'ualp_delete_log_pwd' );
		if ( isset( $_POST['btnSetPassword'] ) && isset( $_POST['_wp_set_password_nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wp_set_password_nonce'] ) ), '_wp_set_password_action' ) ) {
			if ( isset( $_POST['password'] ) && '' != $_POST['password'] ) {
				$password      = sanitize_text_field( wp_unslash( $_POST['password'] ) );
				$password_md5  = md5( $password );
				$matchadminpwd = wp_check_password( $password, $admin_pass );
			}
			if ( isset( $_POST['repeatPassword'] ) && '' != $_POST['repeatPassword'] ) {
				$repeat_password = sanitize_text_field( wp_unslash( $_POST['repeatPassword'] ) );
			}
			if ( isset( $_POST['oldPassword'] ) && '' != $_POST['oldPassword'] ) {
				$old_password     = sanitize_text_field( wp_unslash( $_POST['oldPassword'] ) );
				$old_password_md5 = md5( $old_password );
			}
			if ( isset( $_POST['newPassword'] ) && '' != $_POST['newPassword'] ) {
				$new_password     = sanitize_text_field( wp_unslash( $_POST['newPassword'] ) );
				$new_password_md5 = md5( $new_password );
				$matchadminnewpwd = wp_check_password( $new_password, $admin_pass );
			}
			if ( isset( $_POST['repeatNewPassword'] ) && '' != $_POST['repeatNewPassword'] ) {
				$repeat_new_password = sanitize_text_field( wp_unslash( $_POST['repeatNewPassword'] ) );
			}
			if ( isset( $current_password ) && '' != $current_password ) {
				if ( '' == $old_password ) {
					$message = esc_html__( 'Please Enter Old password', 'user_activity_log_pro' );
					ualp_admin_notice_message( 'error', $message );
				} elseif ( '' == $new_password ) {
					$message = esc_html__( 'Please Enter New password', 'user_activity_log_pro' );
					ualp_admin_notice_message( 'error', $message );
				} elseif ( '' == $repeat_new_password ) {
					$message = esc_html__( 'Please Enter Repeat password', 'user_activity_log_pro' );
					ualp_admin_notice_message( 'error', $message );
				} elseif ( $old_password_md5 != $current_password ) {
					$message = esc_html__( 'Old password not match', 'user_activity_log_pro' );
					ualp_admin_notice_message( 'error', $message );
				} elseif ( '' != $new_password && '' != $repeat_new_password && ( $new_password != $repeat_new_password ) ) {
					$message = esc_html__( 'Password and repeat password not same', 'user_activity_log_pro' );
					ualp_admin_notice_message( 'error', $message );
				} elseif ( '' != $new_password && $new_password_md5 == $current_password ) {
					$message = esc_html__( 'This Password is not valid. You current password and new password is same', 'user_activity_log_pro' );
					ualp_admin_notice_message( 'error', $message );
				} elseif ( 1 == $matchadminnewpwd ) {
					$message = esc_html__( 'This password is not acceptable. This is same as admin password', 'user_activity_log_pro' );
					ualp_admin_notice_message( 'error', $message );
				} elseif ( '' != $new_password && ( $repeat_new_password == $new_password ) && ( $new_password != $current_password ) && ( $old_password_md5 == $current_password ) && 1 != $matchadminnewpwd ) {
					update_option( 'ualp_delete_log_pwd', $new_password_md5 );
					$message = esc_html__( 'Password updated successfully', 'user_activity_log_pro' );
					ualp_admin_notice_message( 'updated', $message );
					do_action( 'ualp_change_password' );
				}
			} else {
				if ( '' == $password ) {
					$message = esc_html__( 'Please Enter password', 'user_activity_log_pro' );
					ualp_admin_notice_message( 'error', $message );
				} elseif ( '' == $repeat_password ) {
					$message = esc_html__( 'Please Enter Repeat password', 'user_activity_log_pro' );
					ualp_admin_notice_message( 'error', $message );
				} elseif ( '' != $password && '' != $repeat_password && ( $password != $repeat_password ) ) {
					$message = esc_html__( 'Password and repeat password not same', 'user_activity_log_pro' );
					ualp_admin_notice_message( 'error', $message );
				} elseif ( 1 == $matchadminpwd ) {
					$message = esc_html__( 'This password is not acceptable. This is same as admin password', 'user_activity_log_pro' );
					ualp_admin_notice_message( 'error', $message );
				} elseif ( '' != $password && $password == $repeat_password && 1 != $matchadminpwd ) {
					update_option( 'ualp_delete_log_pwd', $password_md5 );
					$message = esc_html__( 'Password saved successfully', 'user_activity_log_pro' );
					ualp_admin_notice_message( 'updated', $message );
					do_action( 'ualp_set_password' );
				}
				if ( is_network_admin() && ualp_multi_site_support_return() ) {
					$new_url = network_admin_url( 'admin.php?page=ualp_settings' );
				} else {
					$new_url = admin_url( 'admin.php?page=ualp_settings' );
				}
				wp_safe_redirect( $new_url );
			}
			$current_password = get_option( 'ualp_delete_log_pwd' );
		}
		if ( isset( $_POST['btnResetPassword'] ) && isset( $_POST['_wp_set_password_nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wp_set_password_nonce'] ) ), '_wp_set_password_action' ) ) {
			global $user;
			$user                    = wp_get_current_user();
			$to                      = $user->user_email;
			$old_delete_log_password = get_option( 'ualp_delete_log_pwd' );
			$from                    = get_option( 'ualpFromEmail' );
			$random_passwd           = wp_generate_password();
			$random_hash_passwd      = md5( $random_passwd );

			$headers  = "MIME-Version: 1.0;\r\n";
			$headers .= 'From: ' . wp_strip_all_tags( $from ) . "\r\n";
			$headers .= "Content-Type: text/html; charset: utf-8;\r\n";
			$headers .= "X-Priority: 3\r\n";
			$headers .= 'X-Mailer: PHP' . phpversion() . "\r\n";
			$subject  = esc_html__( 'Reset Password', 'user_activity_log_pro' ) . ' - ' . __( 'User Activity Log Pro', 'user_activity_log_pro' );
			$body     = '';
			ob_start();
			?>
			<div style="background: #F5F5F5; border-width: 1px; border-style: solid; padding-bottom: 20px; margin: 0px auto; width: 750px; height: auto; border-radius: 3px 3px 3px 3px; border-color: #5C5C5C;">
				<div style="border: #FFF 1px solid; background-color: #ffffff !important; margin: 20px 20px 0;height: auto; -moz-border-radius: 3px; padding-top: 15px;">
					<div style="padding: 20px 20px 20px 20px; font-family: Arial, Helvetica, sans-serif;height: auto; color: #333333; font-size: 13px;">
						<div style="width: 100%;">
							<strong>
							<?php
							esc_html_e( 'Dear', 'user_activity_log_pro' );
							echo ' ' . esc_html( $user->display_name );
							?>
							</strong>,<br /><br />
							<?php esc_html_e( 'Here is your New password', 'user_activity_log_pro' ) . ' :- '; ?>  <strong><?php echo esc_html( $random_passwd ); ?></strong><br /><br />
							<?php esc_html_e( 'Thank you', 'user_activity_log_pro' ); ?> ,<br /><br />
						</div>
					</div>
				</div>
			</div>
			<?php
			$body = ob_get_clean();
			$mail = wp_mail( $to, $subject, $body, $headers );
			if ( $mail ) {
				update_option( 'ualp_delete_log_pwd', $random_hash_passwd );
				$message = esc_html__( 'Password Reset Successfully', 'user_activity_log_pro' );
				ualp_admin_notice_message( 'updated', $message );
			} else {
				$message = esc_html__( 'Error in Password Reset', 'user_activity_log_pro' );
				ualp_admin_notice_message( 'error', $message );
			}
		}
		if ( isset( $current_password ) && '' != $current_password ) {
			$subtitle = esc_html__( 'Change Password', 'user_activity_log_pro' );
		} else {
			$subtitle = esc_html__( 'Add Password', 'user_activity_log_pro' );
		}
		if ( is_network_admin() && ualp_multi_site_support_return() ) {
			$adminurl = network_admin_url( 'admin.php' );
		} else {
			$adminurl = admin_url( 'admin.php' );
		}
		$qry_str = isset( $_SERVER['QUERY_STRING'] ) ? sanitize_text_field( wp_unslash( $_SERVER['QUERY_STRING'] ) ) : '';
		?>
		<form class="ualpSettingsForm" action="<?php echo esc_attr( $adminurl ) . '?' . esc_attr( $qry_str ); ?>" method="POST" >
			<div class="padding_left_right_15">
				<h3 class="ualpHeaderTitle"><?php echo esc_html( $subtitle ); ?></h3>
				<table class="widefat" cellspacing="0">
					<?php if ( isset( $current_password ) && '' != $current_password ) { ?>
						<tr>
							<th><?php esc_html_e( 'Old Password', 'user_activity_log_pro' ); ?></th>
							<td>
								<input type="Password" name="oldPassword" value="<?php echo esc_attr( $old_password ); ?>">
								<p class="description"><?php esc_html_e( 'Enter Old Password', 'user_activity_log_pro' ); ?></p>
							</td>
						</tr>
						<tr>
							<th><?php esc_html_e( 'New Password', 'user_activity_log_pro' ); ?></th>
							<td>
								<input type="Password" name="newPassword" value="<?php echo esc_attr( $new_password ); ?>">
								<p class="description"><?php esc_html_e( 'Enter New Password', 'user_activity_log_pro' ); ?></p>
								<div aria-live="polite" id="change-pass-strength-result" style="" class=""></div>
							</td>
						</tr>
						<tr>
							<th><?php esc_html_e( 'Repeat New Password', 'user_activity_log_pro' ); ?></th>
							<td>
								<input type="Password" name="repeatNewPassword" value="<?php echo esc_attr( $repeat_new_password ); ?>">
								<p class="description"><?php esc_html_e( 'Repeat New Password', 'user_activity_log_pro' ); ?></p>
							</td>
						</tr>
						<?php
					} else {
						?>
						<tr>
							<th><?php esc_html_e( 'Password', 'user_activity_log_pro' ); ?></th>
							<td>
								<input type="Password" name="password" value="<?php echo esc_attr( $password ); ?>">
								<p class="description"><?php esc_html_e( 'Enter Password for delete logs', 'user_activity_log_pro' ); ?></p>
								<div aria-live="polite" id="pass-strength-result" style="" class=""></div>
							</td>
						</tr>
						<tr>
							<th><?php esc_html_e( 'Repeat Password', 'user_activity_log_pro' ); ?></th>
							<td>
								<input type="Password" name="repeatPassword" value="<?php echo esc_attr( $repeat_password ); ?>">
								<p class="description"><?php esc_html_e( 'Repeat Password', 'user_activity_log_pro' ); ?></p>
							</td>
						</tr>
						<?php
					}
					?>
				</table>
				<input type="hidden" id="currentpwd" name="currentpwd" value="<?php echo esc_attr( $current_password ); ?>">
				<input type="hidden" id="adminpwd" name="adminpwd" value="<?php echo esc_attr( $admin_pass ); ?>">
				<span class="report-note"><b><?php esc_html_e( 'Note', 'user_activity_log_pro' ); ?>&nbsp;:&nbsp;</b><?php esc_html_e( 'Password should be 20 characters long, with one special character and number.', 'user_activity_log_pro' ); ?></span>
				<?php
				wp_nonce_field( '_wp_set_password_action', '_wp_set_password_nonce' );
				?>
				<p class="submit">
					<input class="button button-primary" type="submit" value="<?php esc_attr_e( 'Save Changes', 'user_activity_log_pro' ); ?>" name="btnSetPassword">
					<?php if ( current_user_can( 'administrator' ) && isset( $current_password ) && '' != $current_password ) { ?>
						<input class="button button-default reset_delete_log_passwd" type="submit" value="<?php esc_attr_e( 'Reset Password', 'user_activity_log_pro' ); ?>" name="btnResetPassword">
					<?php } ?>
				</p>
			</div>
		</form>
		<?php
	}
}

if ( ! function_exists( 'ualp_custom_event_settings' ) ) {
	/**
	 * User activity custom event settings.
	 */
	function ualp_custom_event_settings() {
		if ( isset( $_POST['btnAddCustomEvent'] ) && isset( $_POST['_wp_custom_event_nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wp_custom_event_nonce'] ) ), '_wp_custom_event_action' ) ) {
			$custom_event_name = array();
			$custom_event_data = array();
			$i                 = 0;
			$empty_count       = 0;
			if ( ( isset( $_POST['custom_event_name'] ) && '' != $_POST['custom_event_name'] ) ||
				( isset( $_POST['custom_event_label'] ) && '' != $_POST['custom_event_label'] ) ||
				( isset( $_POST['custom_event_type'] ) && '' != $_POST['custom_event_type'] ) ||
				( isset( $_POST['custom_event_description'] ) && '' != $_POST['custom_event_description'] ) ) {
				$custom_event_name        = array_map( 'sanitize_text_field', wp_unslash( $_POST['custom_event_name'] ) );
				$custom_event_label       = array_map( 'sanitize_text_field', wp_unslash( $_POST['custom_event_label'] ) );
				$custom_event_type        = array_map( 'sanitize_text_field', wp_unslash( $_POST['custom_event_type'] ) );
				$custom_event_description = array_map( 'sanitize_text_field', wp_unslash( $_POST['custom_event_description'] ) );
			}
			$custom_event_name_count = count( (array) $custom_event_name );
			if ( is_array( $custom_event_name ) && ! empty( $custom_event_name ) ) {
				if ( $custom_event_name_count > 0 ) {
					foreach ( $custom_event_name as $custom_event_name_single ) {
						if ( '' != $custom_event_name_single ) {
							$custom_event_data[] = array(
								'custom_event_name'        => $custom_event_name_single,
								'custom_event_label'       => $custom_event_label[ $i ],
								'custom_event_type'        => $custom_event_type[ $i ],
								'custom_event_description' => $custom_event_description[ $i ],
							);
							$i++;
						}
					}
				}
			}
			if ( is_array( $custom_event_data ) && ! empty( $custom_event_data ) ) {
				$custom_event_data_count = count( $custom_event_data );
				if ( $custom_event_data_count >= 1 ) {
					foreach ( $custom_event_data as $custom_event_single_data ) {
						$event_name  = $custom_event_single_data['custom_event_name'];
						$event_label = $custom_event_single_data['custom_event_label'];
						$event_type  = $custom_event_single_data['custom_event_type'];
						$event_desc  = $custom_event_single_data['custom_event_description'];
						if ( '' == $event_name || '' == $event_label || '' == $event_type || '' == $event_desc ) {
							$empty_count++;
						}
					}
				}
			}

			if ( 0 == $empty_count && ! empty( $custom_event_data ) ) {
				update_option( 'custom_event_data', $custom_event_data );
				$class   = 'updated';
				$message = esc_html__( 'Custom event saved successfully.', 'user_activity_log_pro' );
				do_action( 'ualp_custom_event_log_updated' );
			} elseif ( 0 == $empty_count && is_array( $custom_event_name ) && ! empty( $custom_event_data ) && $custom_event_name_count > 0 ) {
				update_option( 'custom_event_data', $custom_event_data );
				$class   = 'updated';
				$message = esc_html__( 'Custom event deleted successfully.', 'user_activity_log_pro' );
				do_action( 'ualp_custom_event_log_updated' );
			} else {
				update_option( 'custom_event_data', $custom_event_data );
				$class   = 'error';
				$message = esc_html__( 'Custom event not saved. Fill the empty field.', 'user_activity_log_pro' );
			}
			ualp_admin_notice_message( $class, $message );
		}
		$custom_event_data = get_option( 'custom_event_data' );
		?>
		<script>
			/*add new and remove custom event functionality*/
			jQuery(document).ready(function () {
				jQuery('.ualp_add_new_event_btn').click(function () {
                    var new_row = '<tr>';
                    new_row += '<td><span><?php esc_html_e('Event Name', 'user_activity_log_pro'); echo ' ('; esc_html_e( 'Hook Name', 'user_activity_log_pro'); echo ' ) : '; ?></span><br/><input class="ualp_event_medium" type="text" name="custom_event_name[]" placeholder="<?php esc_html_e('Ex', 'user_activity_log_pro');echo ': wp_login'; ?>" /></td>';
                    new_row += '<td><span><?php esc_html_e('Event Label', 'user_activity_log_pro'); echo ' : '; ?></span><br/><input class="ualp_event_medium" type="text" name="custom_event_label[]" placeholder="<?php esc_html_e('Ex', 'user_activity_log_pro'); echo ' : '; esc_html_e('Login', 'user_activity_log_pro'); ?>" /></td>';
                    new_row += '<td><span><?php esc_html_e('Event Type', 'user_activity_log_pro'); echo ' : '; ?></span><br/><input class="ualp_event_medium" type="text" name="custom_event_type[]" placeholder="<?php esc_html_e('Ex', 'user_activity_log_pro'); echo ' : '; esc_html_e('User', 'user_activity_log_pro'); ?>" /></td>';
                    new_row += '<td><span><?php esc_html_e('Event Description', 'user_activity_log_pro'); echo ' : '; ?></span><br/><input class="ualp_event_large" type="text" name="custom_event_description[]" placeholder="<?php esc_html_e('Ex', 'user_activity_log_pro'); echo ' : '; esc_html_e('User Login from custom event', 'user_activity_log_pro'); ?>" /></td>';
                    new_row += '<td class="ulap_last_column"><span title="<?php esc_html_e('Remove', 'user_activity_log_pro'); ?>" class="ualp_remove_event_btn dashicons dashicons-trash"></span></td>';
                    new_row += '</tr>';
                    jQuery('.custom_event_table').append(new_row);
					jQuery('.ualp_remove_event_btn').on('click', function () {
						if( jQuery('.custom_event_table tr').length > 1 ) {
							jQuery(this).closest('tr').remove();
						}
					});
                });
				jQuery('.ualp_remove_event_btn').on('click', function () {
					if( jQuery('.custom_event_table tr').length > 1 ) {
						jQuery(this).closest('tr').remove();
					}
				});
			});
		</script>
		<?php
		if ( is_network_admin() && ualp_multi_site_support_return() ) {
			$adminurl = network_admin_url( 'admin.php' );
		} else {
			$adminurl = admin_url( 'admin.php' );
		}
		$qry_str = isset( $_SERVER['QUERY_STRING'] ) ? sanitize_text_field( wp_unslash( $_SERVER['QUERY_STRING'] ) ) : '';
		?>
		<form class="ualpSettingsForm" action="<?php echo esc_attr( $adminurl ) . '?' . esc_attr( $qry_str ); ?>" method="POST" >
			<div class="padding_left_right_15">
				<h3 class="ualpHeaderTitle"><?php esc_html_e( 'Add Custom Event', 'user_activity_log_pro' ); ?></h3>
				<table class="widefat custom_event_table" cellspacing="0">
					<?php
					$j = 1;
					if ( is_array( $custom_event_data ) && ! empty( $custom_event_data ) ) {
						foreach ( $custom_event_data as $custom_event_data_single ) {
							?>
							<tr>
								<td>
									<span>
									<?php
									esc_html_e( 'Event Name', 'user_activity_log_pro' );
									echo ' ( ';
									esc_html_e( 'Hook Name', 'user_activity_log_pro' );
									echo ' ) : ';
									?>
									</span><br/>
									<input class="ualp_event_medium" type="text" name="custom_event_name[]" placeholder="<?php esc_html_e( 'Ex', 'user_activity_log_pro' ); echo ': wp_login'; ?>" value="<?php echo esc_attr( $custom_event_data_single['custom_event_name'] ); ?>" />
								</td>
								<td>
									<span>
									<?php
									esc_html_e( 'Event Label', 'user_activity_log_pro' );
									echo ' : ';
									?>
									</span><br/>
									<input class="ualp_event_medium" type="text" name="custom_event_label[]" placeholder="<?php esc_html_e( 'Ex', 'user_activity_log_pro' ); echo ': '; esc_html_e( 'Login', 'user_activity_log_pro' ); ?>" value="<?php echo esc_attr( $custom_event_data_single['custom_event_label'] ); ?>" />
								</td>
								<td>
									<span>
									<?php
									esc_html_e( 'Event Type', 'user_activity_log_pro' );
									echo ' : ';
									?>
									</span><br/>
									<input class="ualp_event_medium" type="text" name="custom_event_type[]" placeholder="<?php esc_html_e( 'Ex', 'user_activity_log_pro' ); echo ': '; esc_html_e( 'User', 'user_activity_log_pro' ); ?>" value="<?php echo esc_attr( $custom_event_data_single['custom_event_type'] ); ?>" />
								</td>
								<td>
									<span>
									<?php
									esc_html_e( 'Event Description', 'user_activity_log_pro' );
									echo ' : ';
									?>
									</span><br/>
									<input class="ualp_event_large" type="text" name="custom_event_description[]" placeholder="<?php esc_html_e( 'Ex', 'user_activity_log_pro' ); echo ': '; esc_html_e( 'User Login from custom event', 'user_activity_log_pro' ); ?>" value="<?php echo esc_attr( $custom_event_data_single['custom_event_description'] ); ?>" />
								</td>
								<td class="ulap_last_column">
									<span title="<?php esc_html_e( 'Remove', 'user_activity_log_pro' ); ?>" class="ualp_remove_event_btn dashicons dashicons-trash"></span>
								</td>
							</tr>
							<?php
							$j++;
						}
					} else {
						if ( is_array( $custom_event_data ) && ! empty( $custom_event_data ) ) {
							foreach ( $custom_event_data as $custom_event_data_single ) {
							?>
							<tr>
								<td>
									<span>
									<?php
									esc_html_e( 'Event Name', 'user_activity_log_pro' );
									echo '( ';
									esc_html_e( 'Hook Name', 'user_activity_log_pro' );
									echo ' ) : ';
									?>
									</span><br/>
									<input class="ualp_event_medium" type="text" name="custom_event_name[]" placeholder="<?php esc_html_e( 'Ex', 'user_activity_log_pro' ); echo ': wp_login'; ?>" value="<?php echo esc_attr( $custom_event_data_single['custom_event_name'] ); ?>" />
								</td>
								<td>
									<span>
									<?php
									esc_html_e( 'Event Label', 'user_activity_log_pro' );
									echo ' : ';
									?>
									</span><br/>
									<input class="ualp_event_medium" type="text" name="custom_event_label[]" placeholder="<?php esc_html_e( 'Ex', 'user_activity_log_pro' ); echo ': '; esc_html_e( 'Login', 'user_activity_log_pro' ); ?>" value="<?php echo esc_attr( $custom_event_data_single['custom_event_label'] ); ?>" />
								</td>
								<td>
									<span>
									<?php
									esc_html_e( 'Event Type', 'user_activity_log_pro' );
									echo ' : ';
									?>
									</span><br/>
									<input class="ualp_event_medium" type="text" name="custom_event_type[]" placeholder="<?php esc_html_e( 'Ex', 'user_activity_log_pro' ); echo ': '; esc_html_e( 'User', 'user_activity_log_pro' ); ?>" value="<?php echo esc_attr( $custom_event_data_single['custom_event_type'] ); ?>" />
								</td>
								<td>
									<span>
									<?php
									esc_html_e( 'Event Description', 'user_activity_log_pro' );
									echo ' : ';
									?>
									</span><br/>
									<input class="ualp_event_large" type="text" name="custom_event_description[]" placeholder="<?php esc_html_e( 'Ex', 'user_activity_log_pro' ); echo ': '; esc_html_e( 'User Login from custom event', 'user_activity_log_pro' ); ?>" value="<?php echo esc_attr( $custom_event_data_single['custom_event_description'] ); ?>" />
								</td>
								<td class="ulap_last_column">
									<span title="<?php esc_html_e( 'Remove', 'user_activity_log_pro' ); ?>" class="ualp_remove_event_btn dashicons dashicons-trash"></span>
								</td>
							</tr>
						<?php }
						}
					} ?>
				</table>
				<div title="<?php esc_html_e( 'Add New', 'user_activity_log_pro' ); ?>" class="ualp_add_new_event_btn dashicons dashicons-plus-alt"></div>
				<?php
				wp_nonce_field( '_wp_custom_event_action', '_wp_custom_event_nonce' );
				?>
				<p class="submit">
					<input class="button button-primary" type="submit" value="<?php esc_attr_e( 'Save Changes', 'user_activity_log_pro' ); ?>" name="btnAddCustomEvent">
				</p>
			</div>
		</form>
		<?php
	}
}


if ( ! function_exists( 'ualp_db_settings' ) ) {
	/**
	 * User External DB Settings.
	 */
	function ualp_db_settings() {
		global $wpdb, $externaldb;
		$dbuser      = '';
		$dbpassword  = '';
		$dbhost      = '';
		$dbname      = '';
		$tableprefix = '';
		$dbuser      = get_option( 'database_username' );
		$dbpassword  = get_option( 'database_password' );
		$dbhost      = get_option( 'database_hostname' );
		$dbname      = get_option( 'database_name' );
		$tableprefix = get_option( 'table_prefix' );
		if ( isset( $_POST['btnSubmitDbData'] ) && isset( $_POST['_wp_db_settings_nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wp_db_settings_nonce'] ) ), '_wp_db_settings_nonce' ) ) {

			$dbhost      = isset( $_POST['database_hostname'] ) ? trim( sanitize_text_field( wp_unslash( $_POST['database_hostname'] ) ) ) : '';
			$dbname      = isset( $_POST['database_name'] ) ? trim( sanitize_text_field( wp_unslash( $_POST['database_name'] ) ) ) : '';
			$dbuser      = isset( $_POST['database_username'] ) ? trim( sanitize_text_field( wp_unslash( $_POST['database_username'] ) ) ) : '';
			$dbpassword  = isset( $_POST['database_password'] ) ? trim( sanitize_text_field( wp_unslash( $_POST['database_password'] ) ) ) : '';
			$tableprefix = isset( $_POST['table_prefix'] ) ? trim( sanitize_text_field( wp_unslash( $_POST['table_prefix'] ) ) ) : '';
			$message     = esc_html__( 'External database settings updated successfully.', 'user_activity_log_pro' );

			if ( '' == $tableprefix && '' == $dbname && '' == $dbhost && '' == $dbuser && '' == $dbpassword ) {
				update_option( 'table_prefix', $tableprefix );
				update_option( 'database_name', $dbname );
				update_option( 'database_hostname', $dbhost );
				update_option( 'database_username', $dbuser );
				update_option( 'database_password', $dbpassword );
			} else {
				$db = new wpdb( $dbuser, $dbpassword, $dbname, $dbhost );
				if ( mysqli_connect_errno() ) {
					// Connection Error.
					exit( "Couldn't connect to the database: " . mysqli_connect_error() );
				} else {
					update_option( 'table_prefix', $tableprefix );
					update_option( 'database_name', $dbname );
					update_option( 'database_hostname', $dbhost );
					update_option( 'database_username', $dbuser );
					update_option( 'database_password', $dbpassword );
					ualp_admin_notice_message( 'updated', $message );
					do_action( 'ualp_db_settings_updated' );
					ualp_create_table();
					ualp_create_detail_table();
					ualp_create_external_table();
					ualp_create_detail_external_table();
					ualp_create_login_external_table();
					ualp_create_email_notification_table();
					ualp_create_sms_notification_table();
					ualp_create_email_notification_external_table();
					ualp_create_schedule_report();
					ualp_create_schedule_external_report();
				}
			}
		}
		if ( is_network_admin() && ualp_multi_site_support_return() ) {
			$adminurl = network_admin_url( 'admin.php' );
		} else {
			$adminurl = admin_url( 'admin.php' );
		}
		$qry_str = isset( $_SERVER['QUERY_STRING'] ) ? sanitize_text_field( wp_unslash( $_SERVER['QUERY_STRING'] ) ) : '';
		?>
		<form class="ualpSettingsForm" action="<?php echo esc_attr( $adminurl ) . '?' . esc_attr( $qry_str ); ?>" method="POST">
			<div class="padding_left_right_15">
				<p class="margin_bottom_30 date_error_msg"><b><?php esc_html_e( "Note: Keep below fields blank if you want to store existing data to current site's database.", 'user_activity_log_pro' ); ?></b></p>
				<table>
					<tr><th colspan="2"></th></tr>
					<tr>
						<th><?php esc_html_e( 'Database Username', 'user_activity_log_pro' ); ?></th>
						<td>
							<input type="text" value="<?php echo esc_attr( $dbuser ); ?>" name="database_username">
							<p>
								<?php
									esc_html_e( 'Database username used to connect with database.', 'user_activity_log_pro' );
								?>
							</p>
						</td>
					</tr>
					<tr>
						<th><?php esc_html_e( 'Database Password', 'user_activity_log_pro' ); ?></th>
						<td>
							<input type="text" value="<?php echo esc_attr( $dbpassword ); ?>" name="database_password">
							<p>
								<?php
									esc_html_e( 'Database password used to connect with database.', 'user_activity_log_pro' );
								?>
							</p>
						</td>
					</tr>
					<tr>
						<th><?php esc_html_e( 'Database Hostname', 'user_activity_log_pro' ); ?></th>
						<td>
							<input type="text" value="<?php echo esc_attr( $dbhost ); ?>" name="database_hostname">
							<p>
								<?php
									esc_html_e( 'Database hostname used to connect with database.', 'user_activity_log_pro' );
								?>
							</p>
						</td>
					</tr>
					<tr>
						<th><?php esc_html_e( 'Database Name', 'user_activity_log_pro' ); ?></th>
						<td>
							<input type="text" value="<?php echo esc_attr( $dbname ); ?>" name="database_name">
							<p>
								<?php
									esc_html_e( 'Database name where you want to store user activity log.', 'user_activity_log_pro' );
								?>
							</p>
						</td>
					</tr>
					<tr>
						<th><?php esc_html_e( 'Table Prefix', 'user_activity_log_pro' ); ?></th>
						<td>
							<input type="text" value="<?php echo esc_attr( $tableprefix ); ?>" name="table_prefix">
							<p>
								<?php
									esc_html_e( 'Table prefix where you want to store user activity log.', 'user_activity_log_pro' );
								?>
							</p>
						</td>
					</tr>
				</table>
				<?php
				wp_nonce_field( '_wp_db_settings_nonce', '_wp_db_settings_nonce' );
				?>
				<p class="submit">
					<input type="submit" name="btnSubmitDbData" class="button button-primary" id="submit3" value="<?php esc_attr_e( 'Save Changes', 'user_activity_log_pro' ); ?>" >
					<input type="submit" name="btnResetDbData" class="button-secondary" value="<?php esc_html_e( 'Reset Changes', 'user_activity_log_pro' ); ?>"  onclick="return confirm('<?php esc_js( esc_html_e( 'Are you sure to reset data?', 'user_activity_log_pro' ) ); ?>')" >
				</p>
			</div>
		</form>
		<?php
	}
}

if ( ! function_exists( 'ualp_db_migrate' ) ) {
	/**
	 * User External DB Migration.
	 */
	function ualp_db_migrate() {
		global $wpdb;
		$tableprefix = '';
		$tableprefix = get_option( 'table_prefix' );
		$dbuser      = get_option( 'database_username' );
		$dbpassword  = get_option( 'database_password' );
		$dbname      = get_option( 'database_name' );
		$dbhost      = get_option( 'database_hostname' );
		if ( isset( $_POST['btnDbMigrateData'] ) && isset( $_POST['_wp_migrate_settings_nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wp_migrate_settings_nonce'] ) ), '_wp_migrate_settings_nonce' ) ) {
			if ( ( isset( $dbuser ) && '' != $dbuser ) && ( isset( $dbpassword ) ) && ( isset( $dbname ) && '' != $dbname ) && ( isset( $dbhost ) && '' != $dbhost ) ) {
				$message = esc_html__( 'Data migration from current database to external database has been done successfully.', 'user_activity_log_pro' );
				ualp_admin_notice_message( 'updated', $message );
				ualp_migrate_current_to_external();
				do_action( 'ualp_db_settings_updated' );
			}
		}
		if ( isset( $_POST['btnDbMigrateToCurrentData'] ) && isset( $_POST['btnDbMigrateToCurrentData'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wp_migrate_settings_nonce'] ) ), '_wp_migrate_settings_nonce' ) ) {
			if ( ( isset( $dbuser ) && '' != $dbuser ) && ( isset( $dbpassword ) ) && ( isset( $dbname ) && '' != $dbname ) && ( isset( $dbhost ) && '' != $dbhost ) ) {
				$message = esc_html__( 'Data migration from external database to current database has been done successfully.', 'user_activity_log_pro' );
				ualp_admin_notice_message( 'updated', $message );
				ualp_migrate_external_to_current();
				do_action( 'ualp_db_settings_updated' );
			}
		}
		if ( is_network_admin() && ualp_multi_site_support_return() ) {
			$adminurl = network_admin_url( 'admin.php' );
		} else {
			$adminurl = admin_url( 'admin.php' );
		}
		$qry_str = isset( $_SERVER['QUERY_STRING'] ) ? sanitize_text_field( wp_unslash( $_SERVER['QUERY_STRING'] ) ) : '';
		?>
		<form class="ualpSettingsForm" action="<?php echo esc_attr( $adminurl ) . '?' . esc_attr( $qry_str ); ?>" method="POST">
			<div class="padding_left_right_15">
				<?php
				$tableprefix = '';
				wp_nonce_field( '_wp_migrate_settings_nonce', '_wp_migrate_settings_nonce' );
				$dbuser      = get_option( 'database_username' );
				$dbpassword  = get_option( 'database_password' );
				$dbhost      = get_option( 'database_hostname' );
				$dbname      = get_option( 'database_name' );
				$tableprefix = get_option( 'table_prefix' );
				if ( ! isset( $dbuser ) || '' == $dbuser || ! isset( $dbhost ) || '' == $dbhost || ! isset( $dbname ) || '' == $dbname || ! isset( $tableprefix ) || '' == $tableprefix ) {
					?>
					<p class="margin_bottom_30 date_error_msg"><b><?php esc_html_e( 'Note: Please fill the database integration options to migrate data.', 'user_activity_log_pro' ); ?></b></p>
					<?php
				}
				?>
				<p class="submit">
					<input type="submit" name="btnDbMigrateData" class="button-primary" value="<?php esc_html_e( 'Migrate data from current database to external database', 'user_activity_log_pro' ); ?>"  onclick="return confirm('<?php esc_js( esc_html_e( 'Are you sure to Migrate data from current database to external database?', 'user_activity_log_pro' ) ); ?>')" >&nbsp;&nbsp;&nbsp;
					<input type="submit" name="btnDbMigrateToCurrentData" class="button-secondary" value="<?php esc_html_e( 'Migrate data from external database to current database', 'user_activity_log_pro' ); ?>"  onclick="return confirm('<?php esc_js( esc_html_e( 'Are you sure to Migrate data from external database to current database?', 'user_activity_log_pro' ) ); ?>')" >
				</p>
			</div>
		</form>
		<?php
	}
}
if ( ! function_exists( 'ualp_exclude_log' ) ) {
	/**
	 * Exclude Log.
	 */
	function ualp_exclude_log() {
		global $allhookdetails;
		$args_kses = ualp_args_kses();
		if ( isset( $_POST['btnAddExcludeLog'] ) && isset( $_POST['_wp_exclude_log_nonce'] ) && wp_verify_nonce( sanitize_text_field( wp_unslash( $_POST['_wp_exclude_log_nonce'] ) ), '_wp_exclude_log_action' ) ) {
			$exclude_log_data  = array();
			$exclude_log_users = array();
			$i                 = 0;
			$empty_count       = 0;
			$j                 = 0;
			if ( ( isset( $_POST['exclude_log_users'] ) && '' != $_POST['exclude_log_users'] ) ||
			( isset( $_POST['exclude_logs_hook'] ) && '' != $_POST['exclude_logs_hook'] ) || ( isset( $_POST['exclude_logs_ips'] ) ) ) {
				$exclude_log_users = isset( $_POST['exclude_log_users'] ) ? array_map( 'sanitize_text_field', wp_unslash( $_POST['exclude_log_users'] ) ) : '';
				$exclude_logs_hook = isset( $_POST['exclude_logs_hook'] ) ? array_map( 'sanitize_text_field', wp_unslash( $_POST['exclude_logs_hook'] ) ) : '';
				foreach ( $exclude_log_users as $all_exclude_log_users ) {
					$j++;
					$exclude_logs_ip[] = isset( $_POST[ 'exclude_logs_ips_' . $j ] ) ? array_map( 'sanitize_text_field', wp_unslash( $_POST[ 'exclude_logs_ips_' . $j ] ) ) : '';
				}
			}
			if ( is_array( $exclude_log_users ) && ! empty( $exclude_log_users ) ) {
				$exclude_log_users_count = count( $exclude_log_users );
				if ( $exclude_log_users_count > 0 ) {
					foreach ( $exclude_log_users as $exclude_log_users_single ) {
						if ( '' != $exclude_log_users_single ) {
							$exclude_log_data[] = array(
								'exclude_log_users' => $exclude_log_users[ $i ],
								'exclude_logs_hook' => $exclude_logs_hook[ $i ],
								'exclude_logs_ips'  => $exclude_logs_ip[ $i ],
							);
							$i++;
						}
					}
				}
			}
			if ( is_array( $exclude_log_data ) && ! empty( $exclude_log_data ) ) {
				$exclude_log_data_count = count( $exclude_log_data );
				if ( $exclude_log_data_count >= 1 ) {
					foreach ( $exclude_log_data as $exclude_log_single_data ) {
						$exclude_log  = $exclude_log_single_data['exclude_log_users'];
						$exclude_hook = $exclude_log_single_data['exclude_logs_hook'];
						$excludeips   = $exclude_log_single_data['exclude_logs_ips'];
						if ( '' == $exclude_log || '' == $exclude_hook || '' == $excludeips ) {
							$empty_count++;
						}
					}
				}
			}
			update_option( 'exclude_log_data', $exclude_log_data );
			$class   = 'updated';
			$message = esc_html__( 'Exclude Log saved successfully.', 'user_activity_log_pro' );
			do_action( 'ualp_exclude_log_updated' );
			ualp_admin_notice_message( $class, $message );
		}
		$excludelogdata = get_option( 'exclude_log_data' );
		$adminurl       = admin_url( 'admin.php' );
		$editable_roles = get_editable_roles();
		$users_list     = get_users();
		$post_types     = ualp_posttypes();
		$taxonomies     = ualp_taxonomies();
		$all_hook       = array( 'activated_plugin', 'deactivated_plugin', 'delete_plugin', 'plugin_modify', 'plugin_installed', 'plugin_updated', 'ualp_export_activity_log', 'ualp_general_settings_updated', 'ualp_hook_log_settings_updated', 'ualp_manage_role', 'ualp_set_password', 'ualp_change_password', 'ualp_email_settings_updated', 'ualp_custom_event_log_updated', 'wp_login', 'wp_login_failed', 'wp_logout', 'delete_user', 'user_register', 'insert_user_meta', 'theme_file_modify', 'switch_theme', 'customize_save', 'theme_updated', 'theme_installed', 'delete_site_transient_update_themes', 'wp_insert_comment', 'edit_comment', 'delete_comment', 'trash_comment', 'spam_comment', 'restore_comment', 'comment_approve', 'comment_unapprove', 'wp_create_nav_menu', 'delete_nav_menu', 'admin_head', 'wp_head', '404_page', '_core_updated_successfully', 'import_end', 'export_wp', 'widget_update_callback', 'updated_option', 'dbdelta_queries', 'wp_privacy_personal_data_exporters', 'wp_privacy_personal_data_erasers', 'wp_update_nav_menu', 'pre_insert_term', 'wp_update_term_parent', 'delete_themes', 'before_delete_post', 'ualpSettingsUpdated', 'check_admin_referer' );
		foreach ( $post_types as $post_type ) {
			if ( 'wpcf7_contact_form' != $post_type->name && 'custom_css' != $post_type->name && 'customize_changeset' != $post_type->name && 'attachment' != $post_type->name && 'edd_discount' != $post_type->name && 'oembed_cache' != $post_type->name && 'wp_block' != $post_type->name ) {
				array_push( $all_hook, $post_type->name . '_create', $post_type->name . '_update', $post_type->name . '_delete' );
			}
			if ( 'attachment' == $post_type->name ) {
				array_push( $all_hook, 'add_' . $post_type->name, 'edit_' . $post_type->name, 'delete_' . $post_type->name );
			}
		}
		foreach ( $taxonomies as $taxonomy ) {
			if ( 'wpcf7_contact_form' != $taxonomy->name && 'custom_css' != $taxonomy->name && 'customize_changeset' != $taxonomy->name && 'product_visibility' != $taxonomy->name ) {
				array_push( $all_hook, $taxonomy->name . '_create', $taxonomy->name . '_update', $taxonomy->name . '_delete' );
			}
		}
		if ( class_exists( 'Woocommerce' ) ) {
			array_push( $all_hook, 'woocommerce_updated_option', 'woocommerce_new_product_variation', 'wp_ajax_woocommerce_remove_variations', 'woocommerce_save_product_variation' );
		}
		if ( is_plugin_active( 'buddypress/bp-loader.php' ) ) {
			array_push( $all_hook, 'groups_create_group', 'groups_update_group', 'groups_before_delete_group', 'groups_leave_group', 'groups_join_group', 'groups_promote_member', 'groups_demote_member', 'groups_ban_member', 'groups_unban_member', 'groups_remove_member', 'xprofile_field_after_save', 'xprofile_fields_deleted_field', 'xprofile_group_after_save', 'xprofile_groups_deleted_group' );
		}
		if ( is_plugin_active( 'easy-digital-downloads/easy-digital-downloads.php' ) ) {
			array_push( $all_hook, 'edd_activate_discount', 'edd_deactivate_discount', 'edd_add_discount', 'edd_delete_discount', 'transition_post_status' );
		}
		if ( is_plugin_active( 'gravityforms/gravityforms.php' ) ) {
			array_push( $all_hook, 'gf_export_forms', 'notification_duplicated', 'gform_entry_created', 'gf_resend_notifications', 'gform_pre_note_deleted', 'gform_post_export_entries', 'gform_post_note_added', 'gform_delete_entries', 'gform_post_form_views_deleted', 'gform_post_form_activated', 'gform_post_form_deactivated', 'gform_before_delete_form', 'gform_post_form_trashed', 'gform_post_form_restored', 'gform_post_form_duplicated', 'gform_after_save_form', 'gform_forms_post_import', 'gform_pre_notification_deleted', 'gform_pre_confirmation_deleted', 'wp_ajax_rg_update_notification_active', 'wp_ajax_rg_update_confirmation_active', 'gform_pre_confirmation_save', 'gform_pre_notification_save', 'gform_delete_lead', 'gform_update_status', 'gform_update_is_read', 'gform_update_is_starred', 'gravityforms_updated_option' );
		}
		if ( is_plugin_active( 'wordfence/wordfence.php' ) ) {
			array_push( $all_hook, 'wordfence_updateIssueStatus', 'wordfence_saveWAFConfig', 'wordfence_wafConfigureAutoPrepend', 'wordfence_whitelistBulkEnable', 'wordfence_whitelistBulkDisable', 'wordfence_whitelistBulkDelete', 'wordfence_updateConfig', 'wordfence_loadLiveTraffic', 'wordfence_loadCacheExclusions', 'wordfence_saveCacheConfig', 'wordfence_cache_clear', 'wordfence_reverseLookup', 'wordfence_whois', 'wordfence_clearPageCache', 'wordfence_saveConfig', 'wordfence_blockIP', 'wordfence_addTwoFactor', 'wordfence_exportSettings', 'wordfence_clearAllBlocked', 'wordfence_blockIPUARange', 'wordfence_unblockRange', 'wordfence_saveCountryBlocking', 'wordfence_saveDebuggingConfig', 'wordfence_startPasswdAudit', 'wordfence_saveScanSchedule', 'wordfence_updateWAFRules', 'wordfence_sendTestEmail', 'wordfence_sendDiagnostic', 'wordfence_updateAllIssues', 'wordfence_sendActivityLog', 'wordfence_addCacheExclusion', 'wordfence_daily_autoUpdate', 'wordfence_update_blocked_IPs', 'wordfence_start_scheduled_scan', 'wordfence_doScan', 'wordfence_daily_cron', 'wordfence_hourly_cron', 'wordfence_email_activity_report', 'wordfence_scan', 'wordfence_updateAlertEmail', 'wordfence_restoreFile', 'wordfence_deletePasswdAudit', 'wordfence_weakPasswordsFix', 'wordfence_killPasswdAudit', 'wordfence_importSettings', 'wordfence_bulkOperation', 'wordfence_deleteFile', 'wordfence_deleteDatabaseOption', 'wordfence_removeExclusion', 'wordfence_deleteIssue', 'wordfence_unlockOutIP', 'wordfence_unblockIP', 'wordfence_permBlockIP', 'wordfence_downloadHtaccess', 'wordfence_checkFalconHtaccess', 'wordfence_removeFromCache', 'wordfence_autoUpdateChoice', 'wordfence_saveCacheOptions', 'wordfence_adminEmailChoice', 'wordfence_getCacheStats', 'wordfence_killScan', 'wordfence_tourClosed', 'wordfence_removeCacheExclusion', 'wordfence_unblockNetwork', 'wordfence_permanentlyBlockAllIPs', 'wordfence_deleteAdminUser', 'wordfence_revokeAdminUser', 'wordfence_hideFileHtaccess', 'wordfence_loadBlockRanges' );
		}
		if ( is_plugin_active( 'wordpress-seo/wp-seo.php' ) ) {
			array_push( $all_hook, 'wpseo_handle_import', 'wpseo_import', 'wpseo_save_auth_code', 'updated_post_meta' );
		}
		if ( is_plugin_active( 'contact-form-7/wp-contact-form-7.php' ) ) {
			array_push( $all_hook, 'wpcf7_after_create' );
			array_push( $all_hook, 'wpcf7_after_update' );
			array_push( $all_hook, 'wpcf7_admin_notices' );
		}
		if ( is_plugin_active( 'blog-designer-pro/blog-designer-pro.php' ) ) {
			array_push( $all_hook, 'bdp_update_shortcode', 'bdp_reset_shortcode', 'bdp_reset_archive_page', 'bdp_add_shortcode', 'bdp_update_single_file', 'bdp_add_archive_layout', 'bdp_delete_shortcode', 'bdp_delete_single_template', 'bdp_copy_single_template', 'bdp_duplicate_layout_settings', 'bdp_import_blog_layout_settings', 'bdp_import_archive_layout_settings', 'bdp_export_blog_layout_settings', 'bdp_export_archive_layout_settings', 'bdp_update_archive_page', 'bdp_import_single_layout_settings', 'bdp_export_single_layout_settings', 'bdp_delete_archive', 'bdp_delete_single_layout', 'bdp_update_single_page', 'bdp_reset_single_page', 'bdp_add_single_layout', 'bdp_duplicate_single_layout_settings' );
		}
		if ( is_plugin_active( 'portfolio-designer/portfolio-designer.php' ) ) {
			array_push( $all_hook, 'import_sample_portfolio_data', 'create_portfolio_layout', 'duplicate_portfolio_layout', 'delete_portfolio_layout', 'update_portfolio_layout', 'reset_portfolio_layout', 'create_post_type', 'update_post_type', 'export_portfolio_layout', 'delete_post_type' );
		}
		if ( is_plugin_active( 'all-in-one-reservation/all-in-one-reservation.php' ) ) {
			array_push( $all_hook, 'save_reservation_settings', 'rest_update_reservation_status', 'reset_reservation_settings' );
		}
		if ( is_plugin_active( 'wc-checkout-fields-editor/wc-checkout-fields-editor.php' ) ) {
			array_push( $all_hook, 'save_wcfe_settings', 'wp_ajax_wcfe_get_checkout_required_fields' );
		}
		if ( is_plugin_active( 'easy-digital-downloads/easy-digital-downloads.php' ) ) {
			array_push( $all_hook, 'edd_activate_discount', 'edd_settings', 'edd_deactivate_discount', 'edd_add_discount', 'edd_delete_discount' );
		}
		if ( is_plugin_active( 'bbpress/bbpress.php' ) ) {
			array_push( $all_hook, 'bbpress_settings' );
		}
		if ( is_plugin_active( 'autoptimize/autoptimize.php' ) ) {
			array_push( $all_hook, 'wp_ajax_autoptimize_delete_cache', 'autoptimizer_settings' );
		}
		if ( is_plugin_active( 'users-ultra/xoousers.php' ) ) {
			array_push( $all_hook, 'wp_ajax_save_fields_settings', 'wp_ajax_delete_profile_field', 'wp_ajax_sort_fileds_list', 'wp_ajax_custom_fields_reset', 'wp_ajax_create_uploader_folder', 'wp_ajax_package_add_new', 'wp_ajax_package_delete', 'wp_ajax_package_edit_confirm', 'wp_ajax_sync_users', 'wp_ajax_user_resend_activation_link', 'wp_ajax_delete_uultra_transtient', 'wp_ajax_delete_uultra_stats', 'wp_ajax_delete_uultra_reviews', 'wp_ajax_edit_photo_cate_conf', 'wp_ajax_edit_photo_cate_del', 'wp_ajax_edit_photo_cate', 'userultra_admin_menu_hook' );
		}
		if ( is_plugin_active( 'user-switching/user-switching.php' ) ) {
			array_push( $all_hook, 'switch_to_user', 'switch_back_user', 'switch_off_user' );
		}
		if ( is_plugin_active( 'advanced-custom-fields/acf.php' ) ) {
			array_push( $all_hook, 'export_acf' );
		}
		global $externaldb;
		$ips     = $externaldb->get_results(
			'SELECT DISTINCT ip_address FROM ' . UALP_TABLENAME . '
                WHERE 1 = 1
                ORDER BY ip_address
        '
		);
		$qry_str = isset( $_SERVER['QUERY_STRING'] ) ? sanitize_text_field( wp_unslash( $_SERVER['QUERY_STRING'] ) ) : '';
		?>
	<form class="ualpSettingsForm" action="<?php echo esc_attr( $adminurl ) . '?' . esc_attr( $qry_str ); ?>" method="POST" >
		<div class="padding_left_right_15">
			<h3 class="ualpHeaderTitle"><?php esc_html_e( 'Add Exclude Log Rules', 'user_activity_log_pro' ); ?></h3>
			<div>
				<div class="button ualp_add_new_exclude_log_btn">+ <?php esc_html_e( 'Add Log', 'user_activity_log_pro' ); ?></div>
				<table class="wp-list-table widefat custom_exclude_log_table fixed">
				<thead>
					<tr>
						<td style="width:20px;"><input type="checkbox" id="cb-all-exclude-select" name="all_exclude_log"></td>
						<td style="width:33.33%;"><?php esc_html_e( 'Author or role', 'user_activity_log_pro' ); ?></td>
						<td style="width:33.33%;"><?php esc_html_e( 'Activity', 'user_activity_log_pro' ); ?></td>
						<?php if ( $ips && ualp_allow_ip() ) { ?>
							<td style="width:25%;"><?php esc_html_e( 'IP Address', 'user_activity_log_pro' ); ?></td>
							<?php
						}
						?>
						<td style="width:33.33%;"><?php esc_html_e( 'Action', 'user_activity_log_pro' ); ?></td>
					</tr>
				</thead>
				<tbody>
					<?php
					$j = 1;
					if ( is_array( $excludelogdata ) && ! empty( $excludelogdata ) ) {
						$i = 0;
						foreach ( $excludelogdata as $excludelogdatasingle ) {
							$i++;
							?>
							<tr class="log_show">
								<td style="width:20px;"><input type="checkbox" id="cb-exclude-select" name="exclude_log[]"></td>
								<td>
									<select style="width:100%;" class="chosen-filter" name="exclude_log_users[]">
										<option value="all"><?php esc_html_e( 'All Users and Roles', 'user_activity_log_pro' ); ?></option>                       
										<optgroup class="select2-result-selectable" label="Roles">
											<?php
											$editable_roles = get_editable_roles();
											foreach ( $editable_roles as $role => $details ) {
												$role_label = esc_attr( $role );
												?>
												<option value="<?php echo esc_attr( $role ); ?>" <?php selected( $excludelogdatasingle['exclude_log_users'], $role ); ?>><?php echo esc_html( $role ); ?></option>
												<?php
											}
											?>
													
										</optgroup>                                 
										<optgroup class="select2-result-selectable" label="User">                
										<?php
										foreach ( $users_list as $data ) {
											$user_id   = $data->ID;
											$user_name = $data->user_login;
											?>
											<option value="<?php echo esc_attr( $user_name ); ?>"  <?php selected( $excludelogdatasingle['exclude_log_users'], $user_name ); ?>><?php echo esc_html( $user_name ); ?></option>
											<?php
										}
										?>
											<option value="0" <?php selected( $excludelogdatasingle['exclude_log_users'], '0' ); ?>>Guest Users(Not Logged In Users)</option>               
										</optgroup>                                                    
									</select>
								</td>
								<td>
								<?php

								echo ' <select name="exclude_logs_hook[]" class="chosen-filter" style="width:100%;">';
								echo '<option value="all">All Activity</option>';
								foreach ( $all_hook as $hook ) {
									$label = $hook;
									foreach ( $allhookdetails as $key => $val ) {
										if ( $hook == $key ) {
											$label = $val;
										}
									}
									echo '<option value="' . esc_attr( $hook ) . '" ' . selected( $excludelogdatasingle['exclude_logs_hook'], $hook ) . '>' . esc_html( $label ) . '</option>';
								}
								echo '</select>';
								?>
								</td>
								<?php
								if ( $ips && ualp_allow_ip() ) {
									$output = array();
									foreach ( $ips as $ip ) {
										if ( empty( $excludelogdatasingle['exclude_logs_ips'] ) ) {
											$excludelogdatasingle['exclude_logs_ips'] = array();
										}
										if ( isset( $ip->ip_address ) && '' != $ip->ip_address ) {
											$output[] = sprintf( '<option value="%s" %s>%s</option>', $ip->ip_address, in_array( $ip->ip_address, $excludelogdatasingle['exclude_logs_ips'] ) ? 'selected="selected"' : '', $ip->ip_address );
										}
									}
									?>
									<td>
									<?php
									echo ' <select name="exclude_logs_ips_' . esc_attr( $i ) . '[]" class="chosen-filter" style="width:100%;" data-placeholder="' . esc_attr__( 'All Ip-address', 'user_activity_log_pro' ) . '" multiple>';
									echo wp_kses( implode( '', array_unique( $output ) ), $args_kses );
									echo '</select>';
									?>
									</td>
									<?php
								}
								?>
								<td><a class="exclude_log_remove" href="#">Delete</a></td>
							</tr>

							<?php
						}
					}
					?>
						<tr class="hidden_log"><td class="colspanchange" colspan="5"><?php esc_html_e( 'No rules found.', 'user_activity_log_pro' ); ?></td></tr>		 
				</tbody>
				<tfoot>
					<tr>
						<td style="width:20px;"><input type="checkbox" id="cb-all-exclude-select" name="all_exclude_log"></td>
						<td style="width:33.33%;"><?php esc_html_e( 'Author or role', 'user_activity_log_pro' ); ?></td>
						<td style="width:33.33%;"><?php esc_html_e( 'Activity', 'user_activity_log_pro' ); ?></td>
						<?php if ( $ips && ualp_allow_ip() ) { ?>
							<td style="width:25%;"><?php esc_html_e( 'IP Address', 'user_activity_log_pro' ); ?></td>
							<?php
						}
						?>
						<td style="width:33.33%;"><?php esc_html_e( 'Action', 'user_activity_log_pro' ); ?></td>
					</tr>
				</tfoot>
				</table>
				<?php
				wp_nonce_field( '_wp_exclude_log_action', '_wp_exclude_log_nonce' );
				?>
				<input type="button" class="button delete_exclude_log" value="Delete Selected Rules" disabled="">
				<p class="submit">
					<input class="button button-primary" type="submit" value="<?php esc_attr_e( 'Save Changes', 'user_activity_log_pro' ); ?>" name="btnAddExcludeLog">
				</p>
			</div>

	</div>
	<script>
		/*add new and remove custom event functionality*/
		jQuery(document).ready(function () {
			var rows= jQuery('.custom_exclude_log_table tbody tr.log_show').length;
			jQuery('.custom_exclude_log_table tbody tr.log_show:even').addClass('alternate');
			if(rows == 0 ) {
				jQuery('.hidden_log').show();
			} else {
				jQuery('.hidden_log').hide();
			}
			jQuery('.ualp_add_new_exclude_log_btn').click(function () {
				var rows= jQuery('.custom_exclude_log_table tbody tr.log_show').length;
				var selectrow= rows+1;
				var new_row = '<tr class="log_show">';
				new_row +='<td style="width:20px;"><input type="checkbox" id="cb-exclude-select" name="exclude_log[]"></td>';

				new_row +='<td><select style="width:100%;" class="chosen-filter" name="exclude_log_users[]"><option value="all"><?php esc_html_e('All Users and Roles', 'user_activity_log_pro'); ?></option><optgroup class="select2-result-selectable" label="Roles"><?php $editable_roles = get_editable_roles();
                foreach($editable_roles as $role => $details){
                    $role_label = esc_attr($role);
                    ?><option value="<?php echo esc_attr( $role ); ?>"><?php echo esc_html( $role ); ?></option><?php
                }
                ?></optgroup><optgroup class="select2-result-selectable" label="User"><?php 
                foreach ($users_list as $data) {
                $user_id = $data->ID;
                $user_name = $data->user_login;
                ?><option value="<?php echo esc_attr( $user_name ); ?>"><?php echo esc_html( $user_name ); ?></option><?php } ?><option value="0">Guest Users(Not Logged In Users)</option></optgroup> </select></td>';
                new_row +='<td> <?php 
                        echo ' <select name="exclude_logs_hook[]" class="chosen-filter" style="width:100%;">';
                        echo '<option value="all">All Activity</option>';
                        foreach ($all_hook as $hook) {
                            $label = $hook;
                            foreach ($allhookdetails as $key => $val) {
                                if($hook == $key) {
                                    $label = $val;
                                }
                            }
                            echo '<option value="'.esc_attr( $hook ).'">'.esc_html( $label ).'</option>';
                        }
                        echo '</select>';
                        ?></td>';
               <?php if ($ips && ualp_allow_ip()) { ?>
                new_row +='<td><select name="exclude_logs_ips_'+selectrow+'[]" class="chosen-filter" style="width:100%;" data-placeholder="All Ip-address" multiple> <?php
                foreach ($ips as $ip) {
                         
                    if(isset($ip->ip_address) && $ip->ip_address != ''){
                        echo '<option value="'.esc_attr( $ip->ip_address ).'" >'.esc_html( $ip->ip_address ).'</option>';
                    }
                }
                echo '</select>';?></td>';
			   <?php } ?>
				new_row +='<td><a class="exclude_log_remove" href="#">Delete</a></td>';
				new_row += '</tr>';
				jQuery('.custom_exclude_log_table tbody').append(new_row);
				jQuery('.custom_exclude_log_table tbody tr:even').addClass('alternate');
				jQuery('.chosen-filter').select2();
				var final_rows= jQuery('.custom_exclude_log_table tbody tr.log_show').length;
				if(final_rows == 0 ) {
					jQuery('.hidden_log').show();
				} else {
					jQuery('.hidden_log').hide();
				}
				jQuery('.exclude_log_remove').on('click', function () {
					jQuery(this).closest('tr').remove();
					var rows= jQuery('.custom_exclude_log_table tbody tr.log_show').length;
					if(rows == 0 ) {
						jQuery('.hidden_log').show();
					}
				});
			});
			jQuery('.exclude_log_remove').on('click', function () {
				jQuery(this).closest('tr').remove();
				var rows= jQuery('.custom_exclude_log_table tbody tr.log_show').length;
				if(rows == 0 ) {
					jQuery('.hidden_log').show();
				}
			});
		});
	</script>
	</form>
		<?php
	}
}
